@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2000 David Faure <faure@kde.org>
+   Copyright (C) 2004 Nicolas GOUTTE <goutte@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -13,30 +14,40 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
-#ifndef CSVEXPORT_TEST_H
-#define CSVEXPORT_TEST_H
+#ifndef CSVEXPORT_H
+#define CSVEXPORT_H
 
-#include <koFilter.h>
+#include <QString>
+//Added by qt3to4:
+#include <QByteArray>
+#include <KoFilter.h>
 
-class QCString;
-class QString;
-class KoDocument;
-
-class CSVExport : public KoFilter {
+namespace KSpread
+{
+class Sheet;
+class Doc;
+}
 
+class CSVExport : public KoFilter
+{
     Q_OBJECT
 
 public:
-    CSVExport(KoFilter *parent, const char*name);
+    CSVExport(QObject* parent, const QStringList &);
     virtual ~CSVExport() {}
-    /** do the filtering :) */
-    virtual const bool E_filter(const QString &file, const KoDocument * const document,
-                                const QString &from, const QString &to,
-                                const QString &config=QString::null);
-};
-#endif
 
+    virtual KoFilter::ConversionStatus convert(const QByteArray & from, const QByteArray & to);
+
+private:
+    QString exportCSVCell(const KSpread::Doc* doc, KSpread::Sheet const * const sheet,
+                          int col, int row, QChar const & textQuote, QChar csvDelimiter);
+
+private:
+    QString m_eol; ///< End of line (LF, CR or CRLF)
+};
+
+#endif // CSVEXPORT_H
