@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2002 Ariya Hidayat <ariyahidayat@yahoo.de>
+   Copyright (C) 2002 Ariya Hidayat <ariya@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 #ifndef __AMIPROIMPORT_H
 #define __AMIPROIMPORT_H
 
-#include <qstring.h>
+#include <QString>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <KoFilter.h>
 #include <KoStore.h>
@@ -29,14 +31,14 @@ class AmiProImport : public KoFilter
 {
     Q_OBJECT
 
-  public:
+public:
 
-    AmiProImport(KoFilter *parent, const char *name, const QStringList&);
+    AmiProImport(QObject* parent, const QStringList&);
     AmiProImport();
 
     virtual ~AmiProImport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // __AMIPROIMPORT_H
