@@ -23,22 +23,20 @@
 #include <KoFilter.h>
 #include <KoStore.h>
 
-#include <qcstring.h>
-
-class HancomWordImport : public KoFilter 
+class HancomWordImport : public KoFilter
 {
-Q_OBJECT
+    Q_OBJECT
 
 public:
 
-    HancomWordImport ( QObject *parent, const char* name, const QStringList& );
+    HancomWordImport(QObject *parent, const QStringList&);
     virtual ~HancomWordImport();
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 
 private:
-  class Private;
-  Private* d;
+    class Private;
+    Private* d;
 };
 
 #endif // HANCOMWORDIMPORT_H
