@@ -21,8 +21,8 @@
 #ifndef KWORD_1_3_IMPORT_H
 #define KWORD_1_3_IMPORT_H
 
-#include <qstring.h>
-#include <qcstring.h>
+#include <QString>
+#include <q3cstring.h>
 
 #include <KoFilter.h>
 
@@ -30,18 +30,19 @@ class QIODevice;
 class KoStore;
 class KWord13Document;
 
-class KWord13Import : public KoFilter {
+class KWord13Import : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    KWord13Import(KoFilter *parent, const char *name, const QStringList &);
+    KWord13Import(QObject* parent, const QStringList &);
     virtual ~KWord13Import() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 protected:
-    bool parseInfo( QIODevice* io, KWord13Document& kwordDocument );
-    bool parseRoot( QIODevice* io, KWord13Document& kwordDocument );
-    bool postParse( KoStore* store, KWord13Document& doc );    
+    bool parseInfo(QIODevice* io, KWord13Document& kwordDocument);
+    bool parseRoot(QIODevice* io, KWord13Document& kwordDocument);
+    bool postParse(KoStore* store, KWord13Document& doc);
 };
 #endif // KWORD_1_3_IMPORT_H
