@@ -20,10 +20,12 @@
 #ifndef __LATEXIMPORT_H__
 #define __LATEXIMPORT_H__
 
-#include <qstring.h>
-#include <qtextstream.h>
-#include <qfile.h>
-#include <qobject.h>
+#include <QString>
+#include <QTextStream>
+#include <QFile>
+#include <QObject>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <KoFilter.h>
 #include <KoStore.h>
@@ -31,13 +33,13 @@
 class LATEXImport : public KoFilter
 {
 
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-		LATEXImport(KoFilter *parent, const char *name, const QStringList&);
-		virtual ~LATEXImport() {}
+public:
+    LATEXImport(QObject* parent, const QStringList&);
+    virtual ~LATEXImport() {}
 
-		virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif /* __LATEXIMPORT_H__ */
