@@ -35,7 +35,7 @@
 #define EXPORTTAGPROCESSING_H
 
 #include <qdom.h>
-#include <qvaluelist.h>
+#include <QList>
 
 #include <kdemacros.h>
 #include <kdebug.h>
@@ -54,33 +54,30 @@ class KWEFKWordLeader;
 
 class TagProcessing
 {
-    public:
-        TagProcessing ()
-        {}
+public:
+    TagProcessing() {}
 
-        TagProcessing (const QString& n,
-                       void (*p)(QDomNode, void *, KWEFKWordLeader*),
-                       void *d) : name (n), processor (p), data (d)
-        {}
+    TagProcessing(const QString& n,
+                  void (*p)(QDomNode, void *, KWEFKWordLeader*),
+                  void *d) : name(n), processor(p), data(d) {}
 
-        /**
-         * @brief No-op constructor
-         *
-         * Avoids a warning that a tag is unknown.
-         */
-        TagProcessing (const QString& _name) : name(_name), processor(0), data(0)
-        {}
+    /**
+     * @brief No-op constructor
+     *
+     * Avoids a warning that a tag is unknown.
+     */
+    TagProcessing(const QString& _name) : name(_name), processor(0), data(0) {}
 
-        QString  name;
-        void     (*processor)(QDomNode, void *, KWEFKWordLeader*);
-        void    *data;
+    QString  name;
+    void (*processor)(QDomNode, void *, KWEFKWordLeader*);
+    void    *data;
 };
 
-void ProcessSubtags     ( const QDomNode             &parentNode,
-                          QValueList<TagProcessing>  &tagProcessingList,
-                          KWEFKWordLeader            *leader);
+void ProcessSubtags(const QDomNode             &parentNode,
+                    QList<TagProcessing>  &tagProcessingList,
+                    KWEFKWordLeader            *leader);
 
-void AllowNoSubtags ( const QDomNode& myNode, KWEFKWordLeader *leader );
+void AllowNoSubtags(const QDomNode& myNode, KWEFKWordLeader *leader);
 
 
 /**
@@ -97,43 +94,42 @@ class AttrProcessing
 {
 public:
 
-    enum AttrType
-        { AttrNull = 0, AttrInt, AttrDouble, AttrBool, AttrQString };
+    enum AttrType { AttrNull = 0, AttrInt, AttrDouble, AttrBool, AttrQString };
 
     /**
      * @brief Default constructor
      */
-    AttrProcessing () : type( AttrNull ), data(0) {}
+    AttrProcessing() : type(AttrNull), data(0) {}
 
     /**
      * @brief Old constructor (without type checking)
      * @deprecated do not use for new code
      */
-    AttrProcessing ( const QString& n, const QString& t, void *d ) KDE_DEPRECATED;
+    KDE_CONSTRUCTOR_DEPRECATED AttrProcessing(const QString& n, const QString& t, void *d);
 
     /**
      * @brief No-op constructor
      *
      * This just avoids that a warning is triggered that an attribute is not known
      */
-    AttrProcessing ( const QString& _name )
-        : name( _name ), type( AttrNull ), data( 0 ) {}
+    AttrProcessing(const QString& _name)
+            : name(_name), type(AttrNull), data(0) {}
 
     /**
      * @brief Integer constructor
      *
      * Get the attribute value as an integer
      */
-    AttrProcessing ( const QString& _name, int& i )
-        : name( _name ), type( AttrInt ), data( &i ) {}
+    AttrProcessing(const QString& _name, int& i)
+            : name(_name), type(AttrInt), data(&i) {}
 
     /**
      * @brief Double constructor
      *
      * Get the attribute value as a double
      */
-    AttrProcessing ( const QString& _name, double& d )
-        : name( _name ), type( AttrDouble ), data( &d ) {}
+    AttrProcessing(const QString& _name, double& d)
+            : name(_name), type(AttrDouble), data(&d) {}
 
     /**
      * @brief Boolean constructor
@@ -141,16 +137,16 @@ public:
      * Get the attribute value as a bool
      * (The values "yes,", "no", "true", "false", "0" and "1" are supported)
      */
-    AttrProcessing ( const QString& _name, bool& flag )
-        : name( _name ), type( AttrBool ), data( &flag ) {}
+    AttrProcessing(const QString& _name, bool& flag)
+            : name(_name), type(AttrBool), data(&flag) {}
 
     /**
      * @brief String constructor
      *
      * Get the attribute value as a QString
      */
-    AttrProcessing ( const QString& _name, QString& str )
-        : name( _name ), type( AttrQString ), data( &str ) {}
+    AttrProcessing(const QString& _name, QString& str)
+            : name(_name), type(AttrQString), data(&str) {}
 
 
 public:
@@ -159,9 +155,9 @@ public:
     void     *data;
 };
 
-void ProcessAttributes ( const QDomNode              &myNode,
-                         QValueList<AttrProcessing>  &attrProcessingList );
+void ProcessAttributes(const QDomNode              &myNode,
+                       QList<AttrProcessing>  &attrProcessingList);
 
-void AllowNoAttributes ( const QDomNode& myNode );
+void AllowNoAttributes(const QDomNode& myNode);
 
 #endif
