@@ -21,24 +21,27 @@
 #ifndef RTFEXPORT_H
 #define RTFEXPORT_H
 
-#include <qstring.h>
-#include <qtextstream.h>
-#include <qfile.h>
-#include <qobject.h>
+#include <QString>
+#include <QTextStream>
+#include <QFile>
+#include <QObject>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <KoFilter.h>
 #include <KoStore.h>
 
 
-class RTFExport : public KoFilter {
+class RTFExport : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    RTFExport(KoFilter *parent, const char *name, const QStringList &);
+    RTFExport(QObject* parent, const QStringList &);
     virtual ~RTFExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // RTFEXPORT_H
