@@ -21,33 +21,35 @@
 
 #include <starwriterimport.h>
 
-#include <qdatastream.h>
-#include <qiodevice.h>
-#include <qstring.h>
+#include <QDataStream>
+#include <QIODevice>
+#include <QString>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <KoFilterChain.h>
 #include <kgenericfactory.h>
 
 #include "pole.h"
 
-typedef KGenericFactory<StarWriterImport, KoFilter> StarWriterImportFactory;
+typedef KGenericFactory<StarWriterImport> StarWriterImportFactory;
 K_EXPORT_COMPONENT_FACTORY(libstarwriterimport, StarWriterImportFactory("kofficefilters"))
 
 // Get unsigned 24-bits integer at given offset
-static inline Q_UINT32 readU24(QByteArray array, Q_UINT32 p)
+static inline quint32 readU24(const QByteArray& array, quint32 p)
 {
-   Q_UINT8* ptr = (Q_UINT8*) array.data();
-   return (Q_UINT32) (ptr[p] + (ptr[p+1] << 8) + (ptr[p+2] << 16));
+    quint8* ptr = (quint8*) array.data();
+    return (quint32)(ptr[p] + (ptr[p+1] << 8) + (ptr[p+2] << 16));
 }
 
 // Get unsigned 16-bits integer at given offset
-static inline Q_UINT16 readU16(QByteArray array, Q_UINT32 p)
+static inline quint16 readU16(const QByteArray& array, quint32 p)
 {
-   Q_UINT8* ptr = (Q_UINT8*) array.data();
-   return (Q_UINT16) (ptr[p] + (ptr[p+1] << 8));
+    quint8* ptr = (quint8*) array.data();
+    return (quint16)(ptr[p] + (ptr[p+1] << 8));
 }
 
-StarWriterImport::StarWriterImport(KoFilter *, const char *, const QStringList&) : KoFilter()
+StarWriterImport::StarWriterImport(KoFilter *, const char *, const QStringList&) : KoFilter(parent)
 {
     hasHeader = false;
     hasFooter = false;
@@ -58,7 +60,7 @@ StarWriterImport::~StarWriterImport()
 {
 }
 
-KoFilter::ConversionStatus StarWriterImport::convert(const QCString& from, const QCString& to)
+KoFilter::ConversionStatus StarWriterImport::convert(const QByteArray& from, const QByteArray& to)
 {
     // Check for proper conversion
     // When 4.x is supported, use also: || (from != "application/x-starwriter")
@@ -67,7 +69,7 @@ KoFilter::ConversionStatus StarWriterImport::convert(const QCString& from, const
 
     // Read streams
     POLE::Storage storage;
-    storage.open(m_chain->inputFile().latin1());
+    storage.open(m_chain->inputFile().toLatin1());
 
     POLE::Stream* stream;
 
@@ -99,9 +101,9 @@ KoFilter::ConversionStatus StarWriterImport::convert(const QCString& from, const
     // Prepare storage device and return
     KoStoreDevice *out = m_chain->storageFile("maindoc.xml", KoStore::Write);
     if (out) {
-        QCString cstring = maindoc.utf8();
+        QByteArray cstring = maindoc.utf8();
         cstring.prepend("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-        out->writeBlock((const char*) cstring, cstring.length());
+        out->write((const char*) cstring, cstring.length());
     }
 
     return KoFilter::OK;
@@ -129,7 +131,7 @@ bool StarWriterImport::checkDocumentVersion()
     if (StarWriterDocument[0x05] != 'R') return false;
 
     // Password-protection is not supported for the moment
-    Q_UINT16 flags = readU16(StarWriterDocument, 0x0A);
+    quint16 flags = readU16(StarWriterDocument, 0x0A);
     if (flags & 0x0008) return false;
 
     return true;
@@ -183,8 +185,8 @@ bool StarWriterImport::addBody()
 {
     // Find the starting point, by:
     // 1. skipping the header
-    Q_UINT32 len = StarWriterDocument[0x07];
-    Q_UINT32 p = len;
+    quint32 len = StarWriterDocument[0x07];
+    quint32 p = len;
 
     // 2. skipping 8 more bytes
     p += 0x08;
@@ -192,16 +194,17 @@ bool StarWriterImport::addBody()
     // 3. skipping useless sections
     char c = StarWriterDocument[p];
     while (c != 'N') {
-        len = readU24(StarWriterDocument, p+1);
+        len = readU24(StarWriterDocument, p + 1);
         p += len;
         c = StarWriterDocument[p];
-    };   // there is at least one empty paragraph!
+    }
+    ;   // there is at least one empty paragraph!
 
     // Select nodes and pass them to parseNodes()
-    len = readU24(StarWriterDocument, p+1);
+    len = readU24(StarWriterDocument, p + 1);
     QByteArray data(len);
-    for (Q_UINT32 k=0; k<len; k++)
-      data[k] = StarWriterDocument[p+k];
+    for (quint32 k = 0; k < len; k++)
+        data[k] = StarWriterDocument[p+k];
     bool retval = parseNodes(data);
 
     // add proper tags
@@ -216,50 +219,50 @@ QString StarWriterImport::convertToKWordString(QByteArray s)
 {
     QString result;
 
-    for (Q_UINT32 i = 0x00; i < s.size(); i++)
+    for (quint32 i = 0x00; i < s.size(); i++)
         if (s[i] == '&') result += "&amp;";
         else if (s[i] == '<') result += "&lt;";
         else if (s[i] == '>') result += "&gt;";
         else if (s[i] == '"') result += "&quot;";
         else if (s[i] == 0x27) result += "&apos;";
-        else if (s[i] == 0x09) result += "\t";
-        // FIXME: more to add here
-        //        (manual breaks, soft-hyphens, non-breaking spaces, variables)
+        else if (s[i] == 0x09) result += '\t';
+    // FIXME: more to add here
+    //        (manual breaks, soft-hyphens, non-breaking spaces, variables)
         else result += QChar(s[i]);
 
     return result;
 }
 
-bool StarWriterImport::parseNodes(QByteArray n)
+bool StarWriterImport::parseNodes(const QByteArray& n)
 {
     QByteArray s;
-    Q_UINT32 len, p;
+    quint32 len, p;
 
     // Loop
     p = 0x09;   // is this a fixed value? is it the same for headers/footers?
 
     while (p < n.size()) {
         char c = n[p];
-        len = readU24(n, p+1);
+        len = readU24(n, p + 1);
 
         s.resize(len);
-        for (Q_UINT32 k = 0x00; k < len; k++)
+        for (quint32 k = 0x00; k < len; k++)
             s[k] = n[p+k];
 
         switch (c) {
-            case 'T':
-                //if ((s[0x0A] == 0x01) && (s[0x0B] == 0x00) && (s[0x0C] == 0xFF)) {
-                //    if (!parseGraphics(s)) return false;
-                //}
-                //else {
-                    if (!parseText(s)) return false;
-                //}
-                break;
-            case 'E':
-                if (!parseTable(s)) return false;
-                break;
-            default:
-                break;
+        case 'T':
+            //if ((s[0x0A] == 0x01) && (s[0x0B] == 0x00) && (s[0x0C] == 0xFF)) {
+            //    if (!parseGraphics(s)) return false;
+            //}
+            //else {
+            if (!parseText(s)) return false;
+            //}
+            break;
+        case 'E':
+            if (!parseTable(s)) return false;
+            break;
+        default:
+            break;
         };
         p += len;
     };
@@ -267,20 +270,20 @@ bool StarWriterImport::parseNodes(QByteArray n)
     return true;
 }
 
-bool StarWriterImport::parseText(QByteArray n)
+bool StarWriterImport::parseText(const QByteArray& n)
 {
     QByteArray s;
-    Q_UINT16 len;
-    Q_UINT32 p;
+    quint16 len;
+    quint32 p;
     QString text;
-    // Q_UINT16 attributeStart, attributeEnd, formatPos, formatLen;
+    // quint16 attributeStart, attributeEnd, formatPos, formatLen;
     // QString pAttributes, cAttributes, tempCAttributes;
     // QStringList cAttributesList;
 
     // Retrieve the paragraph (text-only)
     len = readU16(n, 0x09);
     s.resize(len);
-    for (Q_UINT16 k = 0x00; k < len; k++)
+    for (quint16 k = 0x00; k < len; k++)
         s[k] = n[0x0B+k];
 
     /*
@@ -333,91 +336,91 @@ bool StarWriterImport::parseText(QByteArray n)
     return true;
 }
 
-bool StarWriterImport::parseTable(QByteArray n)
+bool StarWriterImport::parseTable(const QByteArray& n)
 {
-/*
-    QByteArray s;
-    Q_UINT32 len, len2;
-    Q_UINT16 len3;
-    Q_UINT32 p, p2;
-    QString text;
-    QString tableCell, tableText, tableName;
-    Q_UINT8 row, column;
+    /*
+        QByteArray s;
+        quint32 len, len2;
+        quint16 len3;
+        quint32 p, p2;
+        QString text;
+        QString tableCell, tableText, tableName;
+        quint8 row, column;
 
-    // Set table name
-    tableName = QString("Table %1").arg(tablesNumber);
-    tablesNumber++;
+        // Set table name
+        tableName = QString("Table %1").arg(tablesNumber);
+        tablesNumber++;
 
-    // Skip useless sections and retrieve the right point
-    p = 0x13;
-    while (n[p] != 'L') {
-        len = readU24(n, p+1);
-        p += len;
-    }
-
-    row = 0;
-
-    // Read rows
-    while (n[p] == 'L') {
-        column = 0;
-
-        // Find the first 't'
-        while (n[p] != 't') p++;
-
-        // Read cells
-        while (n[p] == 't') {
-            // Get cell length
-            len2 = readU24(n, p+1);
-            p2 = p + len2;
-
-            // Find the 'T' section
-            while (n[p] != 'T') p++;
-
-            // Get cell text/value
-            len3 = readU16(n, p+0x09);
-            s.resize(len3);
-            for (Q_UINT16 k = 0x00; k < len3; k++)
-                s[k] = n[p+0x0B+k];
-            text = convertToKWordString(s);
-
-            // FIXME: check this stuff
-            QString frameName = QString("%1 Cell %2,%3").arg(tableName).arg(row).arg(column);
-            tableText.append(QString(" <FRAMESET name=\"%1\" frameType=\"1\" frameInfo=\"0\" removable=\"0\" visible=\"1\" grpMgr=\"%2\" row=\"%3\" col=\"%4\" rows=\"1\" cols=\"1\" protectSize=\"0\">\n").arg(frameName).arg(tableName).arg(row).arg(column));
-            tableText.append(" <FRAME runaround=\"1\" copy=\"0\" newFrameBehavior=\"1\" runaroundSide=\"biggest\" autoCreateNewFrame=\"0\" bleftpt=\"2.8\" brightpt=\"2.8\" btoppt=\"2.8\" bbottompt=\"2.8\" runaroundGap=\"2.8\" />\n");
-            tableText.append("  <PARAGRAPH>\n");
-            tableText.append("   <TEXT xml:space=\"preserve\">" + text + "</TEXT>\n");
-            tableText.append("  </PARAGRAPH>\n");
-            tableText.append(" </FRAMESET>\n");
-
-            // Skip other sections or bytes
-            p = p2;
-
-            // Increase column pointers
-            column++;
+        // Skip useless sections and retrieve the right point
+        p = 0x13;
+        while (n[p] != 'L') {
+            len = readU24(n, p+1);
+            p += len;
         }
 
-        // Increase row pointer
-        row++;
-    }
+        row = 0;
 
-    // Add everything to tablesStuff
-    tablesStuff.append(tableText);
+        // Read rows
+        while (n[p] == 'L') {
+            column = 0;
 
-    // Add anchor to bodyStuff
-    bodyStuff.append("  <PARAGRAPH>\n");
-    bodyStuff.append("   <TEXT xml:space=\"preserve\">#</TEXT>\n");
-    bodyStuff.append("   <FORMATS>\n");
-    bodyStuff.append("    <FORMAT id=\"6\" pos=\"0\" len=\"1\">\n");
-    bodyStuff.append(QString("    <ANCHOR type=\"frameset\" instance=\"%1\" />\n").arg(tableName));
-    bodyStuff.append("    </FORMAT>\n");
-    bodyStuff.append("   </FORMATS>\n");
-    bodyStuff.append("  </PARAGRAPH>\n");
+            // Find the first 't'
+            while (n[p] != 't') p++;
 
-*/
+            // Read cells
+            while (n[p] == 't') {
+                // Get cell length
+                len2 = readU24(n, p+1);
+                p2 = p + len2;
+
+                // Find the 'T' section
+                while (n[p] != 'T') p++;
+
+                // Get cell text/value
+                len3 = readU16(n, p+0x09);
+                s.resize(len3);
+                for (quint16 k = 0x00; k < len3; k++)
+                    s[k] = n[p+0x0B+k];
+                text = convertToKWordString(s);
+
+                // FIXME: check this stuff
+                QString frameName = QString("%1 Cell %2,%3").arg(tableName).arg(row).arg(column);
+                tableText.append(QString(" <FRAMESET name=\"%1\" frameType=\"1\" frameInfo=\"0\" removable=\"0\" visible=\"1\" grpMgr=\"%2\" row=\"%3\" col=\"%4\" rows=\"1\" cols=\"1\" protectSize=\"0\">\n").arg(frameName).arg(tableName).arg(row).arg(column));
+                tableText.append(" <FRAME runaround=\"1\" copy=\"0\" newFrameBehavior=\"1\" runaroundSide=\"biggest\" autoCreateNewFrame=\"0\" bleftpt=\"2.8\" brightpt=\"2.8\" btoppt=\"2.8\" bbottompt=\"2.8\" runaroundGap=\"2.8\" />\n");
+                tableText.append("  <PARAGRAPH>\n");
+                tableText.append("   <TEXT xml:space=\"preserve\">" + text + "</TEXT>\n");
+                tableText.append("  </PARAGRAPH>\n");
+                tableText.append(" </FRAMESET>\n");
+
+                // Skip other sections or bytes
+                p = p2;
+
+                // Increase column pointers
+                column++;
+            }
+
+            // Increase row pointer
+            row++;
+        }
+
+        // Add everything to tablesStuff
+        tablesStuff.append(tableText);
+
+        // Add anchor to bodyStuff
+        bodyStuff.append("  <PARAGRAPH>\n");
+        bodyStuff.append("   <TEXT xml:space=\"preserve\">#</TEXT>\n");
+        bodyStuff.append("   <FORMATS>\n");
+        bodyStuff.append("    <FORMAT id=\"6\" pos=\"0\" len=\"1\">\n");
+        bodyStuff.append(QString("    <ANCHOR type=\"frameset\" instance=\"%1\" />\n").arg(tableName));
+        bodyStuff.append("    </FORMAT>\n");
+        bodyStuff.append("   </FORMATS>\n");
+        bodyStuff.append("  </PARAGRAPH>\n");
+
+    */
     return true;
 }
 
-bool StarWriterImport::parseGraphics(QByteArray n)
+bool StarWriterImport::parseGraphics(const QByteArray& n)
 {
     return true;
 }
