@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2002 Ariya Hidayat <ariyahidayat@yahoo.de>
+   Copyright (C) 2002 Ariya Hidayat <ariya@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,48 +20,48 @@
 #ifndef __WMLPARSER_H
 #define __WMLPARSER_H
 
-#include <qvaluelist.h>
-#include <qstring.h>
+#include <QList>
+#include <QString>
 
 class WMLFormat
 {
-  public:
+public:
     int pos, len;
     bool bold, italic, underline;
     enum { Normal, Big, Small } fontsize;
     QString link, href;
     WMLFormat();
-    WMLFormat( const WMLFormat& );
-    WMLFormat& operator= ( const WMLFormat& );
-    void assign( const WMLFormat& );
+    WMLFormat(const WMLFormat&);
+    WMLFormat& operator= (const WMLFormat&);
+    void assign(const WMLFormat&);
 };
 
 class WMLLayout
 {
-  public:
+public:
     enum { Left, Center, Right } align;
     WMLLayout();
-    WMLLayout( const WMLLayout& );
-    WMLLayout& operator=( const WMLLayout& );
-    void assign( const WMLLayout& );
+    WMLLayout(const WMLLayout&);
+    WMLLayout& operator=(const WMLLayout&);
+    void assign(const WMLLayout&);
 };
 
-typedef QValueList<WMLFormat> WMLFormatList;
+typedef QList<WMLFormat> WMLFormatList;
 
 class WMLParser
 {
-  public:
-    WMLParser(){};
-    virtual ~WMLParser(){};
-    virtual void parse( const char* filename );
+public:
+    WMLParser() {}
+    virtual ~WMLParser() {}
+    virtual void parse(const char* filename);
 
     virtual bool doOpenDocument();
     virtual bool doCloseDocument();
-    virtual bool doOpenCard( QString id, QString title );
+    virtual bool doOpenCard(const QString& id, const QString& title);
     virtual bool doCloseCard();
-    virtual bool doParagraph( QString, WMLFormatList, WMLLayout );
+    virtual bool doParagraph(const QString&, WMLFormatList, WMLLayout);
     virtual bool doBeginTable();
-    virtual bool doTableCell( unsigned row, unsigned col );
+    virtual bool doTableCell(unsigned row, unsigned col);
     virtual bool doEndTable();
 };
 
