@@ -20,12 +20,10 @@
 #ifndef __WPIMPORT_H
 #define __WPIMPORT_H
 
-#include <qstring.h>
-#include <qdatastream.h>
-#include <qobject.h>
-#include <qptrlist.h>
-#include <qcstring.h>
-#include <qmemarray.h>
+#include <QString>
+#include <QDataStream>
+#include <QObject>
+#include <QVector>
 #include <qdom.h>
 
 #include <KoFilter.h>
@@ -36,18 +34,18 @@ class WPImport : public KoFilter
     Q_OBJECT
 
 
-  protected:
+protected:
 
     QString m_output;
 
-  public:
+public:
 
-    WPImport(KoFilter *parent, const char *name, const QStringList&);
+    WPImport(QObject* parent, const QStringList&);
     WPImport();
 
     virtual ~WPImport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 
 };
 #endif // __WPIMPORT_H
