@@ -21,8 +21,11 @@
 #define __XSLTEXPORTDIA_H__
 
 #include <xsltdialog.h>
-#include <qstringlist.h>
+#include <QStringList>
+//Added by qt3to4:
+#include <QByteArray>
 #include <kurl.h>
+#include <kconfiggroup.h>
 
 class KoStoreDevice;
 class KConfig;
@@ -34,9 +37,10 @@ class XSLTExportDia : public XSLTDialog
     QString _fileOut;
     KoStoreDevice* _in;
     /** xslt file current */
-    KURL _currentFile;
-    QCString _format;
+    KUrl _currentFile;
+    QByteArray _format;
     KConfig* _config;
+    KConfigGroup grp;
     /** List of the most recent xslt file used. */
     QStringList _recentList;
 
@@ -46,10 +50,12 @@ class XSLTExportDia : public XSLTDialog
     QStringList _namesList;
 
 public:
-    XSLTExportDia( KoStoreDevice*, const QCString &format, QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
+    XSLTExportDia(KoStoreDevice*, const QByteArray &format, QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
     ~XSLTExportDia();
 
-    void setOutputFile(QString file) { _fileOut = file; }
+    void setOutputFile(QString file) {
+        _fileOut = file;
+    }
 
 public slots:
     virtual void cancelSlot();
