@@ -32,9 +32,9 @@
 using namespace KFormDesigner;
 
 FormScript::FormScript(Form *form, ScriptManager *manager, const char *name)
- : QObject(manager, name), m_manager(manager), m_form(form)
+        : QObject(manager, name), m_manager(manager), m_form(form)
 {
-	m_script = manager->krossManager()->getScriptContainer(form->widget()->name());
+    m_script = manager->krossManager()->getScriptContainer(form->widget()->name());
 }
 
 FormScript::~FormScript()
@@ -44,64 +44,60 @@ FormScript::~FormScript()
 QString
 FormScript::getCode(const QString &)
 {
-	/// \todo Allow to select only one function
-	return m_script->getCode();
+    /// \todo Allow to select only one function
+    return m_script->getCode();
 }
 
 void
 FormScript::setCode(const QString &code)
 {
-	m_script->setCode(code);
+    m_script->setCode(code);
 }
 
 void
 FormScript::appendCode(const QString &code)
 {
-	QString s = m_script->getCode();
-	s.append("\n");
-	s.append(code);
-	m_script->setCode(s);
+    QString s = m_script->getCode();
+    s.append("\n");
+    s.append(code);
+    m_script->setCode(s);
 }
 
 bool
 FormScript::execute(const QString &functionName)
 {
-	/// \todo  support return value and arguments
-	try {
-		m_script->callFunction(functionName);
-	}
-	catch(Kross::Api::Exception& e) {
-		kdDebug() << QString("EXCEPTION type='%1' description='%2'").arg(e.type()).arg(e.description()) << endl;
-		return false;
-	}
-	return true;
+    /// \todo  support return value and arguments
+    try {
+        m_script->callFunction(functionName);
+    } catch (Kross::Api::Exception& e) {
+        kDebug() << QString("EXCEPTION type='%1' description='%2'").arg(e.type()).arg(e.description());
+        return false;
+    }
+    return true;
 }
 
 void
 FormScript::connectEvents()
 {
-	if(!m_form || !m_form->objectTree())
-		return;
-	// first call addQObject for each widget in the Form
-	ObjectTreeDict *dict = m_form->objectTree()->dict();
-	ObjectTreeDictIterator it(*dict);
-	for(; it.current(); ++it)
-		m_script->addQObject(it.current()->widget());
-	m_script->addQObject(m_form->widget());
+    if (!m_form || !m_form->objectTree())
+        return;
+    // first call addQObject for each widget in the Form
+    ObjectTreeHash *hash = m_form->objectTree()->hash();
+    foreach (ObjectTreeItem *item, *hash) {
+        m_script->addQObject(item->widget());
+    }
+    m_script->addQObject(m_form->widget());
 
-	// Then we connect all signals
-	QValueListConstIterator<Event*> endIt = m_list.constEnd();
-	for(QValueListConstIterator<Event*> it = m_list.constBegin(); it != endIt; ++it) {
-		if( (*it)->type() == Event::Slot) {
-			connect((*it)->sender(), (*it)->signal(), (*it)->receiver(), (*it)->slot());
-		}
-		else if( (*it)->type() == Event::UserFunction) {
-			m_script->connect((*it)->sender(), (*it)->signal(), (*it)->slot());
-		}
-		else if( (*it)->type() == Event::Action) {
-			/// \todo connect signals with actions
-		}
-	}
+    // Then we connect all signals
+    foreach (Event* e, m_list) {
+        if (e->type() == Event::Slot) {
+            connect(e->sender(), e->signal(), e->receiver(), e->slot());
+        } else if (e->type() == Event::UserFunction) {
+            m_script->connect(e->sender(), e->signal(), e->slot());
+        } else if (e->type() == Event::Action) {
+            /// \todo connect signals with actions
+        }
+    }
 }
 
 
