@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2003-2006 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2003-2006 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -17,39 +17,42 @@
  * Boston, MA 02110-1301, USA.
 */
 
-#include <kexidb/global.h>
+#include "global.h"
 
 using namespace KexiDB;
 
 DatabaseVersionInfo::DatabaseVersionInfo()
 {
-	major = 0;
-	minor = 0;
+    major = 0;
+    minor = 0;
 }
 
 DatabaseVersionInfo::DatabaseVersionInfo(uint majorVersion, uint minorVersion)
 {
-	major = majorVersion;
-	minor = minorVersion;
+    major = majorVersion;
+    minor = minorVersion;
 }
 
 //------------------------
 
 ServerVersionInfo::ServerVersionInfo()
 {
-	major = 0;
-	minor = 0;
-	release = 0;
+    major = 0;
+    minor = 0;
+    release = 0;
 }
 
 void ServerVersionInfo::clear()
 {
-	major = 0;
-	minor = 0;
-	release = 0;
-	string = QString::null;
+    major = 0;
+    minor = 0;
+    release = 0;
+    string.clear();
 }
 
 //------------------------
 
-DatabaseVersionInfo KexiDB::version() { return KEXIDB_VERSION; }
+DatabaseVersionInfo KexiDB::version()
+{
+    return KEXIDB_VERSION;
+}
