@@ -20,28 +20,26 @@
 #ifndef KEXIRELATIONMAINDLG_H
 #define KEXIRELATIONMAINDLG_H
 
-#include <kexiviewbase.h>
+#include <KexiView.h>
 
-class KexiMainWindow;
-class KexiRelationWidget;
+class KexiRelationsView;
 
-class KexiRelationMainDlg : public KexiViewBase
+class KexiRelationMainDlg : public KexiView
 {
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-		KexiRelationMainDlg(KexiMainWindow *mainWin, QWidget *parent, const char *name = 0);
-		~KexiRelationMainDlg();
+public:
+    KexiRelationMainDlg(QWidget *parent);
+    virtual ~KexiRelationMainDlg();
 
-		virtual QSize sizeHint() const;
+    virtual QSize sizeHint() const;
 
-		virtual QWidget* mainWidget();
+    virtual QWidget* mainWidget();
 
-		virtual QString itemIcon();
+    virtual QString itemIcon();
 
-	private:
-		KexiRelationWidget	*m_rel;
+private:
+    KexiRelationsView *m_rel;
 };
 
 #endif
-
