@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2007 -2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -191,6 +191,16 @@ void ViewListItem::save( QDomElement &element ) const
 ViewListTreeWidget::ViewListTreeWidget( QWidget *parent )
     : QTreeWidget( parent )
 {
+    setWhatsThis( i18nc( "@info:whatsthis",
+        "<para>This is the list of available views and editors.</para>"
+        "<para>You can configure the list by using the context menu:"
+        "<list>"
+        "<item>Rename categories or views</item>"
+        "<item>Configure. Move, remove, rename or edit tool tip for categories or views</item>"
+        "<item>Insert categories and views</item>"
+        "</list>"
+    ) );
+
     header() ->hide();
     setRootIsDecorated( false );
     setItemDelegate( new ViewCategoryDelegate( this, this ) );
@@ -354,7 +364,6 @@ void ViewListWidget::slotItemChanged( QTreeWidgetItem *item, int col )
 
 void ViewListWidget::slotActivated( QTreeWidgetItem *item, QTreeWidgetItem *prev )
 {
-    qDebug()<<"slotActivated:"<<item<<prev<<m_prev;
     if ( m_prev ) {
         m_prev->setData( 0, Qt::BackgroundRole, QVariant() );
     }
@@ -422,8 +431,10 @@ ViewListItem *ViewListWidget::addView( QTreeWidgetItem *category, const QString
         item->setData( 0, Qt::DecorationRole, KIcon( icon ) );
     }
     item->setFlags( item->flags() | Qt::ItemIsEditable );
-    category->insertChild( ( index == -1 ? category->childCount() : index ), item );
-    //kDebug() << "added: " << item->tag();
+    insertViewListItem( item, category, index );
+
+    connect(view, SIGNAL(optionsModified()), SLOT(setModified()));
+
     return item;
 }
 
@@ -442,7 +453,6 @@ void ViewListWidget::setSelected( QTreeWidgetItem *item )
 
 void ViewListWidget::setCurrentItem( QTreeWidgetItem *item )
 {
-    qDebug()<<"setCurrentItem:"<<item<<","<<m_viewlist->currentItem();
     m_viewlist->setCurrentItem( item );
     //kDebug()<<item<<","<<m_viewlist->currentItem();
 }
@@ -547,6 +557,7 @@ void ViewListWidget::slotRemoveCategory()
     }
     takeViewListItem( m_contextitem );
     delete m_contextitem;
+    emit modified();
     m_contextitem = 0;
 }
 
@@ -555,6 +566,7 @@ void ViewListWidget::slotRemoveView()
     if ( m_contextitem ) {
         takeViewListItem( m_contextitem );
         delete m_contextitem;
+        emit modified();
     }
 }
 
@@ -565,6 +577,9 @@ void ViewListWidget::slotEditViewTitle()
         kDebug()<<m_contextitem<<":"<<m_contextitem->type();
         QString title = m_contextitem->text( 0 );
         m_viewlist->editItem( m_contextitem );
+        if ( title != m_contextitem->text( 0 ) ) {
+            emit modified();
+        }
     }
 }
 
@@ -573,15 +588,28 @@ void ViewListWidget::slotConfigureItem()
     if ( m_contextitem == 0 ) {
         return;
     }
+    KDialog *dlg = 0;
     if ( m_contextitem->type() == ViewListItem::ItemType_Category ) {
         kDebug()<<m_contextitem<<":"<<m_contextitem->type();
-        ViewListEditCategoryDialog *dlg = new ViewListEditCategoryDialog( *this, m_contextitem, this );
-        dlg->exec();
-        delete dlg;
+        dlg = new ViewListEditCategoryDialog( *this, m_contextitem, this );
     } else if ( m_contextitem->type() == ViewListItem::ItemType_SubView ) {
-        ViewListEditViewDialog *dlg = new ViewListEditViewDialog( *this, m_contextitem, this );
-        dlg->exec();
-        delete dlg;
+        dlg = new ViewListEditViewDialog( *this, m_contextitem, this );
+    }
+    if ( dlg ) {
+        connect(dlg, SIGNAL(finished(int)), SLOT(slotDialogFinished(int)));
+        dlg->show();
+        dlg->raise();
+        dlg->activateWindow();
+    }
+}
+
+void ViewListWidget::slotDialogFinished( int result )
+{
+    if ( result == QDialog::Accepted ) {
+        emit modified();
+    }
+    if ( sender() ) {
+        sender()->deleteLater();
     }
 }
 
@@ -604,6 +632,7 @@ int ViewListWidget::removeViewListItem( ViewListItem *item )
     int i = p->indexOfChild( item );
     if ( i != -1 ) {
         p->takeChild( i );
+        emit modified();
     }
     return i;
 }
@@ -618,6 +647,7 @@ void ViewListWidget::addViewListItem( ViewListItem *item, QTreeWidgetItem *paren
         index = p->childCount();
     }
     p->insertChild( index, item );
+    emit modified();
 }
 
 int ViewListWidget::takeViewListItem( ViewListItem *item )
@@ -625,7 +655,6 @@ int ViewListWidget::takeViewListItem( ViewListItem *item )
     while ( item->childCount() > 0 ) {
         takeViewListItem( static_cast<ViewListItem*>( item->child( 0 ) ) );
     }
-    //qDebug()<<"takeViewListItem:"<<item<<m_prev;
     int pos = removeViewListItem( item );
     if ( pos != -1 ) {
         emit viewListItemRemoved( item );
@@ -642,7 +671,7 @@ int ViewListWidget::takeViewListItem( ViewListItem *item )
 void ViewListWidget::insertViewListItem( ViewListItem *item, QTreeWidgetItem *parent, int index )
 {
     addViewListItem( item, parent, index );
-    emit viewListItemInserted( item );
+    emit viewListItemInserted( item, static_cast<ViewListItem*>( parent ), index );
 }
 
 void ViewListWidget::setupContextMenus()
@@ -650,15 +679,15 @@ void ViewListWidget::setupContextMenus()
     // NOTE: can't use xml file as there may not be a factory()
     QAction *action;
     // view actions
-    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
+    action = new QAction( KIcon( "edit-rename" ), i18nc( "@action:inmenu rename view", "Rename" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotEditViewTitle() ) );
     m_viewactions.append( action );
 
-    action = new QAction( KIcon( "configure" ), i18n( "Configure..." ), this );
+    action = new QAction( KIcon( "configure" ), i18nc( "@action:inmenu configure view", "Configure..." ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
     m_viewactions.append( action );
 
-    action = new QAction( KIcon( "list-remove" ), i18n( "Remove" ), this );
+    action = new QAction( KIcon( "list-remove" ), i18nc( "@action:inmenu remove view", "Remove" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveView() ) );
     m_viewactions.append( action );
 
@@ -667,15 +696,15 @@ void ViewListWidget::setupContextMenus()
     m_viewactions.append( action );
 
     // Category actions
-    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
+    action = new QAction( KIcon( "edit-rename" ), i18nc( "@action:inmenu rename view category", "Rename" ), this );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( renameCategory() ) );
     m_categoryactions.append( action );
 
-    action = new QAction( KIcon( "configure" ), i18n( "Configure..." ), this );
+    action = new QAction( KIcon( "configure" ), i18nc( "@action:inmenu configure view category", "Configure..." ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
     m_categoryactions.append( action );
 
-    action = new QAction( KIcon( "list-remove" ), i18n( "Remove" ), this );
+    action = new QAction( KIcon( "list-remove" ), i18nc( "@action:inmenu Remove view category", "Remove" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveCategory() ) );
     m_categoryactions.append( action );
 
@@ -684,7 +713,7 @@ void ViewListWidget::setupContextMenus()
     m_categoryactions.append( action );
 
     // list actions
-    action = new QAction( KIcon( "list-add" ), i18nc( "@action Insert View", "Insert..." ), this );
+    action = new QAction( KIcon( "list-add" ), i18nc( "@action:inmenu Insert View", "Insert..." ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotAddView() ) );
     m_listactions.append( action );
 }
@@ -772,6 +801,11 @@ void ViewListWidget::slotScheduleManagerAdded( ScheduleManager *sm )
     }
 }
 
+void ViewListWidget::setModified()
+{
+    emit modified();
+}
+
 }  //KPlato namespace
 
 #include "kptviewlist.moc"
