@@ -53,7 +53,49 @@ class Project;
 class Schedule;
 class XMLLoaderObject;
 
-typedef QPair<DateTime, DateTime> DateTimeInterval;
+class KPLATOKERNEL_EXPORT DateTimeInterval : public QPair<DateTime, DateTime>
+{
+public:
+    DateTimeInterval()
+    : QPair<DateTime, DateTime>()
+    {}
+    DateTimeInterval( const DateTime &t1, const DateTime &t2 )
+    : QPair<DateTime, DateTime>( t1, t2 )
+    {}
+    DateTimeInterval &operator=( const DateTimeInterval &other ) {
+        first = other.first; second = other.second;
+        return *this;
+    }
+    bool isValid() const { return first.isValid() && second.isValid(); }
+    void limitTo( const DateTime &start, const DateTime &end ) {
+        if ( ! first.isValid() || ( start.isValid() && start > first ) ) {
+            first = start;
+        }
+        if ( ! second.isValid() || ( end.isValid() && end < second ) ) {
+            second = end;
+        }
+        if ( isValid() && first > second ) {
+            first = second = DateTime();
+        }
+    }
+    void limitTo( const DateTimeInterval &interval ) {
+        limitTo( interval.first, interval.second );
+    }
+
+    DateTimeInterval limitedTo( const DateTime &start, const DateTime &end ) const {
+        DateTimeInterval i = *this;
+        i.limitTo( start, end );
+        return i;
+    }
+    DateTimeInterval limitedTo( const DateTimeInterval &interval ) const {
+        return limitedTo( interval.first, interval.second );
+    }
+    QString toString() const {
+        return QString( "%1 to %2" )
+                    .arg( first.isValid()?first.toString():"''" )
+                    .arg( second.isValid()?second.toString():"''" );
+    }
+};
 
 /// TimeInterval is defined as a start time and a length.
 /// The end time (start + length) must not exceed midnight
@@ -267,7 +309,7 @@ public:
     int state(int weekday) const;
     void setState(int weekday, int state);
     
-    const QList<TimeInterval*> &intervals(int weekday) const;
+    QList<TimeInterval*> intervals(int weekday) const;
     void setIntervals(int weekday, QList<TimeInterval*>intervals);
     void clearIntervals(int weekday);
     
@@ -454,7 +496,7 @@ public:
     KTimeZone timeZone() const { return m_spec.timeZone(); }
     void setTimeZone( const KTimeZone &tz );
     
-    void setDefault( bool on ) { m_default = on; }
+    void setDefault( bool on );
     bool isDefault() const { return m_default; }
     
 signals:
@@ -572,6 +614,8 @@ public:
     /// Set the work time of a normal day
     void setDay(double hours) { m_day = Duration(hours, Duration::Unit_h); changed(); }
     
+    QList<double> scales() const;
+
     bool load( KoXmlElement &element, XMLLoaderObject &status );
     void save(QDomElement &element) const;
 
