@@ -1075,44 +1075,35 @@ NodeMoveCmd::NodeMoveCmd( Project *project, Node *node, Node *newParent, int new
     m_node( node ),
     m_newparent( newParent ),
     m_newpos( newPos ),
-    m_moved( false ),
-    m_cmd( 0 )
+    m_moved( false )
 {
     m_oldparent = node->parentNode();
     Q_ASSERT( m_oldparent );
-    m_oldpos = m_oldparent->indexOf( node );
-    
-    // Summarytasks can't have resources, so remove resource requests from the new parent
-    foreach ( ResourceGroupRequest *r, newParent->requests().requests() ) {
-        if ( m_cmd == 0 ) m_cmd = new MacroCommand( "" );
-        m_cmd->addCommand( new RemoveResourceGroupRequestCmd( r ) );
-    }
-    // TODO appointments ??
-}
-NodeMoveCmd::~NodeMoveCmd()
-{
-    delete m_cmd;
 }
 void NodeMoveCmd::execute()
 {
     if ( m_project ) {
+        m_oldpos = m_oldparent->indexOf( m_node );
         m_moved = m_project->moveTask( m_node, m_newparent, m_newpos );
-        if ( m_moved && m_cmd ) {
-            m_cmd->execute();
+        if ( m_moved ) {
+            if ( m_cmd.isEmpty() ) {
+                // Summarytasks can't have resources, so remove resource requests from the new parent
+                foreach ( ResourceGroupRequest *r, m_newparent->requests().requests() ) {
+                    m_cmd.addCommand( new RemoveResourceGroupRequestCmd( r ) );
+                }
+                // TODO appointments ??
+            }
+            m_cmd.execute();
         }
     }
-
 }
 void NodeMoveCmd::unexecute()
 {
     if ( m_project && m_moved ) {
         m_moved = m_project->moveTask( m_node, m_oldparent, m_oldpos );
-        if ( m_moved && m_cmd ) {
-            m_cmd->unexecute();
-        }
+        m_cmd.unexecute();
     }
     m_moved = false;
-
 }
 
 AddRelationCmd::AddRelationCmd( Project &project, Relation *rel, const QString& name )
@@ -1314,6 +1305,22 @@ void ModifyResourceRequestUnitsCmd::unexecute()
     m_request->setUnits( m_oldvalue );
 }
 
+ModifyResourceRequestRequiredCmd::ModifyResourceRequestRequiredCmd( ResourceRequest *request, const QList<Resource*> &value, const QString& name )
+    : NamedCommand( name ),
+    m_request( request ),
+    m_newvalue( value )
+{
+    m_oldvalue = request->requiredResources();
+}
+void ModifyResourceRequestRequiredCmd::execute()
+{
+    m_request->setRequiredResources( m_newvalue );
+}
+void ModifyResourceRequestRequiredCmd::unexecute()
+{
+    m_request->setRequiredResources( m_oldvalue );
+}
+
 ModifyResourceGroupRequestUnitsCmd::ModifyResourceGroupRequestUnitsCmd( ResourceGroupRequest *request, int oldvalue, int newvalue, const QString& name )
     : NamedCommand( name ),
     m_request( request ),
@@ -1618,6 +1625,9 @@ RemoveResourceCmd::RemoveResourceCmd( ResourceGroup *group, Resource *resource,
             }
         }
     }
+    if ( resource->account() ) {
+        m_cmd.addCommand( new ResourceModifyAccountCmd( *resource, resource->account(), 0 ) );
+    }
 }
 RemoveResourceCmd::~RemoveResourceCmd()
 {
@@ -1638,6 +1648,7 @@ void RemoveResourceCmd::execute()
         //kDebug()<<"detached:"<<a;
     }*/
     AddResourceCmd::unexecute();
+    m_cmd.execute();
     setSchScheduled( false );
 }
 void RemoveResourceCmd::unexecute()
@@ -1650,6 +1661,7 @@ void RemoveResourceCmd::unexecute()
         r->parent() ->addResourceRequest( r );
         //kDebug()<<"Add request for"<<r->resource()->name();
     }
+    m_cmd.unexecute();
     AddResourceCmd::execute();
     setSchScheduled();
 }
@@ -1917,6 +1929,22 @@ void ModifyResourceCalendarCmd::unexecute()
 
 }
 
+ModifyRequiredResourcesCmd::ModifyRequiredResourcesCmd( Resource *resource, const QList<Resource*> &value, const QString& name )
+        : NamedCommand( name ),
+        m_resource( resource ),
+        m_newvalue( value )
+{
+    m_oldvalue = resource->requiredResources();
+}
+void ModifyRequiredResourcesCmd::execute()
+{
+    m_resource->setRequiredResources( m_newvalue );
+}
+void ModifyRequiredResourcesCmd::unexecute()
+{
+    m_resource->setRequiredResources( m_oldvalue );
+}
+
 RemoveResourceGroupCmd::RemoveResourceGroupCmd( Project *project, ResourceGroup *group, const QString& name )
         : NamedCommand( name ),
         m_group( group ),
@@ -2632,6 +2660,34 @@ void ModifyDefaultAccountCmd::unexecute()
 
 }
 
+ResourceModifyAccountCmd::ResourceModifyAccountCmd( Resource &resource,  Account *oldvalue, Account *newvalue, const QString& name )
+    : NamedCommand( name ),
+    m_resource( resource )
+{
+    m_oldvalue = oldvalue;
+    m_newvalue = newvalue;
+}
+void ResourceModifyAccountCmd::execute()
+{
+    //kDebug();
+    if ( m_oldvalue ) {
+        m_oldvalue->removeRunning( m_resource );
+    }
+    if ( m_newvalue ) {
+        m_newvalue->addRunning( m_resource );
+    }
+}
+void ResourceModifyAccountCmd::unexecute()
+{
+    //kDebug();
+    if ( m_newvalue ) {
+        m_newvalue->removeRunning( m_resource );
+    }
+    if ( m_oldvalue ) {
+        m_oldvalue->addRunning( m_resource );
+    }
+}
+
 ProjectModifyConstraintCmd::ProjectModifyConstraintCmd( Project &node, Node::ConstraintType c, const QString& name )
         : NamedCommand( name ),
         m_node( node ),
@@ -2888,14 +2944,14 @@ void ModifyScheduleManagerSchedulerCmd::unexecute()
     m_sm.setSchedulerPlugin( oldvalue );
 }
 
-CalculateScheduleCmd::CalculateScheduleCmd( Project &node, ScheduleManager &sm, const QString& name )
+CalculateScheduleCmd::CalculateScheduleCmd( Project &node, ScheduleManager *sm, const QString& name )
     : NamedCommand( name ),
     m_node( node ),
     m_sm( sm ),
     m_first( true ),
-    m_oldexpected( m_sm.expected() ),
-    m_oldoptimistic( m_sm.optimistic() ),
-    m_oldpessimistic( m_sm.pessimistic() ),
+    m_oldexpected( m_sm->expected() ),
+    m_oldoptimistic( m_sm->optimistic() ),
+    m_oldpessimistic( m_sm->pessimistic() ),
     m_newexpected( 0 ),
     m_newoptimistic( 0 ),
     m_newpessimistic( 0 )
@@ -2905,23 +2961,37 @@ CalculateScheduleCmd::CalculateScheduleCmd( Project &node, ScheduleManager &sm,
 void CalculateScheduleCmd::execute()
 {
     if ( m_first ) {
-        m_first = false;
-        m_sm.calculateSchedule();
-        m_newexpected = m_sm.expected();
-        m_newoptimistic = m_sm.optimistic();
-        m_newpessimistic = m_sm.pessimistic();
+        QApplication::setOverrideCursor( Qt::WaitCursor );
+        m_sm->calculateSchedule();
+        Q_ASSERT( m_sm );
+        if ( m_sm->calculationResult() != ScheduleManager::CalculationCanceled ) {
+            m_first = false;
+            m_newexpected = m_sm->expected();
+            m_newoptimistic = m_sm->optimistic();
+            m_newpessimistic = m_sm->pessimistic();
+        }
+        QApplication::restoreOverrideCursor();
         return;
     }
-    m_sm.setExpected( m_newexpected );
-    m_sm.setOptimistic( m_newoptimistic );
-    m_sm.setPessimistic( m_newpessimistic );
+    m_sm->setExpected( m_newexpected );
+    m_sm->setOptimistic( m_newoptimistic );
+    m_sm->setPessimistic( m_newpessimistic );
 }
 
 void CalculateScheduleCmd::unexecute()
 {
-    m_sm.setExpected( m_oldexpected );
-    m_sm.setOptimistic( m_oldoptimistic );
-    m_sm.setPessimistic( m_oldpessimistic );
+    if ( m_sm->scheduling() ) {
+        // terminate scheduling
+        QApplication::setOverrideCursor( Qt::WaitCursor );
+        m_sm->stopCalculation();
+        m_first = true;
+        QApplication::restoreOverrideCursor();
+
+    }
+    m_sm->setExpected( m_oldexpected );
+    m_sm->setOptimistic( m_oldoptimistic );
+    m_sm->setPessimistic( m_oldpessimistic );
+
 }
 
 //------------------------
@@ -3138,7 +3208,7 @@ void DocumentModifyStatusCmd::unexecute()
 }
 
 //----------------
-DocumentModifySendAsCmd::DocumentModifySendAsCmd( Document *doc, Document::SendAs value, const QString& name )
+DocumentModifySendAsCmd::DocumentModifySendAsCmd( Document *doc, const Document::SendAs value, const QString& name )
     : NamedCommand( name ),
     m_doc( doc )
 {
@@ -3180,7 +3250,18 @@ InsertProjectCmd::InsertProjectCmd( Project &project, Node *parent, Node *after,
 {
     Q_ASSERT( &project != m_project );
 
-    qDebug()<<"InsertProjectCmd: Merge calendars";
+    // remove unhandled info in tasks
+    foreach ( Node *n, project.allNodes() ) {
+        if ( n->type() == Node::Type_Task ) {
+            Task *t = static_cast<Task*>( n );
+            t->workPackage().clear();
+            while ( ! t->workPackageLog().isEmpty() ) {
+                WorkPackage *wp = t->workPackageLog().at( 0 );
+                t->removeWorkPackage( wp );
+                delete wp;
+            }
+        }
+    }
     QMap<Calendar*, Calendar*> unusedCalendars;
     foreach ( Calendar *c, project.calendars() ) {
         addCalendars( c, 0, unusedCalendars );
@@ -3191,7 +3272,6 @@ InsertProjectCmd::InsertProjectCmd( Project &project, Node *parent, Node *after,
             r->setCalendar( unusedCalendars[ r->calendar() ] );
         }
     }
-    qDebug()<<"InsertProjectCmd: Get requests:";
     // get all requests before resources are merged
     QMap<ResourceGroupRequest*, QPair<Node *, ResourceGroup*> > greqs;
     QMap<ResourceGroupRequest*, QPair<ResourceRequest*, Resource*> > rreqs;
@@ -3201,7 +3281,6 @@ InsertProjectCmd::InsertProjectCmd( Project &project, Node *parent, Node *after,
             continue;
         }
         while ( ResourceGroupRequest *gr = n->requests().requests().value( 0 ) ) {
-            //qDebug()<<"InsertProjectCmd: "<<gr->group()<<existingGroups;
             while ( ResourceRequest *rr = gr->resourceRequests().value( 0 ) ) {
                 rreqs.insertMulti( gr, QPair<ResourceRequest*, Resource*>( rr, rr->resource() ) );
                 // all resource requests shall be reinserted
@@ -3214,45 +3293,33 @@ InsertProjectCmd::InsertProjectCmd( Project &project, Node *parent, Node *after,
             int i = n->requests().takeRequest( gr );
             Q_ASSERT( i >= 0 );
         }
-        qDebug()<<"InsertProjectCmd groupRequest:"<<n->requests().requests().count();
     }
-    qDebug()<<"InsertProjectCmd: Merge resources";
-    //qDebug()<<"InsertProjectCmd: project groups"<<project.resourceGroups();
-    //qDebug()<<"InsertProjectCmd: m_project grps"<<m_project->resourceGroups();
     QMap<ResourceGroup*, ResourceGroup*> existingGroups;
     QMap<Resource*, Resource*> existingResources;
     foreach ( ResourceGroup *g, project.resourceGroups() ) {
         ResourceGroup *gr = m_project->findResourceGroup( g->id() );
         if ( gr == 0 ) {
-            //qDebug()<<"InsertProjectCmd: new group, add to m_project"<<g<<g->name();
             addCommand( new AddResourceGroupCmd( m_project, g, QString("ResourceGroup") ) );
             gr = g;
         } else {
-            //qDebug()<<"InsertProjectCmd: group exists"<<g->name()<<gr->name();
             existingGroups[ gr ] = g;
         }
         foreach ( Resource *r, g->resources() ) {
-            ////qDebug()<<"InsertProjectCmd: res sch"<<r->schedules();
             while ( Schedule *s = r->schedules().values().value( 0 ) ) {
                 r->deleteSchedule( s ); // schedules not handled
             }
             Resource *res = m_project->findResource( r->id() );
             if ( res == 0 ) {
-                //qDebug()<<"InsertProjectCmd: new resource, add to m_project"<<r<<r->name();
                 addCommand( new AddResourceCmd( gr, r, "Resource" ) );
             } else {
-                //qDebug()<<"InsertProjectCmd: resource exists"<<r->name()<<res->name();
                 existingResources[ res ] = r;
             }
         }
     }
-    qDebug()<<"InsertProjectCmd: Insert requests:";
-    //qDebug()<<"InsertProjectCmd: existingGroups"<<existingGroups;
     // Requests: clean up requests to resources already in m_project
     int gi = 0;
     int ri = 0;
     foreach ( ResourceGroupRequest *gr, greqs.keys() ) {
-        //qDebug()<<"InsertProjectCmd: "<<gr->group()<<existingGroups;
         QPair<Node*, ResourceGroup*> pair = greqs[ gr ];
         Node *n = pair.first;
         ResourceGroup *newGroup = pair.second;
@@ -3270,29 +3337,24 @@ InsertProjectCmd::InsertProjectCmd( Project &project, Node *parent, Node *after,
                 newRes = existingResources.keys().value( existingResources.values().indexOf( newRes ) );
             }
             // all resource requests shall be reinserted
-            qDebug()<<"InsertProjectCmd resourceRequest:"<<n->name()<<rr->resource()->name()<<(newRes==rr->resource()?"keep":"swap");
             rr->setResource( newRes );
             addCommand( new AddResourceRequestCmd( gr, rr, QString("Resource %1").arg( ++ri ) ) );
         }
-        qDebug()<<"InsertProjectCmd groupRequest:"<<n->requests().requests().count();
     }
     // Add nodes ( ids are unique, no need to check )
     Node *node_after = after;
     for ( int i = 0; i < project.numChildren(); ++i ) {
         Node *n = project.childNode( i );
         Q_ASSERT( n );
-        //qDebug()<<"InsertProjectCmd: node sch"<<n->schedules();
         while ( Schedule *s = n->schedules().values().value( 0 ) ) {
             n->takeSchedule( s ); // schedules not handled
             delete s;
         }
         n->setParentNode( 0 );
-        //qDebug()<<"InsertProjectCmd: add"<<project.name()<<"->"<<n->name();
         if ( node_after ) {
             addCommand( new TaskAddCmd( m_project, n, node_after, "Task" ) );
             node_after = n;
         } else {
-            qDebug()<<"InsertProjectCmd: add subtask"<<parent->name()<<"->"<<n->name();
             addCommand( new SubtaskAddCmd( m_project, n, parent, "Subtask" ) );
         }
         addChildNodes( n );
@@ -3305,7 +3367,6 @@ InsertProjectCmd::InsertProjectCmd( Project &project, Node *parent, Node *after,
     }
     // Remove nodes from project so they are not deleted
     while ( Node *ch = project.childNode( 0 ) ) {
-        //qDebug()<<"InsertProjectCmd: remove"<<project.name()<<"->"<<ch->name();
         project.takeChildNode( ch );
     }
     foreach ( Node *n, project.allNodes() ) {
@@ -3350,20 +3411,17 @@ void InsertProjectCmd::addCalendars( Calendar *calendar, Calendar *parent, QMap<
 
 void InsertProjectCmd::addChildNodes( Node *node ) {
     // schedules not handled
-    //qDebug()<<"InsertProjectCmd: node sch"<<node->schedules();
     while ( Schedule *s = node->schedules().values().value( 0 ) ) {
         node->takeSchedule( s ); // schedules not handled
         delete s;
     }
     foreach ( Node *n, node->childNodeIterator() ) {
         n->setParentNode( 0 );
-        //qDebug()<<"addChildNodes: add"<<node->name()<<"->"<<n->name();
         addCommand( new SubtaskAddCmd( m_project, n, node, "Subtask" ) );
         addChildNodes( n );
     }
     // Remove child nodes so they are not added twice
     while ( Node *ch = node->childNode( 0 ) ) {
-        //qDebug()<<"addChildNodes: remove"<<node->name()<<"->"<<ch->name();
         node->takeChildNode( ch );
     }
 }
