@@ -97,6 +97,7 @@ public:
     
     virtual bool isBaselined() const;
     virtual bool usePert() const;
+    virtual void setAllowOverbooking( bool state );
     virtual bool allowOverbooking() const;
     virtual bool checkExternalAppointments() const;
 
@@ -117,10 +118,10 @@ public:
     /// Return calculation mode
     int calculationMode() const { return m_calculationMode; }
     /// Return the list of appointments
-    QList<Appointment*> &appointments() { return m_appointments; }
+    QList<Appointment*> appointments() const { return m_appointments; }
     /// Return the list of appointments
     /// @param which specifies which list is returned
-    QList<Appointment*> &appointments( int which);
+    QList<Appointment*> appointments(int which) const;
     /// Adds appointment to this schedule only
     virtual bool add( Appointment *appointment );
     /// Adds appointment to both this resource schedule and node schedule
@@ -131,11 +132,16 @@ public:
     /// Attach the appointment to appropriate list (appointment->calculationMode() specifies list)
     bool attatch( Appointment *appointment );
     
+    DateTime appointmentStartTime() const;
+    DateTime appointmentEndTime() const;
+
     virtual Appointment appointmentIntervals( int which = Scheduling ) const;
 
     virtual bool isOverbooked() const { return false; }
     virtual bool isOverbooked( const KDateTime & /*start*/, const KDateTime & /*end*/ ) const { return false; }
     virtual QStringList overbookedResources() const;
+    /// Returns the first booked interval to @p node that intersects @p interval (limited to @p interval)
+    virtual DateTimeInterval firstBookedInterval( const DateTimeInterval &interval, const Schedule *node ) const;
 
     /// Return the resources that has appointments to this schedule
     virtual QList<Resource*> resources() const;
@@ -144,7 +150,8 @@ public:
     
     virtual EffortCostMap bcwsPrDay() const;
     virtual EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end ) const;
-
+    virtual EffortCostMap plannedEffortCostPrDay( const Resource *resource, const QDate &start, const QDate &end ) const;
+    
     /// Returns the total planned effort for this schedule
     virtual Duration plannedEffort() const;
     /// Returns the total planned effort for this schedule on date
@@ -287,7 +294,7 @@ protected:
     bool resourceOverbooked;
     /// Set if the requested resource is not available
     bool resourceNotAvailable;
-    /// Set if the task cannot be scheduled to fullfill all the constraints
+    /// Set if the task cannot be scheduled to fullfil all the constraints
     bool schedulingError;
     /// Set if the node has not been scheduled
     bool notScheduled;
@@ -412,6 +419,7 @@ public:
     virtual bool isDeleted() const { return m_deleted; }
     
     virtual bool isBaselined() const;
+    virtual void setAllowOverbooking( bool state );
     virtual bool allowOverbooking() const;
     virtual bool checkExternalAppointments() const;
     virtual bool usePert() const;
@@ -462,8 +470,7 @@ public:
     }
     void addCriticalPathNode( Node *node );
     
-    QList<Schedule::Log> logs();
-    const QList<Schedule::Log> &logs() const;
+    QList<Schedule::Log> logs() const;
     virtual void addLog( Schedule::Log &log );
     virtual void clearLogs() { m_log.clear(); m_logPhase.clear(); }
     
@@ -510,9 +517,12 @@ public:
  * Expected-, Optimistic- and Pessimistic schedules.
  * A ScheduleManager can also have child manager(s).
  */
-class KPLATOKERNEL_EXPORT ScheduleManager
+class KPLATOKERNEL_EXPORT ScheduleManager : public QObject
 {
+    Q_OBJECT
 public:
+    enum CalculationResult { CalculationRunning = 0, CalculationDone, CalculationStopped, CalculationCanceled, CalculationError };
+
     explicit ScheduleManager( Project &project, const QString name = QString() );
     ~ScheduleManager();
     
@@ -528,7 +538,7 @@ public:
     
     int removeChild( const ScheduleManager *sm );
     void insertChild( ScheduleManager *sm, int index = -1 );
-    const QList<ScheduleManager*> &children() const { return m_children; }
+    QList<ScheduleManager*> children() const { return m_children; }
     int childCount() const { return m_children.count(); }
     ScheduleManager *childAt( int index ) const { return m_children.value( index ); }
     /// Return list of all child managers (also childrens children)
@@ -581,6 +591,9 @@ public:
     void setSchedulingDirection( bool on );
     bool schedulingDirection() const { return m_schedulingDirection; }
 
+    void setScheduling( bool on );
+    bool scheduling() const { return m_scheduling; }
+
     QList<MainSchedule*> schedules() const;
     int numSchedules() const;
     int indexOf( const MainSchedule *sch ) const;
@@ -603,7 +616,15 @@ public:
     int schedulerPluginIndex() const;
     void setSchedulerPlugin( int index );
 
+    void stopCalculation();
     void calculateSchedule();
+    int calculationResult() const { return m_calculationresult; }
+    void setCalculationResult( int r ) { m_calculationresult = r; }
+
+    int progress() const { return m_progress; }
+    void setProgress( int progress );
+    int maxProgress() const { return m_maxprogress; }
+    void setMaxProgress( int max ) { m_maxprogress = max; }
 
 protected:
     MainSchedule *loadMainSchedule( KoXmlElement &element, XMLLoaderObject &status );
@@ -620,6 +641,9 @@ protected:
     bool m_recalculate;
     DateTime m_recalculateFrom;
     bool m_schedulingDirection;
+    bool m_scheduling;
+    int m_progress;
+    int m_maxprogress;
     MainSchedule *m_expected;
     MainSchedule *m_optimistic;
     MainSchedule *m_pessimistic;
@@ -627,6 +651,8 @@ protected:
     QList<ScheduleManager*> m_children;
 
     QString m_schedulerPluginId;
+    
+    int m_calculationresult;
 };
 
 
