@@ -83,13 +83,14 @@ public:
             ActualEffort *takeEffort( const QDate &date ) { return m_actual.take( date ); }
             /// Returns the total effort for all registered dates
             Duration effort() const;
-            const QMap<QDate, ActualEffort*> &actualEffortMap() const { return m_actual; }
+            QMap<QDate, ActualEffort*> actualEffortMap() const { return m_actual; }
             
             /// Load from document
             bool loadXML(KoXmlElement &element, XMLLoaderObject &status );
             /// Save to document
             void saveXML(QDomElement &element) const;
-        
+            bool contains( const QDate &date ) const { return m_actual.contains( date ); }
+
         private:
             QMap<QDate, ActualEffort*> m_actual;
     };
@@ -179,10 +180,12 @@ public:
     Duration remainingEffort( const QDate &date ) const;
     /// Returns the total actual effort
     Duration actualEffort() const;
-    /// Returns the total actual effort on @ date
+    /// Returns the total actual effort on @p date
     Duration actualEffort( const QDate &date ) const;
     /// Returns the total actual effort upto and including @p date
     Duration actualEffortTo( const QDate &date ) const;
+    /// Returns the actual effort for @p resource on @p date
+    Duration actualEffort( const Resource *resource, const QDate &date ) const;
     /// TODO
     QString note() const;
     /// TODO
@@ -190,10 +193,12 @@ public:
     
     /// Returns the total actual cost
     double actualCost() const;
+    /// Returns the actual cost for @p resource
+    double actualCost( const Resource *resource ) const;
     /// Returns the actual cost on @p date
     double actualCost( const QDate &date ) const;
-    /// Returns the total actual cost for @p resource
-    double actualCost( const Resource *resource ) const;
+    /// Returns the total actual cost for @p resource on @p date
+    double actualCost( const Resource *resource, const QDate &date ) const;
     /// Returns the total actual effort and cost upto and including @p date
     EffortCost actualCostTo( const QDate &date ) const;
     
@@ -219,7 +224,9 @@ public:
     QStringList entrymodeList() const;
     
     EffortCostMap effortCostPrDay(const QDate &start, const QDate &end, long id = -1 ) const;
-    
+    /// Returns the actual effort and cost pr day used by @p resource
+    EffortCostMap effortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const;
+
 protected:
     void copy( const Completion &copy);
     double averageCostPrHour( const QDate &date, long id ) const;
@@ -286,7 +293,7 @@ public:
     const Completion &completion() const;
 
     void addLogEntry( DateTime &dt, const QString &str );
-    const QMap<DateTime, QString> &log() const;
+    QMap<DateTime, QString> log() const;
     QStringList log();
 
     /// Return a list of resources fetched from the appointements or requests
@@ -310,6 +317,9 @@ public:
     DateTime transmitionTime() const { return m_transmitionTime; }
     void setTransmitionTime( const DateTime &dt ) { m_transmitionTime = dt; }
 
+    /// Clear workpackage data
+    void clear();
+
 private:
     Task *m_task;
     ScheduleManager *m_manager;
@@ -401,6 +411,11 @@ public:
      * for the interval start, end inclusive
      */
     virtual EffortCostMap plannedEffortCostPrDay(const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE ) const;
+    /**
+     * Returns a list of planned effort and cost for the @p resource
+     * for the interval @p start, @p end inclusive, useing schedule with identity @p id
+     */
+    virtual EffortCostMap plannedEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE ) const;
     
     /// Returns the total planned effort for this task (or subtasks) 
     virtual Duration plannedEffort( long id = CURRENTSCHEDULE ) const;
@@ -439,7 +454,9 @@ public:
      * for the interval start, end inclusive
      */
     virtual EffortCostMap actualEffortCostPrDay(const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE ) const;
-    
+    /// Returns the actual effort and cost pr day used by @p resource
+    virtual EffortCostMap actualEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const;
+
     /// Returns the effort planned to be used to reach the actual percent finished
     virtual Duration budgetedWorkPerformed( const QDate &date, long id = CURRENTSCHEDULE ) const;
 
@@ -479,18 +496,21 @@ public:
      * @param id Schedule identity. If id is CURRENTSCHEDULE, use current schedule.
      */
     Duration positiveFloat( long id = CURRENTSCHEDULE ) const;
+    void setPositiveFloat( const Duration &fl, long id = CURRENTSCHEDULE ) const;
     /**
      * Return the duration by which the duration of an activity or path 
-     * has to be reduced in order to fullfill a timing- or dependency constraint.
+     * has to be reduced in order to fullfil a timing- or dependency constraint.
      * @param id Schedule identity. If id is CURRENTSCHEDULE, use current schedule.
      */
     Duration negativeFloat( long id = CURRENTSCHEDULE ) const;
+    void setNegativeFloat( const Duration &fl, long id = CURRENTSCHEDULE ) const;
     /**
      * Return the duration by which an activity can be delayed or extended 
      * without affecting the start of any succeeding activity.
      * @param id Schedule identity. If id is CURRENTSCHEDULE, use current schedule.
      */
     Duration freeFloat( long id = CURRENTSCHEDULE ) const;
+    void setFreeFloat( const Duration &fl, long id = CURRENTSCHEDULE ) const;
     /**
      * Return the duration from Early Start to Late Start.
      * @param id Schedule identity. If id is CURRENTSCHEDULE, use current schedule.
@@ -515,7 +535,7 @@ public:
     virtual void setCurrentSchedule(long id);
     
     /**
-     * The assigned resources can not fullfill the estimated effort.
+     * The assigned resources can not fullfil the estimated effort.
      * @param id Schedule identity. If id is CURRENTSCHEDULE, use current schedule.
      */
     virtual bool effortMetError( long id = CURRENTSCHEDULE ) const;
@@ -545,6 +565,29 @@ public:
     QList<Relation*> parentProxyRelations() const { return  m_parentProxyRelations; }
     QList<Relation*> childProxyRelations() const { return  m_childProxyRelations; }
 
+    /**
+     * Calculates and returns the duration of the node.
+     * Uses the correct expected-, optimistic- or pessimistic effort
+     * dependent on @p use.
+     * @param time Where to start calculation.
+     * @param use Calculate using expected-, optimistic- or pessimistic estimate.
+     * @param backward If true, time specifies when the task should end.
+     */
+    virtual Duration duration(const DateTime &time, int use, bool backward);
+
+    /**
+     * Return the duration calculated on bases of the estimates calendar
+     */
+    Duration length(const DateTime &time, const Duration &duration, bool backward);
+    Duration length(const DateTime &time, const Duration &duration, Schedule *sch, bool backward);
+
+    /// Copy info from parent schedule
+    void copySchedule();
+    /// Copy intervals from parent schedule
+    void copyAppointments();
+    /// Copy intervals from parent schedule in the range @p start, @p end
+    void copyAppointments( const DateTime &start, const DateTime &end = DateTime() );
+
 signals:
     void workPackageToBeAdded( Node *node, int row );
     void workPackageAdded( Node *node );
@@ -552,13 +595,6 @@ signals:
     void workPackageRemoved( Node *node );
 
 protected:
-    /// Copy info from parent schedule
-    void copySchedule();
-    /// Copy intervals from parent schedule
-    void copyAppointments();
-    /// Copy intervals from parent schedule in the range @p start, @p end
-    void copyAppointments( const DateTime &start, const DateTime &end = DateTime() );
-    
     /// Check if this node has any dependent child nodes
     virtual bool isEndNode() const;
     /// Check if this node has any dependent parent nodes
@@ -677,37 +713,25 @@ public:
     void setActivityFreeMargin(int value);
 
 protected:
-    /**
-     * Calculates and returns the duration of the node.
-     * Uses the correct expected-, optimistic- or pessimistic effort
-     * dependent on @p use.
-     * @param time Where to start calculation.
-     * @param use Calculate using expected-, optimistic- or pessimistic estimate.
-     * @param backward If true, time specifies when the task should end.
-     */
-    virtual Duration duration(const DateTime &time, int use, bool backward);
     /**
      * Return the duration calculated on bases of the requested resources
      */
     Duration calcDuration(const DateTime &time, const Duration &effort, bool backward);
 
-    /**
-     * Return the duration calculated on bases of the estimates calendar
-     */
-    Duration length(const DateTime &time, const Duration &duration, bool backward);
-
 private:
     DateTime calculateSuccessors(const QList<Relation*> &list, int use);
     DateTime calculatePredeccessors(const QList<Relation*> &list, int use);
     DateTime scheduleSuccessors(const QList<Relation*> &list, int use);
     DateTime schedulePredeccessors(const QList<Relation*> &list, int use);
     
-    // Check appointments if specified in currentschedule
-    DateTime workStartAfter(const DateTime &dt);
-    DateTime workFinishBefore(const DateTime &dt);
-    // Don't check for appointments
-    DateTime workTimeAfter(const DateTime &dt) const;
-    DateTime workTimeBefore(const DateTime &dt) const;
+    /// Fixed duration: Returns @p dt
+    /// Duration with calendar: Returns first available after @p dt
+    /// Has working resource(s) allocated: Returns the earliest time a resource can start work after @p dt, and checks appointments if @p sch is not null.
+    DateTime workTimeAfter(const DateTime &dt, Schedule *sch = 0) const;
+    /// Fixed duration: Returns @p dt
+    /// Duration with calendar: Returns first available before @p dt
+    /// Has working resource(s) allocated: Returns the latest time a resource can finish work, and checks appointments if @p sch is not null.
+    DateTime workTimeBefore(const DateTime &dt, Schedule *sch = 0) const;
     
 private:
     QList<ResourceGroup*> m_resource;
@@ -718,14 +742,6 @@ private:
     // This list store pointers to linked task
     QList<Node*> m_requiredTasks;
 
-    // Attributes used for calculation
-    DateTime m_earlyStartDate;
-    DateTime m_earlyFinishDate;
-    DateTime m_lateStartDate;
-    DateTime m_lateFinishDate;
-    int m_activitySlack;
-    int m_activityFreeMargin;
-
     WorkPackage m_workPackage;
     QList<WorkPackage*> m_packageLog;
 
