@@ -20,6 +20,7 @@
 #include "kptaccountsmodel.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptduration.h"
 #include "kptnode.h"
@@ -79,21 +80,21 @@ QVariant AccountModel::data( const Account *a, int property, int role ) const
 
 QVariant AccountModel::name( const Account *a, int role ) const
 {
-    //kDebug()<<res->name()<<","<<role;
+    //kDebug()<<a->name()<<","<<role;
     switch ( role ) {
         case Qt::DisplayRole:
         case Qt::EditRole:
             return a->name();
         case Qt::ToolTipRole:
-/*FIXME:            if ( a->isDefaultAccount() ) {
+            if ( a->isDefaultAccount() ) {
                 return i18nc( "1=account name", "%1 (Default account)", a->name() );
-            }*/
+            }
             return a->name();
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
-//FIXME:         case Qt::CheckStateRole:
-//             return a->isDefaultAccount() ? Qt::Checked : Qt::Unchecked;
+         case Qt::CheckStateRole:
+             return a->isDefaultAccount() ? Qt::Checked : Qt::Unchecked;
     }
     return QVariant();
 }
@@ -188,10 +189,10 @@ void AccountItemModel::setProject( Project *project )
     if ( m_project ) {
         Accounts *acc = &( m_project->accounts() );
         disconnect( acc , SIGNAL( changed( Account* ) ), this, SLOT( slotAccountChanged( Account* ) ) );
-        
+
         disconnect( acc, SIGNAL( accountAdded( const Account* ) ), this, SLOT( slotAccountInserted( const Account* ) ) );
         disconnect( acc, SIGNAL( accountToBeAdded( const Account*, int ) ), this, SLOT( slotAccountToBeInserted( const Account*, int ) ) );
-        
+
         disconnect( acc, SIGNAL( accountRemoved( const Account* ) ), this, SLOT( slotAccountRemoved( const Account* ) ) );
         disconnect( acc, SIGNAL( accountToBeRemoved( const Account* ) ), this, SLOT( slotAccountToBeRemoved( const Account* ) ) );
     }
@@ -200,10 +201,10 @@ void AccountItemModel::setProject( Project *project )
         Accounts *acc = &( project->accounts() );
         kDebug()<<acc;
         connect( acc, SIGNAL( changed( Account* ) ), this, SLOT( slotAccountChanged( Account* ) ) );
-        
+
         connect( acc, SIGNAL( accountAdded( const Account* ) ), this, SLOT( slotAccountInserted( const Account* ) ) );
         connect( acc, SIGNAL( accountToBeAdded( const Account*, int ) ), this, SLOT( slotAccountToBeInserted( const Account*, int ) ) );
-        
+
         connect( acc, SIGNAL( accountRemoved( const Account* ) ), this, SLOT( slotAccountRemoved( const Account* ) ) );
         connect( acc, SIGNAL( accountToBeRemoved( const Account* ) ), this, SLOT( slotAccountToBeRemoved( const Account* ) ) );
     }
@@ -220,7 +221,7 @@ Qt::ItemFlags AccountItemModel::flags( const QModelIndex &index ) const
     }
     if ( account ( index ) ) {
         switch ( index.column() ) {
-            case AccountModel::Name: flags |= ( Qt::ItemIsEditable /*FIXME:| Qt::ItemIsUserCheckable*/ ); break;
+            case AccountModel::Name: flags |= ( Qt::ItemIsEditable | Qt::ItemIsUserCheckable ); break;
             default: flags |= Qt::ItemIsEditable; break;
         }
     }
@@ -318,13 +319,13 @@ bool AccountItemModel::setName( Account *a, const QVariant &value, int role )
             switch ( value.toInt() ) {
                 case Qt::Unchecked:
                     if ( a->isDefaultAccount() ) {
-                        emit executeCommand( new ModifyDefaultAccountCmd( m_project->accounts(), a, 0, ( "Modify default account" ) ) ); //FIXME i18n
+                        emit executeCommand( new ModifyDefaultAccountCmd( m_project->accounts(), a, 0, i18n( "De-select as default account" ) ) );
                         return true;
                     }
                     break;
                 case Qt::Checked:
                     if ( ! a->isDefaultAccount() ) {
-                        emit executeCommand( new ModifyDefaultAccountCmd( m_project->accounts(), m_project->accounts().defaultAccount(), a, ( "Modify default account" ) ) ); //FIXME i18n
+                        emit executeCommand( new ModifyDefaultAccountCmd( m_project->accounts(), m_project->accounts().defaultAccount(), a, i18n( "Select as default account" ) ) );
                         return true;
                     }
                     break;
@@ -370,7 +371,8 @@ bool AccountItemModel::setData( const QModelIndex &index, const QVariant &value,
     if ( ! index.isValid() ) {
         return ItemModelBase::setData( index, value, role );
     }
-    if ( ( flags( index ) &Qt::ItemIsEditable ) == 0 ) {
+    if ( ( flags( index ) &( Qt::ItemIsEditable | Qt::CheckStateRole ) ) == 0 ) {
+        Q_ASSERT( true );
         return false;
     }
     Account *a = account( index );
@@ -480,12 +482,14 @@ void CostBreakdownItemModel::slotAccountToBeInserted( const Account *parent, int
 
 void CostBreakdownItemModel::slotAccountInserted( const Account *account )
 {
+    Q_UNUSED(account);
     //kDebug()<<account->name();
     endInsertRows();
 }
 
 void CostBreakdownItemModel::slotAccountToBeRemoved( const Account *account )
 {
+
     //kDebug()<<account->name();
     int row = index( account ).row();
     beginRemoveRows( index( account->parent() ), row, row );
@@ -493,12 +497,14 @@ void CostBreakdownItemModel::slotAccountToBeRemoved( const Account *account )
 
 void CostBreakdownItemModel::slotAccountRemoved( const Account *account )
 {
+    Q_UNUSED(account);
     //kDebug()<<account->name();
     endRemoveRows();
 }
 
 void CostBreakdownItemModel::slotNodeChanged( Node *node )
 {
+    Q_UNUSED(node);
     fetchData();
     foreach ( Account *a, m_plannedCostMap.keys() ) {
         QModelIndex idx1 = index( a );
@@ -513,13 +519,13 @@ void CostBreakdownItemModel::setProject( Project *project )
     if ( m_project ) {
         Accounts *acc = &( m_project->accounts() );
         disconnect( acc , SIGNAL( changed( Account* ) ), this, SLOT( slotAccountChanged( Account* ) ) );
-        
+
         disconnect( acc, SIGNAL( accountAdded( const Account* ) ), this, SLOT( slotAccountInserted( const Account* ) ) );
         disconnect( acc, SIGNAL( accountToBeAdded( const Account*, int ) ), this, SLOT( slotAccountToBeInserted( const Account*, int ) ) );
-        
+
         disconnect( acc, SIGNAL( accountRemoved( const Account* ) ), this, SLOT( slotAccountRemoved( const Account* ) ) );
         disconnect( acc, SIGNAL( accountToBeRemoved( const Account* ) ), this, SLOT( slotAccountToBeRemoved( const Account* ) ) );
-    
+
         disconnect( m_project , SIGNAL( nodeChanged( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
         disconnect( m_project , SIGNAL( nodeAdded( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
         disconnect( m_project , SIGNAL( nodeRemoved( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
@@ -529,13 +535,13 @@ void CostBreakdownItemModel::setProject( Project *project )
         Accounts *acc = &( project->accounts() );
         kDebug()<<acc;
         connect( acc, SIGNAL( changed( Account* ) ), this, SLOT( slotAccountChanged( Account* ) ) );
-        
+
         connect( acc, SIGNAL( accountAdded( const Account* ) ), this, SLOT( slotAccountInserted( const Account* ) ) );
         connect( acc, SIGNAL( accountToBeAdded( const Account*, int ) ), this, SLOT( slotAccountToBeInserted( const Account*, int ) ) );
-        
+
         connect( acc, SIGNAL( accountRemoved( const Account* ) ), this, SLOT( slotAccountRemoved( const Account* ) ) );
         connect( acc, SIGNAL( accountToBeRemoved( const Account* ) ), this, SLOT( slotAccountToBeRemoved( const Account* ) ) );
-    
+
         connect( project , SIGNAL( nodeChanged( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
         connect( project , SIGNAL( nodeAdded( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
         connect( project , SIGNAL( nodeRemoved( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
@@ -548,18 +554,10 @@ void CostBreakdownItemModel::setScheduleManager( ScheduleManager *sm )
     if ( m_manager != sm ) {
         m_manager = sm;
         fetchData();
-        updateDates();
         reset();
     }
 }
 
-void CostBreakdownItemModel::updateDates()
-{
-    setStartDate();
-    setEndDate();
-    kDebug()<<m_startmode<<m_start<<m_endmode<<m_end;
-}
-
 long CostBreakdownItemModel::id() const
 {
     return m_manager == 0 ? -1 : m_manager->id();
@@ -576,11 +574,20 @@ EffortCostMap CostBreakdownItemModel::fetchPlannedCost( Account *account )
         }
     }
     m_plannedCostMap.insert( account, ec );
+    QDate s = ec.startDate();
+    if ( ! m_plannedStart.isValid() || s < m_plannedStart ) {
+        m_plannedStart = s;
+    }
+    QDate e = ec.endDate();
+    if ( ! m_plannedEnd.isValid() || e > m_plannedEnd ) {
+        m_plannedEnd = e;
+    }
     return ec;
 }
 
 EffortCostMap CostBreakdownItemModel::fetchActualCost( Account *account )
 {
+    kDebug()<<account->name();
     EffortCostMap ec;
     if ( account->isElement() ) {
         ec = account->actualCost( id() );
@@ -590,6 +597,16 @@ EffortCostMap CostBreakdownItemModel::fetchActualCost( Account *account )
         }
     }
     m_actualCostMap.insert( account, ec );
+    QDate s = ec.startDate();
+    if ( ! m_actualStart.isValid() || s < m_actualStart ) {
+        m_actualStart = s;
+    }
+    QDate e = ec.endDate();
+    if ( ! m_actualEnd.isValid() || e > m_actualEnd ) {
+        m_actualEnd = e;
+    }
+    kDebug()<<account->name()<<ec.totalEffort().toDouble(Duration
+::Unit_h)<<ec.totalCost();
     return ec;
 }
 
@@ -597,6 +614,8 @@ void CostBreakdownItemModel::fetchData()
 {
     //kDebug()<<m_start<<m_end;
     m_plannedCostMap.clear();
+    m_plannedStart = m_plannedEnd = QDate();
+    m_actualStart = m_actualEnd = QDate();
     if ( m_project == 0 || m_manager == 0 ) {
         return;
     }
@@ -670,26 +689,26 @@ QModelIndex CostBreakdownItemModel::index( const Account *account ) const
 int CostBreakdownItemModel::columnCount( const QModelIndex & ) const
 {
     int c = 3;
-    if ( m_start.isValid() && m_end.isValid() ) {
+    if ( startDate().isValid() && endDate().isValid() ) {
         switch ( m_periodtype ) {
             case Period_Day: {
-                c += m_start.daysTo( m_end ) + 1;
+                c += startDate().daysTo( endDate()) + 1;
                 break;
             }
             case Period_Week: {
-                int days = KGlobal::locale()->weekStartDay() - m_start.dayOfWeek();
+                int days = KGlobal::locale()->weekStartDay() - startDate().dayOfWeek();
                 if ( days > 0 ) {
                     days -= 7;
                 }
-                QDate start = m_start.addDays( days );
-                c += (start.daysTo( m_end ) / 7) + 1;
+                QDate start = startDate().addDays( days );
+                c += (start.daysTo( endDate() ) / 7) + 1;
                 break;
             }
             case Period_Month: {
-                int days = m_start.daysInMonth() - m_start.day() + 1;
-                for ( QDate d = m_start; d < m_end; d = d.addDays( days ) ) {
+                int days = startDate().daysInMonth() - startDate().day() + 1;
+                for ( QDate d = startDate(); d < endDate(); d = d.addDays( days ) ) {
                     ++c;
-                    days = qMin( d.daysTo( m_end ), d.daysInMonth() );
+                    days = qMin( d.daysTo( endDate() ), d.daysInMonth() );
                 }
                 break;
             }
@@ -713,16 +732,16 @@ int CostBreakdownItemModel::rowCount( const QModelIndex &parent ) const
 QString CostBreakdownItemModel::formatMoney( double cost1, double cost2 ) const
 {
     if ( m_showmode == ShowMode_Planned ) {
-        return KGlobal::locale()->formatMoney( cost1, "", 0 );
+        return m_project->locale()->formatMoney( cost1, "", 0 );
     }
     if ( m_showmode == ShowMode_Actual ) {
-        return KGlobal::locale()->formatMoney( cost2, "", 0 );
+        return m_project->locale()->formatMoney( cost2, "", 0 );
     }
     if ( m_showmode == ShowMode_Both ) {
-        return QString(m_format).arg( KGlobal::locale()->formatMoney( cost2, "", 0 ) ).arg( KGlobal::locale()->formatMoney( cost1, "", 0 ) );
+        return QString(m_format).arg( m_project->locale()->formatMoney( cost2, "", 0 ) ).arg( m_project->locale()->formatMoney( cost1, "", 0 ) );
     }
     if ( m_showmode == ShowMode_Deviation ) {
-        return KGlobal::locale()->formatMoney( cost1 - cost2, "", 0 );
+        return m_project->locale()->formatMoney( cost1 - cost2, "", 0 );
     }
     return "";
 }
@@ -749,42 +768,42 @@ QVariant CostBreakdownItemModel::data( const QModelIndex &index, int role ) cons
                     case Period_Day: {
                         double planned = 0.0;
                         if ( m_cumulative ) {
-                            planned = pc.costTo( m_start.addDays( col ) );
+                            planned = pc.costTo( startDate().addDays( col ) );
                         } else {
-                            planned = pc.costOnDate( m_start.addDays( col ) );
+                            planned = pc.costOnDate( startDate().addDays( col ) );
                         }
                         double actual = 0.0;
                         if ( m_cumulative ) {
-                            actual = ac.costTo( m_start.addDays( col ) );
+                            actual = ac.costTo( startDate().addDays( col ) );
                         } else {
-                            actual = ac.costOnDate( m_start.addDays( col ) );
+                            actual = ac.costOnDate( startDate().addDays( col ) );
                         }
                         return formatMoney( planned, actual );
                     }
                     case Period_Week: {
-                        int days = KGlobal::locale()->weekStartDay() - m_start.dayOfWeek();
+                        int days = KGlobal::locale()->weekStartDay() - startDate().dayOfWeek();
                         if ( days > 0 ) {
                             days -= 7; ;
                         }
-                        QDate start = m_start.addDays( days );
+                        QDate start = startDate().addDays( days );
                         int week = col;
                         double planned = 0.0;
                         if ( m_cumulative ) {
                             planned = pc.costTo( start.addDays( ++week * 7 ) );
                         } else {
-                            planned = week == 0 ? pc.cost( m_start, m_start.daysTo( start.addDays( 7 ) ) ) : pc.cost( start.addDays( week * 7 ) );
+                            planned = week == 0 ? pc.cost( startDate(), startDate().daysTo( start.addDays( 7 ) ) ) : pc.cost( start.addDays( week * 7 ) );
                         }
                         double actual = 0.0;
                         if ( m_cumulative ) {
                             actual = ac.costTo( start.addDays( ++week * 7 ) );
                         } else {
-                            actual = week == 0 ? ac.cost( m_start, m_start.daysTo( start.addDays( 7 ) ) ) : ac.cost( start.addDays( week * 7 ) );
+                            actual = week == 0 ? ac.cost( startDate(), startDate().daysTo( start.addDays( 7 ) ) ) : ac.cost( start.addDays( week * 7 ) );
                         }
                         return formatMoney( planned, actual );
                     }
                     case Period_Month: {
-                        int days = m_start.daysInMonth() - m_start.day() + 1;
-                        QDate start = m_start;
+                        int days = startDate().daysInMonth() - startDate().day() + 1;
+                        QDate start = startDate();
                         for ( int i = 0; i < col; ++i ) {
                             start = start.addDays( days );
                             days = start.daysInMonth();
@@ -809,13 +828,85 @@ QVariant CostBreakdownItemModel::data( const QModelIndex &index, int role ) cons
                 }
             }
         }
-    }
-    if ( role == Qt::TextAlignmentRole ) {
+    } else if ( role == Qt::ToolTipRole ) {
+        switch ( index.column() ) {
+            case 0: return a->name();
+            case 1: return a->description();
+            case 2: {
+                double act = m_actualCostMap.value( a ).totalCost();
+                double pl = m_plannedCostMap.value( a ).totalCost();
+                return i18n( "Actual total cost: %1, planned total cost: %2", m_project->locale()->formatMoney( act, "", 0 ), m_project->locale()->formatMoney( pl, "", 0 ) );
+            }
+            default: break;
+        }
+    } else if ( role == Qt::TextAlignmentRole ) {
         return headerData( index.column(), Qt::Horizontal, role );
+    } else {
+        switch ( index.column() ) {
+            case 0:
+            case 1: return QVariant();
+            default: {
+                return cost( a, index.column() - 3, role );
+            }
+        }
     }
     return QVariant();
 }
 
+QVariant CostBreakdownItemModel::cost( const Account *a, int offset, int role ) const
+{
+    EffortCostMap costmap;
+    if ( role == Role::Planned ) {
+        costmap = m_plannedCostMap.value( const_cast<Account*>( a ) );
+    } else if ( role == Role::Actual ) {
+        costmap = m_actualCostMap.value( const_cast<Account*>( a ) );
+    } else {
+        return QVariant();
+    }
+    double cost = 0.0;
+    switch ( m_periodtype ) {
+        case Period_Day: {
+            if ( m_cumulative ) {
+                cost = costmap.costTo( startDate().addDays( offset ) );
+            } else {
+                cost = costmap.costOnDate( startDate().addDays( offset ) );
+            }
+            break;
+        }
+        case Period_Week: {
+            int days = KGlobal::locale()->weekStartDay() - startDate().dayOfWeek();
+            if ( days > 0 ) {
+                days -= 7; ;
+            }
+            QDate start = startDate().addDays( days );
+            int week = offset;
+            if ( m_cumulative ) {
+                cost = costmap.costTo( start.addDays( ++week * 7 ) );
+            } else {
+                cost = week == 0 ? costmap.cost( startDate(), startDate().daysTo( start.addDays( 7 ) ) ) : costmap.cost( start.addDays( week * 7 ) );
+            }
+            break;
+        }
+        case Period_Month: {
+            int days = startDate().daysInMonth() - startDate().day() + 1;
+            QDate start = startDate();
+            for ( int i = 0; i < offset; ++i ) {
+                start = start.addDays( days );
+                days = start.daysInMonth();
+            }
+            if ( m_cumulative ) {
+                cost = costmap.costTo( start.addDays( start.daysInMonth() - start.day() + 1 ) );
+            } else {
+                cost = costmap.cost( start, start.daysInMonth() - start.day() + 1);
+            }
+            break;
+        }
+        default:
+            break;
+    }
+    return cost;
+}
+
 int CostBreakdownItemModel::periodType() const
 {
     return m_periodtype;
@@ -837,7 +928,6 @@ int CostBreakdownItemModel::startMode() const
 void CostBreakdownItemModel::setStartMode( int mode )
 {
     m_startmode = mode;
-    setStartDate();
     reset();
 }
 
@@ -849,60 +939,27 @@ int CostBreakdownItemModel::endMode() const
 void CostBreakdownItemModel::setEndMode( int mode )
 {
     m_endmode = mode;
-    setEndDate();
     reset();
 }
 
-void CostBreakdownItemModel::setStartDate()
-{
-    if ( m_startmode == StartMode_Project ) {
-        m_start = m_project->startTime( id() ).date();
-        foreach ( const EffortCostMap &ec, m_plannedCostMap ) {
-            if ( ! ec.startDate().isValid() ) {
-                continue;
-            }
-            if ( ! m_start.isValid() || m_start > ec.startDate() ) {
-                m_start = ec.startDate();
-            }
-        }
-        foreach ( const EffortCostMap &ec, m_actualCostMap ) {
-            if ( ! ec.startDate().isValid() ) {
-                continue;
-            }
-            if ( ! m_start.isValid() || m_start > ec.startDate() ) {
-                m_start = ec.startDate();
-            }
-        }
-    }
-}
-
-void CostBreakdownItemModel::setEndDate()
-{
-    if ( m_endmode == EndMode_Project ) {
-        m_end = m_project->endTime( id() ).date();
-        foreach ( const EffortCostMap &ec, m_plannedCostMap ) {
-            if ( ! ec.endDate().isValid() ) {
-                continue;
-            }
-            if ( ! m_end.isValid() || m_end < ec.endDate() ) {
-                m_end = ec.endDate();
-            }
-        }
-        foreach ( const EffortCostMap &ec, m_actualCostMap ) {
-            if ( ! ec.endDate().isValid() ) {
-                continue;
-            }
-            if ( ! m_end.isValid() || m_end < ec.endDate() ) {
-                m_end = ec.endDate();
-            }
-        }
-    } else if ( m_endmode == EndMode_CurrentDate ) {
-        m_end = QDate::currentDate();
-    }
-}
-
 QDate CostBreakdownItemModel::startDate() const
 {
+    if ( m_project == 0 || m_manager == 0 ) {
+        return m_start;
+    }
+    switch ( m_startmode ) {
+        case StartMode_Project: {
+            QDate d = m_project->startTime( id() ).date();
+            if ( m_plannedStart.isValid() && m_plannedStart < d ) {
+                d = m_plannedStart;
+            }
+            if ( m_actualStart.isValid() && m_actualStart < d ) {
+                d = m_actualStart;
+            }
+            return d;
+        }
+        default: break;
+    }
     return m_start;
 }
 
@@ -915,6 +972,23 @@ void CostBreakdownItemModel::setStartDate( const QDate &date )
 
 QDate CostBreakdownItemModel::endDate() const
 {
+    if ( m_project == 0 || m_manager == 0 ) {
+        return m_end;
+    }
+    switch ( m_endmode ) {
+        case EndMode_Project: {
+            QDate d = m_project->endTime( id() ).date();
+            if ( m_plannedEnd.isValid() && m_plannedEnd > d ) {
+                d = m_plannedEnd;
+            }
+            if ( m_actualEnd.isValid() && m_actualEnd > d ) {
+                d = m_actualEnd;
+            }
+            return d;
+        }
+        case EndMode_CurrentDate: return QDate::currentDate();
+        default: break;
+    }
     return m_end;
 }
 
@@ -960,14 +1034,14 @@ QVariant CostBreakdownItemModel::headerData( int section, Qt::Orientation orient
             int col = section - 3;
             switch ( m_periodtype ) {
                 case Period_Day: {
-                    return m_start.addDays( col ).toString( Qt::ISODate );
+                    return startDate().addDays( col ).toString( Qt::ISODate );
                 }
                 case Period_Week: {
-                    return m_start.addDays( ( col ) * 7 ).weekNumber();
+                    return startDate().addDays( ( col ) * 7 ).weekNumber();
                 }
                 case Period_Month: {
-                    int days = m_start.daysInMonth() - m_start.day() + 1;
-                    QDate start = m_start;
+                    int days = startDate().daysInMonth() - startDate().day() + 1;
+                    QDate start = startDate();
                     for ( int i = 0; i < col; ++i ) {
                         start = start.addDays( days );
                         days = start.daysInMonth();
@@ -984,7 +1058,7 @@ QVariant CostBreakdownItemModel::headerData( int section, Qt::Orientation orient
             switch ( section ) {
                 case 0: return ToolTip::accountName();
                 case 1: return ToolTip::accountDescription();
-                case 2: return i18n( "The total cost for the account" );
+                case 2: return i18n( "The total cost for the account shown as: Actual cost [ Planned cost ]" );
                 default: return QVariant();
             }
         }
@@ -1007,6 +1081,7 @@ Account *CostBreakdownItemModel::account( const QModelIndex &index ) const
 
 void CostBreakdownItemModel::slotAccountChanged( Account *account )
 {
+    Q_UNUSED(account);
     fetchData();
     foreach ( Account *a, m_plannedCostMap.keys() ) {
         QModelIndex idx1 = index( a );
