@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2006 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2006 -2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -29,6 +29,8 @@
 
 #include <KoDocument.h>
 #include <KoShape.h>
+#include <KoPageLayoutWidget.h>
+#include <KoPagePreviewWidget.h>
 
 #include <QAbstractItemModel>
 #include <QAbstractProxyModel>
@@ -97,6 +99,15 @@ PrintingDialog::PrintingDialog( ViewBase *view )
     m_view( view ),
     m_widget( 0 )
 {
+    setPrinterPageLayout();
+}
+
+QAbstractPrintDialog::PrintDialogOptions PrintingDialog::printDialogOptions() const
+{
+    return QAbstractPrintDialog::PrintToFile |
+           QAbstractPrintDialog::PrintPageRange |
+           QAbstractPrintDialog::PrintCollateCopies |
+           QAbstractPrintDialog::DontUseSheet;
 }
 
 PrintingOptions PrintingDialog::printingOptions() const
@@ -109,6 +120,21 @@ void PrintingDialog::setPrintingOptions( const PrintingOptions &opt )
     return m_view->setPrintingOptions( opt );
 }
 
+void PrintingDialog::setPrinterPageLayout()
+{
+    QPrinter &p = printer();
+    KoPageLayout l = m_view->pageLayout();
+    QPrinter::Orientation o;
+    switch ( l.orientation ) {
+        case KoPageFormat::Portrait: o = QPrinter::Portrait; break;
+        case KoPageFormat::Landscape: o = QPrinter::Landscape; break;
+        default: o = QPrinter::Portrait; break;
+    }
+    p.setOrientation( o );
+    p.setPaperSize( KoPageFormat::printerPageSize( l.format ) );
+    p.setPageMargins( l.leftMargin, l.topMargin, l.rightMargin, l.bottomMargin, QPrinter::Point );
+}
+
 void PrintingDialog::startPrinting(RemovePolicy removePolicy )
 {
     PrintingOptions opt;
@@ -116,9 +142,35 @@ void PrintingDialog::startPrinting(RemovePolicy removePolicy )
         opt = m_widget->options();
     }
     setPrintingOptions( opt );
+    setPrinterPageLayout();
     KoPrintingDialog::startPrinting( removePolicy );
 }
 
+QWidget *PrintingDialog::createPageLayoutWidget() const
+{
+    QWidget *widget = new QWidget();
+    widget->setWindowTitle( i18nc( "@title:tab", "Page Layout" ) );
+
+    QHBoxLayout *lay = new QHBoxLayout(widget);
+    lay->setMargin(0);
+    widget->setLayout(lay);
+
+    KoPageLayoutWidget *w = new KoPageLayoutWidget( widget, m_view->pageLayout() );
+    w->showPageSpread( false );
+    w->layout()->setMargin( 0 );
+    lay->addWidget( w );
+
+    KoPagePreviewWidget *prev = new KoPagePreviewWidget( widget );
+    prev->setPageLayout( m_view->pageLayout() );
+    lay->addWidget( prev );
+
+    connect(w, SIGNAL(layoutChanged(const KoPageLayout&)), m_view, SLOT(setPageLayout(const KoPageLayout&)));
+
+    connect (w, SIGNAL(layoutChanged(const KoPageLayout&)), prev, SLOT(setPageLayout(const KoPageLayout&)));
+
+    return widget;
+}
+
 QList<QWidget*> PrintingDialog::createOptionWidgets() const
 {
     //kDebug();
@@ -292,7 +344,9 @@ void PrintingDialog::paint( QPainter &p, const PrintingOptions::Data &options, c
 //--------------
 ViewBase::ViewBase(KoDocument *doc, QWidget *parent)
     : KoView( doc, parent ),
-    m_readWrite( false )
+    m_readWrite( false ),
+    m_proj( 0 ),
+    m_schedulemanager( 0 )
 {
 }
     
@@ -333,12 +387,6 @@ void ViewBase::setGuiActive( bool active ) // virtual slot
     emit guiActivated( this, active );
 }
 
-ViewBase *ViewBase::hitView( const QPoint &glpos )
-{
-    //kDebug()<<glpos;
-    return this;
-}
-
 void ViewBase::slotUpdateReadWrite( bool rw )
 {
     updateReadWrite( rw );
@@ -367,6 +415,16 @@ void ViewBase::createOptionAction()
     addContextAction( actionOptions );
 }
 
+void ViewBase::slotOptionsFinished( int result )
+{
+    if ( result == QDialog::Accepted ) {
+        emit optionsModified();
+    }
+    if ( sender() ) {
+        sender()->deleteLater();
+    }
+}
+
 //----------------------
 TreeViewPrintingDialog::TreeViewPrintingDialog( ViewBase *view, TreeViewBase *treeview, Project *project )
     : PrintingDialog( view ),
@@ -433,6 +491,14 @@ int TreeViewPrintingDialog::firstRow( int page ) const
     return row;
 }
 
+QList<QWidget*> TreeViewPrintingDialog::createOptionWidgets() const
+{
+    QList<QWidget*> lst;
+    lst << createPageLayoutWidget();
+    lst += PrintingDialog::createOptionWidgets();
+    return  lst;
+}
+
 void TreeViewPrintingDialog::printPage( int page, QPainter &painter )
 {
     painter.save();
@@ -440,11 +506,12 @@ void TreeViewPrintingDialog::printPage( int page, QPainter &painter )
     m_firstRow = firstRow( page );
     
     QHeaderView *mh = m_tree->header();
-    int length = mh->length();;
+    int length = mh->length();
     int height = mh->height();
     QRect hRect = headerRect();
     QRect fRect = footerRect();
     QRect pageRect = printer().pageRect();
+    pageRect.moveTo( 0, 0 );
     QRect paperRect = printer().paperRect();
     
     QAbstractItemModel *model = m_tree->model();
@@ -1221,6 +1288,14 @@ int DoubleTreeViewPrintingDialog::firstRow( int page ) const
     return row;
 }
 
+QList<QWidget*> DoubleTreeViewPrintingDialog::createOptionWidgets() const
+{
+    QList<QWidget*> lst;
+    lst << createPageLayoutWidget();
+    lst += PrintingDialog::createOptionWidgets();
+    return  lst;
+}
+
 void DoubleTreeViewPrintingDialog::printPage( int page, QPainter &painter )
 {
     painter.save();
@@ -1234,6 +1309,7 @@ void DoubleTreeViewPrintingDialog::printPage( int page, QPainter &painter )
     QRect hRect = headerRect();
     QRect fRect = footerRect();
     QRect pageRect = printer().pageRect();
+    pageRect.moveTo( 0, 0 );
     QRect paperRect = printer().paperRect();
     
     QAbstractItemModel *model = m_tree->model();
@@ -1358,6 +1434,11 @@ KoPrintJob *DoubleTreeViewBase::createPrintJob( ViewBase *parent )
     return dia;
 }
 
+void DoubleTreeViewBase::expandAll()
+{
+    m_leftview->expandAll();
+}
+
 void DoubleTreeViewBase::setParentsExpanded( const QModelIndex &idx, bool expanded )
 {
     //kDebug()<<idx<<m_leftview->isExpanded( idx )<<m_rightview->isExpanded( idx );
@@ -1458,7 +1539,7 @@ void DoubleTreeViewBase::hideColumns( TreeViewBase *view, const QList<int> &list
     view->setColumnsHidden( list );
 }
 
-void DoubleTreeViewBase::hideColumns( const QList<int> &masterList, QList<int> slaveList )
+void DoubleTreeViewBase::hideColumns( const QList<int> &masterList, const QList<int> slaveList )
 {
     m_leftview->setColumnsHidden( masterList );
     m_rightview->setColumnsHidden( slaveList );
@@ -1670,7 +1751,7 @@ void DoubleTreeViewBase::setStretchFactors()
 {
     int lc = m_leftview->header()->count() - m_leftview->header()->hiddenSectionCount();
     int rc = m_rightview->header()->count() - m_rightview->header()->hiddenSectionCount();
-    setStretchFactor( indexOf( m_rightview ), qMax( 1, qMin( 4, rc / lc ) ) );
+    setStretchFactor( indexOf( m_rightview ), qMax( 1, qMin( 4, rc / qMax( 1, lc ) ) ) );
     //kDebug()<<this<<"set stretch factor="<<qMax( 1, qMin( 4, rc / lc ) );
 }
 
