@@ -25,7 +25,6 @@
 #include <QList>
 #include <QHash>
 
-#include <QThread>
 #include <klocale.h>
 #include <kconfiggroup.h>
 
@@ -45,27 +44,53 @@
 #include "kis_random_sub_accessor.h"
 #include "kis_selection.h"
 #include "kis_node.h"
-#include "kis_painterly_overlay.h"
-
+#include "commands/kis_paintdevice_convert_type_command.h"
 #include "kis_datamanager.h"
 
 #include "kis_selection_component.h"
 #include "kis_pixel_selection.h"
-#include "kis_paint_device_jobs.h"
 #include "kis_repeat_iterators_pixel.h"
+#include <KoColorModelStandardIds.h>
+
+#include "tiles3/kis_hline_iterator.h"
+#include "tiles3/kis_vline_iterator.h"
+#include "tiles3/kis_rect_iterator.h"
+#include "tiles3/kis_random_accessor.h"
 
 class KisPaintDevice::Private
 {
 
 public:
-
     KisNodeWSP parent;
+    KisDefaultBounds defaultBounds;
     qint32 x;
     qint32 y;
-    KoColorSpace * colorSpace;
+    KoColorSpace* colorSpace;
     qint32 pixelSize;
     qint32 nChannels;
-    KisPainterlyOverlaySP painterlyOverlay;
+
+
+    struct Cache {
+
+        Cache() : self(0) {
+            invalidate();
+        }
+
+        KisPaintDevice* self;
+        bool thumbnailsValid;
+
+        QMap<int, QMap<int, QImage> > thumbnails;
+
+
+        void invalidate() {
+            thumbnailsValid = false;
+            if (self)
+                self->dataManager()->invalidateExactBounds();
+        }
+
+    };
+
+    Cache cache;
 
 };
 
@@ -77,6 +102,7 @@ KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& n
 
     Q_ASSERT(colorSpace);
 
+    m_d->cache.self = this;
     m_d->x = 0;
     m_d->y = 0;
 
@@ -92,23 +118,25 @@ KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& n
 
     m_d->parent = 0;
 
-    m_d->colorSpace = colorSpace->clone();
+    m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(colorSpace);
 
 }
 
-KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, const QString& name)
+KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, KisDefaultBounds defaultBounds, const QString& name)
         : QObject(0)
         , m_d(new Private())
 {
     setObjectName(name);
     Q_ASSERT(colorSpace);
 
+    m_d->cache.self = this;
     m_d->x = 0;
     m_d->y = 0;
 
-    m_d->parent = parent;
+    setParentNode(parent);
+    setDefaultBounds(defaultBounds);
 
-    m_d->colorSpace = colorSpace->clone();
+    m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(colorSpace);
 
     Q_ASSERT(m_d->colorSpace);
 
@@ -117,7 +145,7 @@ KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpac
 
     quint8 *defaultPixel = new quint8[ m_d->pixelSize ];
     m_d->colorSpace->fromQColor(Qt::black, defaultPixel);
-    m_d->colorSpace->setAlpha(defaultPixel, OPACITY_TRANSPARENT, 1);
+    m_d->colorSpace->setOpacity(defaultPixel, OPACITY_TRANSPARENT_U8, 1);
 
     m_datamanager = new KisDataManager(m_d->pixelSize, defaultPixel);
     delete [] defaultPixel;
@@ -128,9 +156,10 @@ KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpac
 
 KisPaintDevice::KisPaintDevice(const KisPaintDevice& rhs)
         : QObject()
-        , KisShared(rhs)
+        , KisShared()
         , m_d(new Private())
 {
+    m_d->cache.self = this;
     if (this != &rhs) {
         m_d->parent = 0;
         if (rhs.m_datamanager) {
@@ -141,7 +170,7 @@ KisPaintDevice::KisPaintDevice(const KisPaintDevice& rhs)
         }
         m_d->x = rhs.m_d->x;
         m_d->y = rhs.m_d->y;
-        m_d->colorSpace = rhs.m_d->colorSpace->clone();
+        m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(rhs.m_d->colorSpace);
 
         m_d->pixelSize = rhs.m_d->pixelSize;
 
@@ -151,29 +180,46 @@ KisPaintDevice::KisPaintDevice(const KisPaintDevice& rhs)
 
 KisPaintDevice::~KisPaintDevice()
 {
-    delete m_d->colorSpace;
+    KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
     delete m_d;
 }
 
 void KisPaintDevice::setDirty(const QRect & rc)
 {
-
+    m_d->cache.invalidate();
     if (m_d->parent.isValid())
         m_d->parent->setDirty(rc);
 }
 
 void KisPaintDevice::setDirty(const QRegion & region)
 {
+    m_d->cache.invalidate();
     if (m_d->parent.isValid())
         m_d->parent->setDirty(region);
 }
 
 void KisPaintDevice::setDirty()
 {
+    m_d->cache.invalidate();
     if (m_d->parent.isValid())
         m_d->parent->setDirty();
 }
 
+void KisPaintDevice::setParentNode(KisNodeWSP parent)
+{
+    m_d->parent = parent;
+}
+
+void KisPaintDevice::setDefaultBounds(KisDefaultBounds defaultBounds)
+{
+    m_d->defaultBounds = defaultBounds;
+}
+
+KisDefaultBounds KisPaintDevice::defaultBounds() const
+{
+    return m_d->defaultBounds;
+}
+
 void KisPaintDevice::move(qint32 x, qint32 y)
 {
     QRect dirtyRegion = extent();
@@ -203,15 +249,24 @@ void KisPaintDevice::extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
 
 QRect KisPaintDevice::extent() const
 {
-    qint32 x, y, w, h;
-    m_datamanager->extent(x, y, w, h);
-    x += m_d->x;
-    y += m_d->y;
+    QRect extent;
 
-    return QRect(x, y, w, h);
+    quint8 defaultOpacity = colorSpace()->opacityU8(defaultPixel());
+    if (defaultOpacity != OPACITY_TRANSPARENT_U8) {
+        extent = m_d->defaultBounds.bounds();
+    } else {
+        qint32 x, y, w, h;
+        m_datamanager->extent(x, y, w, h);
+        x += m_d->x;
+        y += m_d->y;
+        extent = QRect(x, y, w, h);
+    }
+
+    return extent;
 }
 
-void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const {
+void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
+{
     QRect rc = exactBounds();
     x = rc.x();
     y = rc.y();
@@ -222,9 +277,19 @@ void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) con
 
 QRect KisPaintDevice::exactBounds() const
 {
+    quint8 defaultOpacity = colorSpace()->opacityU8(defaultPixel());
+    if(defaultOpacity != OPACITY_TRANSPARENT_U8) {
+        return m_d->defaultBounds.bounds();
+    }
+
+    if (m_datamanager->validExactBounds()) {
+        return m_datamanager->exactBounds();
+    }
+
     // Solution n°2
     qint32  x, y, w, h, boundX2, boundY2, boundW2, boundH2;
     QRect rc = extent();
+
     x = boundX2 = rc.x();
     y = boundY2 = rc.y();
     w = boundW2 = rc.width();
@@ -250,7 +315,7 @@ QRect KisPaintDevice::exactBounds() const
 
     found = false;
 
-    for (qint32 y2 = y + h; y2 >= y ; --y2) {
+    for (qint32 y2 = y + h - 1; y2 >= y ; --y2) {
         KisHLineConstIterator it = const_cast<KisPaintDevice *>(this)->createHLineConstIterator(x, y2, w);
         while (!it.isDone() && found == false) {
             if (memcmp(it.rawData(), defaultPixel, m_d->pixelSize) != 0) {
@@ -285,7 +350,7 @@ QRect KisPaintDevice::exactBounds() const
     // Look for right edge )
     {
 
-        for (qint32 x2 = x + w; x2 >= x; --x2) {
+        for (qint32 x2 = x + w - 1; x2 >= x; --x2) {
 
             KisVLineConstIterator it = const_cast<KisPaintDevice *>(this)->createVLineConstIterator(x2, boundY2, boundH2);
             while (!it.isDone() && found == false) {
@@ -301,7 +366,9 @@ QRect KisPaintDevice::exactBounds() const
             if (found) break;
         }
     }
-    return QRect(boundX2, boundY2, boundW2, boundH2);
+    m_datamanager->setExactBounds(QRect(boundX2, boundY2, boundW2, boundH2));
+
+    return m_datamanager->exactBounds();
 }
 
 
@@ -320,6 +387,7 @@ void KisPaintDevice::crop(const QRect & r)
 
 void KisPaintDevice::setDefaultPixel(const quint8 *defPixel)
 {
+    m_d->cache.invalidate();
     m_datamanager->setDefaultPixel(defPixel);
 }
 
@@ -330,17 +398,20 @@ const quint8 *KisPaintDevice::defaultPixel() const
 
 void KisPaintDevice::clear()
 {
+    m_d->cache.invalidate();
     m_datamanager->clear();
 }
 
 void KisPaintDevice::fill(qint32 x, qint32 y, qint32 w, qint32 h, const quint8 *fillPixel)
 {
+    m_d->cache.invalidate();
     m_datamanager->clear(x, y, w, h, fillPixel);
 }
 
 
 void KisPaintDevice::clear(const QRect & rc)
 {
+    m_d->cache.invalidate();
     m_datamanager->clear(rc.x(), rc.y(), rc.width(), rc.height(), m_datamanager->defaultPixel());
 }
 
@@ -354,23 +425,25 @@ bool KisPaintDevice::write(KoStore *store)
 
 bool KisPaintDevice::read(KoStore *store)
 {
+    m_d->cache.invalidate();
     bool retval = m_datamanager->read(store);
     emit ioProgress(100);
 
     return retval;
 }
 
-void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
+QUndoCommand* KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
+    m_d->cache.invalidate();
+    dbgImage << this << colorSpace()->id() << dstColorSpace->id() << renderingIntent;
     if (*colorSpace() == *dstColorSpace) {
-        return;
+        return 0;
     }
 
     KisPaintDevice dst(dstColorSpace);
     dst.setX(x());
     dst.setY(y());
 
-
     qint32 x, y, w, h;
     QRect rc = exactBounds();
     x = rc.x();
@@ -406,57 +479,68 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
     }
 
     KisDataManagerSP oldData = m_datamanager;
-    const KoColorSpace *oldColorSpace = m_d->colorSpace;
+    KoColorSpace *oldColorSpace = m_d->colorSpace;
+
+    KisPaintDeviceConvertTypeCommand* cmd = new KisPaintDeviceConvertTypeCommand(this,
+            oldData,
+            oldColorSpace,
+            dst.m_datamanager,
+            dstColorSpace);
 
     setDataManager(dst.m_datamanager, dstColorSpace);
 
-    emit colorSpaceChanged(dstColorSpace);
-    delete oldColorSpace;
+    return cmd;
 }
 
 void KisPaintDevice::setProfile(const KoColorProfile * profile)
 {
     if (profile == 0) return;
-
+    m_d->cache.invalidate();
     const KoColorSpace * dstSpace =
-        KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(), profile);
-    if (dstSpace)
-        m_d->colorSpace = dstSpace->clone();
+        KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
+    if (dstSpace) {
+        KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
+        m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(dstSpace);
+    }
     emit profileChanged(profile);
 }
 
 void KisPaintDevice::setDataManager(KisDataManagerSP data, const KoColorSpace * colorSpace)
 {
+    m_d->cache.invalidate();
     m_datamanager = data;
-    m_d->colorSpace = colorSpace->clone();
+    KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
+    m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(colorSpace);
     m_d->pixelSize = m_d->colorSpace->pixelSize();
     m_d->nChannels = m_d->colorSpace->channelCount();
 
+    emit colorSpaceChanged(colorSpace);
 }
 
-void KisPaintDevice::convertFromQImage(const QImage& image, const QString &srcProfileName,
+void KisPaintDevice::convertFromQImage(const QImage& _image, const QString &srcProfileName,
                                        qint32 offsetX, qint32 offsetY)
 {
-    QImage img = image;
+    QImage image = _image;
 
-    if (img.format() != QImage::Format_ARGB32) {
-        img = img.convertToFormat(QImage::Format_ARGB32);
+    if (image.format() != QImage::Format_ARGB32) {
+        image = image.convertToFormat(QImage::Format_ARGB32);
     }
     // Don't convert if not no profile is given and both paint dev and qimage are rgba.
     if (srcProfileName.isEmpty() && colorSpace()->id() == "RGBA") {
-        writeBytes(img.bits(), 0, 0, img.width(), img.height());
+        writeBytes(image.bits(), offsetX, offsetY, image.width(), image.height());
     } else {
-        quint8 * dstData = new quint8[img.width() * img.height() * pixelSize()];
+        quint8 * dstData = new quint8[image.width() * image.height() * pixelSize()];
         KoColorSpaceRegistry::instance()
-            ->colorSpace("RGBA", srcProfileName)
-            ->convertPixelsTo(img.bits(), dstData, colorSpace(), img.width() * img.height());
-            
-        writeBytes(dstData, offsetX, offsetY, img.width(), img.height());
+        ->colorSpace(RGBAColorModelID.id(), Integer8BitsColorDepthID.id(), srcProfileName)
+        ->convertPixelsTo(image.bits(), dstData, colorSpace(), image.width() * image.height());
+
+        writeBytes(dstData, offsetX, offsetY, image.width(), image.height());
         delete[] dstData;
     }
+    m_d->cache.invalidate();
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile)
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile) const
 {
     qint32 x1;
     qint32 y1;
@@ -475,7 +559,7 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile)
     return convertToQImage(dstProfile, x1, y1, w, h);
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h)
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h) const
 {
 
     if (w < 0)
@@ -504,40 +588,45 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint3
     return image;
 }
 
-KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h) const
+KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h, const KisSelection * selection, QRect rect) const
 {
-    KisPaintDeviceSP thumbnail = KisPaintDeviceSP(new KisPaintDevice(colorSpace(), "thumbnail"));
+    KisPaintDeviceSP thumbnail = new KisPaintDevice(colorSpace());
     thumbnail->clear();
 
-    int srcw, srch;
-    const QRect e = extent();
-    srcw = e.width();
-    srch = e.height();
+    int srcWidth, srcHeight;
+    int srcX0, srcY0;
+    QRect e;
+    if (!rect.isValid())
+        e = extent();
+    else
+        e = rect;
+    e.getRect(&srcX0, &srcY0, &srcWidth, &srcHeight);
 
-    if (w > srcw) {
-        w = srcw;
-        h = qint32(double(srcw) / w * h);
+
+    if (w > srcWidth) {
+        w = srcWidth;
+        h = qint32(double(srcWidth) / w * h);
     }
-    if (h > srch) {
-        h = srch;
-        w = qint32(double(srch) / h * w);
+    if (h > srcHeight) {
+        h = srcHeight;
+        w = qint32(double(srcHeight) / h * w);
     }
 
-    if (srcw > srch)
-        h = qint32(double(srch) / srcw * w);
-    else if (srch > srcw)
-        w = qint32(double(srcw) / srch * h);
+    if (srcWidth > srcHeight)
+        h = qint32(double(srcHeight) / srcWidth * w);
+    else if (srcHeight > srcWidth)
+        w = qint32(double(srcWidth) / srcHeight * h);
 
-    KisRandomConstAccessorPixel iter = createRandomConstAccessor(0, 0);
-    KisRandomAccessorPixel dstIter = thumbnail->createRandomAccessor( 0, 0 );
+    KisRandomConstAccessorPixel iter = createRandomConstAccessor(0, 0, selection);
+    KisRandomAccessorPixel dstIter = thumbnail->createRandomAccessor(0, 0);
 
     for (qint32 y = 0; y < h; ++y) {
-        qint32 iY = (y * srch) / h;
+        qint32 iY = srcY0 + (y * srcHeight) / h;
         for (qint32 x = 0; x < w; ++x) {
-            qint32 iX = (x * srcw) / w;
+            qint32 iX = srcX0 + (x * srcWidth) / w;
             iter.moveTo(iX, iY);
-            dstIter.moveTo(x,  y );
-            memcpy( dstIter.rawData(), iter.rawData(), m_d->pixelSize );
+            dstIter.moveTo(x,  y);
+            memcpy(dstIter.rawData(), iter.rawData(), m_d->pixelSize);
         }
     }
 
@@ -545,34 +634,33 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h) const
 
 }
 
-KisPainterlyOverlaySP KisPaintDevice::painterlyOverlay()
+QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h, const KisSelection *selection, QRect rect)
 {
-    return m_d->painterlyOverlay;
-}
-
-void KisPaintDevice::createPainterlyOverlay()
-{
-    if (!m_d->painterlyOverlay) {
-        m_d->painterlyOverlay = new KisPainterlyOverlay();
-        emit(painterlyOverlayCreated());
+    if (m_d->cache.thumbnailsValid) {
+        if (m_d->cache.thumbnails.contains(w) && m_d->cache.thumbnails[w].contains(h)) {
+            QImage thumbnail = m_d->cache.thumbnails[w][h];
+            if (!thumbnail.isNull()) {
+                return thumbnail;
+            }
+        }
+    } else {
+        m_d->cache.thumbnails.clear();
     }
 
-}
+    KisPaintDeviceSP dev = createThumbnailDevice(w, h, selection, rect);
 
-void KisPaintDevice::removePainterlyOverlay()
-{
-    m_d->painterlyOverlay = 0;
-}
+    QImage thumbnail = dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile());
 
+    m_d->cache.thumbnails[w][h] = thumbnail;
+    m_d->cache.thumbnailsValid = true;
 
-QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h)
-{
-    KisPaintDeviceSP dev = createThumbnailDevice(w, h);
-    return dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile());
+    return thumbnail;
 }
 
 KisRectIteratorPixel KisPaintDevice::createRectIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
+
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -594,6 +682,7 @@ KisRectConstIteratorPixel KisPaintDevice::createRectConstIterator(qint32 left, q
 
 KisHLineIteratorPixel  KisPaintDevice::createHLineIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -602,6 +691,43 @@ KisHLineIteratorPixel  KisPaintDevice::createHLineIterator(qint32 x, qint32 y, q
     return KisHLineIteratorPixel(m_datamanager.data(), selectionDm, x, y, w, m_d->x, m_d->y);
 }
 
+
+KisHLineIteratorSP KisPaintDevice::createHLineIteratorNG(qint32 x, qint32 y, qint32 w)
+{
+    m_d->cache.invalidate();
+    return new KisHLineIterator2(m_datamanager.data(), x, y, w, m_d->x, m_d->y, true);
+}
+
+KisHLineConstIteratorSP KisPaintDevice::createHLineConstIteratorNG(qint32 x, qint32 y, qint32 w) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    return new KisHLineIterator2(dm, x, y, w, m_d->x, m_d->y, false);
+}
+
+KisVLineIteratorSP KisPaintDevice::createVLineIteratorNG(qint32 x, qint32 y, qint32 w)
+{
+    m_d->cache.invalidate();
+    return new KisVLineIterator2(m_datamanager.data(), x, y, w, m_d->x, m_d->y, true);
+}
+
+KisVLineConstIteratorSP KisPaintDevice::createVLineConstIteratorNG(qint32 x, qint32 y, qint32 w) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    return new KisVLineIterator2(dm, x, y, w, m_d->x, m_d->y, false);
+}
+
+KisRectIteratorSP KisPaintDevice::createRectIteratorNG(qint32 x, qint32 y, qint32 w, qint32 h)
+{
+    m_d->cache.invalidate();
+    return new KisRectIterator2(m_datamanager.data(), x, y, w, h, m_d->x, m_d->y, true);
+}
+
+KisRectConstIteratorSP KisPaintDevice::createRectConstIteratorNG(qint32 x, qint32 y, qint32 w, qint32 h) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    return new KisRectIterator2(dm, x, y, w, h, m_d->x, m_d->y, false);
+}
+
 KisHLineConstIteratorPixel  KisPaintDevice::createHLineConstIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection) const
 {
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
@@ -613,7 +739,7 @@ KisHLineConstIteratorPixel  KisPaintDevice::createHLineConstIterator(qint32 x, q
     return KisHLineConstIteratorPixel(dm, selectionDm, x, y, w, m_d->x, m_d->y);
 }
 
-KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection ) const
+KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection) const
 {
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
@@ -624,7 +750,7 @@ KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(
     return KisRepeatHLineConstIteratorPixel(dm, selectionDm, x, y, w, m_d->x, m_d->y, _dataWidth);
 }
 
-KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection ) const
+KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection) const
 {
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
@@ -637,6 +763,7 @@ KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(
 
 KisVLineIteratorPixel  KisPaintDevice::createVLineIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -658,6 +785,7 @@ KisVLineConstIteratorPixel  KisPaintDevice::createVLineConstIterator(qint32 x, q
 
 KisRandomAccessorPixel KisPaintDevice::createRandomAccessor(qint32 x, qint32 y, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -677,8 +805,22 @@ KisRandomConstAccessorPixel KisPaintDevice::createRandomConstAccessor(qint32 x,
     return KisRandomConstAccessorPixel(dm, selectionDm, x, y, m_d->x, m_d->y);
 }
 
+KisRandomAccessorSP KisPaintDevice::createRandomAccessorNG(qint32 x, qint32 y)
+{
+    m_d->cache.invalidate();
+    return new KisRandomAccessor2(m_datamanager.data(), x, y, m_d->x, m_d->y, true);
+}
+
+KisRandomConstAccessorSP KisPaintDevice::createRandomConstAccessorNG(qint32 x, qint32 y) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+
+    return new KisRandomAccessor2(dm, x, y, m_d->x, m_d->y, false);
+}
+
 KisRandomSubAccessorPixel KisPaintDevice::createRandomSubAccessor(const KisSelection * selection) const
 {
+    m_d->cache.invalidate();
     // XXX: shouldn't we use the selection here?
     Q_UNUSED(selection);
     KisPaintDevice* pd = const_cast<KisPaintDevice*>(this);
@@ -694,20 +836,24 @@ void KisPaintDevice::clearSelection(KisSelectionSP selection)
         KisHLineIterator devIt = createHLineIterator(r.x(), r.y(), r.width());
         KisHLineConstIterator selectionIt = selection->createHLineIterator(r.x(), r.y(), r.width());
 
+        const quint8* defaultPixel_ = defaultPixel();
+        bool transparentDefault = (m_d->colorSpace->opacityU8(defaultPixel_) == OPACITY_TRANSPARENT_U8);
         for (qint32 y = 0; y < r.height(); y++) {
 
             while (!devIt.isDone()) {
                 // XXX: Optimize by using stretches
 
                 m_d->colorSpace->applyInverseAlphaU8Mask(devIt.rawData(), selectionIt.rawData(), 1);
-
+                if (transparentDefault && m_d->colorSpace->opacityU8(devIt.rawData()) == OPACITY_TRANSPARENT_U8) {
+                    memcpy(devIt.rawData(), defaultPixel_, m_d->colorSpace->pixelSize());
+                }
                 ++devIt;
                 ++selectionIt;
             }
             devIt.nextRow();
             selectionIt.nextRow();
         }
-
+        m_datamanager->purge(r.translated(-m_d->x, -m_d->y));
         setDirty(r);
     }
 }
@@ -735,9 +881,9 @@ void KisPaintDevice::applySelectionMask(KisSelectionSP mask)
     }
 }
 
-bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c)
+bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c) const
 {
-    KisHLineConstIteratorPixel iter = createHLineIterator(x, y, 1);
+    KisHLineConstIteratorPixel iter = createHLineConstIterator(x, y, 1);
 
     const quint8 *pix = iter.rawData();
 
@@ -749,11 +895,11 @@ bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c)
 }
 
 
-bool KisPaintDevice::pixel(qint32 x, qint32 y, KoColor * kc)
+bool KisPaintDevice::pixel(qint32 x, qint32 y, KoColor * kc) const
 {
-    KisHLineIteratorPixel iter = createHLineIterator(x, y, 1);
+    KisHLineConstIteratorPixel iter = createHLineConstIterator(x, y, 1);
 
-    quint8 *pix = iter.rawData();
+    const quint8 *pix = iter.rawData();
 
     if (!pix) return false;
 
@@ -767,7 +913,7 @@ bool KisPaintDevice::setPixel(qint32 x, qint32 y, const QColor& c)
     KisHLineIteratorPixel iter = createHLineIterator(x, y, 1);
 
     colorSpace()->fromQColor(c, iter.rawData());
-
+    m_d->cache.invalidate();
     return true;
 }
 
@@ -783,7 +929,7 @@ bool KisPaintDevice::setPixel(qint32 x, qint32 y, const KoColor& kc)
         pix = kc.data();
         memcpy(iter.rawData(), pix, m_d->colorSpace->pixelSize());
     }
-
+    m_d->cache.invalidate();
     return true;
 }
 
@@ -814,7 +960,7 @@ void KisPaintDevice::setY(qint32 y)
 }
 
 
-void KisPaintDevice::readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h)
+void KisPaintDevice::readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h) const
 {
     m_datamanager->readBytes(data, x - m_d->x, y - m_d->y, w, h);
 }
@@ -827,11 +973,13 @@ void KisPaintDevice::readBytes(quint8 * data, const QRect &rect)
 void KisPaintDevice::writeBytes(const quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h)
 {
     m_datamanager->writeBytes(data, x - m_d->x, y - m_d->y, w, h);
+    m_d->cache.invalidate();
 }
 
 void KisPaintDevice::writeBytes(const quint8 * data, const QRect &rect)
 {
     writeBytes(data, rect.x(), rect.y(), rect.width(), rect.height());
+    m_d->cache.invalidate();
 }
 
 QVector<quint8*> KisPaintDevice::readPlanarBytes(qint32 x, qint32 y, qint32 w, qint32 h)
@@ -842,6 +990,7 @@ QVector<quint8*> KisPaintDevice::readPlanarBytes(qint32 x, qint32 y, qint32 w, q
 void KisPaintDevice::writePlanarBytes(QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h)
 {
     m_datamanager->writePlanarBytes(planes, channelSizes(), x, y, w, h);
+    m_d->cache.invalidate();
 }
 
 
