@@ -24,6 +24,7 @@
 
 #include <KoColorSpaceConstants.h>
 
+#include "kis_distance_information.h"
 #include "kis_global.h"
 #include "kis_types.h"
 #include <krita_export.h>
@@ -108,9 +109,6 @@ public:
     /// begin a transaction with the given command
     void beginTransaction(KisTransaction* command);
 
-    /// Return the current transcation
-    KisTransaction  * transaction();
-
     /// Returns the current paint device.
     const KisPaintDeviceSP device() const;
     KisPaintDeviceSP device();
@@ -131,6 +129,16 @@ public:
                 qint32 sx, qint32 sy,
                 qint32 sw, qint32 sh);
 
+    /**
+     * Blast the specific region from src to the current paint device using a \ref KisFixedPaintDevice
+     * @param selection the selection will be used between (0,0) and (sw-1,sh-1).
+     */
+    void bitBlt(qint32 dx, qint32 dy,
+                const KisPaintDeviceSP srcdev,
+                const KisFixedPaintDeviceSP selection,
+                qint32 sx, qint32 sy,
+                qint32 sw, qint32 sh);
+                
     /**
      * Convenience method that uses QPoint and QRect
      */
@@ -160,6 +168,30 @@ public:
      */
     void bltFixed(const QPoint & pos, const KisFixedPaintDeviceSP src, const QRect & srcRect);
 
+    /**
+     * Transfer the specified region from src onto the current paint device with selection. 
+     * Src is a * fixed-size paint device: this means that src must have the same colorspace as
+     * the destination device. Selection is also fixed-size paint device and it's colorspace has
+     * to be alpha8. Assert if there is wrong colorspace for selection
+     *
+     * @param dx the destination x-coordinate
+     * @param dy the destination y-coordinate
+     * @param op a pointer to the composite op use to blast the pixels from src on dst
+     * @param src the source device
+     * @param selection the selection stored in fixed device
+     * @param sx the source x-coordinate
+     * @param sy the source y-coordinate
+     * @param sw the width of the region
+     * @param sh the height of the region
+     */
+    void bltFixed(qint32 dx, qint32 dy,
+                  const KisFixedPaintDeviceSP srcDev,
+                  const KisFixedPaintDeviceSP selection,
+                  qint32 sx, qint32 sy,
+                  qint32 sw, qint32 sh);
+    
+    
+    
 
     /**
      * The methods below are 'higher' level than the above methods. They need brushes, colors
@@ -184,9 +216,9 @@ public:
      * @return the drag distance, that is the remains of the distance between p1 and p2 not covered
      * because the currenlty set brush has a spacing greater than that distance.
      */
-    double paintLine(const KisPaintInformation &pi1,
+    KisDistanceInformation paintLine(const KisPaintInformation &pi1,
                      const KisPaintInformation &pi2,
-                     double savedDist = -1);
+                     const KisDistanceInformation& savedDist = KisDistanceInformation());
 
     /**
      * Draw a Bezier curve between pos1 and pos2 using control points 1 and 2.
@@ -195,11 +227,11 @@ public:
      * @return the drag distance, that is the remains of the distance between p1 and p2 not covered
      * because the currenlty set brush has a spacing greater than that distance.
      */
-    double paintBezierCurve(const KisPaintInformation &pi1,
+    KisDistanceInformation paintBezierCurve(const KisPaintInformation &pi1,
                             const QPointF &control1,
                             const QPointF &control2,
                             const KisPaintInformation &pi2,
-                            const double savedDist = -1);
+                            const KisDistanceInformation& savedDist = KisDistanceInformation());
     /**
      * Fill the given vector points with the points needed to draw the Bezier curve between
      * pos1 and pos2 using control points 1 and 2, excluding the final pos2.
@@ -256,7 +288,7 @@ public:
     void paintPolygon(const vQPointF& points);
 
     /** Draw a spot at pos using the currently set paint op, brush and color */
-    void paintAt(const KisPaintInformation &pos);
+    double paintAt(const KisPaintInformation &pos);
 
     /**
      * Stroke the given QPainterPath.
@@ -273,6 +305,7 @@ public:
      * specified end position.
      *
      * XXX: this method really should work with subpixel precision for start and end position
+     * XXX: this method does not use the composite op
      */
     void drawLine(const QPointF & start, const QPointF & end);
 
@@ -281,6 +314,7 @@ public:
      * specified end position.
      *
      * XXX: this method really should work with subpixel precision for start and end position
+     * XXX: this method does not use the composite op
      */
     void drawDDALine(const QPointF & start, const QPointF & end);
 
@@ -289,12 +323,15 @@ public:
      * end position.
      *
      * XXX: this method really should work with subpixel precision for start and end position
+     * XXX: this method does not use the composite op
      */
     void drawWobblyLine(const QPointF & start, const QPointF & end);
 
     /**
      * Paint an unstroked, one-pixel wide line from the specified start to the specified
      * end position using the Wu algorithm
+     *
+     * XXX: this method does not use the composite op
      */
     void drawWuLine(const QPointF & start, const QPointF & end);
 
@@ -305,6 +342,7 @@ public:
      *
      * XXX: the width should be set in doubles, not integers.
      * XXX: this method really should work with subpixel precision for start and end position
+     * XXX: this method does not use the composite op
      */
     void drawThickLine(const QPointF & start, const QPointF & end, int startWidth, int endWidth);
 
@@ -343,10 +381,10 @@ public:
     KisPaintOp* paintOp() const;
 
     /// Set the current pattern
-    void setPattern(KisPattern * pattern);
+    void setPattern(const KisPattern * pattern);
 
     /// Returns the currently set pattern
-    KisPattern * pattern() const;
+    const KisPattern * pattern() const;
 
     /**
      * Set the color that will be used to paint with, and convert it
@@ -373,10 +411,10 @@ public:
     const KoColor &fillColor() const;
 
     /// Set the current generator (a generator can be used to fill an area
-    void setGenerator(KisFilterConfiguration * generator);
+    void setGenerator(const KisFilterConfiguration * generator);
 
     /// @return the current generator configuration
-    KisFilterConfiguration * generator() const;
+    const KisFilterConfiguration * generator() const;
 
     /// This enum contains the styles with which we can fill things like polygons and ellipses
     enum FillStyle {
@@ -460,6 +498,13 @@ public:
     */
     void setMaskImageSize(qint32 width, qint32 height);
 
+    /**
+     * If the alpha channel is locked, the alpha values of the paint device we are painting on
+     * will not change.
+     */
+    void setLockAlpha(bool protect);
+    bool alphaLocked() const;
+
 protected:
     /// Initialize, set everything to '0' or defaults
     void init();
