@@ -17,33 +17,36 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
- 
+
 #ifndef _KIS_WDG_BLUR_H_
 #define _KIS_WDG_BLUR_H_
 
-// TODO: remove that
-#define LCMS_HEADER <lcms.h>
-// TODO: remove it !
-
-#include <kis_filter_config_widget.h>
+#include <kis_config_widget.h>
 
 class KisFilter;
-class WdgBlur;
+class Ui_WdgBlur;
 
-class KisWdgBlur : public KisFilterConfigWidget
+class KisWdgBlur : public KisConfigWidget
 {
     Q_OBJECT
-    public:
-        KisWdgBlur( KisFilter* nfilter, QWidget * parent, const char * name);
-        inline WdgBlur* widget() { return m_widget; };
-        virtual void setConfiguration(KisFilterConfiguration*);
-    private slots:
-        void linkSpacingToggled(bool);
-        void spinBoxHalfWidthChanged(int );
-        void spinBoxHalfHeightChanged(int );
-    private:
-        bool m_halfSizeLink;
-        WdgBlur* m_widget;
+public:
+    KisWdgBlur(QWidget * parent);
+    inline const Ui_WdgBlur* widget() const {
+        return m_widget;
+    }
+    virtual void setConfiguration(const KisPropertiesConfiguration*);
+    virtual KisPropertiesConfiguration* configuration() const;
+
+private slots:
+
+    void linkSpacingToggled(bool);
+    void spinBoxHalfWidthChanged(int);
+    void spinBoxHalfHeightChanged(int);
+
+private:
+
+    bool m_halfSizeLink;
+    Ui_WdgBlur* m_widget;
 };
 
 #endif
