@@ -20,15 +20,16 @@
 #ifndef LEVEL_H
 #define LEVEL_H
 
-#include <kparts/plugin.h>
+#include <QObject>
+#include <QVariant>
 
 class KisColorSpace;
 class KisColorAdjustment;
 
-class LevelFilter : public KParts::Plugin
+class LevelFilter : public QObject
 {
 public:
-    LevelFilter(QObject *parent, const QStringList &);
+    LevelFilter(QObject *parent, const QVariantList &);
     virtual ~LevelFilter();
 };
 
