@@ -21,15 +21,16 @@
 #ifndef WAVEFILTER_H
 #define WAVEFILTER_H
 
-#include <kparts/plugin.h>
+#include <QObject>
+#include <QVariant>
 #include "filter/kis_filter.h"
 
 class KisConfigWidget;
 
-class KritaWaveFilter : public KParts::Plugin
+class KritaWaveFilter : public QObject
 {
 public:
-    KritaWaveFilter(QObject *parent, const QStringList &);
+    KritaWaveFilter(QObject *parent, const QVariantList &);
     virtual ~KritaWaveFilter();
 };
 
@@ -57,7 +58,7 @@ public:
 public:
     virtual int overlapMarginNeeded(const KisFilterConfiguration* _config) const;
 
-    virtual KisConfigWidget * createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, const KisImageSP image = 0) const;
+    virtual KisConfigWidget * createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, const KisImageWSP image = 0) const;
 };
 
 #endif
