@@ -112,20 +112,21 @@ struct KisBrush::Private {
 };
 
 KisBrush::KisBrush()
-        : KoResource("")
-        , d(new Private)
+    : KoResource("")
+    , d(new Private)
 {
 }
 
 KisBrush::KisBrush(const QString& filename)
-        : KoResource(filename)
-        , d(new Private)
+    : KoResource(filename)
+    , d(new Private)
 {
 }
 
 KisBrush::KisBrush(const KisBrush& rhs)
-        : KoResource("")
-        , d(new Private)
+    : KoResource("")
+    , KisShared()
+    , d(new Private)
 {
     m_image = rhs.m_image;
     d->brushType = rhs.d->brushType;
@@ -141,7 +142,7 @@ KisBrush::~KisBrush()
     delete d;
 }
 
-QImage KisBrush::img() const
+QImage KisBrush::image() const
 {
     return m_image;
 }
@@ -184,10 +185,11 @@ void KisBrush::setHotSpot(QPointF pt)
     d->hotSpot = QPointF(x, y);
 }
 
-QPointF KisBrush::hotSpot(double scaleX, double scaleY) const
+QPointF KisBrush::hotSpot(double scaleX, double scaleY, double rotation) const
 {
-    double w = width() * scaleX;
-    double h = height() * scaleY;
+    Q_UNUSED(scaleY);
+    double w = maskWidth( scaleX, rotation);
+    double h = maskHeight( scaleX, rotation);
 
     // The smallest brush we can produce is a single pixel.
     if (w < 1) {
@@ -222,13 +224,13 @@ bool KisBrush::canPaintFor(const KisPaintInformation& /*info*/)
     return true;
 }
 
-void KisBrush::setImage(const QImage& img)
+void KisBrush::setImage(const QImage& image)
 {
-    Q_ASSERT(!img.isNull());
-    m_image = img;
+    Q_ASSERT(!image.isNull());
+    m_image = image;
 
-    setWidth(img.width());
-    setHeight(img.height());
+    setWidth(image.width());
+    setHeight(image.height());
 
     clearScaledBrushes();
 
@@ -254,7 +256,10 @@ KisBrushSP KisBrush::fromXML(const QDomElement& element)
 qint32 KisBrush::maskWidth(double scale, double angle) const
 {
     double width_ = width() * scale;
+    if(angle == 0.0) return qint32(width_ + 1);
+
     double height_ = height() * scale;
+
     // Add one for sub-pixel shift
     if (angle >= 0.0 && angle < M_PI_2) {
         return qAbs(static_cast<qint32>(ceil(width_ * cos(angle) + height_ * sin(angle)) + 1));
@@ -269,8 +274,11 @@ qint32 KisBrush::maskWidth(double scale, double angle) const
 
 qint32 KisBrush::maskHeight(double scale, double angle) const
 {
-    double width_ = width() * scale;
     double height_ = height() * scale;
+    if(angle == 0.0) return qint32(height_ + 1);
+
+    double width_ = width() * scale;
+
     // Add one for sub-pixel shift
     if (angle >= 0.0 && angle < M_PI_2) {
         return qAbs(static_cast<qint32>(ceil(width_ * sin(angle) + height_ * cos(angle)) + 1));
@@ -321,10 +329,10 @@ void KisBrush::mask(KisFixedPaintDeviceSP dst, const KisPaintDeviceSP src, doubl
 
 
 void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
-        ColoringInformation* coloringInformation,
-        double scaleX, double scaleY, double angle,
-        const KisPaintInformation& info_,
-        double subPixelX, double subPixelY) const
+                                                   ColoringInformation* coloringInformation,
+                                                   double scaleX, double scaleY, double angle,
+                                                   const KisPaintInformation& info_,
+                                                   double subPixelX, double subPixelY) const
 {
     Q_ASSERT(valid());
     Q_UNUSED(angle);
@@ -337,9 +345,14 @@ void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
 
     KisQImagemaskSP outputMask = createMask(scale, subPixelX, subPixelY);
 
+    if (angle != 0)
+    {
+        outputMask->rotation(angle);
+    }
+
     qint32 maskWidth = outputMask->width();
     qint32 maskHeight = outputMask->height();
-
+    
     if (coloringInformation) {
 
         // old bounds
@@ -350,7 +363,7 @@ void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
 
         if (maskWidth * maskHeight <= bounds.width() * bounds.height()) {
             // just clear the data in dst,
-            memset(dst->data(), OPACITY_TRANSPARENT, maskWidth * maskHeight * dst->pixelSize());
+            memset(dst->data(), OPACITY_TRANSPARENT_U8, maskWidth * maskHeight * dst->pixelSize());
         } else {
             dst->initialize();
         }
@@ -371,15 +384,15 @@ void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
         }
     } else {
         // Mask everything out
-        cs->setAlpha(dst->data(), OPACITY_TRANSPARENT, dst->bounds().width() * dst->bounds().height());
+        cs->setOpacity(dst->data(), OPACITY_TRANSPARENT_U8, dst->bounds().width() * dst->bounds().height());
     }
 
     int rowWidth = dst->bounds().width();
 
-    quint8* maskPointer = outputMask->data();
     quint8* rowPointer = dabPointer;
 
     for (int y = 0; y < maskHeight; y++) {
+        quint8* maskPointer = outputMask->scanline(y);
         for (int x = 0; x < maskWidth; x++) {
             if (coloringInformation) {
                 if (color) {
@@ -405,15 +418,14 @@ void KisBrush::generateMaskAndApplyMaskOrCreateDab(KisFixedPaintDeviceSP dst,
     }
 }
 
-KisFixedPaintDeviceSP KisBrush::image(const KoColorSpace * colorSpace,
-                                      double scale, double angle,
-                                      const KisPaintInformation& info,
-                                      double subPixelX, double subPixelY) const
+KisFixedPaintDeviceSP KisBrush::paintDevice(const KoColorSpace * colorSpace,
+                                            double scale, double angle,
+                                            const KisPaintInformation& info,
+                                            double subPixelX, double subPixelY) const
 {
     Q_ASSERT(valid());
     Q_UNUSED(colorSpace);
     Q_UNUSED(info);
-    Q_UNUSED(angle);
     if (d->scaledBrushes.isEmpty()) {
         createScaledBrushes();
     }
@@ -449,6 +461,11 @@ KisFixedPaintDeviceSP KisBrush::image(const KoColorSpace * colorSpace,
 
         }
     }
+    
+    if (angle != 0.0)
+    {
+        outputImage = outputImage.transformed(QMatrix().rotate(-angle * 180 / M_PI));
+    }
 
     int outputWidth = outputImage.width();
     int outputHeight = outputImage.height();
@@ -504,19 +521,19 @@ void KisBrush::createScaledBrushes() const
         const_cast<KisBrush*>(this)->clearScaledBrushes();
     }
 
-    if (img().isNull()) {
+    if (image().isNull()) {
         return;
     }
     // Construct a series of brushes where each one's dimensions are
     // half the size of the previous one.
-    int width = img().width() * MAXIMUM_SCALE;
-    int height = img().height() * MAXIMUM_SCALE;
+    int width = image().width() * MAXIMUM_SCALE;
+    int height = image().height() * MAXIMUM_SCALE;
 
     QImage scaledImage;
     while (true) {
 
-        if (width >= img().width() && height >= img().height()) {
-            scaledImage = scaleImage(img(), width, height);
+        if (width >= image().width() && height >= image().height()) {
+            scaledImage = scaleImage(image(), width, height);
         } else {
             // Scale down the previous image once we're below 1:1.
             scaledImage = scaleImage(scaledImage, width, height);
@@ -525,8 +542,8 @@ void KisBrush::createScaledBrushes() const
         KisQImagemaskSP scaledMask = KisQImagemaskSP(new KisQImagemask(scaledImage, hasColor()));
         Q_CHECK_PTR(scaledMask);
 
-        double xScale = static_cast<double>(width) / img().width();
-        double yScale = static_cast<double>(height) / img().height();
+        double xScale = static_cast<double>(width) / image().width();
+        double yScale = static_cast<double>(height) / image().height();
         double scale = xScale;
 
         d->scaledBrushes.append(KisScaledBrush(scaledMask, hasColor() ? scaledImage : QImage(), scale, xScale, yScale));
@@ -623,10 +640,10 @@ KisQImagemaskSP KisBrush::scaleMask(const KisScaledBrush *srcBrush, double scale
 
             double yInterp = srcY - topY;
 
-            quint8 topLeft = (leftX >= 0 && leftX < srcWidth && topY >= 0 && topY < srcHeight) ? srcMask->alphaAt(leftX, topY) : OPACITY_TRANSPARENT;
-            quint8 bottomLeft = (leftX >= 0 && leftX < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcMask->alphaAt(leftX, topY + 1) : OPACITY_TRANSPARENT;
-            quint8 topRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY >= 0 && topY < srcHeight) ? srcMask->alphaAt(leftX + 1, topY) : OPACITY_TRANSPARENT;
-            quint8 bottomRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcMask->alphaAt(leftX + 1, topY + 1) : OPACITY_TRANSPARENT;
+            quint8 topLeft = (leftX >= 0 && leftX < srcWidth && topY >= 0 && topY < srcHeight) ? srcMask->alphaAt(leftX, topY) : OPACITY_TRANSPARENT_U8;
+            quint8 bottomLeft = (leftX >= 0 && leftX < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcMask->alphaAt(leftX, topY + 1) : OPACITY_TRANSPARENT_U8;
+            quint8 topRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY >= 0 && topY < srcHeight) ? srcMask->alphaAt(leftX + 1, topY) : OPACITY_TRANSPARENT_U8;
+            quint8 bottomRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcMask->alphaAt(leftX + 1, topY + 1) : OPACITY_TRANSPARENT_U8;
 
             double a = 1 - xInterp;
             double b = 1 - yInterp;
@@ -637,10 +654,10 @@ KisQImagemaskSP KisBrush::scaleMask(const KisScaledBrush *srcBrush, double scale
                                      + (1 - a) * b * topRight
                                      + (1 - a) * (1 - b) * bottomRight + 0.5);
 
-            if (d < OPACITY_TRANSPARENT) {
-                d = OPACITY_TRANSPARENT;
-            } else if (d > OPACITY_OPAQUE) {
-                d = OPACITY_OPAQUE;
+            if (d < OPACITY_TRANSPARENT_U8) {
+                d = OPACITY_TRANSPARENT_U8;
+            } else if (d > OPACITY_OPAQUE_U8) {
+                d = OPACITY_OPAQUE_U8;
             }
 
             dstMask->setAlphaAt(dstX, dstY, static_cast<quint8>(d));
@@ -658,7 +675,11 @@ QImage KisBrush::scaleImage(const KisScaledBrush *srcBrush, double scale, double
 
     QImage dstImage(dstWidth, dstHeight, QImage::Format_ARGB32);
 
-    const QImage srcImage = srcBrush->image();
+    QImage srcImage = srcBrush->image();
+    if (srcImage.format() != QImage::Format_ARGB32)
+    {
+        srcImage = srcImage.convertToFormat(QImage::Format_ARGB32);
+    }
 
     // Compute scales to map the scaled brush onto the required scale.
     double xScale = srcBrush->xScale() / scale;
@@ -668,13 +689,25 @@ QImage KisBrush::scaleImage(const KisScaledBrush *srcBrush, double scale, double
     int srcHeight = srcImage.height();
 
     for (int dstY = 0; dstY < dstHeight; dstY++) {
+
+        double srcY = (dstY - subPixelY + 0.5) * yScale;
+        srcY -= 0.5;
+        int topY = static_cast<int>(srcY);
+        if (srcY < 0) {
+            topY--;
+        }
+
+        QRgb *dstPixel = reinterpret_cast<QRgb *>(dstImage.scanLine(dstY));
+        QRgb *srcPixel = reinterpret_cast<QRgb *>(srcImage.scanLine(topY));
+        QRgb *srcPixelPlusOne = reinterpret_cast<QRgb *>(srcImage.scanLine(topY + 1));
+
         for (int dstX = 0; dstX < dstWidth; dstX++) {
 
             double srcX = (dstX - subPixelX + 0.5) * xScale;
-            double srcY = (dstY - subPixelY + 0.5) * yScale;
+
 
             srcX -= 0.5;
-            srcY -= 0.5;
+
 
             int leftX = static_cast<int>(srcX);
 
@@ -684,18 +717,12 @@ QImage KisBrush::scaleImage(const KisScaledBrush *srcBrush, double scale, double
 
             double xInterp = srcX - leftX;
 
-            int topY = static_cast<int>(srcY);
-
-            if (srcY < 0) {
-                topY--;
-            }
-
             double yInterp = srcY - topY;
 
-            QRgb topLeft = (leftX >= 0 && leftX < srcWidth && topY >= 0 && topY < srcHeight) ? srcImage.pixel(leftX, topY) : qRgba(0, 0, 0, 0);
-            QRgb bottomLeft = (leftX >= 0 && leftX < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcImage.pixel(leftX, topY + 1) : qRgba(0, 0, 0, 0);
-            QRgb topRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY >= 0 && topY < srcHeight) ? srcImage.pixel(leftX + 1, topY) : qRgba(0, 0, 0, 0);
-            QRgb bottomRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcImage.pixel(leftX + 1, topY + 1) : qRgba(0, 0, 0, 0);
+            QRgb topLeft = (leftX >= 0 && leftX < srcWidth && topY >= 0 && topY < srcHeight) ? srcPixel[leftX] : qRgba(0, 0, 0, 0);
+            QRgb bottomLeft = (leftX >= 0 && leftX < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcPixelPlusOne[leftX] : qRgba(0, 0, 0, 0);
+            QRgb topRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY >= 0 && topY < srcHeight) ? srcPixel[leftX + 1] : qRgba(0, 0, 0, 0);
+            QRgb bottomRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcPixelPlusOne[leftX + 1] : qRgba(0, 0, 0, 0);
 
             double a = 1 - xInterp;
             double b = 1 - yInterp;
@@ -742,17 +769,22 @@ QImage KisBrush::scaleImage(const KisScaledBrush *srcBrush, double scale, double
                 alpha = 255;
             }
 
-            dstImage.setPixel(dstX, dstY, qRgba(red, green, blue, alpha));
+            dstPixel[dstX] = qRgba(red, green, blue, alpha);
         }
     }
 
     return dstImage;
 }
 
-QImage KisBrush::scaleImage(const QImage& srcImage, int width, int height)
+QImage KisBrush::scaleImage(const QImage& _srcImage, int width, int height)
 {
     QImage scaledImage;
+    QImage srcImage = _srcImage; // detaches!
     //QString filename;
+    if (srcImage.format() != QImage::Format_ARGB32)
+    {
+        srcImage = srcImage.convertToFormat(QImage::Format_ARGB32);
+    }
 
     int srcWidth = srcImage.width();
     int srcHeight = srcImage.height();
@@ -770,13 +802,24 @@ QImage KisBrush::scaleImage(const QImage& srcImage, int width, int height)
         scaledImage = QImage(width, height, srcImage.format());
 
         for (int dstY = 0; dstY < height; dstY++) {
+
+            double srcY = (dstY + 0.5) * yScale;
+            srcY -= 0.5;
+            int topY = static_cast<int>(srcY);
+
+            if (srcY < 0) {
+                topY--;
+            }
+
+            QRgb *dstPixel = reinterpret_cast<QRgb *>(scaledImage.scanLine(dstY));
+            QRgb *srcPixel = reinterpret_cast<QRgb *>(srcImage.scanLine(topY));
+            QRgb *srcPixelPlusOne = reinterpret_cast<QRgb *>(srcImage.scanLine(topY + 1));
+
             for (int dstX = 0; dstX < width; dstX++) {
 
                 double srcX = (dstX + 0.5) * xScale;
-                double srcY = (dstY + 0.5) * yScale;
 
                 srcX -= 0.5;
-                srcY -= 0.5;
 
                 int leftX = static_cast<int>(srcX);
 
@@ -786,18 +829,12 @@ QImage KisBrush::scaleImage(const QImage& srcImage, int width, int height)
 
                 double xInterp = srcX - leftX;
 
-                int topY = static_cast<int>(srcY);
-
-                if (srcY < 0) {
-                    topY--;
-                }
-
                 double yInterp = srcY - topY;
 
-                QRgb topLeft = (leftX >= 0 && leftX < srcWidth && topY >= 0 && topY < srcHeight) ? srcImage.pixel(leftX, topY) : qRgba(0, 0, 0, 0);
-                QRgb bottomLeft = (leftX >= 0 && leftX < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcImage.pixel(leftX, topY + 1) : qRgba(0, 0, 0, 0);
-                QRgb topRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY >= 0 && topY < srcHeight) ? srcImage.pixel(leftX + 1, topY) : qRgba(0, 0, 0, 0);
-                QRgb bottomRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcImage.pixel(leftX + 1, topY + 1) : qRgba(0, 0, 0, 0);
+                QRgb topLeft = (leftX >= 0 && leftX < srcWidth && topY >= 0 && topY < srcHeight) ? srcPixel[leftX] : qRgba(0, 0, 0, 0);
+                QRgb bottomLeft = (leftX >= 0 && leftX < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcPixelPlusOne[leftX] : qRgba(0, 0, 0, 0);
+                QRgb topRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY >= 0 && topY < srcHeight) ? srcPixel[leftX] : qRgba(0, 0, 0, 0);
+                QRgb bottomRight = (leftX + 1 >= 0 && leftX + 1 < srcWidth && topY + 1 >= 0 && topY + 1 < srcHeight) ? srcPixelPlusOne[leftX + 1] : qRgba(0, 0, 0, 0);
 
                 double a = 1 - xInterp;
                 double b = 1 - yInterp;
@@ -870,7 +907,7 @@ QImage KisBrush::scaleImage(const QImage& srcImage, int width, int height)
                     alpha = 255;
                 }
 
-                scaledImage.setPixel(dstX, dstY, qRgba(red, green, blue, alpha));
+                dstPixel[dstX] = qRgba(red, green, blue, alpha);
             }
         }
 
@@ -922,12 +959,13 @@ KisQImagemaskSP KisBrush::scaleSinglePixelMask(double scale, quint8 maskValue, d
     double b = subPixelY;
 
     for (int y = 0; y < dstHeight; y++) {
+
         for (int x = 0; x < dstWidth; x++) {
 
-            quint8 topLeft = (x > 0 && y > 0) ? maskValue : OPACITY_TRANSPARENT;
-            quint8 bottomLeft = (x > 0 && y < srcHeight) ? maskValue : OPACITY_TRANSPARENT;
-            quint8 topRight = (x < srcWidth && y > 0) ? maskValue : OPACITY_TRANSPARENT;
-            quint8 bottomRight = (x < srcWidth && y < srcHeight) ? maskValue : OPACITY_TRANSPARENT;
+            quint8 topLeft = (x > 0 && y > 0) ? maskValue : OPACITY_TRANSPARENT_U8;
+            quint8 bottomLeft = (x > 0 && y < srcHeight) ? maskValue : OPACITY_TRANSPARENT_U8;
+            quint8 topRight = (x < srcWidth && y > 0) ? maskValue : OPACITY_TRANSPARENT_U8;
+            quint8 bottomRight = (x < srcWidth && y < srcHeight) ? maskValue : OPACITY_TRANSPARENT_U8;
 
             // Bi-linear interpolation
             int d = static_cast<int>(a * b * topLeft
@@ -939,10 +977,10 @@ KisQImagemaskSP KisBrush::scaleSinglePixelMask(double scale, quint8 maskValue, d
             // has 0.25 the value of the 1x1.
             d = static_cast<int>(d * scale * scale + 0.5);
 
-            if (d < OPACITY_TRANSPARENT) {
-                d = OPACITY_TRANSPARENT;
-            } else if (d > OPACITY_OPAQUE) {
-                d = OPACITY_OPAQUE;
+            if (d < OPACITY_TRANSPARENT_U8) {
+                d = OPACITY_TRANSPARENT_U8;
+            } else if (d > OPACITY_OPAQUE_U8) {
+                d = OPACITY_OPAQUE_U8;
             }
 
             outputMask->setAlphaAt(x, y, static_cast<quint8>(d));
@@ -965,6 +1003,9 @@ QImage KisBrush::scaleSinglePixelImage(double scale, QRgb pixel, double subPixel
     double b = subPixelY;
 
     for (int y = 0; y < dstHeight; y++) {
+
+        QRgb *dstPixel = reinterpret_cast<QRgb *>(outputImage.scanLine(y));
+
         for (int x = 0; x < dstWidth; x++) {
 
             QRgb topLeft = (x > 0 && y > 0) ? pixel : qRgba(0, 0, 0, 0);
@@ -1024,7 +1065,7 @@ QImage KisBrush::scaleSinglePixelImage(double scale, QRgb pixel, double subPixel
                 alpha = 255;
             }
 
-            outputImage.setPixel(x, y, qRgba(red, green, blue, alpha));
+            dstPixel[x] = qRgba(red, green, blue, alpha);
         }
     }
 
@@ -1040,8 +1081,9 @@ QImage KisBrush::interpolate(const QImage& image1, const QImage& image2, double
 
     QImage outputImage(width, height, QImage::Format_ARGB32);
 
-    for (int x = 0; x < width; x++) {
-        for (int y = 0; y < height; y++) {
+    for (int y = 0; y < height; y++) {
+        QRgb *dstPixel = reinterpret_cast<QRgb *>(outputImage.scanLine(y));
+        for (int x = 0; x < width; x++) {
             QRgb image1pixel = image1.pixel(x, y);
             QRgb image2pixel = image2.pixel(x, y);
 
@@ -1075,7 +1117,7 @@ QImage KisBrush::interpolate(const QImage& image1, const QImage& image2, double
                 alpha = 255;
             }
 
-            outputImage.setPixel(x, y, qRgba(red, green, blue, alpha));
+            dstPixel[x] = qRgba(red, green, blue, alpha);
         }
     }
 
@@ -1091,11 +1133,9 @@ void KisBrush::resetBoundary()
 void KisBrush::generateBoundary() const
 {
     KisFixedPaintDeviceSP dev;
-    int w = width();
-    int h = height();
 
     if (brushType() == IMAGE || brushType() == PIPE_IMAGE) {
-        dev = image(KoColorSpaceRegistry::instance()->colorSpace("RGBA", 0), 1.0, 0.0, KisPaintInformation());
+        dev = paintDevice(KoColorSpaceRegistry::instance()->rgb8(), 1.0, 0.0, KisPaintInformation());
     } else {
         const KoColorSpace* cs = KoColorSpaceRegistry::instance()->rgb8();
         dev = new KisFixedPaintDevice(cs);
@@ -1119,8 +1159,8 @@ void KisBrush::generateBoundary() const
 #endif
     }
 
-    d->boundary = new KisBoundary(dev.data());
-    d->boundary->generateBoundary(w, h);
+    d->boundary = new KisBoundary(dev);
+    d->boundary->generateBoundary();
 }
 
 const KisBoundary* KisBrush::boundary() const
