@@ -44,6 +44,7 @@
 #include <KoCompositeOp.h>
 #include <KoShapeManager.h>
 #include <KoProgressUpdater.h>
+#include <KoUpdater.h>
 
 #include <kis_global.h>
 #include <canvas/kis_canvas2.h>
@@ -58,11 +59,17 @@
 #include <widgets/kis_cmb_idlist.h>
 #include <kis_statusbar.h>
 #include <kis_transform_worker.h>
+#include <kis_pixel_selection.h>
+#include <kis_shape_selection.h>
+#include <kis_selection_manager.h>
+
+#include <KoShapeTransformCommand.h>
 
 #include "flake/kis_node_shape.h"
 #include "flake/kis_layer_container_shape.h"
 #include "flake/kis_shape_layer.h"
 #include "kis_canvas_resource_provider.h"
+#include "widgets/kis_progress_widget.h"
 
 namespace
 {
@@ -76,9 +83,9 @@ public:
 public:
     virtual void redo();
     virtual void undo();
-    void transformArgs(double &sx, double &sy, QPointF &translate, double &a);
-    KisSelectionSP origSelection(QPoint &startPos, QPoint &endPos);
-
+    void transformArgs(double &sx, double &sy, QPointF &translate, double &a) const;
+    KisSelectionSP origSelection(QPoint &startPos, QPoint &endPos) const;
+    void setNewPosition(int x, int y);
 private:
     double m_scaleX;
     double m_scaleY;
@@ -88,6 +95,7 @@ private:
     KisSelectionSP m_origSelection;
     QPoint m_originalTopLeft;
     QPoint m_originalBottomRight;
+    QPoint m_newPosition;
 };
 
 TransformCmd::TransformCmd(KisToolTransform *tool, KisNodeSP node, double scaleX, double scaleY, QPointF translate, double a, KisSelectionSP origSel, QPoint originalTopLeft, QPoint originalBottomRight)
@@ -107,7 +115,7 @@ TransformCmd::~TransformCmd()
 {
 }
 
-void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, double &a)
+void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, double &a) const
 {
     sx = m_scaleX;
     sy = m_scaleY;
@@ -115,7 +123,7 @@ void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, dou
     a = m_a;
 }
 
-KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &originalBottomRight)
+KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &originalBottomRight) const
 {
     originalTopLeft = m_originalTopLeft;
     originalBottomRight = m_originalBottomRight;
@@ -125,12 +133,21 @@ KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &orig
 void TransformCmd::redo()
 {
     KisSelectedTransaction::redo();
+    layer()->paintDevice()->move(m_newPosition);
 }
 
 void TransformCmd::undo()
 {
     KisSelectedTransaction::undo();
+    layer()->paintDevice()->move(m_originalTopLeft);
 }
+
+void TransformCmd::setNewPosition(int x, int y)
+{
+    m_newPosition.setX(x);
+    m_newPosition.setY(y);
+}
+
 }
 
 KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
@@ -169,18 +186,17 @@ void KisToolTransform::deactivate()
     m_canvas->updateCanvas(QRect(m_originalTopLeft, m_originalBottomRight));
 }
 
-void KisToolTransform::activate(bool temporary)
+void KisToolTransform::activate(ToolActivation toolActivation, const QSet<KoShape*> &)
 {
-    Q_UNUSED(temporary);
+    Q_UNUSED(toolActivation);
 
     if (currentNode() && currentNode()->paintDevice()) {
-        //connect(m_subject, commandExecuted(K3Command *c), this, notifyCommandAdded( KCommand * c));
-        //m_subject->undoAdapter()->setCommandHistoryListener( this );
+        image()->undoAdapter()->setCommandHistoryListener(this);
 
-        TransformCmd * cmd = 0;
+        const TransformCmd * cmd = 0;
 
         if (image()->undoAdapter()->presentCommand())
-            cmd = dynamic_cast<TransformCmd*>(image()->undoAdapter()->presentCommand());
+            cmd = dynamic_cast<const TransformCmd*>(image()->undoAdapter()->presentCommand());
 
         if (cmd == 0) {
             initHandles();
@@ -193,7 +209,7 @@ void KisToolTransform::activate(bool temporary)
         }
     }
     currentNode() =
-        m_canvas->resourceProvider()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeSP>();
+        m_canvas->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeSP>();
 }
 
 void KisToolTransform::initHandles()
@@ -209,7 +225,9 @@ void KisToolTransform::initHandles()
     KisSelectionSP selection = currentSelection();
     if (selection) {
         QRect r = selection->selectedExactRect();
-        m_origSelection = new KisSelection(*selection.data());
+        m_origSelection = new KisSelection();
+        KisPixelSelectionSP origPixelSelection = new KisPixelSelection(*selection->getOrCreatePixelSelection().data());
+        m_origSelection->setPixelSelection(origPixelSelection);
         r.getRect(&x, &y, &w, &h);
     } else {
         dev->exactBounds(x, y, w, h);
@@ -229,13 +247,13 @@ void KisToolTransform::initHandles()
 
 void KisToolTransform::mousePressEvent(KoPointerEvent *e)
 {
-    KisImageSP img = image();
+    KisImageWSP kisimage = image();
 
     if (!currentNode())
         return;
 
-    if (img && currentNode()->paintDevice() && e->button() == Qt::LeftButton) {
-        QPointF mousePos = QPointF(e->point.x() * img->xRes(), e->point.y() * img->yRes());
+    if (kisimage && currentNode()->paintDevice() && e->button() == Qt::LeftButton) {
+        QPointF mousePos = QPointF(e->point.x() * kisimage->xRes(), e->point.y() * kisimage->yRes());
         switch (m_function) {
         case ROTATE:
             m_clickoffset = mousePos - m_translate;
@@ -337,8 +355,8 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
     QPointF bottomleft = m_bottomleft;
     QPointF bottomright = m_bottomright;
 
-    KisImageSP img = image();
-    QPointF mousePos = QPointF(e->point.x() * img->xRes(), e->point.y() * img->yRes());
+    KisImageWSP kisimage = image();
+    QPointF mousePos = QPointF(e->point.x() * kisimage->xRes(), e->point.y() * kisimage->yRes());
 
     if (m_selecting) {
         m_canvas->updateCanvas(QRect(m_originalTopLeft, m_originalBottomRight));
@@ -596,11 +614,11 @@ void KisToolTransform::paint(QPainter& gc, const KoViewConverter &converter)
     pen.setWidth(0);
 
     recalcOutline();
-    KisImageSP img = image();
-    QPointF topleft = converter.documentToView(QPointF(m_topleft.x() / img->xRes(), m_topleft.y() / img->yRes()));
-    QPointF topright = converter.documentToView(QPointF(m_topright.x() / img->xRes(), m_topright.y() / img->yRes()));
-    QPointF bottomleft = converter.documentToView(QPointF(m_bottomleft.x() / img->xRes(), m_bottomleft.y() / img->yRes()));
-    QPointF bottomright = converter.documentToView(QPointF(m_bottomright.x() / img->xRes(), m_bottomright.y() / img->yRes()));
+    KisImageWSP kisimage = image();
+    QPointF topleft = converter.documentToView(QPointF(m_topleft.x() / kisimage->xRes(), m_topleft.y() / kisimage->yRes()));
+    QPointF topright = converter.documentToView(QPointF(m_topright.x() / kisimage->xRes(), m_topright.y() / kisimage->yRes()));
+    QPointF bottomleft = converter.documentToView(QPointF(m_bottomleft.x() / kisimage->xRes(), m_bottomleft.y() / kisimage->yRes()));
+    QPointF bottomright = converter.documentToView(QPointF(m_bottomright.x() / kisimage->xRes(), m_bottomright.y() / kisimage->yRes()));
 
     QRectF handleRect(-4, -4, 8, 8);
 
@@ -629,44 +647,91 @@ void KisToolTransform::transform()
         return;
 
     KisCanvas2 *canvas = dynamic_cast<KisCanvas2 *>(m_canvas);
-    if(!canvas)
+    if (!canvas)
         return;
 
     QPointF t = m_translate - rot(m_originalCenter.x() * m_scaleX, m_originalCenter.y() * m_scaleY);
-    KoProgressUpdater updater(canvas->view()->statusBar()->progress());
-    updater.start( 100, i18n("Transform") );
-    KoUpdater progress = updater.startSubtask();
+    KoProgressUpdater* updater = canvas->view()->createProgressUpdater();
+    updater->start(100, i18n("Transform"));
+    KoUpdaterPtr progress = updater->startSubtask();
 
     // This mementoes the current state of the active device.
     TransformCmd * transaction = new TransformCmd(this, currentNode(), m_scaleX,
             m_scaleY, m_translate, m_a, m_origSelection, m_originalTopLeft, m_originalBottomRight);
 
-    // Copy the original state back.
+
+    //Copy the original state back.
     QRect rc = m_origDevice->extent();
     rc = rc.normalized();
     currentNode()->paintDevice()->clear();
     KisPainter gc(currentNode()->paintDevice());
-    gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, m_origDevice, rc.x(), rc.y(), rc.width(), rc.height());
+    gc.setCompositeOp(COMPOSITE_COPY);
+    gc.bitBlt(rc.topLeft(), m_origDevice, rc);
     gc.end();
 
     // Also restore the original selection.
     if (m_origSelection) {
         QRect rc = m_origSelection->selectedRect();
         rc = rc.normalized();
-        // XXX_SELECTION This used to create a new selection!
-        if (currentSelection()) currentSelection()->clear();
-        KisPainter sgc(KisPaintDeviceSP(currentSelection().data()));
-        sgc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, KisPaintDeviceSP(m_origSelection.data()), rc.x(), rc.y(), rc.width(), rc.height());
-        sgc.end();
-    } else
-        if (currentSelection())
-            currentSelection()->clear();
+        if (currentSelection()) {
+            currentSelection()->getOrCreatePixelSelection()->clear();
+            KisPainter sgc(KisPaintDeviceSP(currentSelection()->getOrCreatePixelSelection()));
+            sgc.setCompositeOp(COMPOSITE_COPY);
+            sgc.bitBlt(rc.topLeft(), m_origSelection, rc);
+            sgc.end();
+        }
+    } else if (currentSelection())
+        currentSelection()->clear();
 
     // Perform the transform. Since we copied the original state back, this doesn't degrade
     // after many tweaks. Since we started the transaction before the copy back, the memento
     // has the previous state.
-    KisTransformWorker worker(currentNode()->paintDevice(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), &progress, m_filter);
-    worker.run();
+    if (m_origSelection) {
+        KisPaintDeviceSP tmpDevice = new KisPaintDevice(m_origDevice->colorSpace());
+        QRect selectRect = currentSelection()->selectedRect();
+        KisPainter gc(tmpDevice, currentSelection());
+        gc.bitBlt(selectRect.topLeft(), m_origDevice, selectRect);
+        gc.end();
+
+        KisTransformWorker worker(tmpDevice, m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), progress, m_filter);
+        worker.run();
+
+        currentNode()->paintDevice()->clearSelection(currentSelection());
+
+        KisTransformWorker selectionWorker(currentSelection()->getOrCreatePixelSelection(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), progress, m_filter);
+        selectionWorker.run();
+
+
+        if (currentSelection()->hasShapeSelection()) {
+            QMatrix resolutionMatrix;
+            resolutionMatrix.scale(1 / image()->xRes(), 1 / image()->yRes());
+            QPointF center = resolutionMatrix.map(m_originalCenter);
+
+            QMatrix matrix;
+            matrix.translate(center.x(), center.y());
+            matrix.rotate(m_a / M_PI*180);
+            matrix.translate(-center.x(), -center.y());
+
+            KisShapeSelection* shapeSelection = static_cast<KisShapeSelection*>(currentSelection()->shapeSelection());
+            QList<KoShape *> shapes = shapeSelection->shapeManager()->shapes();
+            QList<QMatrix> m_oldMatrixList;
+            QList<QMatrix> m_newMatrixList;
+            foreach(KoShape *shape, shapes) {
+                m_oldMatrixList << shape->transformation();
+                m_newMatrixList << shape->transformation()*matrix;
+            }
+            KoShapeTransformCommand* cmd = new KoShapeTransformCommand(shapes, m_oldMatrixList, m_newMatrixList, transaction);
+            cmd->redo();
+        }
+
+        QRect tmpRc = tmpDevice->extent();
+        KisPainter painter(currentNode()->paintDevice());
+        painter.bitBlt(tmpRc.topLeft(), tmpDevice, tmpRc);
+        painter.end();
+    } else {
+        KisTransformWorker worker(currentNode()->paintDevice(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), progress, m_filter);
+        worker.run();
+    }
 
 // XXX_PROGRESS, XXX_LAYERS
 //     // If canceled, go back to the memento
@@ -679,20 +744,27 @@ void KisToolTransform::transform()
 
     currentNode()->paintDevice()->setDirty(rc); // XXX: This is not enough - should union with new extent
 
+    canvas->view()->selectionManager()->selectionChanged();
+    if (currentSelection() && currentSelection()->hasShapeSelection())
+        canvas->view()->selectionManager()->shapeSelectionChanged();
+
     // Else add the command -- this will have the memento from the previous state,
     // and the transformed state from the original device we cached in our activated()
     // method.
     if (transaction) {
+        transaction->setNewPosition(currentNode()->paintDevice()->x(), currentNode()->paintDevice()->y());
         if (image()->undo())
             image()->undoAdapter()->addCommand(transaction);
         else
             delete transaction;
     }
+    updater->deleteLater();
+
 }
 
-void KisToolTransform::notifyCommandAdded(QUndoCommand * command)
+void KisToolTransform::notifyCommandAdded(const QUndoCommand * command)
 {
-    TransformCmd * cmd = dynamic_cast<TransformCmd*>(command);
+    const TransformCmd * cmd = dynamic_cast<const TransformCmd*>(command);
     if (cmd == 0) {
         // The last added command wasn't one of ours;
         // we should reset to the new state of the canvas.
@@ -701,13 +773,13 @@ void KisToolTransform::notifyCommandAdded(QUndoCommand * command)
     }
 }
 
-void KisToolTransform::notifyCommandExecuted(QUndoCommand * command)
+void KisToolTransform::notifyCommandExecuted(const QUndoCommand * command)
 {
     Q_UNUSED(command);
-    TransformCmd * cmd = 0;
+    const TransformCmd * cmd = 0;
 
     if (image()->undoAdapter()->presentCommand())
-        cmd = dynamic_cast<TransformCmd*>(image()->undoAdapter()->presentCommand());
+        cmd = dynamic_cast<const TransformCmd*>(image()->undoAdapter()->presentCommand());
 
     if (cmd == 0) {
         // The command now on the top of the stack isn't one of ours
@@ -732,7 +804,7 @@ QWidget* KisToolTransform::createOptionWidget()
     m_optWidget = new WdgToolTransform(0);
     Q_CHECK_PTR(m_optWidget);
     m_optWidget->setObjectName(toolId() + " option widget");
-    
+
     m_optWidget->cmbFilter->clear();
     m_optWidget->cmbFilter->setIDList(KisFilterStrategyRegistry::instance()->listKeys());
 
@@ -757,6 +829,7 @@ QWidget* KisToolTransform::createOptionWidget()
     m_optWidget->textLabel2->hide();
     m_optWidget->textLabel3->hide();
     m_optWidget->textLabel4->hide();
+    m_optWidget->setFixedHeight(m_optWidget->sizeHint().height());
     return m_optWidget;
 }
 
