@@ -1,6 +1,7 @@
 /*
  *  Copyright (c) 2006-2008 Boudewijn Rempt <boud@valdyas.org>
  *  Copyright (c) 2007 Thomas Zander <zander@kde.org>
+ *  Copyright (c) 2009 Cyrille Berger <cberger@cberger.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,18 +31,19 @@
 #include <QList>
 #include <QMap>
 #include <QDebug>
+#include <QUndoCommand>
 
 #include <ktemporaryfile.h>
 #include <kicon.h>
+#include <kdebug.h>
 
 #include <KoColorSpace.h>
 #include <KoCompositeOp.h>
-#include <KoDataCenter.h>
+#include <KoDataCenterBase.h>
 #include <KoDocument.h>
 #include <KoEmbeddedDocumentSaver.h>
 #include <KoGenStyle.h>
 #include <KoImageCollection.h>
-#include <KoOdfLoadingContext.h>
 #include <KoOdfReadStore.h>
 #include <KoOdfStylesReader.h>
 #include <KoOdfWriteStore.h>
@@ -61,6 +63,8 @@
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
 #include <KoSelection.h>
+#include <KoShapeMoveCommand.h>
+#include <KoShapeTransformCommand.h>
 
 #include <kis_types.h>
 #include <kis_image.h>
@@ -71,38 +75,55 @@
 #include "kis_node_visitor.h"
 #include "kis_effect_mask.h"
 
+#include "kis_shape_layer_paste.h"
+
 class KisShapeLayer::Private
 {
 public:
     KoViewConverter * converter;
     qint32 x;
     qint32 y;
-    KisPaintDeviceSP projection;
-    KisPaintDeviceSP filteredProjection;
+    KisPaintDeviceSP paintDevice;
     KisShapeLayerCanvas * canvas;
     KoShapeControllerBase* controller;
 };
 
 KisShapeLayer::KisShapeLayer(KoShapeContainer * parent,
                              KoShapeControllerBase* controller,
-                             KisImageSP img,
+                             KisImageWSP image,
                              const QString &name,
                              quint8 opacity)
-        : KisExternalLayer(img, name, opacity)
+        : KisExternalLayer(image, name, opacity)
         , m_d(new Private())
 {
     KoShapeContainer::setParent(parent);
-    setShapeId(KIS_SHAPE_LAYER_ID);
+    initShapeLayer(controller);
+}
 
-    m_d->converter = new KisImageViewConverter(image());
-    m_d->x = 0;
-    m_d->y = 0;
-    m_d->projection = new KisPaintDevice(img->colorSpace());
-    m_d->canvas = new KisShapeLayerCanvas(this, m_d->converter);
-    m_d->canvas->setProjection(m_d->projection);
-    m_d->controller = controller;
+KisShapeLayer::KisShapeLayer(const KisShapeLayer& _rhs)
+        : KisExternalLayer(_rhs)
+        , KoShapeLayer(_rhs)
+        , m_d(new Private())
+{
+    kDebug() << "copying rhs" << &_rhs << "to" << this;
+
+    KoShapeContainer::setParent(_rhs.KoShapeContainer::parent());
+    initShapeLayer(_rhs.m_d->controller);
+
+    KoShapeOdfSaveHelper saveHelper(_rhs.childShapes());
+    KoDrag drag;
+    drag.setOdf(KoOdf::mimeType(KoOdf::Text), saveHelper);
+    QMimeData* mimeData = drag.mimeData();
+
+    Q_ASSERT(mimeData->hasFormat(KoOdf::mimeType(KoOdf::Text)));
+
+    KisShapeLayerShapePaste paste(this, m_d->controller);
+    bool success = paste.paste(KoOdf::Text, mimeData);
+    Q_ASSERT(success);
+    if (!success) {
+        warnUI << "Could not paste shape layer";
+    }
 
-    connect(m_d->canvas->shapeManager(), SIGNAL(selectionChanged()), this, SLOT(selectionChanged()));
 }
 
 KisShapeLayer::~KisShapeLayer()
@@ -112,61 +133,66 @@ KisShapeLayer::~KisShapeLayer()
     delete m_d;
 }
 
+void KisShapeLayer::initShapeLayer(KoShapeControllerBase* controller)
+{
+    setShapeId(KIS_SHAPE_LAYER_ID);
+
+    m_d->converter = new KisImageViewConverter(image());
+    m_d->x = 0;
+    m_d->y = 0;
+    m_d->paintDevice = new KisPaintDevice(image()->colorSpace());
+    m_d->canvas = new KisShapeLayerCanvas(this, m_d->converter);
+    m_d->canvas->setProjection(m_d->paintDevice);
+    m_d->controller = controller;
+
+    connect(m_d->canvas->shapeManager(), SIGNAL(selectionChanged()), this, SLOT(selectionChanged()));
+}
+
 bool KisShapeLayer::allowAsChild(KisNodeSP node) const
 {
-    if (node->inherits("KisMask"))
-        return true;
-    else
-        return false;
+    return node->inherits("KisMask");
 }
 
 void KisShapeLayer::addChild(KoShape *object)
 {
+    QRect updatedRect = m_d->converter->documentToView(object->boundingRect()).toRect();
+
     KoShapeLayer::addChild(object);
     m_d->canvas->shapeManager()->add(object);
 
-    setDirty(m_d->converter->documentToView(object->boundingRect()).toRect());
+    setDirty(updatedRect);
 }
 
 void KisShapeLayer::removeChild(KoShape *object)
 {
+    QRect updatedRect = m_d->converter->documentToView(object->boundingRect()).toRect();
+
     m_d->canvas->shapeManager()->remove(object);
     KoShapeLayer::removeChild(object);
+
+    setDirty(updatedRect);
 }
 
 QIcon KisShapeLayer::icon() const
 {
-    return KIcon("bookmarks");
+    return KIcon("bookmark-new");
 }
 
-void KisShapeLayer::updateProjection(const QRect& rc)
+KisPaintDeviceSP KisShapeLayer::original() const
 {
-    dbgImage << "KisShapeLayer::updateProjection()" << rc;
-
-    KoProperties props;
-    props.setProperty("visible", true);
-    QList<KisNodeSP> masks = childNodes(QStringList("KisEffectMask"), props);
-
-    if(masks.empty()) {
-        m_d->filteredProjection = 0; // Don't use the filtered projection anymore
-    } else {
-        if( !m_d->filteredProjection || !(*m_d->filteredProjection->colorSpace() == *m_d->projection->colorSpace() ) ) {
-            m_d->filteredProjection = new KisPaintDevice(m_d->projection->colorSpace());
-        } else {
-            m_d->filteredProjection->clear(rc);
-        }
-        applyEffectMasks(m_d->projection, m_d->filteredProjection, rc);
-    }
+    return m_d->paintDevice;
 }
 
-KisPaintDeviceSP KisShapeLayer::projection() const
+KisPaintDeviceSP KisShapeLayer::paintDevice() const
 {
-    if(m_d->filteredProjection)
-    {
-        return m_d->filteredProjection;
-    } else {
-        return m_d->projection;
-    }
+    return 0;
+}
+
+QRect KisShapeLayer::repaintOriginal(KisPaintDeviceSP original,
+                                     const QRect& rect)
+{
+    Q_UNUSED(original);
+    return rect;
 }
 
 qint32 KisShapeLayer::x() const
@@ -174,35 +200,21 @@ qint32 KisShapeLayer::x() const
     return m_d->x;
 }
 
-void KisShapeLayer::setX(qint32 x)
-{
-    if (x == m_d->x) return;
-    m_d->x = x;
-    setDirty();
-}
-
 qint32 KisShapeLayer::y() const
 {
     return m_d->y;
 }
 
+void KisShapeLayer::setX(qint32 x)
+{
+    m_d->x = x;
+    // FIXME: setDirty();
+}
+
 void KisShapeLayer::setY(qint32 y)
 {
-    if (y == m_d->y) return;
     m_d->y = y;
-    setDirty();
-}
-
-QRect KisShapeLayer::extent() const
-{
-    QRect rc = boundingRect().toRect();
-    return QRectF(rc.x() * image()->xRes(), rc.y() * image()->yRes(), rc.width() * image()->xRes(), rc.height() * image()->yRes()).toRect();
-}
-
-QRect KisShapeLayer::exactBounds() const
-{
-    QRect rc = boundingRect().toRect();
-    return QRectF(rc.x() * image()->xRes(), rc.y() * image()->yRes(), rc.width() * image()->xRes(), rc.height() * image()->yRes()).toRect();
+    //FIXME: setDirty();
 }
 
 bool KisShapeLayer::accept(KisNodeVisitor& visitor)
@@ -239,19 +251,18 @@ bool KisShapeLayer::saveLayer(KoStore * store) const
     QRectF rc = boundingRect();
     page.width = rc.width();
     page.height = rc.height();
-    if ( page.width > page.height ) {
+    if (page.width > page.height) {
         page.orientation = KoPageFormat::Landscape;
-    }
-    else {
-         page.orientation = KoPageFormat::Portrait;
+    } else {
+        page.orientation = KoPageFormat::Portrait;
     }
 
     KoGenStyles mainStyles;
     KoGenStyle pageLayout = page.saveOdf();
-    QString layoutName = mainStyles.lookup(pageLayout, "PL");
-    KoGenStyle masterPage(KoGenStyle::StyleMaster);
+    QString layoutName = mainStyles.insert(pageLayout, "PL");
+    KoGenStyle masterPage(KoGenStyle::MasterPageStyle);
     masterPage.addAttribute("style:page-layout-name", layoutName);
-    mainStyles.lookup(masterPage, "Default", KoGenStyles::DontForceNumbering);
+    mainStyles.insert(masterPage, "Default", KoGenStyles::DontAddNumberToName);
 
     KTemporaryFile contentTmpFile;
     contentTmpFile.open();
@@ -274,7 +285,7 @@ bool KisShapeLayer::saveLayer(KoStore * store) const
     contentTmpWriter.endElement(); // office:drawing
     contentTmpWriter.endElement(); // office:body
 
-    mainStyles.saveOdfAutomaticStyles(docWriter, false);
+    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, docWriter);
 
     // And now we can copy over the contents from the tempfile to the real one
     contentTmpFile.seek(0);
@@ -297,8 +308,8 @@ bool KisShapeLayer::saveLayer(KoStore * store) const
 
     manifestWriter->addManifestEntry("settings.xml", "text/xml");
 
-    if (! shapeContext.saveDataCenter( documentContext.odfStore.store(),
-                                       documentContext.odfStore.manifestWriter() ))
+    if (! shapeContext.saveDataCenter(documentContext.odfStore.store(),
+                                      documentContext.odfStore.manifestWriter()))
         return false;
 
     // Write out manifest file
@@ -310,93 +321,81 @@ bool KisShapeLayer::saveLayer(KoStore * store) const
     return true;
 }
 
-bool KisShapeLayer::loadLayer( KoStore* store )
+bool KisShapeLayer::loadLayer(KoStore* store)
 {
-    KoOdfReadStore odfStore( store );
+    KoOdfReadStore odfStore(store);
     QString errorMessage;
 
-    odfStore.loadAndParse( errorMessage );
+    odfStore.loadAndParse(errorMessage);
 
-    if ( !errorMessage.isEmpty() ) {
-        qDebug() << errorMessage;
+    if (!errorMessage.isEmpty()) {
+        warnKrita << errorMessage;
         return false;
     }
 
     KoXmlElement contents = odfStore.contentDoc().documentElement();
 
-//    qDebug() <<"Start loading OASIS document..." << contents.text();
-//    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().localName();
-//    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().namespaceURI();
-//    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().isElement();
+    //    qDebug() <<"Start loading OASIS document..." << contents.text();
+    //    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().localName();
+    //    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().namespaceURI();
+    //    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().isElement();
 
-    KoXmlElement body( KoXml::namedItemNS( contents, KoXmlNS::office, "body" ) );
+    KoXmlElement body(KoXml::namedItemNS(contents, KoXmlNS::office, "body"));
 
-    if( body.isNull() )
-    {
-        qDebug() <<"No office:body found!";
+    if (body.isNull()) {
         //setErrorMessage( i18n( "Invalid OASIS document. No office:body tag found." ) );
         return false;
     }
 
-    body = KoXml::namedItemNS( body, KoXmlNS::office, "drawing");
-    if(body.isNull())
-    {
-        qDebug() <<"No office:drawing found!";
+    body = KoXml::namedItemNS(body, KoXmlNS::office, "drawing");
+    if (body.isNull()) {
         //setErrorMessage( i18n( "Invalid OASIS document. No office:drawing tag found." ) );
         return false;
     }
 
-    KoXmlElement page( KoXml::namedItemNS( body, KoXmlNS::draw, "page" ) );
-    if(page.isNull())
-    {
-        qDebug() <<"No office:drawing found!";
+    KoXmlElement page(KoXml::namedItemNS(body, KoXmlNS::draw, "page"));
+    if (page.isNull()) {
         //setErrorMessage( i18n( "Invalid OASIS document. No draw:page tag found." ) );
         return false;
     }
 
     KoXmlElement * master = 0;
-    if( odfStore.styles().masterPages().contains( "Standard" ) )
-        master = odfStore.styles().masterPages().value( "Standard" );
-    else if( odfStore.styles().masterPages().contains( "Default" ) )
-        master = odfStore.styles().masterPages().value( "Default" );
-    else if( ! odfStore.styles().masterPages().empty() )
+    if (odfStore.styles().masterPages().contains("Standard"))
+        master = odfStore.styles().masterPages().value("Standard");
+    else if (odfStore.styles().masterPages().contains("Default"))
+        master = odfStore.styles().masterPages().value("Default");
+    else if (! odfStore.styles().masterPages().empty())
         master = odfStore.styles().masterPages().begin().value();
 
-    if( master )
-    {
+    if (master) {
         const KoXmlElement *style = odfStore.styles().findStyle(
-            master->attributeNS( KoXmlNS::style, "page-layout-name", QString() ) );
+                                        master->attributeNS(KoXmlNS::style, "page-layout-name", QString()));
         KoPageLayout pageLayout;
-        pageLayout.loadOdf( *style );
-        setSize( QSizeF( pageLayout.width, pageLayout.height ) );
-    }
-    else
-    {
-        kWarning() << "No master page found!";
-        return false;
+        pageLayout.loadOdf(*style);
+        setSize(QSizeF(pageLayout.width, pageLayout.height));
     }
+    // We work fine without a master page
 
-    KoOdfLoadingContext context( odfStore.styles(), odfStore.store() );
+    KoOdfLoadingContext context(odfStore.styles(), odfStore.store());
     context.setManifestFile(QString("tar:/") + odfStore.store()->currentPath() + "META-INF/manifest.xml");
-    KoShapeLoadingContext shapeContext( context, m_d->controller->dataCenterMap() );
+    KoShapeLoadingContext shapeContext(context, m_d->controller->resourceManager());
 
 
     KoXmlElement layerElement;
-    forEachElement( layerElement, context.stylesReader().layerSet() )
-    {
-        KoShapeLayer * l = new KoShapeLayer();
-        if( !loadOdf( layerElement, shapeContext ) ) {
+    forEachElement(layerElement, context.stylesReader().layerSet()) {
+        // FIXME: investigate what is this
+        //        KoShapeLayer * l = new KoShapeLayer();
+        if (!loadOdf(layerElement, shapeContext)) {
             kWarning() << "Could not load shape layer!";
             return false;
         }
     }
 
     KoXmlElement child;
-    forEachElement( child, page )
-    {
-        KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf( child, shapeContext );
-        if( shape ) {
-            addChild( shape );
+    forEachElement(child, page) {
+        KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf(child, shapeContext);
+        if (shape) {
+            addChild(shape);
         }
     }
 
@@ -404,15 +403,49 @@ bool KisShapeLayer::loadLayer( KoStore* store )
 
 }
 
-
-KisPaintDeviceSP KisShapeLayer::paintDevice() const
-{
-    return 0;
-}
-
 void KisShapeLayer::selectionChanged()
 {
-    emit selectionChanged( m_d->canvas->shapeManager()->selection()->selectedShapes());
+    emit selectionChanged(m_d->canvas->shapeManager()->selection()->selectedShapes());
+}
+
+QUndoCommand* KisShapeLayer::crop(const QRect & rect) {
+    QRectF croppedRect = m_d->converter->viewToDocument(rect);
+    QList<KoShape*> shapes = m_d->canvas->shapeManager()->shapes();
+    if(shapes.isEmpty())
+        return 0;
+
+    QList<QPointF> previousPositions;
+    QList<QPointF> newPositions;
+    foreach(const KoShape* shape, shapes) {
+        previousPositions.append(shape->position());
+        newPositions.append(shape->position() - croppedRect.topLeft());
+    }
+    return new KoShapeMoveCommand(shapes, previousPositions, newPositions);
+}
+
+QUndoCommand* KisShapeLayer::transform(double  xscale, double  yscale, double  xshear, double  yshear, double angle, qint32  translatex, qint32  translatey) {
+
+    Q_UNUSED(xshear);
+    Q_UNUSED(yshear);
+    QPointF transF = m_d->converter->viewToDocument(QPoint(translatex, translatey));
+    QList<KoShape*> shapes = m_d->canvas->shapeManager()->shapes();
+    if(shapes.isEmpty())
+        return 0;
+
+    QMatrix matrix;
+    matrix.translate(transF.x(), transF.y());
+    matrix.scale(xscale,yscale);
+    matrix.rotate(angle*180/M_PI);
+
+    QList<QMatrix> oldTransformations;
+    QList<QMatrix> newTransformations;
+    foreach(const KoShape* shape, shapes) {
+        QMatrix oldTransform = shape->transformation();
+        oldTransformations.append(oldTransform);
+
+        newTransformations.append(oldTransform*matrix);
+    }
+    return new KoShapeTransformCommand(shapes, oldTransformations, newTransformations);
 }
 
 #include "kis_shape_layer.moc"
