@@ -41,6 +41,7 @@
 #include <KoShapeLoadingContext.h>
 #include <KoXmlWriter.h>
 #include <KoStore.h>
+#include <KoShapeController.h>
 #include <KoShapeSavingContext.h>
 #include <KoStoreDevice.h>
 
@@ -51,14 +52,15 @@
 #include "kis_image.h"
 #include "kis_selection.h"
 #include "kis_shape_selection_canvas.h"
+#include "kis_shape_layer_paste.h"
 
 #include <kis_debug.h>
 
-KisShapeSelection::KisShapeSelection(KisImageSP image, KisSelectionSP selection)
-        : KoShapeLayer(new KisShapeSelectionModel(image, selection, this))
+KisShapeSelection::KisShapeSelection(KisImageWSP image, KisSelectionWSP selection)
+        : KoShapeLayer(m_model = new KisShapeSelectionModel(image, selection, this))
         , m_image(image)
 {
-    Q_ASSERT( m_image );
+    Q_ASSERT(m_image);
     setShapeId("KisShapeSelection");
     setSelectable(false);
     m_dirty = false;
@@ -69,21 +71,36 @@ KisShapeSelection::KisShapeSelection(KisImageSP image, KisSelectionSP selection)
 
 KisShapeSelection::~KisShapeSelection()
 {
+    m_model->setShapeSelection(0);
     delete m_canvas;
 }
 
-KisShapeSelection::KisShapeSelection(const KisShapeSelection& rhs)
-    : KoShapeLayer( rhs )
+KisShapeSelection::KisShapeSelection(const KisShapeSelection& rhs, KisSelection* selection)
+        : KoShapeLayer(m_model = new KisShapeSelectionModel(rhs.m_image, selection, this))
 {
     m_dirty = rhs.m_dirty;
     m_image = rhs.m_image;
     m_canvas = new KisShapeSelectionCanvas();
-    m_canvas->shapeManager()->add( this );
+    m_canvas->shapeManager()->add(this);
+
+    KoShapeOdfSaveHelper saveHelper(rhs.childShapes());
+    KoDrag drag;
+    drag.setOdf(KoOdf::mimeType(KoOdf::Text), saveHelper);
+    QMimeData* mimeData = drag.mimeData();
+
+    Q_ASSERT(mimeData->hasFormat(KoOdf::mimeType(KoOdf::Text)));
+
+    KisShapeLayerShapePaste paste(this, 0);
+    bool success = paste.paste(KoOdf::Text, mimeData);
+    Q_ASSERT(success);
+    if (!success) {
+        warnUI << "Could not paste shape layer";
+    }
 }
 
-KisSelectionComponent* KisShapeSelection::clone()
+KisSelectionComponent* KisShapeSelection::clone(KisSelection* selection)
 {
-    return new KisShapeSelection( *this );
+    return new KisShapeSelection(*this, selection);
 }
 
 bool KisShapeSelection::saveSelection(KoStore * store) const
@@ -110,19 +127,18 @@ bool KisShapeSelection::saveSelection(KoStore * store) const
     QRectF rc = boundingRect();
     page.width = rc.width();
     page.height = rc.height();
-    if ( page.width > page.height ) {
+    if (page.width > page.height) {
         page.orientation = KoPageFormat::Landscape;
-    }
-    else {
-         page.orientation = KoPageFormat::Portrait;
+    } else {
+        page.orientation = KoPageFormat::Portrait;
     }
 
     KoGenStyles mainStyles;
     KoGenStyle pageLayout = page.saveOdf();
-    QString layoutName = mainStyles.lookup(pageLayout, "PL");
-    KoGenStyle masterPage(KoGenStyle::StyleMaster);
+    QString layoutName = mainStyles.insert(pageLayout, "PL");
+    KoGenStyle masterPage(KoGenStyle::MasterPageStyle);
     masterPage.addAttribute("style:page-layout-name", layoutName);
-    mainStyles.lookup(masterPage, "Default", KoGenStyles::DontForceNumbering);
+    mainStyles.insert(masterPage, "Default", KoGenStyles::DontAddNumberToName);
 
     KTemporaryFile contentTmpFile;
     contentTmpFile.open();
@@ -145,7 +161,7 @@ bool KisShapeSelection::saveSelection(KoStore * store) const
     contentTmpWriter.endElement(); // office:drawing
     contentTmpWriter.endElement(); // office:body
 
-    mainStyles.saveOdfAutomaticStyles(docWriter, false);
+    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, docWriter);
 
     // And now we can copy over the contents from the tempfile to the real one
     contentTmpFile.seek(0);
@@ -166,7 +182,7 @@ bool KisShapeSelection::saveSelection(KoStore * store) const
 
     manifestWriter->addManifestEntry("settings.xml", "text/xml");
 
-    if (! shapeContext.saveDataCenter( documentContext.odfStore.store(), documentContext.odfStore.manifestWriter() ))
+    if (! shapeContext.saveDataCenter(documentContext.odfStore.store(), documentContext.odfStore.manifestWriter()))
         return false;
 
     // Write out manifest file
@@ -178,14 +194,14 @@ bool KisShapeSelection::saveSelection(KoStore * store) const
     return true;
 }
 
-bool KisShapeSelection::loadSelection( KoStore* store )
+bool KisShapeSelection::loadSelection(KoStore* store)
 {
-    KoOdfReadStore odfStore( store );
+    KoOdfReadStore odfStore(store);
     QString errorMessage;
 
-    odfStore.loadAndParse( errorMessage );
+    odfStore.loadAndParse(errorMessage);
 
-    if ( !errorMessage.isEmpty() ) {
+    if (!errorMessage.isEmpty()) {
         qDebug() << errorMessage;
         return false;
     }
@@ -197,74 +213,63 @@ bool KisShapeSelection::loadSelection( KoStore* store )
 //    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().namespaceURI();
 //    qDebug() <<"Start loading OASIS contents..." << contents.lastChild().isElement();
 
-    KoXmlElement body( KoXml::namedItemNS( contents, KoXmlNS::office, "body" ) );
+    KoXmlElement body(KoXml::namedItemNS(contents, KoXmlNS::office, "body"));
 
-    if( body.isNull() )
-    {
-        qDebug() <<"No office:body found!";
+    if (body.isNull()) {
+        qDebug() << "No office:body found!";
         //setErrorMessage( i18n( "Invalid OASIS document. No office:body tag found." ) );
         return false;
     }
 
-    body = KoXml::namedItemNS( body, KoXmlNS::office, "drawing");
-    if(body.isNull())
-    {
-        qDebug() <<"No office:drawing found!";
+    body = KoXml::namedItemNS(body, KoXmlNS::office, "drawing");
+    if (body.isNull()) {
+        qDebug() << "No office:drawing found!";
         //setErrorMessage( i18n( "Invalid OASIS document. No office:drawing tag found." ) );
         return false;
     }
 
-    KoXmlElement page( KoXml::namedItemNS( body, KoXmlNS::draw, "page" ) );
-    if(page.isNull())
-    {
-        qDebug() <<"No office:drawing found!";
+    KoXmlElement page(KoXml::namedItemNS(body, KoXmlNS::draw, "page"));
+    if (page.isNull()) {
+        qDebug() << "No office:drawing found!";
         //setErrorMessage( i18n( "Invalid OASIS document. No draw:page tag found." ) );
         return false;
     }
 
     KoXmlElement * master = 0;
-    if( odfStore.styles().masterPages().contains( "Standard" ) )
-        master = odfStore.styles().masterPages().value( "Standard" );
-    else if( odfStore.styles().masterPages().contains( "Default" ) )
-        master = odfStore.styles().masterPages().value( "Default" );
-    else if( ! odfStore.styles().masterPages().empty() )
+    if (odfStore.styles().masterPages().contains("Standard"))
+        master = odfStore.styles().masterPages().value("Standard");
+    else if (odfStore.styles().masterPages().contains("Default"))
+        master = odfStore.styles().masterPages().value("Default");
+    else if (! odfStore.styles().masterPages().empty())
         master = odfStore.styles().masterPages().begin().value();
 
-    if( master )
-    {
+    if (master) {
         const KoXmlElement *style = odfStore.styles().findStyle(
-            master->attributeNS( KoXmlNS::style, "page-layout-name", QString() ) );
+                                        master->attributeNS(KoXmlNS::style, "page-layout-name", QString()));
         KoPageLayout pageLayout;
-        pageLayout.loadOdf( *style );
-        setSize( QSizeF( pageLayout.width, pageLayout.height ) );
-    }
-    else
-    {
+        pageLayout.loadOdf(*style);
+        setSize(QSizeF(pageLayout.width, pageLayout.height));
+    } else {
         kWarning() << "No master page found!";
         return false;
     }
 
-    QMap<QString, KoDataCenter*> dataCenterMap;
-    KoOdfLoadingContext context( odfStore.styles(), odfStore.store() );
-    KoShapeLoadingContext shapeContext( context, dataCenterMap );
-
+    KoOdfLoadingContext context(odfStore.styles(), odfStore.store());
+    KoShapeLoadingContext shapeContext(context, 0);
 
     KoXmlElement layerElement;
-    forEachElement( layerElement, context.stylesReader().layerSet() )
-    {
-        KoShapeLayer * l = new KoShapeLayer();
-        if( !loadOdf( layerElement, shapeContext ) ) {
+    forEachElement(layerElement, context.stylesReader().layerSet()) {
+        if (!loadOdf(layerElement, shapeContext)) {
             kWarning() << "Could not load shape layer!";
             return false;
         }
     }
 
     KoXmlElement child;
-    forEachElement( child, page )
-    {
-        KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf( child, shapeContext );
-        if( shape ) {
-            addChild( shape );
+    forEachElement(child, page) {
+        KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf(child, shapeContext);
+        if (shape) {
+            addChild(shape);
         }
     }
 
@@ -288,17 +293,12 @@ void KisShapeSelection::removeChild(KoShape *object)
 QPainterPath KisShapeSelection::selectionOutline()
 {
     if (m_dirty) {
-        QList<KoShape*> shapesList = iterator();
+        QList<KoShape*> shapesList = childShapes();
 
         QPainterPath outline;
-        KoPathShape* pathShape;
         foreach(KoShape * shape, shapesList) {
-            pathShape = dynamic_cast<KoPathShape*>(shape);
-            if (pathShape) {
-                QMatrix shapeMatrix = shape->absoluteTransformation(0);
-
-                outline = outline.united(shapeMatrix.map(shape->outline()));
-            }
+            QMatrix shapeMatrix = shape->absoluteTransformation(0);
+            outline = outline.united(shapeMatrix.map(shape->outline()));
         }
         m_outline = outline;
         m_dirty = false;
@@ -314,8 +314,8 @@ void KisShapeSelection::paintComponent(QPainter& painter, const KoViewConverter&
 
 void KisShapeSelection::renderToProjection(KisSelection* projection)
 {
-    Q_ASSERT( projection );
-    Q_ASSERT( m_image );
+    Q_ASSERT(projection);
+    Q_ASSERT(m_image);
     QMatrix resolutionMatrix;
     resolutionMatrix.scale(m_image->xRes(), m_image->yRes());
 
@@ -325,14 +325,14 @@ void KisShapeSelection::renderToProjection(KisSelection* projection)
 
 void KisShapeSelection::renderToProjection(KisSelection* projection, const QRect& r)
 {
-    Q_ASSERT( projection );
+    Q_ASSERT(projection);
     renderSelection(projection, r);
 }
 
 void KisShapeSelection::renderSelection(KisSelection* projection, const QRect& r)
 {
-    Q_ASSERT( projection );
-    Q_ASSERT( m_image );
+    Q_ASSERT(projection);
+    Q_ASSERT(m_image);
 
     QMatrix resolutionMatrix;
     resolutionMatrix.scale(m_image->xRes(), m_image->yRes());
@@ -354,9 +354,9 @@ void KisShapeSelection::renderSelection(KisSelection* projection, const QRect& r
     for (qint32 x = r.x(); x < r.x() + r.width(); x += MASK_IMAGE_WIDTH) {
         for (qint32 y = r.y(); y < r.y() + r.height(); y += MASK_IMAGE_HEIGHT) {
 
-            maskPainter.fillRect(polygonMaskImage.rect(), QColor(OPACITY_TRANSPARENT, OPACITY_TRANSPARENT, OPACITY_TRANSPARENT, 255));
+            maskPainter.fillRect(polygonMaskImage.rect(), QColor(OPACITY_TRANSPARENT_U8, OPACITY_TRANSPARENT_U8, OPACITY_TRANSPARENT_U8, 255));
             maskPainter.translate(-x, -y);
-            maskPainter.fillPath(resolutionMatrix.map(selectionOutline()), QColor(OPACITY_OPAQUE, OPACITY_OPAQUE, OPACITY_OPAQUE, 255));
+            maskPainter.fillPath(resolutionMatrix.map(selectionOutline()), QColor(OPACITY_OPAQUE_U8, OPACITY_OPAQUE_U8, OPACITY_OPAQUE_U8, 255));
             maskPainter.translate(x, y);
 
             qint32 rectWidth = qMin(r.x() + r.width() - x, MASK_IMAGE_WIDTH);
@@ -371,7 +371,7 @@ void KisShapeSelection::renderSelection(KisSelection* projection, const QRect& r
         }
     }
     KisPainter painter(projection);
-    painter.bitBlt(r.x(), r.y(), COMPOSITE_OVER, tmpMask, r.x(), r.y(), r.width(), r.height());
+    painter.bitBlt(r.x(), r.y(), tmpMask, r.x(), r.y(), r.width(), r.height());
     painter.end();
     dbgRender << "Shape selection rendering: " << t.elapsed();
 }
@@ -387,19 +387,9 @@ KoShapeManager* KisShapeSelection::shapeManager() const
 }
 
 KisShapeSelectionFactory::KisShapeSelectionFactory(QObject* parent)
-        : KoShapeFactory(parent, "KisShapeSelection", "selection shape container")
+        : KoShapeFactoryBase(parent, "KisShapeSelection", "selection shape container")
 {
-}
-
-KoShape* KisShapeSelectionFactory::createDefaultShape() const
-{
-    return 0;
-}
-
-KoShape* KisShapeSelectionFactory::createShape(const KoProperties* params) const
-{
-    Q_UNUSED(params);
-    return 0;
+    setHidden(true);
 }
 
 #include "kis_shape_selection.moc"
