@@ -15,53 +15,63 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-
 #include "kis_canvas_resource_provider.h"
+
+#ifdef Q_WS_X11
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <fixx11h.h>
+#include <QX11Info>
+#endif
+
 #include <QImage>
 #include <QPainter>
 
 #include <KoCanvasBase.h>
 #include <KoID.h>
 #include <KoColorSpaceRegistry.h>
-
-#include "colorprofiles/KoIccColorProfile.h"
-
+#include <KoColorModelStandardIds.h>
+#include <KoColorProfile.h>
 #include <KoAbstractGradient.h>
+
 #include <kis_pattern.h>
 #include <kis_paint_device.h>
 #include <filter/kis_filter_configuration.h>
 #include <kis_image.h>
 #include <kis_group_layer.h>
 #include <kis_paintop_preset.h>
+#include <kis_paintop_settings.h>
+#include "ko_favorite_resource_manager.h"
+
 #include "kis_exposure_visitor.h"
 #include "kis_config.h"
 #include "kis_view2.h"
 #include "canvas/kis_canvas2.h"
-#include <kis_paintop_settings.h>
 
 KisCanvasResourceProvider::KisCanvasResourceProvider(KisView2 * view)
         : m_view(view)
 {
+    m_fGChanged = true;
 }
 
 KisCanvasResourceProvider::~KisCanvasResourceProvider()
 {
 }
 
-void KisCanvasResourceProvider::setCanvasResourceProvider(KoCanvasResourceProvider * resourceProvider)
+void KisCanvasResourceProvider::setResourceManager(KoResourceManager *resourceManager)
 {
-    m_resourceProvider = resourceProvider;
+    m_resourceManager = resourceManager;
 
     QVariant v;
     v.setValue(KoColor(Qt::black, m_view->image()->colorSpace()));
-    m_resourceProvider->setResource(KoCanvasResource::ForegroundColor, v);
+    m_resourceManager->setResource(KoCanvasResource::ForegroundColor, v);
 
     v.setValue(KoColor(Qt::white, m_view->image()->colorSpace()));
-    m_resourceProvider->setResource(KoCanvasResource::BackgroundColor, v);
+    m_resourceManager->setResource(KoCanvasResource::BackgroundColor, v);
 
     resetDisplayProfile();
 
-    connect(m_resourceProvider, SIGNAL(resourceChanged(int, const QVariant &)),
+    connect(m_resourceManager, SIGNAL(resourceChanged(int, const QVariant &)),
             this, SLOT(slotResourceChanged(int, const QVariant&)));
 
 }
@@ -74,22 +84,22 @@ KoCanvasBase * KisCanvasResourceProvider::canvas() const
 
 KoColor KisCanvasResourceProvider::bgColor() const
 {
-    return m_resourceProvider->resource(KoCanvasResource::BackgroundColor).value<KoColor>();
+    return m_resourceManager->resource(KoCanvasResource::BackgroundColor).value<KoColor>();
 }
 
 KoColor KisCanvasResourceProvider::fgColor() const
 {
-    return m_resourceProvider->resource(KoCanvasResource::ForegroundColor).value<KoColor>();
+    return m_resourceManager->resource(KoCanvasResource::ForegroundColor).value<KoColor>();
 }
 
 float KisCanvasResourceProvider::HDRExposure() const
 {
-    return static_cast<float>(m_resourceProvider->resource(HdrExposure).toDouble());
+    return static_cast<float>(m_resourceManager->resource(HdrExposure).toDouble());
 }
 
 void KisCanvasResourceProvider::setHDRExposure(float exposure)
 {
-    m_resourceProvider->setResource(HdrExposure, static_cast<double>(exposure));
+    m_resourceManager->setResource(HdrExposure, static_cast<double>(exposure));
     KisExposureVisitor eV(exposure);
     m_view->image()->projection()->colorSpace()->profile()->setProperty("exposure", exposure);
     m_view->image()->rootLayer()->accept(eV);
@@ -100,26 +110,26 @@ void KisCanvasResourceProvider::setHDRExposure(float exposure)
 
 KisPattern * KisCanvasResourceProvider::currentPattern() const
 {
-    return static_cast<KisPattern*>(m_resourceProvider->resource(CurrentPattern).value<void *>());
+    return static_cast<KisPattern*>(m_resourceManager->resource(CurrentPattern).value<void *>());
 }
 
 KisFilterConfiguration * KisCanvasResourceProvider::currentGeneratorConfiguration() const
 {
-    return static_cast<KisFilterConfiguration*>(m_resourceProvider->
+    return static_cast<KisFilterConfiguration*>(m_resourceManager->
             resource(CurrentGeneratorConfiguration).value<void *>());
 }
 
 
 KoAbstractGradient* KisCanvasResourceProvider::currentGradient() const
 {
-    return static_cast<KoAbstractGradient*>(m_resourceProvider->resource(CurrentGradient).value<void *>());
+    return static_cast<KoAbstractGradient*>(m_resourceManager->resource(CurrentGradient).value<void *>());
 }
 
 
 void KisCanvasResourceProvider::resetDisplayProfile()
 {
     // XXX: The X11 monitor profile overrides the settings
-    m_displayProfile = KoIccColorProfile::getScreenProfile();
+    m_displayProfile = KisCanvasResourceProvider::getScreenProfile();
 
     if (m_displayProfile == 0) {
         KisConfig cfg;
@@ -135,7 +145,7 @@ const KoColorProfile * KisCanvasResourceProvider::currentDisplayProfile() const
 
 }
 
-KisImageSP KisCanvasResourceProvider::currentImage() const
+KisImageWSP KisCanvasResourceProvider::currentImage() const
 {
     return m_view->image();
 }
@@ -147,7 +157,7 @@ KisNodeSP KisCanvasResourceProvider::currentNode() const
 
 KisPaintOpPresetSP KisCanvasResourceProvider::currentPreset() const
 {
-    KisPaintOpPresetSP preset = m_resourceProvider->resource(CurrentPaintOpPreset).value<KisPaintOpPresetSP>();
+    KisPaintOpPresetSP preset = m_resourceManager->resource(CurrentPaintOpPreset).value<KisPaintOpPresetSP>();
     return preset;
 }
 
@@ -156,7 +166,7 @@ void KisCanvasResourceProvider::slotPatternActivated(KoResource * res)
 {
     KisPattern * pattern = dynamic_cast<KisPattern*>(res);
     QVariant v = qVariantFromValue((void *) pattern);
-    m_resourceProvider->setResource(CurrentPattern, v);
+    m_resourceManager->setResource(CurrentPattern, v);
     emit sigPatternChanged(pattern);
 }
 
@@ -164,7 +174,7 @@ void KisCanvasResourceProvider::slotGeneratorConfigurationActivated(KisFilterCon
 {
     KisFilterConfiguration * generatorConfiguration = dynamic_cast<KisFilterConfiguration*>(res);
     QVariant v = qVariantFromValue((void *) generatorConfiguration);
-    m_resourceProvider->setResource(CurrentGeneratorConfiguration, v);
+    m_resourceManager->setResource(CurrentGeneratorConfiguration, v);
     emit sigGeneratorConfigurationChanged(generatorConfiguration);
 }
 
@@ -173,19 +183,23 @@ void KisCanvasResourceProvider::slotGradientActivated(KoResource *res)
 
     KoAbstractGradient * gradient = dynamic_cast<KoAbstractGradient*>(res);
     QVariant v = qVariantFromValue((void *) gradient);
-    m_resourceProvider->setResource(CurrentGradient, v);
+    m_resourceManager->setResource(CurrentGradient, v);
     emit sigGradientChanged(gradient);
 }
 
 void KisCanvasResourceProvider::slotPaintOpPresetActivated(const KisPaintOpPresetSP preset)
 {
-    Q_ASSERT( preset->valid() );
-    Q_ASSERT( !preset->paintOp().id().isEmpty() );
-    Q_ASSERT( preset->settings() );
+
+    Q_ASSERT(preset->valid());
+    Q_ASSERT(!preset->paintOp().id().isEmpty());
+    Q_ASSERT(preset->settings());
     if (!preset) return;
+
+    dbgUI << "slotPaintOpPresetActivated" << preset->paintOp();
+
     QVariant v;
     v.setValue(preset);
-    m_resourceProvider->setResource(CurrentPaintOpPreset, v);
+    m_resourceManager->setResource(CurrentPaintOpPreset, v);
     emit sigPaintOpPresetChanged(preset);
 }
 
@@ -194,15 +208,17 @@ void KisCanvasResourceProvider::setBGColor(const KoColor& c)
 
     QVariant v;
     v.setValue(c);
-    m_resourceProvider->setResource(KoCanvasResource::BackgroundColor, v);
+    m_resourceManager->setResource(KoCanvasResource::BackgroundColor, v);
     emit sigBGColorChanged(c);
 }
 
 void KisCanvasResourceProvider::setFGColor(const KoColor& c)
 {
+    m_fGChanged = true;
+
     QVariant v;
     v.setValue(c);
-    m_resourceProvider->setResource(KoCanvasResource::ForegroundColor, v);
+    m_resourceManager->setResource(KoCanvasResource::ForegroundColor, v);
     emit sigFGColorChanged(c);
 }
 
@@ -220,19 +236,19 @@ void KisCanvasResourceProvider::slotNodeActivated(const KisNodeSP node)
 {
     QVariant v;
     v.setValue(node);
-    m_resourceProvider->setResource(CurrentKritaNode, v);
+    m_resourceManager->setResource(CurrentKritaNode, v);
     emit sigNodeChanged(currentNode());
 }
 
 
 void KisCanvasResourceProvider::slotImageSizeChanged()
 {
-    if (KisImageSP image = m_view->image()) {
+    if (KisImageWSP image = m_view->image()) {
         float fw = image->width() / image->xRes();
         float fh = image->height() / image->yRes();
 
         QSizeF postscriptSize(fw, fh);
-        m_resourceProvider->setResource(KoCanvasResource::PageSize, postscriptSize);
+        m_resourceManager->setResource(KoCanvasResource::PageSize, postscriptSize);
     }
 }
 
@@ -246,6 +262,7 @@ void KisCanvasResourceProvider::slotResourceChanged(int key, const QVariant & re
 {
     switch (key) {
     case(KoCanvasResource::ForegroundColor):
+        m_fGChanged = true;
         emit sigFGColorChanged(res.value<KoColor>());
         break;
     case(KoCanvasResource::BackgroundColor):
@@ -270,4 +287,57 @@ void KisCanvasResourceProvider::slotResourceChanged(int key, const QVariant & re
     };
 }
 
+const KoColorProfile *KisCanvasResourceProvider::getScreenProfile(int screen)
+{
+#ifdef Q_WS_X11
+
+    Atom type;
+    int format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    quint8 * str;
+
+    static Atom icc_atom = XInternAtom(QX11Info::display(), "_ICC_PROFILE", True);
+
+    if (XGetWindowProperty(QX11Info::display(),
+                           QX11Info::appRootWindow(screen),
+                           icc_atom,
+                           0,
+                           INT_MAX,
+                           False,
+                           XA_CARDINAL,
+                           &type,
+                           &format,
+                           &nitems,
+                           &bytes_after,
+                           (unsigned char **) &str) == Success
+       ) {
+        QByteArray bytes(nitems, '\0');
+        bytes = QByteArray::fromRawData((char*)str, (quint32)nitems);
+
+        return KoColorSpaceRegistry::instance()->createColorProfile(RGBAColorModelID.id(), Integer8BitsColorDepthID.id(), bytes);
+        return 0;
+    } else {
+        return 0;
+    }
+#else
+    return 0;
+
+#endif
+}
+
+void KisCanvasResourceProvider::slotPainting()
+{
+
+    if (m_fGChanged && m_enablefGChange) {
+        emit sigFGColorUsed(fgColor());
+        m_fGChanged = false;
+    }
+}
+
+void KisCanvasResourceProvider::slotResetEnableFGChange(bool b)
+{
+    m_enablefGChange = b;
+}
+
 #include "kis_canvas_resource_provider.moc"
