@@ -40,6 +40,7 @@
 #include <KoColorProfile.h>
 #include <KoSelection.h>
 #include <KoShapeManager.h>
+#include <KoProgressUpdater.h>
 
 #include <filter/kis_filter_configuration.h>
 #include <filter/kis_filter.h>
@@ -56,8 +57,9 @@
 #include <kis_selection.h>
 #include <flake/kis_shape_layer.h>
 #include <kis_shear_visitor.h>
-#include <kis_transform_worker.h>
+#include <kis_transform_visitor.h>
 #include <kis_undo_adapter.h>
+#include <kis_painter.h>
 
 #include "kis_config.h"
 #include "kis_cursor.h"
@@ -65,7 +67,6 @@
 #include "dialogs/kis_dlg_adjustment_layer.h"
 #include "dialogs/kis_dlg_layer_properties.h"
 #include "dialogs/kis_dlg_generator_layer.h"
-#include "dialogs/kis_dlg_new_layer.h"
 #include "kis_doc2.h"
 #include "kis_filter_manager.h"
 #include "commands/kis_image_commands.h"
@@ -79,32 +80,21 @@
 #include "canvas/kis_canvas2.h"
 #include "widgets/kis_meta_data_merge_strategy_chooser_widget.h"
 #include "widgets/kis_wdg_generator.h"
-#include "kis_layer_box.h"
+#include "kis_progress_widget.h"
 #include "kis_node_commands_adapter.h"
+#include "kis_node_manager.h"
 
 KisLayerManager::KisLayerManager(KisView2 * view, KisDoc2 * doc)
         : m_view(view)
         , m_doc(doc)
-        , m_imgFlatten(0)
-        , m_imgMergeLayer(0)
-        , m_actionAdjustmentLayer(0)
-        , m_actionGeneratorLayer(0)
-        , m_layerAdd(0)
-        , m_layerAddCloneLayer(0)
-        , m_layerAddShapeLayer(0)
-        , m_layerBottom(0)
-        , m_layerDup(0)
-        , m_layerHide(0)
-        , m_layerLower(0)
-        , m_layerProperties(0)
-        , m_layerRaise(0)
-        , m_layerRm(0)
+        , m_imageFlatten(0)
+        , m_imageMergeLayer(0)
         , m_layerSaveAs(0)
-        , m_layerTop(0)
         , m_actLayerVis(false)
-        , m_imgResizeToLayer(0)
+        , m_imageResizeToLayer(0)
+        , m_flattenLayer(0)
         , m_activeLayer(0)
-        , m_commandsAdapter( new KisNodeCommandsAdapter( m_view ) )
+        , m_commandsAdapter(new KisNodeCommandsAdapter(m_view))
 {
 }
 
@@ -131,8 +121,7 @@ void KisLayerManager::activateLayer(KisLayerSP layer)
     m_activeLayer = layer;
     emit sigLayerActivated(layer);
     layersUpdated();
-    if(layer)
-    {
+    if (layer) {
         m_view->resourceProvider()->slotNodeActivated(layer.data());
     }
 }
@@ -140,91 +129,27 @@ void KisLayerManager::activateLayer(KisLayerSP layer)
 
 void KisLayerManager::setup(KActionCollection * actionCollection)
 {
-    m_imgFlatten  = new KAction(i18n("&Flatten image"), this);
-    actionCollection->addAction("flatten_image", m_imgFlatten);
-    m_imgFlatten->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_E));
-    connect(m_imgFlatten, SIGNAL(triggered()), this, SLOT(flattenImage()));
+    m_imageFlatten  = new KAction(i18n("&Flatten image"), this);
+    actionCollection->addAction("flatten_image", m_imageFlatten);
+    m_imageFlatten->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_E));
+    connect(m_imageFlatten, SIGNAL(triggered()), this, SLOT(flattenImage()));
 
-    m_imgMergeLayer  = new KAction(i18n("&Merge with Layer Below"), this);
-    actionCollection->addAction("merge_layer", m_imgMergeLayer);
-    m_imgMergeLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_E));
-    connect(m_imgMergeLayer, SIGNAL(triggered()), this, SLOT(mergeLayer()));
+    m_imageMergeLayer  = new KAction(i18n("&Merge with Layer Below"), this);
+    actionCollection->addAction("merge_layer", m_imageMergeLayer);
+    m_imageMergeLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_E));
+    connect(m_imageMergeLayer, SIGNAL(triggered()), this, SLOT(mergeLayer()));
 
-    m_layerAdd  = new KAction(i18n("&Add..."), this);
-    actionCollection->addAction("insert_layer", m_layerAdd);
-    m_layerAdd->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_N));
-    connect(m_layerAdd, SIGNAL(triggered()), this, SLOT(layerAdd()));
-
-    m_actionAdjustmentLayer  = new KAction(i18n("&Filter Layer"), this);
-    actionCollection->addAction("insert_adjustment_layer", m_actionAdjustmentLayer);
-    connect(m_actionAdjustmentLayer, SIGNAL(triggered()), this, SLOT(addAdjustmentLayer()));
-
-    m_actionGeneratorLayer  = new KAction(i18n("&Generator Layer"), this);
-    actionCollection->addAction("insert_Generator_layer", m_actionGeneratorLayer);
-    connect(m_actionGeneratorLayer, SIGNAL(triggered()), this, SLOT(addGeneratorLayer()));
-
-    m_layerAddCloneLayer  = new KAction(i18n("&Clone Layer"), this);
-    actionCollection->addAction("insert_clone_layer", m_layerAddCloneLayer);
-    connect(m_layerAddCloneLayer, SIGNAL(triggered()), this, SLOT(addCloneLayer()));
-
-    m_layerAddShapeLayer  = new KAction(i18n("&Shape Layer"), this);
-    actionCollection->addAction("insert_shape_layer", m_layerAddShapeLayer);
-    connect(m_layerAddShapeLayer, SIGNAL(triggered()), this, SLOT(addShapeLayer()));
-
-    m_layerRm  = new KAction(i18n("&Remove"), this);
-    actionCollection->addAction("remove_layer", m_layerRm);
-    connect(m_layerRm, SIGNAL(triggered()), this, SLOT(layerRemove()));
-
-    m_layerDup  = new KAction(i18n("Duplicate"), this);
-    actionCollection->addAction("duplicate_layer", m_layerDup);
-    connect(m_layerDup, SIGNAL(triggered()), this, SLOT(layerDuplicate()));
-
-    m_layerHide  = new KToggleAction(i18n("&Hide"), this);
-    actionCollection->addAction("hide_layer", m_layerHide);
-    connect(m_layerHide, SIGNAL(triggered()), this,  SLOT(layerToggleVisible()));
-
-    m_layerHide->setCheckedState(KGuiItem(i18n("&Show")));
-    m_layerHide->setChecked(false);
-
-    m_layerRaise  = new KAction(KIcon("go-up"), i18n("Raise"), this);
-    actionCollection->addAction("raiselayer", m_layerRaise);
-    m_layerRaise->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_BracketRight));
-    connect(m_layerRaise, SIGNAL(triggered()), this, SLOT(layerRaise()));
-
-    m_layerLower  = new KAction(KIcon("go-down"), i18n("Lower"), this);
-    actionCollection->addAction("lowerlayer", m_layerLower);
-    m_layerLower->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_BracketLeft));
-    connect(m_layerLower, SIGNAL(triggered()), this, SLOT(layerLower()));
-
-    m_layerTop  = new KAction(KIcon("go-top"), i18n("To Top"), this);
-    actionCollection->addAction("toplayer", m_layerTop);
-    m_layerTop->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_BracketRight));
-    connect(m_layerTop, SIGNAL(triggered()), this, SLOT(layerFront()));
-
-    m_layerBottom  = new KAction(KIcon("go-down"), i18n("To Bottom"), this);
-    actionCollection->addAction("bottomlayer", m_layerBottom);
-    m_layerBottom->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_BracketLeft));
-    connect(m_layerBottom, SIGNAL(triggered()), this, SLOT(layerBack()));
-
-    m_layerProperties  = new KAction(i18n("Properties..."), this);
-    actionCollection->addAction("layer_properties", m_layerProperties);
-    connect(m_layerProperties, SIGNAL(triggered()), this, SLOT(layerProperties()));
+    m_flattenLayer  = new KAction(i18n("&Flatten Layer"), this);
+    actionCollection->addAction("flatten_layer", m_flattenLayer);
+    connect(m_flattenLayer, SIGNAL(triggered()), this, SLOT(flattenLayer()));
 
     m_layerSaveAs  = new KAction(KIcon("document-save"), i18n("Save Layer as Image..."), this);
     actionCollection->addAction("save_layer_as_image", m_layerSaveAs);
     connect(m_layerSaveAs, SIGNAL(triggered()), this, SLOT(saveLayerAsImage()));
 
-    KAction * action  = new KAction(KIcon("view-split-left-right"), i18n("Flip on &X Axis"), this);
-    actionCollection->addAction("mirrorLayerX", action);
-    connect(action, SIGNAL(triggered()), this, SLOT(mirrorLayerX()));
-
-    action  = new KAction(KIcon("view-split-top-bottom"), i18n("Flip on &Y Axis"), this);
-    actionCollection->addAction("mirrorLayerY", action);
-    connect(action, SIGNAL(triggered()), this, SLOT(mirrorLayerY()));
-
-    m_imgResizeToLayer  = new KAction(i18n("Resize Image to Size of Current Layer"), this);
-    actionCollection->addAction("resizeimgtolayer", m_imgResizeToLayer);
-    connect(m_imgResizeToLayer, SIGNAL(triggered()), this, SLOT(imgResizeToActiveLayer()));
+    m_imageResizeToLayer  = new KAction(i18n("Size Canvas to Size of Current Layer"), this);
+    actionCollection->addAction("resizeimagetolayer", m_imageResizeToLayer);
+    connect(m_imageResizeToLayer, SIGNAL(triggered()), this, SLOT(imageResizeToActiveLayer()));
 }
 
 void KisLayerManager::addAction(QAction * action)
@@ -234,67 +159,53 @@ void KisLayerManager::addAction(QAction * action)
 
 void KisLayerManager::updateGUI()
 {
-
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
     KisLayerSP layer;
     qint32 nlayers = 0;
     qint32 nvisible = 0;
 
 
-    if (img) {
+    if (image) {
         layer = m_activeLayer;
-        nlayers = img->nlayers();
-        nvisible = nlayers - img->nHiddenLayers();
+        nlayers = image->nlayers();
+        nvisible = nlayers - image->nHiddenLayers();
     }
 
-    bool enable = img && layer && layer->visible() && !layer->userLocked() && !layer->systemLocked();
+    bool enable = image && layer && layer->visible() && !layer->userLocked() && !layer->systemLocked();
 
-    m_layerDup->setEnabled(enable);
-    m_layerRm->setEnabled(enable);
-    m_layerHide->setEnabled(img && layer);
-    m_layerProperties->setEnabled(enable);
     m_layerSaveAs->setEnabled(enable);
-    m_layerRaise->setEnabled(enable && layer->prevSibling());
-    m_layerLower->setEnabled(enable && layer->nextSibling());
-    m_layerTop->setEnabled(enable && nlayers > 1 && layer.data() != img->rootLayer()->firstChild().data());
-    m_layerBottom->setEnabled(enable && nlayers > 1 && layer.data() != img->rootLayer()->lastChild().data());
 
-    // XXX these should be named layer instead of img
-    m_imgFlatten->setEnabled(nlayers > 1);
-    m_imgMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
+    // XXX these should be named layer instead of image
+    m_imageFlatten->setEnabled(nlayers > 1);
+    m_imageMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
+    m_flattenLayer->setEnabled(nlayers > 1 && layer && layer->firstChild());
 
-
-    // XXX: Check whether the colorspace was really changed
-    if (activeDevice()) {
-        emit currentColorSpaceChanged(activeDevice()->colorSpace());
-    }
-
-    m_imgResizeToLayer->setEnabled(activeLayer());
+    m_imageResizeToLayer->setEnabled(activeLayer());
 
     if (m_view->statusBar())
-        m_view->statusBar()->setProfile(img);
+        m_view->statusBar()->setProfile(image);
 }
 
-void KisLayerManager::imgResizeToActiveLayer()
+void KisLayerManager::imageResizeToActiveLayer()
 {
     KisLayerSP layer;
     KisUndoAdapter * undoAdapter = m_view->undoAdapter();
 
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (img && (layer = activeLayer())) {
+    if (image && (layer = activeLayer())) {
 
         if (undoAdapter && undoAdapter->undo()) {
             undoAdapter->beginMacro(i18n("Resize Image to Size of Current Layer"));
         }
 
-        img->lock();
+        image->lock();
 
         QRect r = layer->exactBounds();
-        img->resize(r.width(), r.height(), r.x(), r.y(), true);
+        image->resize(r.width(), r.height(), r.x(), r.y(), true);
 
-        img->unlock();
+        image->unlock();
 
         if (undoAdapter && undoAdapter->undo()) {
             undoAdapter->endMacro();
@@ -302,52 +213,6 @@ void KisLayerManager::imgResizeToActiveLayer()
     }
 }
 
-void KisLayerManager::layerCompositeOp(const KoCompositeOp* compositeOp)
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    m_doc->addCommand(new KisLayerCompositeOpCommand(layer, layer->compositeOp(), compositeOp));
-}
-
-// range: 0 - 100
-void KisLayerManager::layerOpacity(double opacity, bool final)
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    opacity = int(opacity * 2.55 + 0.5);
-    if (opacity > 255)
-        opacity = 255;
-
-    if (opacity == layer->opacity()) return;
-
-    if (!final) {
-        layer->setOpacity(static_cast<int>(opacity));
-        layer->setDirty();
-    } else {
-        m_doc->addCommand(new KisLayerOpacityCommand(layer, layer->opacity(), static_cast<int>(opacity)));
-    }
-}
-
-void KisLayerManager::layerToggleVisible()
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    layer->setVisible(!layer->visible());
-
-    layer->setDirty();
-}
-
-void KisLayerManager::layerToggleLocked()
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    layer->setUserLocked(!layer->userLocked());
-}
-
 void KisLayerManager::actLayerVisChanged(int show)
 {
     m_actLayerVis = (show != 0);
@@ -374,43 +239,55 @@ void KisLayerManager::layerProperties()
         if (prev) dev = prev->projection();
 
         KisDlgAdjLayerProps dlg(dev, alayer->image(), alayer->filter(), alayer->name(), i18n("Filter Layer Properties"), m_view, "dlgadjlayerprops");
+        dlg.resize(dlg.minimumSizeHint());
         KisFilterConfiguration* config = dlg.filterConfiguration();
         QString before;
-        if (config)
-            before = config->toLegacyXML();
+        if (config) {
+            before = config->toXML();
+        }
         if (dlg.exec() == QDialog::Accepted) {
 
             QString after;
-            if (dlg.filterConfiguration())
-                after = dlg.filterConfiguration()->toLegacyXML();
-            alayer->setName(dlg.layerName());
-            KisChangeFilterCmd<KisAdjustmentLayerSP> * cmd
-            = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
-                    dlg.filterConfiguration(),
-                    before,
-                    after);
-            cmd->redo();
-            m_view->undoAdapter()->addCommand(cmd);
-            m_doc->setModified(true);
+            if (dlg.filterConfiguration()) {
+                after = dlg.filterConfiguration()->toXML();
+            }
+            if (after != before) {
+                alayer->setName(dlg.layerName());
+
+                KisChangeFilterCmd<KisAdjustmentLayerSP> * cmd
+                = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
+                        dlg.filterConfiguration(),
+                        before,
+                        after);
+                cmd->redo();
+                m_view->undoAdapter()->addCommand(cmd);
+                m_doc->setModified(true);
+            }
         }
     } else if (KisGeneratorLayerSP alayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()))) {
 
         KisDlgGeneratorLayer dlg(alayer->name(), m_view);
         dlg.setCaption(i18n("Generator Layer Properties"));
 
-        QString before = alayer->generator()->toLegacyXML();
+        QString before = alayer->generator()->toXML();
         dlg.setConfiguration(alayer->generator());
+        dlg.resize(dlg.minimumSizeHint());
 
         if (dlg.exec() == QDialog::Accepted) {
 
-            KisChangeGeneratorCmd<KisGeneratorLayerSP> * cmd
-            = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
-                    dlg.configuration(),
-                    before,
-                    dlg.configuration()->toLegacyXML());
-            cmd->redo();
-            m_view->undoAdapter()->addCommand(cmd);
-            m_doc->setModified(true);
+            QString after = dlg.configuration()->toXML();
+            if (after != before) {
+                KisChangeGeneratorCmd<KisGeneratorLayerSP> * cmd
+                = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
+                        dlg.configuration(),
+                        before,
+                        after
+                                                                );
+
+                cmd->redo();
+                m_view->undoAdapter()->addCommand(cmd);
+                m_doc->setModified(true);
+            }
 
         }
     } else {
@@ -419,6 +296,8 @@ void KisLayerManager::layerProperties()
                                   layer->compositeOp(),
                                   cs,
                                   layer->channelFlags());
+        dlg.resize(dlg.minimumSizeHint());
+
         if (dlg.exec() == QDialog::Accepted) {
 
             QBitArray newChannelFlags = dlg.getChannelFlags();
@@ -434,13 +313,13 @@ void KisLayerManager::layerProperties()
 
             if (layer->name() != dlg.getName() ||
                     layer->opacity() != dlg.getOpacity() ||
-                    layer->compositeOp() != dlg.getCompositeOp() ||
+                    layer->compositeOp()->id() != dlg.getCompositeOp() ||
                     oldChannelFlags != newChannelFlags
                ) {
                 QApplication::setOverrideCursor(KisCursor::waitCursor());
                 m_view->undoAdapter()->addCommand(new KisLayerPropsCommand(layer,
                                                   layer->opacity(), dlg.getOpacity(),
-                                                  layer->compositeOp(), dlg.getCompositeOp(),
+                                                  layer->compositeOpId(), dlg.getCompositeOp(),
                                                   layer->name(), dlg.getName(),
                                                   oldChannelFlags, newChannelFlags));
                 QApplication::restoreOverrideCursor();
@@ -453,25 +332,25 @@ void KisLayerManager::layerProperties()
 
 void KisLayerManager::layerAdd()
 {
-    KisImageSP img = m_view->image();
-    if (img && activeLayer()) {
+    KisImageWSP image = m_view->image();
+    if (image && activeLayer()) {
         addLayer(activeLayer()->parent(), activeLayer());
-    } else if (img)
-        addLayer(img->rootLayer(), KisLayerSP(0));
+    } else if (image)
+        addLayer(image->rootLayer(), KisLayerSP(0));
 }
 
 void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
-    if (img) {
+    KisImageWSP image = m_view->image();
+    if (image) {
         KisConfig cfg;
         QString profilename;
-        KisLayerSP layer = new KisPaintLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE, img->colorSpace());
+        KisLayerSP layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, image->colorSpace());
         if (layer) {
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
+            layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
             m_view->canvas()->update();
-            m_view->layerBox()->setCurrentNode( layer );
+            m_view->nodeManager()->activateNode(layer);
         } else {
             KMessageBox::error(m_view, i18n("Could not add layer to image."), i18n("Layer Error"));
         }
@@ -480,11 +359,11 @@ void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
-    if (img) {
-        KisLayerSP layer = KisLayerSP(new KisGroupLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE));
+    KisImageWSP image = m_view->image();
+    if (image) {
+        KisLayerSP layer = KisLayerSP(new KisGroupLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8));
         if (layer) {
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
+            layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
             m_view->canvas()->update();
         } else {
@@ -496,21 +375,21 @@ void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addCloneLayer()
 {
-    KisImageSP img = m_view->image();
-    if (img && activeLayer()) {
+    KisImageWSP image = m_view->image();
+    if (image && activeLayer()) {
         addCloneLayer(activeLayer()->parent(), activeLayer());
-    } else if (img)
-        addCloneLayer(img->rootLayer(), KisLayerSP(0));
+    } else if (image)
+        addCloneLayer(image->rootLayer(), KisLayerSP(0));
 }
 
 void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
-    if (img) {
+    KisImageWSP image = m_view->image();
+    if (image) {
         // Check whether we are not cloning a parent layer
         if (KisGroupLayer * from = dynamic_cast<KisGroupLayer*>(m_activeLayer.data())) {
             KisNodeSP parent = parent;
-            while (parent && parent != img->root()) {
+            while (parent && parent != image->root()) {
                 if (parent.data() == from) {
                     // The chosen layer is one of our own parents -- this will
                     // lead to cyclic behaviour when updating. Don't do that!
@@ -520,11 +399,11 @@ void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
             }
         }
 
-        KisLayerSP layer = new KisCloneLayer(m_activeLayer, img.data(), img->nextLayerName(), OPACITY_OPAQUE);
+        KisLayerSP layer = new KisCloneLayer(m_activeLayer, image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
 
         if (layer) {
 
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
+            layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
 
             m_view->canvas()->update();
@@ -538,27 +417,27 @@ void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addShapeLayer()
 {
-    KisImageSP img = m_view->image();
-    if (img && activeLayer()) {
+    KisImageWSP image = m_view->image();
+    if (image && activeLayer()) {
         addShapeLayer(activeLayer()->parent(), activeLayer());
-    } else if (img)
-        addShapeLayer(img->rootLayer(), KisLayerSP(0));
+    } else if (image)
+        addShapeLayer(image->rootLayer(), KisLayerSP(0));
 }
 
 
 void KisLayerManager::addShapeLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
-    if (img) {
+    KisImageWSP image = m_view->image();
+    if (image) {
         // XXX: Make work with nodes!
         KisLayer * parentLayer = dynamic_cast<KisLayer*>(parent.data());
         KoShapeContainer * parentContainer =
             dynamic_cast<KoShapeContainer*>(m_doc->shapeForNode(parentLayer));
         if (!parentContainer) return;
 
-        KisLayerSP layer = new KisShapeLayer(parentContainer, m_doc->shapeController(), img.data(), img->nextLayerName(), OPACITY_OPAQUE);
+        KisLayerSP layer = new KisShapeLayer(parentContainer, m_doc->shapeController(), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
         if (layer) {
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
+            layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent, above.data());
             m_view->canvas()->update();
         } else {
@@ -577,8 +456,8 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
 {
     Q_ASSERT(parent);
 
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP l = activeLayer();
 
@@ -588,13 +467,15 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
     if (l->selection())
         selection = l->selection();
     else
-        selection = img->globalSelection();
+        selection = image->globalSelection();
 
     KisAdjustmentLayerSP adjl = addAdjustmentLayer(parent, above, QString(), 0, selection);
 
-    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), dev, adjl->image(), img->nextLayerName(), i18n("New Filter Layer"), m_view, "dlgadjustmentlayer");
+    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), dev, adjl->image(), image->nextLayerName(), i18n("New Filter Layer"), m_view, "dlgadjustmentlayer");
+    dlg.resize(dlg.minimumSizeHint());
+
     if (dlg.exec() != QDialog::Accepted) {
-        m_view->image()->removeNode(adjl);
+        m_commandsAdapter->undoLastCommand();
     } else {
         adjl->setName(dlg.layerName());
     }
@@ -605,12 +486,12 @@ KisAdjustmentLayerSP KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNo
 {
     Q_ASSERT(parent);
 
-    KisImageSP img = m_view->image();
-    if (!img) return 0;
+    KisImageWSP image = m_view->image();
+    if (!image) return 0;
 
-    KisAdjustmentLayerSP l = new KisAdjustmentLayer(img, name, filter, selection);
+    KisAdjustmentLayerSP l = new KisAdjustmentLayer(image, name, filter, selection);
     m_commandsAdapter->addNode(l.data(), parent, above);
-    l->setDirty(img->bounds());
+    l->setDirty(image->bounds());
     return l;
 }
 
@@ -623,10 +504,12 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above)
 {
     Q_ASSERT(parent);
 
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
+
+    KisDlgGeneratorLayer dlg(image->nextLayerName(), m_view);
+    dlg.resize(dlg.minimumSizeHint());
 
-    KisDlgGeneratorLayer dlg(img->nextLayerName(), m_view);
     if (dlg.exec() == QDialog::Accepted) {
         KisSelectionSP selection = m_view->selection();
         KisFilterConfiguration * generator = dlg.configuration();
@@ -641,24 +524,24 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above, const
     Q_ASSERT(parent);
     Q_ASSERT(generator);
 
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
-    KisGeneratorLayerSP l = new KisGeneratorLayer(img, name, generator, selection);
+    KisGeneratorLayerSP l = new KisGeneratorLayer(image, name, generator, selection);
     m_commandsAdapter->addNode(l.data(), parent, above.data());
     if (l->selection())
         l->setDirty(l->selection()->selectedExactRect());
     else
-        l->setDirty(img->bounds());
+        l->setDirty(image->bounds());
 
 }
 
 
 void KisLayerManager::layerRemove()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (img) {
+    if (image) {
         KisLayerSP layer = activeLayer();
         if (layer) {
             QRect extent = layer->extent();
@@ -677,9 +560,9 @@ void KisLayerManager::layerRemove()
 
 void KisLayerManager::layerDuplicate()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (!img)
+    if (!image)
         return;
 
     KisLayerSP active = activeLayer();
@@ -701,10 +584,10 @@ void KisLayerManager::layerDuplicate()
 
 void KisLayerManager::layerRaise()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP layer;
 
-    if (!img)
+    if (!image)
         return;
 
     layer = activeLayer();
@@ -715,24 +598,24 @@ void KisLayerManager::layerRaise()
 
 void KisLayerManager::layerLower()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP layer;
 
-    if (!img)
+    if (!image)
         return;
 
     layer = activeLayer();
 
-    m_commandsAdapter->lower( layer );
+    m_commandsAdapter->lower(layer);
     layer->parent()->setDirty();
 }
 
 void KisLayerManager::layerFront()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP layer;
 
-    if (!img)
+    if (!image)
         return;
 
     layer = activeLayer();
@@ -742,8 +625,8 @@ void KisLayerManager::layerFront()
 
 void KisLayerManager::layerBack()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer;
 
@@ -778,14 +661,13 @@ void KisLayerManager::mirrorLayerX()
         Q_CHECK_PTR(t);
     }
 
-    QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection() );
+    QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection());
     m_activeLayer->setDirty(dirty);
 
     if (t) m_view->undoAdapter()->addCommand(t);
 
     m_doc->setModified(true);
     layersUpdated();
-
     m_view->canvas()->update();
 }
 
@@ -800,7 +682,7 @@ void KisLayerManager::mirrorLayerY()
         Q_CHECK_PTR(t);
     }
 
-    QRect dirty = KisTransformWorker::mirrorY(dev, m_view->selection() );
+    QRect dirty = KisTransformWorker::mirrorY(dev, m_view->selection());
     m_activeLayer->setDirty(dirty);
 
     if (t) m_view->undoAdapter()->addCommand(t);
@@ -818,19 +700,23 @@ void KisLayerManager::scaleLayer(double sx, double sy, KisFilterStrategy *filter
     if (!layer) return;
 
     KisSelectedTransaction * t = 0;
-    if (m_view->undoAdapter() && m_view->undoAdapter()->undo() && layer->selection()) {
+    if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
         t = new KisSelectedTransaction(i18n("Scale Layer"), layer);
         Q_CHECK_PTR(t);
     }
 
-    // XXX_PROGRESS: pass KoUpdater
-    KisTransformWorker worker(layer->paintDevice(), sx, sy, 0, 0, 0.0, 0, 0, 0, filterStrategy);
+    KoProgressUpdater* updater = m_view->createProgressUpdater();
+    KoUpdaterPtr u = updater->startSubtask();
+
+    KisTransformWorker worker(layer->paintDevice(), sx, sy, 0, 0, 0.0, 0, 0, u, filterStrategy);
     worker.run();
 
     if (t) m_view->undoAdapter()->addCommand(t);
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
+
+    updater->deleteLater();
 }
 
 void KisLayerManager::rotateLayer(double radians)
@@ -840,10 +726,8 @@ void KisLayerManager::rotateLayer(double radians)
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    KisSelectedTransaction * t = 0;
     if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
-        t = new KisSelectedTransaction(i18n("Rotate Layer"), layer);
-        Q_CHECK_PTR(t);
+        m_view->undoAdapter()->beginMacro(i18n("Rotate Layer"));
     }
 
     KisFilterStrategy *filter = KisFilterStrategyRegistry::instance()->value("Triangle");
@@ -857,10 +741,11 @@ void KisLayerManager::rotateLayer(double radians)
     double cy = r.y() + r.height() / 2.0;
     qint32 tx = qint32(cx * cos(radians) - cy * sin(radians) - cx + 0.5);
     qint32 ty = qint32(cy * cos(radians) + cx * sin(radians) - cy + 0.5);
-    KisTransformWorker tw(layer->paintDevice(), 1.0, 1.0, 0, 0, radians, -tx, -ty, 0, filter);
-    tw.run();
+    KisTransformVisitor visitor(m_view->image(), 1.0, 1.0, 0, 0, radians, -tx, -ty, 0, filter);
+    layer->accept(visitor);
 
-    if (t) m_view->undoAdapter()->addCommand(t);
+    if (m_view->undoAdapter() && m_view->undoAdapter()->undo())
+        m_view->undoAdapter()->endMacro();
 
     m_doc->setModified(true);
     layersUpdated();
@@ -879,11 +764,11 @@ void KisLayerManager::shearLayer(double angleX, double angleY)
         undo->beginMacro(i18n("Shear layer"));
     }
 
-    KoProgressUpdater updater(m_view->statusBar()->progress());
-    updater.start( 100, i18n("Shear layer") );
-    KoUpdater up = updater.startSubtask();
+    KoProgressUpdater* updater = m_view->statusBar()->progress()->createUpdater();
+    updater->start(100, i18n("Shear layer"));
+    KoUpdaterPtr up = updater->startSubtask();
 
-    KisShearVisitor v(angleX, angleY, &up);
+    KisShearVisitor v(angleX, angleY, up);
     v.setUndoAdapter(undo);
     layer->accept(v);
 
@@ -892,16 +777,19 @@ void KisLayerManager::shearLayer(double angleX, double angleY)
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
+
+    m_view->statusBar()->progress()->detachUpdater(updater);
+    updater->deleteLater();
 }
 
 void KisLayerManager::flattenImage()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (img) {
+    if (image) {
         bool doIt = true;
 
-        if (img->nHiddenLayers() > 0) {
+        if (image->nHiddenLayers() > 0) {
             int answer = KMessageBox::warningYesNo(m_view,
                                                    i18n("The image contains hidden layers that will be lost."),
                                                    i18n("Flatten Image"),
@@ -914,15 +802,15 @@ void KisLayerManager::flattenImage()
         }
 
         if (doIt) {
-            img->flatten();
+            image->flatten();
         }
     }
 }
 
 void KisLayerManager::mergeLayer()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = activeLayer();
     if (!layer) return;
@@ -930,8 +818,8 @@ void KisLayerManager::mergeLayer()
     const KisMetaData::MergeStrategy* strategy = KisMetaDataMergeStrategyChooserWidget::showDialog(m_view);
     if (!strategy) return;
 
-    KisLayerSP  newLayer = img->mergeLayer(layer, strategy);
-    if(newLayer) {
+    KisLayerSP  newLayer = image->mergeLayer(layer, strategy);
+    if (newLayer) {
         newLayer->setDirty();
     }
 
@@ -939,6 +827,21 @@ void KisLayerManager::mergeLayer()
 
 }
 
+void KisLayerManager::flattenLayer()
+{
+    KisImageWSP image = m_view->image();
+    if (!image) return;
+
+    KisLayerSP layer = activeLayer();
+    if (!layer) return;
+
+    KisLayerSP newLayer = image->flattenLayer(layer);
+    if (newLayer) {
+        newLayer->setDirty();
+    }
+    m_view->updateGUI();
+}
+
 void KisLayerManager::layersUpdated()
 {
     KisLayerSP layer = activeLayer();
@@ -963,24 +866,35 @@ void KisLayerManager::saveLayerAsImage()
     KUrl url = fd.selectedUrl();
     QString mimefilter = fd.currentMimeFilter();
 
+    if (mimefilter.isNull()) {
+        KMimeType::Ptr mime = KMimeType::findByUrl(url);
+        mimefilter = mime->name();
+    }
+
     if (url.isEmpty())
         return;
 
-
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP l = activeLayer();
     if (!l) return;
 
-    QRect r = l->exactBounds();
+    QRect r = image->bounds();
 
     KisDoc2 d;
     d.prepareForImport();
 
-    KisImageSP dst = KisImageSP(new KisImage(d.undoAdapter(), r.width(), r.height(), img->colorSpace(), l->name()));
+    KisImageWSP dst = new KisImage(d.undoAdapter(), r.width(), r.height(), image->colorSpace(), l->name());
+    dst->setResolution(image->xRes(), image->yRes());
     d.setCurrentImage(dst);
-    dst->addNode(l->clone(), dst->rootLayer(), KisLayerSP(0));
+    KisPaintLayer* paintLayer = new KisPaintLayer(dst, "projection", l->opacity());
+    KisPainter gc(paintLayer->paintDevice());
+    gc.bitBlt(QPoint(0, 0), l->projection(), l->projection()->exactBounds());
+    dst->addNode(paintLayer, dst->rootLayer(), KisLayerSP(0));
+
+    dst->resize(paintLayer->exactBounds());
+    dst->refreshGraph();
 
     d.setOutputMimeType(mimefilter.toLatin1());
     d.exportDocument(url);
