@@ -29,6 +29,7 @@
 #include <kis_types.h>
 #include <kis_node.h>
 #include <kis_selection.h>
+#include <kis_selection_mask.h>
 #include <kis_layer.h>
 #include <kis_mask.h>
 #include <kis_image.h>
@@ -57,6 +58,8 @@ struct KisNodeManager::Private {
     KisNodeSP activeNode;
     KisNodeManager* self;
     KisNodeCommandsAdapter* commandsAdapter;
+    KisNodeSP activeNodeBeforeMove;
+    
     void slotLayersChanged(KisGroupLayerSP);
 };
 
@@ -75,6 +78,8 @@ KisNodeManager::KisNodeManager(KisView2 * view, KisDoc2 * doc)
     m_d->self = this;
     m_d->commandsAdapter = new KisNodeCommandsAdapter(view);
     connect(m_d->view->image(), SIGNAL(sigPostLayersChanged(KisGroupLayerSP)), SLOT(slotLayersChanged(KisGroupLayerSP)));
+    connect(m_d->view->image(), SIGNAL(sigAboutToMoveNode(KisNode*,int,int)), SLOT(aboutToMoveNode()));
+    connect(m_d->view->image(), SIGNAL(sigNodeHasBeenMoved(KisNode*,int,int)), SLOT(nodeHasBeenMoved()));
 }
 
 KisNodeManager::~KisNodeManager()
@@ -109,21 +114,12 @@ KisNodeSP KisNodeManager::activeNode()
     return m_d->activeNode;
 }
 
-KisPaintDeviceSP KisNodeManager::activePaintDevice()
-{
-    if (m_d->maskManager->activeMask()) {
-        return m_d->maskManager->activeDevice();
-    } else {
-        return m_d->layerManager->activeDevice();
-    }
-}
-
 const KoColorSpace* KisNodeManager::activeColorSpace()
 {
     Q_ASSERT(m_d->maskManager);
 
     if (m_d->maskManager->activeDevice()) {
-        Q_ASSERT(m_d->maskManager->activeDevice());
+//        Q_ASSERT(m_d->maskManager->activeDevice());
         return m_d->maskManager->activeDevice()->colorSpace();
     } else {
         Q_ASSERT(m_d->layerManager);
@@ -211,6 +207,13 @@ void KisNodeManager::moveNode(KisNodeSP node, KisNodeSP activeNode)
     KisNodeSP above;
 
     getNewNodeLocation(node->metaObject()->className(), parent, above, activeNode);
+    if (node->inherits("KisSelectionMask") && parent->inherits("KisLayer")) {
+        KisSelectionMask *m = dynamic_cast<KisSelectionMask*>(node.data());
+        KisLayer *l = dynamic_cast<KisLayer*>(parent.data());
+        KisSelectionMaskSP selMask = l->selectionMask();
+        if (m && m->active() && l && l->selectionMask())
+            selMask->setActive(false);
+    }
     m_d->commandsAdapter->moveNode(node, parent, above);
     node->setDirty(node->extent());
 }
@@ -218,6 +221,13 @@ void KisNodeManager::moveNode(KisNodeSP node, KisNodeSP activeNode)
 void KisNodeManager::moveNodeAt(KisNodeSP node, KisNodeSP parent, int index)
 {
     if (allowAsChild(parent->metaObject()->className(), node->metaObject()->className())) {
+        if (node->inherits("KisSelectionMask") && parent->inherits("KisLayer")) {
+            KisSelectionMask *m = dynamic_cast<KisSelectionMask*>(node.data());
+            KisLayer *l = dynamic_cast<KisLayer*>(parent.data());
+            KisSelectionMaskSP selMask = l->selectionMask();
+            if (m && m->active() && l && l->selectionMask())
+                selMask->setActive(false);
+        }
         m_d->commandsAdapter->moveNode(node, parent, index);
     }
 }
@@ -332,6 +342,13 @@ void KisNodeManager::nodesUpdated()
 
 }
 
+KisPaintDeviceSP KisNodeManager::activePaintDevice()
+{
+    return m_d->maskManager->activeMask() ?
+        m_d->maskManager->activeDevice() :
+        m_d->layerManager->activeDevice();
+}
+
 void KisNodeManager::nodeProperties(KisNodeSP node)
 {
     if (node->inherits("KisLayer")) {
@@ -341,18 +358,60 @@ void KisNodeManager::nodeProperties(KisNodeSP node)
     }
 }
 
-void KisNodeManager::nodeOpacityChanged(qreal opacity, bool final)
+qint32 KisNodeManager::convertOpacityToInt(qreal opacity)
 {
-    m_d->layerManager->layerOpacity(opacity, final);
+    /**
+     * Scales opacity from the range 0...1
+     * to the integer range 0...255
+     */
+
+    return qMin(255, int(opacity * 2.55 + 0.5));
+}
+
+void KisNodeManager::setNodeOpacity(KisNodeSP node, qint32 opacity,
+                                    bool finalChange)
+{
+    if (!node) return;
+    if (node->opacity() == opacity) return;
+
+    if (!finalChange) {
+        node->setOpacity(opacity);
+        node->setDirty();
+    } else {
+        m_d->commandsAdapter->setOpacity(node, opacity);
+    }
+}
+
+void KisNodeManager::setNodeCompositeOp(KisNodeSP node,
+                                        const KoCompositeOp* compositeOp)
+{
+    if (!node) return;
+    if (node->compositeOp() == compositeOp) return;
+
+    m_d->commandsAdapter->setCompositeOp(node, compositeOp);
+}
+
+void KisNodeManager::nodeOpacityChanged(qreal opacity, bool finalChange)
+{
+    KisNodeSP node = activeNode();
+
+    setNodeOpacity(node, convertOpacityToInt(opacity), finalChange);
 }
 
 void KisNodeManager::nodeCompositeOpChanged(const KoCompositeOp* op)
 {
-    m_d->layerManager->layerCompositeOp(op);
+    KisNodeSP node = activeNode();
+
+    setNodeCompositeOp(node, op);
 }
 
-void KisNodeManager::duplicateActiveNode(KisNodeSP node)
+void KisNodeManager::duplicateActiveNode()
 {
+    KisNodeSP node = activeNode();
+
+    // FIXME: can't imagine how it may happen
+    Q_ASSERT(node);
+
     if (node->inherits("KisLayer")) {
         m_d->layerManager->layerDuplicate();
     } else if (node->inherits("KisMask")) {
@@ -409,25 +468,45 @@ void KisNodeManager::nodeToBottom()
 void KisNodeManager::removeNode(KisNodeSP node)
 {
 //     QRect bounds = node->exactBounds();
+
+    //do not delete root layer
+    if(node->parent()==0)
+        return;
+
     m_d->commandsAdapter->removeNode(node);
 //     m_image->rootLayer()->setDirty(bounds);
 }
 
 void KisNodeManager::mirrorNodeX()
 {
-    if (m_d->maskManager->activeMask()) {
-        return m_d->maskManager->mirrorMaskX();
-    } else {
-        return m_d->layerManager->mirrorLayerX();
+    KisNodeSP node = activeNode();
+    if (node->inherits("KisLayer")) {
+        m_d->layerManager->mirrorLayerX();
+    } else if (node->inherits("KisMask")) {
+        m_d->maskManager->mirrorMaskX();
     }
 }
 
 void KisNodeManager::mirrorNodeY()
 {
-    if (m_d->maskManager->activeMask()) {
-        return m_d->maskManager->mirrorMaskY();
-    } else {
-        return m_d->layerManager->mirrorLayerY();
+    KisNodeSP node = activeNode();
+    if (node->inherits("KisLayer")) {
+        m_d->layerManager->mirrorLayerY();
+    } else if (node->inherits("KisMask")) {
+        m_d->maskManager->mirrorMaskY();
+    }
+}
+
+void KisNodeManager::aboutToMoveNode()
+{
+    m_d->activeNodeBeforeMove = activeNode();
+}
+
+void KisNodeManager::nodeHasBeenMoved()
+{
+    if (m_d->activeNodeBeforeMove) {
+        activateNode(m_d->activeNodeBeforeMove);
+        m_d->activeNodeBeforeMove = 0;
     }
 }
 
