@@ -44,17 +44,19 @@
 #include <KoDocumentInfo.h>
 #include <KoID.h>
 #include <KoColorSpaceRegistry.h>
-#include <colorprofiles/KoIccColorProfile.h>
+#include <KoColorProfile.h>
 
 #include <kis_doc2.h>
 #include <kis_image.h>
 #include <kis_iterators_pixel.h>
 #include <kis_layer.h>
 #include <kis_paint_device.h>
+#include <kis_transaction.h>
 #include <kis_paint_layer.h>
 #include <kis_group_layer.h>
 #include <kis_meta_data_io_backend.h>
 #include <kis_meta_data_store.h>
+#include <KoColorModelStandardIds.h>
 
 namespace
 {
@@ -80,24 +82,26 @@ int getColorTypeforColorSpace(const KoColorSpace * cs , bool alpha)
 }
 
 
-QString getColorSpaceForColorType(int color_type, int color_nb_bits)
+QPair<QString, QString> getColorSpaceForColorType(int color_type, int color_nb_bits)
 {
-    if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
-        if (color_nb_bits == 16) {
-            return "GRAYA16";
-        } else if (color_nb_bits <= 8) {
-            return "GRAYA";
+    QPair<QString, QString> r;
+
+    if (color_type ==  PNG_COLOR_TYPE_PALETTE) {
+        r.first = RGBAColorModelID.id();
+        r.second = Integer8BitsColorDepthID.id();
+    } else {
+        if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
+            r.first = GrayAColorModelID.id();
+        } else if (color_type == PNG_COLOR_TYPE_RGB_ALPHA || color_type == PNG_COLOR_TYPE_RGB) {
+            r.first = RGBAColorModelID.id();
         }
-    } else if (color_type == PNG_COLOR_TYPE_RGB_ALPHA || color_type == PNG_COLOR_TYPE_RGB) {
         if (color_nb_bits == 16) {
-            return "RGBA16";
+            r.second = Integer16BitsColorDepthID.id();
         } else if (color_nb_bits <= 8) {
-            return "RGBA";
+            r.second = Integer8BitsColorDepthID.id();
         }
-    } else if (color_type ==  PNG_COLOR_TYPE_PALETTE) {
-        return "RGBA"; // <-- we will convert the index image to RGBA
     }
-    return "";
+    return r;
 }
 
 
@@ -191,13 +195,13 @@ QByteArray png_read_raw_profile(png_textp text)
 {
     QByteArray profile;
 
-    unsigned char unhex[103] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 11, 12,
-                                13, 14, 15
-                               };
+    static unsigned char unhex[103] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 11, 12,
+                                       13, 14, 15
+                                      };
 
     png_charp sp = text[0].text + 1;
     /* look for newline */
@@ -260,7 +264,7 @@ KisPNGConverter::KisPNGConverter(KisDoc2 *doc, KisUndoAdapter *adapter)
     m_adapter = adapter;
     m_stop = false;
     m_max_row = 0;
-    m_img = 0;
+    m_image = 0;
 }
 
 KisPNGConverter::~KisPNGConverter()
@@ -404,7 +408,12 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
 
     png_byte signature[8];
     iod->peek((char*)signature, 8);
+
+#if PNG_LIBPNG_VER < 10400
     if (!png_check_sig(signature, 8)) {
+#else
+    if (png_sig_cmp(signature, 0, 8) != 0) {
+#endif
         iod->close();
         return (KisImageBuilder_RESULT_BAD_FETCH);
     }
@@ -471,8 +480,8 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
 #endif
 
     // Determine the colorspace
-    QString csName = getColorSpaceForColorType(color_type, color_nb_bits);
-    if (csName.isEmpty()) {
+    QPair<QString, QString> csName = getColorSpaceForColorType(color_type, color_nb_bits);
+    if (csName.first.isEmpty()) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
         iod->close();
         return KisImageBuilder_RESULT_UNSUPPORTED_COLORSPACE;
@@ -485,14 +494,14 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     png_uint_32 proflen;
 
 
-    KoColorProfile* profile = 0;
+    const KoColorProfile* profile = 0;
     if (png_get_iCCP(png_ptr, info_ptr, &profile_name, &compression_type, &profile_data, &proflen)) {
         QByteArray profile_rawdata;
         // XXX: Hardcoded for icc type -- is that correct for us?
         if (QString::compare(profile_name, "icc") == 0) {
             profile_rawdata.resize(proflen);
             memcpy(profile_rawdata.data(), profile_data, proflen);
-            profile = new KoIccColorProfile(profile_rawdata);
+            profile = KoColorSpaceRegistry::instance()->createColorProfile(csName.first, csName.second, profile_rawdata);
             Q_CHECK_PTR(profile);
             if (profile) {
 //                 dbgFile << "profile name: " << profile->productName() << " profile description: " << profile->productDescription() << " information sur le produit: " << profile->productInfo();
@@ -506,14 +515,23 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     } else {
         dbgFile << "no embedded profile, will use the default profile";
     }
+    
+    // Check that the profile is used by the color space
+    if (profile && !KoColorSpaceRegistry::instance()->colorSpaceFactory(
+        KoColorSpaceRegistry::instance()->colorSpaceId(
+      csName.first, csName.second))->profileIsCompatible(profile)) {
+        warnFile << "The profile " << profile->name() << " is not compatible with the color space model " << csName.first << " " << csName.second;
+        delete profile;
+        profile = 0;
+    }
 
     // Retrieve a pointer to the colorspace
     const KoColorSpace* cs;
     if (profile && profile->isSuitableForOutput()) {
         dbgFile << "image has embedded profile: " << profile -> name() << "\n";
-        cs = KoColorSpaceRegistry::instance()->colorSpace(csName, profile);
+        cs = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, profile);
     } else
-        cs = KoColorSpaceRegistry::instance()->colorSpace(KoID(csName, ""), "");
+        cs = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, 0);
 
     if (cs == 0) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
@@ -523,37 +541,35 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     // Create the cmsTransform if needed
     KoColorTransformation* transform = 0;
     if (profile && !profile->isSuitableForOutput()) {
-        transform = KoColorSpaceRegistry::instance()->colorSpace(csName, profile)->createColorConverter(cs);
+        transform = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, profile)->createColorConverter(cs);
     }
 
     // Creating the KisImageWSP
-    if (m_img == 0) {
-        m_img = new KisImage(m_adapter, width, height, cs, "built image");
-        Q_CHECK_PTR(m_img);
-        m_img->lock();
+    if (m_image == 0) {
+        m_image = new KisImage(m_adapter, width, height, cs, "built image");
+        Q_CHECK_PTR(m_image);
+        m_image->lock();
         if (profile && !profile->isSuitableForOutput()) {
             KisAnnotationSP annotation;
-            // XXX we hardcode icc, this is correct for lcms?
-            // XXX productName(), or just "ICC Profile"?
-            KoIccColorProfile* iccprofile = dynamic_cast<KoIccColorProfile*>(profile);
-            if (iccprofile && !iccprofile->rawData().isEmpty())
-                annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
-            m_img -> addAnnotation(annotation);
+            if (profile->type() == "icc" && !profile->rawData().isEmpty())
+                annotation = new  KisAnnotation("icc", profile->name(), profile->rawData());
+            m_image -> addAnnotation(annotation);
         }
     }
-    
+
     // Read resolution
     int unit_type;
     png_uint_32 x_resolution, y_resolution;
-    
-    png_get_pHYs(png_ptr, info_ptr,&x_resolution,&y_resolution, &unit_type);
-    if (unit_type == PNG_RESOLUTION_METER)
-    {
-        m_img->setResolution((double) POINT_TO_CM(x_resolution)/100.0, (double) POINT_TO_CM(y_resolution)/100.0 );
+
+    png_get_pHYs(png_ptr, info_ptr, &x_resolution, &y_resolution, &unit_type);
+    if (unit_type == PNG_RESOLUTION_METER) {
+        m_image->setResolution((double) POINT_TO_CM(x_resolution) / 100.0, (double) POINT_TO_CM(y_resolution) / 100.0); // It is the "invert" macro because we convert from pointer-per-inchs to points
     }
 
     double coeff = quint8_MAX / (double)(pow(2, color_nb_bits) - 1);
-    KisPaintLayerSP layer = new KisPaintLayer(m_img.data(), m_img -> nextLayerName(), UCHAR_MAX);
+    KisPaintLayerSP layer = new KisPaintLayer(m_image.data(), m_image -> nextLayerName(), UCHAR_MAX);
+    KisTransaction("", layer -> paintDevice());
+
     // Read comments/texts...
     png_text* text_ptr;
     int num_comments;
@@ -708,7 +724,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
             return KisImageBuilder_RESULT_UNSUPPORTED;
         }
     }
-    m_img->addNode(layer.data(), m_img->rootLayer().data());
+    m_image->addNode(layer.data(), m_image->rootLayer().data());
 
     png_read_end(png_ptr, end_info);
     iod->close();
@@ -717,7 +733,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
 
     delete reader;
-    m_img->unlock();
+    m_image->unlock();
     return KisImageBuilder_RESULT_OK;
 
 }
@@ -728,7 +744,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const KUrl& uri)
     if (uri.isEmpty())
         return KisImageBuilder_RESULT_NO_URI;
 
-    if (!KIO::NetAccess::exists(uri, false, qApp -> activeWindow())) {
+    if (!KIO::NetAccess::exists(uri, KIO::NetAccess::SourceSide, qApp -> activeWindow())) {
         return KisImageBuilder_RESULT_NOT_EXIST;
     }
 
@@ -759,11 +775,11 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const KUrl& uri)
 
 KisImageWSP KisPNGConverter::image()
 {
-    return m_img;
+    return m_image;
 }
 
 
-KisImageBuilder_Result KisPNGConverter::buildFile(const KUrl& uri, KisImageWSP img, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
+KisImageBuilder_Result KisPNGConverter::buildFile(const KUrl& uri, KisImageWSP image, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
 {
     dbgFile << "Start writing PNG File " << uri;
     if (uri.isEmpty())
@@ -773,14 +789,14 @@ KisImageBuilder_Result KisPNGConverter::buildFile(const KUrl& uri, KisImageWSP i
         return KisImageBuilder_RESULT_NOT_LOCAL;
     // Open a QIODevice for writing
     QFile *fp = new QFile(uri.toLocalFile());
-    KisImageBuilder_Result result = buildFile(fp, img, device, annotationsStart, annotationsEnd, options, metaData);
+    KisImageBuilder_Result result = buildFile(fp, image, device, annotationsStart, annotationsEnd, options, metaData);
     delete fp;
     return result;
 // TODO: if failure do            KIO::del(uri); // async
 
 }
 
-KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageWSP img, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
+KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageWSP image, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
 {
     if (!iodevice->open(QIODevice::WriteOnly)) {
         dbgFile << "Failed to open PNG File for writing";
@@ -790,12 +806,12 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageW
     if (!device)
         return KisImageBuilder_RESULT_INVALID_ARG;
 
-    if (!img)
+    if (!image)
         return KisImageBuilder_RESULT_EMPTY;
 
     // Setup the writing callback of libpng
-    int height = img->height();
-    int width = img->width();
+    int height = image->height();
+    int width = image->width();
     // Initialize structures
     png_structp png_ptr =  png_create_write_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
     if (!png_ptr) {
@@ -846,7 +862,7 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageW
     int num_palette = 0;
     if (!options.alpha && options.tryToSaveAsIndexed && KoID(device->colorSpace()->id()) == KoID("RGBA")) { // png doesn't handle indexed images and alpha, and only have indexed for RGB8
         palette = new png_color[255];
-        KisRectConstIteratorPixel it = device->createRectConstIterator(0, 0, img->width(), img->height());
+        KisRectConstIteratorPixel it = device->createRectConstIterator(0, 0, image->width(), image->height());
         bool toomuchcolor = false;
         while (!it.isDone()) {
             const quint8* c = it.rawData();
@@ -915,8 +931,8 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageW
 
         if ((*it) -> type().startsWith(QString("krita_attribute:"))) { //
             // Attribute
-#ifdef __GNUC__                                                 \
-    #warning "it should be possible to save krita_attributes in the \"CHUNKs\""
+#ifdef __GNUC__
+#warning "it should be possible to save krita_attributes in the \"CHUNKs\""
 #endif
             dbgFile << "cannot save this annotation : " << (*it) -> type();
         } else { // Profile
@@ -994,13 +1010,13 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageW
         }
 #endif
     }
-
+#if 0 // Unimplemented?
     // Save resolution
     int unit_type;
     png_uint_32 x_resolution, y_resolution;
-    
-    png_set_pHYs(png_ptr, info_ptr, CM_TO_POINT(img->xRes()) * 100.0, CM_TO_POINT(img->yRes()) * 100.0, PNG_RESOLUTION_METER);
-    
+#endif
+    png_set_pHYs(png_ptr, info_ptr, CM_TO_POINT(image->xRes()) * 100.0, CM_TO_POINT(image->yRes()) * 100.0, PNG_RESOLUTION_METER); // It is the "invert" macro because we convert from pointer-per-inchs to points
+
     // Save the information to the file
     png_write_info(png_ptr, info_ptr);
     png_write_flush(png_ptr);
