@@ -21,8 +21,6 @@
  */
 
 #include "kis_selection_manager.h"
-#include "dialogs/kis_dlg_apply_profile.h" // If this isn't first, I get
-// errors in qslider.h. Weird... (BSAR)
 #include <QApplication>
 #include <QClipboard>
 #include <QColor>
@@ -34,11 +32,12 @@
 #include <kstandardaction.h>
 #include <kactioncollection.h>
 
+#include "KoCanvasController.h"
 #include "KoChannelInfo.h"
 #include "KoIntegerMaths.h"
 #include <KoDocument.h>
 #include <KoMainWindow.h>
-#include <KoQueryTrader.h>
+#include <KoDocumentEntry.h>
 #include <KoViewConverter.h>
 #include <KoSelection.h>
 #include <KoShapeManager.h>
@@ -100,7 +99,6 @@ KisSelectionManager::KisSelectionManager(KisView2 * view, KisDoc2 * doc)
         m_reselect(0),
         m_invert(0),
         m_toNewLayer(0),
-        m_feather(0),
         m_smooth(0),
         m_load(0),
         m_save(0),
@@ -114,10 +112,10 @@ KisSelectionManager::KisSelectionManager(KisView2 * view, KisDoc2 * doc)
     Q_ASSERT(selection);
     connect(selection, SIGNAL(selectionChanged()), this, SLOT(shapeSelectionChanged()));
 
-    KisSelectionDecoration* decoration = new KisSelectionDecoration( m_view );
+    KisSelectionDecoration* decoration = new KisSelectionDecoration(m_view);
     connect(this, SIGNAL(currentSelectionChanged()), decoration, SLOT(selectionChanged()));
     decoration->setVisible(true);
-    m_view->canvasBase()->addDecoration( decoration );
+    m_view->canvasBase()->addDecoration(decoration);
 }
 
 KisSelectionManager::~KisSelectionManager()
@@ -138,14 +136,17 @@ void KisSelectionManager::setup(KActionCollection * collection)
     collection->addAction("paste_new", m_pasteNew);
     connect(m_pasteNew, SIGNAL(triggered()), this, SLOT(pasteNew()));
 
+    m_pasteAt = new KAction(i18n("Paste at cursor"), this);
+    collection->addAction("paste_at", m_pasteAt);
+    connect(m_pasteAt, SIGNAL(triggered()), this, SLOT(pasteAt()));
+
     m_selectAll = collection->addAction(KStandardAction::SelectAll,  "select_all", this, SLOT(selectAll()));
 
     m_deselect = collection->addAction(KStandardAction::Deselect,  "deselect", this, SLOT(deselect()));
 
-
     m_clear = collection->addAction(KStandardAction::Clear,  "clear", this, SLOT(clear()));
 
-    m_delete = new KAction(KIcon("edit-delete"), i18n("D&elete"), this);
+    m_delete = new KAction(KIcon("edit-delete"), i18n("D&elete Selection"), this);
     collection->addAction("delete", m_delete);
     m_delete->setShortcut(QKeySequence(Qt::Key_Delete));
     connect(m_delete, SIGNAL(triggered()), this, SLOT(deleteSelection()));
@@ -155,7 +156,7 @@ void KisSelectionManager::setup(KActionCollection * collection)
     m_reselect->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_D));
     connect(m_reselect, SIGNAL(triggered()), this, SLOT(reselect()));
 
-    m_invert  = new KAction(i18n("&Invert"), this);
+    m_invert  = new KAction(i18n("&Invert Selection"), this);
     collection->addAction("invert", m_invert);
     m_invert->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_I));
     connect(m_invert, SIGNAL(triggered()), this, SLOT(invert()));
@@ -170,11 +171,6 @@ void KisSelectionManager::setup(KActionCollection * collection)
     m_cutToNewLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_J));
     connect(m_cutToNewLayer, SIGNAL(triggered()), this, SLOT(cutToNewLayer()));
 
-    m_feather  = new KAction(i18n("Feather"), this);
-    collection->addAction("feather", m_feather);
-    m_feather->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_D));
-    connect(m_feather, SIGNAL(triggered()), this, SLOT(feather()));
-
     m_fillForegroundColor  = new KAction(i18n("Fill with Foreground Color"), this);
     collection->addAction("fill_selection_foreground_color", m_fillForegroundColor);
     m_fillForegroundColor->setShortcut(QKeySequence(Qt::ALT + Qt::Key_Backspace));
@@ -191,7 +187,7 @@ void KisSelectionManager::setup(KActionCollection * collection)
 
     m_toggleDisplaySelection  = new KToggleAction(i18n("Display Selection"), this);
     collection->addAction("toggle_display_selection", m_toggleDisplaySelection);
-    m_toggleDisplaySelection->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
+    m_toggleDisplaySelection->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_H));
     connect(m_toggleDisplaySelection, SIGNAL(triggered()), this, SLOT(toggleDisplaySelection()));
 
     m_toggleDisplaySelection->setCheckedState(KGuiItem(i18n("Hide Selection")));
@@ -247,18 +243,18 @@ void KisSelectionManager::updateGUI()
         return;
     }
 
-    KisImageSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP l;
     KisPaintDeviceSP dev;
 
     bool enable = false;
 
-    if (img && m_view->activeDevice() && m_view->activeLayer()) {
+    if (image && m_view->activeDevice() && m_view->activeLayer()) {
         l = m_view->activeLayer();
 
         enable = l && !l->userLocked() && l->visible();
 #if 0 // XXX_SELECTION (how are we going to handle deselect and
-        // reselectt now?
+        // reselect now?
         if (l->inherits("KisAdjustmentLayer")
                 if (dev && !adjLayer)
                     m_reselect->setEnabled(dev->selectionDeselected());
@@ -274,12 +270,11 @@ void KisSelectionManager::updateGUI()
     m_fillPattern->setEnabled(enable);
 
     m_cutToNewLayer->setEnabled(enable && l->selection());
-    m_selectAll->setEnabled(!img.isNull());
+    m_selectAll->setEnabled(!image.isNull());
 
     bool hasPixelSelection = enable && l->selection() && l->selection()->hasPixelSelection()
                              && !m_view->selection()->isDeselected();
     m_invert->setEnabled(hasPixelSelection);
-    m_feather->setEnabled(hasPixelSelection);
 
     m_smooth->setEnabled(enable);
 //    m_load->setEnabled(enable);
@@ -300,7 +295,7 @@ void KisSelectionManager::updateGUI()
         QListIterator<QAction *> i(m_pluginActions);
 
         while (i.hasNext()) {
-            i.next()->setEnabled(!img.isNull());
+            i.next()->setEnabled(!image.isNull());
         }
     }
 
@@ -308,7 +303,7 @@ void KisSelectionManager::updateGUI()
     // the current layer is locked.
 
     enable = false;
-    if (img && l) {
+    if (image && l) {
         enable = l->selection() && l->visible();
 
 
@@ -336,8 +331,8 @@ void KisSelectionManager::updateGUI()
     }
 
     m_copy->setEnabled(enable || shapeCopyEnable);
-    m_paste->setEnabled(!img.isNull() && (m_clipboard->hasClip() || shapePasteEnable));
-    m_pasteNew->setEnabled(!img.isNull() && m_clipboard->hasClip());
+    m_paste->setEnabled(!image.isNull() && (m_clipboard->hasClip() || shapePasteEnable));
+    m_pasteNew->setEnabled(!image.isNull() && m_clipboard->hasClip());
     m_toNewLayer->setEnabled(enable);
 
     if (shapeLayer || ((m_view->selection()
@@ -369,8 +364,8 @@ void KisSelectionManager::selectionChanged()
 
 void KisSelectionManager::cut()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
@@ -383,10 +378,11 @@ void KisSelectionManager::cut()
     Q_CHECK_PTR(t);
 
     layer->paintDevice()->clearSelection(m_view->selection());
-#if 0 // XXX_SELECTION
-    dev->deselect();
-#endif
+    QRect rect = m_view->selection()->selectedRect();
+    deselect();
+
     m_view->document()->addCommand(t);
+    layer->setDirty(rect);
 }
 
 void KisSelectionManager::copy()
@@ -399,17 +395,17 @@ void KisSelectionManager::copy()
         m_view->canvasBase()->toolProxy()->copy();
     } else {
 
-        KisImageSP img = m_view->image();
-        if (!img) return;
+        KisImageWSP image = m_view->image();
+        if (!image) return;
 
         KisPaintDeviceSP dev = m_view->activeDevice();
         if (!dev) return;
 
         KisSelectionSP selection = m_view->selection();
 
-        QRect r = (selection) ? selection->selectedExactRect() : img->bounds();
+        QRect r = (selection) ? selection->selectedExactRect() : image->bounds();
 
-        KisPaintDeviceSP clip = new KisPaintDevice(dev->colorSpace(), "clip");
+        KisPaintDeviceSP clip = new KisPaintDevice(dev->colorSpace());
         Q_CHECK_PTR(clip);
 
         const KoColorSpace * cs = clip->colorSpace();
@@ -419,7 +415,8 @@ void KisSelectionManager::copy()
         // Copy image data
         KisPainter gc;
         gc.begin(clip);
-        gc.bitBlt(0, 0, COMPOSITE_COPY, dev, r.x(), r.y(), r.width(), r.height());
+        gc.setCompositeOp(COMPOSITE_COPY);
+        gc.bitBlt(0, 0, dev, r.x(), r.y(), r.width(), r.height());
         gc.end();
 
         if (selection) {
@@ -443,7 +440,7 @@ void KisSelectionManager::copy()
             }
         }
 
-        m_clipboard->setClip(clip);
+        m_clipboard->setClip(clip, r.topLeft());
     }
 
     selectionChanged();
@@ -452,54 +449,49 @@ void KisSelectionManager::copy()
 
 void KisSelectionManager::paste()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
+    
+    //figure out where to position the clip
+    // XXX: Fix this for internal points & zoom! (BSAR)
+    QWidget * w = m_view->canvas();
+    QPoint center = QPoint(w->width() / 2, w->height() / 2);
+    QPoint bottomright = QPoint(w->width(), w->height());
+    if (bottomright.x() > image->width())
+        center.setX(image->width() / 2);
+    if (bottomright.y() > image->height())
+        center.setY(image->height() / 2);
 
-    KisPaintDeviceSP clip = m_clipboard->clip();
+    const KoCanvasBase* canvasBase = m_view->canvasBase();
+    const KoViewConverter* viewConverter = m_view->canvasBase()->viewConverter();
+
+    KisPaintDeviceSP clip = m_clipboard->clip(
+        QPoint(
+            viewConverter->viewToDocumentX(canvasBase->canvasController()->canvasOffsetX()) + center.x(),
+            viewConverter->viewToDocumentY(canvasBase->canvasController()->canvasOffsetY()) + center.y()));
 
     if (clip) {
-        KisPaintLayer *layer = new KisPaintLayer(img.data(), img->nextLayerName() + i18n("(pasted)"), OPACITY_OPAQUE);
+        KisPaintLayer *layer = new KisPaintLayer(image.data(), image->nextLayerName() + i18n("(pasted)"), OPACITY_OPAQUE_U8, clip);
         Q_CHECK_PTR(layer);
 
-        QRect r = clip->exactBounds();
-        KisPainter gc;
-        gc.begin(layer->paintDevice());
-        gc.bitBlt(0, 0, COMPOSITE_COPY, clip, r.x(), r.y(), r.width(), r.height());
-        gc.end();
-
-        //figure out where to position the clip
-        // XXX: Fix this for internal points & zoom! (BSAR)
-        QWidget * w = m_view->canvas();
-        QPoint center = QPoint(w->width() / 2, w->height() / 2);
-        QPoint bottomright = QPoint(w->width(), w->height());
-        if (bottomright.x() > img->width())
-            center.setX(img->width() / 2);
-        if (bottomright.y() > img->height())
-            center.setY(img->height() / 2);
-        center -= QPoint(r.width() / 2, r.height() / 2);
-        layer->setX(center.x());
-        layer->setY(center.y());
-
-        /*XXX CBR have an idea of asking the user if he is about to paste a clip in another cs than that of
-          the image if that is what he want rather than silently converting
-          if ( ! ( *clip->colorSpace == *img ->colorSpace()) )
-          if (dlg->exec() == QDialog::Accepted)
-          layer->convertTo(img->colorSpace());
-        */
-        if( m_view->activeLayer() )
-        {
+        if (m_view->activeLayer()) {
             m_adapter->addNode(layer , m_view->activeLayer()->parent(), m_view->activeLayer().data());
         } else {
-            m_adapter->addNode(layer , img->rootLayer(), 0 );
+            m_adapter->addNode(layer , image->rootLayer(), 0);
         }
         layer->setDirty();
     } else
         m_view->canvasBase()->toolProxy()->paste();
 }
 
+void KisSelectionManager::pasteAt()
+{
+    //XXX
+}
+
 void KisSelectionManager::pasteNew()
 {
-    KisPaintDeviceSP clip = m_clipboard->clip();
+    KisPaintDeviceSP clip = m_clipboard->clip(QPoint(0,0));
     if (!clip) return;
 
     QRect r = clip->exactBounds();
@@ -511,22 +503,23 @@ void KisSelectionManager::pasteNew()
     const QByteArray mimetype = KoDocument::readNativeFormatMimeType();
     KoDocumentEntry entry = KoDocumentEntry::queryByMimeType(mimetype);
 
-    KisDoc2 * doc = dynamic_cast<KisDoc2*>(entry.createDoc());
+    KisDoc2* doc = dynamic_cast<KisDoc2*>(entry.createDoc());
     if (!doc) return;
 
     Q_ASSERT(doc->undoAdapter() != 0);
     doc->undoAdapter()->setUndo(false);
 
-    KisImageSP img = new KisImage(doc->undoAdapter(), r.width(), r.height(),
-                                  KoColorSpaceRegistry::instance()->colorSpace( clip->colorSpace()->id(), clip->colorSpace()->profile() ), "Pasted" ); // TODO should be translated ?
-    KisPaintLayerSP layer = new KisPaintLayer(img.data(), clip->objectName(), OPACITY_OPAQUE, clip->colorSpace());
+    KisImageWSP image = new KisImage(doc->undoAdapter(), r.width(), r.height(),
+                                   KoColorSpaceRegistry::instance()->colorSpace(clip->colorSpace()->colorModelId().id(), clip->colorSpace()->colorDepthId().id(), clip->colorSpace()->profile()), "Pasted");    // TODO should be translated ?
+    KisPaintLayerSP layer = new KisPaintLayer(image.data(), clip->objectName(), OPACITY_OPAQUE_U8, clip->colorSpace());
 
     KisPainter p(layer->paintDevice());
-    p.bitBlt(0, 0, COMPOSITE_COPY, clip, OPACITY_OPAQUE, r.x(), r.y(), r.width(), r.height());
+    p.setCompositeOp(COMPOSITE_COPY);
+    p.bitBlt(0, 0, clip, r.x(), r.y(), r.width(), r.height());
     p.end();
 
-    img->addNode(layer.data(), img->rootLayer());
-    doc->setCurrentImage(img);
+    image->addNode(layer.data(), image->rootLayer());
+    doc->setCurrentImage(image);
 
     doc->undoAdapter()->setUndo(true);
 
@@ -537,18 +530,18 @@ void KisSelectionManager::pasteNew()
 
 void KisSelectionManager::selectAll()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     QUndoCommand* selectionCmd = new QUndoCommand(i18n("Select All"));
 
-    if (!m_view->selection())
-        new KisSetGlobalSelectionCommand(img, selectionCmd);
-    KisSelectionSP selection = m_view->selection();
+    if (!image->globalSelection())
+        new KisSetGlobalSelectionCommand(image, selectionCmd);
+    KisSelectionSP selection = image->globalSelection();
 
-    new KisSelectionTransaction(QString(), img, selection, selectionCmd);
+    new KisSelectionTransaction(QString(), image, selection, selectionCmd);
 
-    selection->getOrCreatePixelSelection()->select(img->bounds());
+    selection->getOrCreatePixelSelection()->select(image->bounds());
 
     m_view->selectionManager()->selectionChanged();
     m_view->document()->addCommand(selectionCmd);
@@ -556,8 +549,8 @@ void KisSelectionManager::selectAll()
 
 void KisSelectionManager::deselect()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP sel = m_view->selection();
     if (!sel) return;
@@ -565,22 +558,17 @@ void KisSelectionManager::deselect()
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
 
-    if (layer->selectionMask()) {
-        KisDeselectLocalSelectionCommand* cmd = new KisDeselectLocalSelectionCommand(img, layer->selectionMask());
+    if (image->globalSelection()) {
+        KisDeselectGlobalSelectionCommand* cmd = new KisDeselectGlobalSelectionCommand(image);
         m_view->document()->addCommand(cmd);
-    } else {
-        if (img->globalSelection()) {
-            KisDeselectGlobalSelectionCommand* cmd = new KisDeselectGlobalSelectionCommand(img);
-            m_view->document()->addCommand(cmd);
-        }
     }
 }
 
 
 void KisSelectionManager::clear()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
     if (!dev) return;
@@ -596,8 +584,8 @@ void KisSelectionManager::clear()
         dev->setDirty();
     }
 
-    dev->setDirty(img->bounds());
-    img->undoAdapter()->addCommand(t);
+    dev->setDirty(image->bounds());
+    image->undoAdapter()->addCommand(t);
 }
 
 void KisSelectionManager::deleteSelection()
@@ -609,8 +597,8 @@ void KisSelectionManager::deleteSelection()
 
 void KisSelectionManager::fill(const KoColor& color, bool fillWithPattern, const QString& transactionText)
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
     if (!dev) return;
@@ -622,24 +610,24 @@ void KisSelectionManager::fill(const KoColor& color, bool fillWithPattern, const
     KisFillPainter painter(filled);
 
     if (fillWithPattern) {
-        painter.fillRect(0, 0, img->width(), img->height(),
+        painter.fillRect(0, 0, image->width(), image->height(),
                          m_view->resourceProvider()->currentPattern());
     } else {
-        painter.fillRect(0, 0, img->width(), img->height(), color);
+        painter.fillRect(0, 0, image->width(), image->height(), color);
     }
 
     painter.end();
 
     KisPainter painter2(dev, selection);
 
-    if (img->undo()) painter2.beginTransaction(transactionText);
-    painter2.bltSelection(0, 0, COMPOSITE_OVER, filled, OPACITY_OPAQUE,
-                          0, 0, img->width(), img->height());
+    if (image->undo()) painter2.beginTransaction(transactionText);
+
+    painter2.bitBlt(0, 0, filled, 0, 0, image->width(), image->height());
 
     dev->setDirty();
 
-    if (img->undo()) {
-        img->undoAdapter()->addCommand(painter2.endTransaction());
+    if (image->undo()) {
+        image->undoAdapter()->addCommand(painter2.endTransaction());
     }
 }
 
@@ -660,39 +648,34 @@ void KisSelectionManager::fillPattern()
 
 void KisSelectionManager::reselect()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
 
-    if (layer->selectionMask() && layer->selectionMask()->deleselectedSelection()) {
-        KisReselectLocalSelectionCommand* cmd = new KisReselectLocalSelectionCommand(img, layer->selectionMask());
+    if (image->globalSelection()) {
+        KisReselectGlobalSelectionCommand* cmd = new KisReselectGlobalSelectionCommand(image);
         m_view->document()->addCommand(cmd);
-    } else {
-        if (img->globalSelection()) {
-            KisReselectGlobalSelectionCommand* cmd = new KisReselectGlobalSelectionCommand(img);
-            m_view->document()->addCommand(cmd);
-        }
     }
 }
 
 
 void KisSelectionManager::invert()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP selection = m_view->selection();
     if (!selection) return;
 
     KisPixelSelectionSP s = selection->getOrCreatePixelSelection();
 
-    KisSelectionTransaction * t = new KisSelectionTransaction(i18n("Invert"), img, selection);
+    KisSelectionTransaction * t = new KisSelectionTransaction(i18n("Invert"), image, selection);
     Q_CHECK_PTR(t);
 
     s->invert();
-    s->setDirty(img->bounds());
+    s->setDirty(image->bounds());
 
     selectionChanged();
     m_view->document()->addCommand(t);
@@ -700,8 +683,8 @@ void KisSelectionManager::invert()
 
 void KisSelectionManager::copySelectionToNewLayer()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
@@ -712,8 +695,8 @@ void KisSelectionManager::copySelectionToNewLayer()
 
 void KisSelectionManager::cutToNewLayer()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
     if (!dev) return;
@@ -722,54 +705,10 @@ void KisSelectionManager::cutToNewLayer()
     paste();
 }
 
-
-void KisSelectionManager::feather()
-{
-    KisImageSP img = m_view->image();
-    if (!img) return;
-    if (!m_view->selection())
-        return;
-
-    KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
-    KisSelectionTransaction * t = new KisSelectionTransaction(i18n("Feather..."), img, m_view->selection());
-    Q_CHECK_PTR(t);
-
-
-    // XXX: we should let gaussian blur & others influence alpha channels as well
-    // (on demand of the caller)
-
-    KisConvolutionPainter painter(selection);
-
-    KisConvolutionKernelSP k = new KisConvolutionKernel(3, 3, 0, 16);
-
-    k->data()[0] = 1;
-    k->data()[1] = 2;
-    k->data()[2] = 1;
-    k->data()[3] = 2;
-    k->data()[4] = 4;
-    k->data()[5] = 2;
-    k->data()[6] = 1;
-    k->data()[7] = 2;
-    k->data()[8] = 1;
-
-    QRect rect = selection->selectedExactRect();
-    // Make sure we've got enough space around the edges.
-    rect = QRect(rect.x() - 3, rect.y() - 3, rect.width() + 6, rect.height() + 6);
-    rect &= QRect(0, 0, img->width(), img->height());
-    painter.setChannelFlags(selection->colorSpace()->channelFlags(false, true, false, false));
-    painter.applyMatrix(k, selection, rect.x(), rect.y(), rect.width(), rect.height(), BORDER_AVOID);
-    painter.end();
-
-    m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
-    selectionChanged();
-
-}
-
 void KisSelectionManager::toggleDisplaySelection()
 {
     KisCanvasDecoration* decoration = m_view->canvasBase()->decoration("selection");
-    if(decoration)
+    if (decoration)
         decoration->toggleVisibility();
 }
 
@@ -783,14 +722,14 @@ bool KisSelectionManager::displaySelection()
 
 void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
 
     //determine the layerSize
-    QRect layerSize = img->bounds();
+    QRect layerSize = image->bounds();
 
     /*
       Any bugs in this function are probably also in thin_region
@@ -802,14 +741,14 @@ void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
     if (xradius <= 0 || yradius <= 0)
         return;
 
-    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Grow"), img, m_view->selection());
+    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Grow Selection"), image, m_view->selection());
 
     max = new quint8* [layerSize.width() + 2 * xradius];
     buf = new quint8* [yradius + 1];
     for (qint32 i = 0; i < yradius + 1; i++) {
         buf[i] = new quint8[layerSize.width()];
     }
-    quint8* buffer = new quint8[(layerSize.width() + 2 * xradius) * (yradius + 1)];
+    quint8* buffer = new quint8[(layerSize.width() + 2 * xradius) *(yradius + 1)];
     for (qint32 i = 0; i < layerSize.width() + 2 * xradius; i++) {
         if (i < xradius)
             max[i] = buffer;
@@ -901,24 +840,24 @@ void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
     delete[] out;
 
     m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
+    m_view->selection()->setDirty(image->bounds());
     selectionChanged();
 }
 
 void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
 {
 
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
 
-    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Shrink"), img, m_view->selection());
+    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Shrink Selection"), image, m_view->selection());
     Q_CHECK_PTR(t);
 
     //determine the layerSize
-    QRect layerSize = img->bounds();
+    QRect layerSize = image->bounds();
     /*
       pretty much the same as fatten_region only different
       blame all bugs in this function on jaycox@gimp.org
@@ -956,11 +895,10 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
                 max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius)];
         else if (i < layerSize.width() + xradius)
             max[i] = &buffer[(yradius + 1) * (i - xradius)];
+        else if (edge_lock)
+            max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius - 1)];
         else
-            if (edge_lock)
-                max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius - 1)];
-            else
-                max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius)];
+            max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius)];
     }
     if (!edge_lock)
         for (qint32 j = 0 ; j < xradius + 1; j++) max[0][j] = 0;
@@ -1052,7 +990,7 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
     delete[] out;
 
     m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
+    m_view->selection()->setDirty(image->bounds());
     selectionChanged();
 }
 
@@ -1060,8 +998,8 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
 
 void KisSelectionManager::smooth()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     if (!m_view->activeLayer()) return;
@@ -1114,7 +1052,7 @@ void KisSelectionManager::smooth()
 
     delete[] out;
 #if 0
-    dev->setDirty(img->bounds());
+    dev->setDirty(image->bounds());
 #endif
 }
 
@@ -1122,8 +1060,8 @@ void KisSelectionManager::smooth()
 
 void KisSelectionManager::erode()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP selection = m_view->selection();
     if (!selection) return;
@@ -1190,8 +1128,8 @@ void KisSelectionManager::erode()
 
 void KisSelectionManager::dilate()
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP selection = m_view->selection();
     if (!selection) return;
@@ -1255,16 +1193,16 @@ void KisSelectionManager::dilate()
 
 void KisSelectionManager::border(qint32 xradius, qint32 yradius)
 {
-    KisImageSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
 
     //determine the layerSize
-    QRect layerSize = img->bounds();
+    QRect layerSize = image->bounds();
 
-    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Border"), img, m_view->selection());
+    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Border Selection"), image, m_view->selection());
     Q_CHECK_PTR(t);
 
     quint8  *buf[3];
@@ -1304,7 +1242,7 @@ void KisSelectionManager::border(qint32 xradius, qint32 yradius)
         delete[] transition;
 
         m_view->document()->addCommand(t);
-        m_view->selection()->setDirty(img->bounds());
+        m_view->selection()->setDirty(image->bounds());
         selectionChanged();
         return;
     }
@@ -1399,13 +1337,12 @@ void KisSelectionManager::border(qint32 xradius, qint32 yradius)
                         max[x] = yradius;
                     else
                         max[x]--;
-                } else
-                    if (transition[-max[x]][x])
-                        max[x] = -max[x];
-                    else if (transition[-max[x] + 1][x])
-                        max[x] = -max[x] + 1;
-                    else
-                        max[x]--;
+                } else if (transition[-max[x]][x])
+                    max[x] = -max[x];
+                else if (transition[-max[x] + 1][x])
+                    max[x] = -max[x] + 1;
+                else
+                    max[x]--;
             } else
                 max[x]--;
             if (max[x] < -yradius - 1)
@@ -1471,7 +1408,55 @@ void KisSelectionManager::border(qint32 xradius, qint32 yradius)
 
 
     m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
+    m_view->selection()->setDirty(image->bounds());
+    selectionChanged();
+}
+
+void KisSelectionManager::feather(qint32 radius)
+{
+    KisImageWSP image = m_view->image();
+    if (!image) return;
+    if (!m_view->selection())
+        return;
+
+    KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
+    KisSelectionTransaction * t = new KisSelectionTransaction(i18n("Feather Selection"), image, m_view->selection());
+    Q_CHECK_PTR(t);
+
+    // compute horizontal kernel
+    const uint kernelSize = radius * 2 + 1;
+    Matrix<qreal, Dynamic, Dynamic> gaussianMatrix(1, kernelSize);
+
+    const qreal multiplicand = 1 / (2 * M_PI * radius * radius);
+    const qreal exponentMultiplicand = 1 / (2 * radius * radius);
+
+    for (uint x = 0; x < kernelSize; x++)
+    {
+        uint xDistance = qAbs((int)radius - (int)x);
+        gaussianMatrix(0, x) = multiplicand * exp( -(qreal)((xDistance * xDistance) + (radius * radius)) * exponentMultiplicand );
+    }
+
+    KisConvolutionKernelSP kernelHoriz = KisConvolutionKernel::fromMatrix(gaussianMatrix, 0, gaussianMatrix.sum());
+    KisConvolutionKernelSP kernelVertical = KisConvolutionKernel::fromMatrix(gaussianMatrix.transpose(), 0, gaussianMatrix.sum());
+
+    QRect rect = selection->selectedExactRect();
+    // Make sure we've got enough space around the edges.
+    rect.adjust(-kernelSize, -kernelSize, kernelSize, kernelSize);
+    rect &= QRect(0, 0, image->width(), image->height());
+
+    KisPaintDeviceSP interm = new KisPaintDevice(selection->colorSpace());
+    KisConvolutionPainter horizPainter(interm);
+    horizPainter.setChannelFlags(interm->colorSpace()->channelFlags(false, true, false, false));
+    horizPainter.applyMatrix(kernelHoriz, selection, rect.topLeft(), rect.topLeft(), rect.size(), BORDER_AVOID);
+    horizPainter.end();
+
+    KisConvolutionPainter verticalPainter(selection);
+    verticalPainter.setChannelFlags(selection->colorSpace()->channelFlags(false, true, false, false));
+    verticalPainter.applyMatrix(kernelVertical, interm, rect.topLeft(), rect.topLeft(), rect.size(), BORDER_AVOID);
+    verticalPainter.end();
+
+    m_view->document()->addCommand(t);
+    m_view->selection()->setDirty(image->bounds());
     selectionChanged();
 }
 
