@@ -62,7 +62,7 @@
 // KOffice
 #include <KoCanvasBase.h>
 #include <KoCanvasController.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 #include <KoViewConverter.h>
 
 // KDE
@@ -114,10 +114,10 @@ void CellToolBase::Private::updateLocationComboBox()
 }
 
 #define ACTION_EXEC( name, command ) { \
-    QAction *a = q->action(name); \
-    const bool blocked = a->blockSignals(true); \
-    a->command; \
-    a->blockSignals(blocked); \
+        QAction *a = q->action(name); \
+        const bool blocked = a->blockSignals(true); \
+        a->command; \
+        a->blockSignals(blocked); \
     }
 
 void CellToolBase::Private::updateActions(const Cell& cell)
@@ -129,18 +129,8 @@ void CellToolBase::Private::updateActions(const Cell& cell)
     ACTION_EXEC("italic", setChecked(style.italic()));
     ACTION_EXEC("underline", setChecked(style.underline()));
     ACTION_EXEC("strikeOut", setChecked(style.strikeOut()));
-    // workaround for bug #59291 (crash upon starting from template)
-    // certain Qt and Fontconfig combination fail miserably if can not
-    // find the font name (e.g. not installed in the system)
-    QStringList fontList;
-    KFontChooser::getFontList(fontList, 0);
-    QString fontFamily = style.fontFamily();
-    for (QStringList::Iterator it = fontList.begin(); it != fontList.end(); ++it) {
-        if ((*it).toLower() == fontFamily.toLower()) {
-            static_cast<KFontAction*>(q->action("font"))->setFont(fontFamily);
-            break;
-        }
-    }
+
+    static_cast<KFontAction*>(q->action("font"))->setFont(style.fontFamily());
     static_cast<KFontSizeAction*>(q->action("fontSize"))->setFontSize(style.fontSize());
     // -- horizontal alignment actions --
     ACTION_EXEC("alignLeft", setChecked(style.halign() == Style::Left));
@@ -846,10 +836,10 @@ bool CellToolBase::Private::formatKeyPress(QKeyEvent * _ev)
 
     case Qt::Key_Ampersand:
         command->setText(i18n("Change Border"));
-        command->setTopBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setBottomBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setLeftBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
-        command->setRightBorderPen(QPen(q->canvas()->resourceProvider()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setTopBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setBottomBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setLeftBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
+        command->setRightBorderPen(QPen(q->canvas()->resourceManager()->foregroundColor().toQColor(), 1, Qt::SolidLine));
         break;
 
     default:
@@ -1189,7 +1179,7 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
             actions.append(popupMenuActions["separator3"]);
             actions.append(popupMenuActions["insertCell"]);
             actions.append(popupMenuActions["deleteCell"]);
-        } else if (q->selection()->isColumnSelected() && q->selection()->isSingular()) {
+        } else if (q->selection()->isColumnSelected()) {
             actions.append(q->action("resizeCol"));
             actions.append(popupMenuActions["adjustColumn"]);
             actions.append(popupMenuActions["separator4"]);
@@ -1226,7 +1216,7 @@ QList<QAction*> CellToolBase::Private::popupActionList() const
                     break;
                 }
             }
-        } else if (q->selection()->isRowSelected() && q->selection()->isSingular()) {
+        } else if (q->selection()->isRowSelected()) {
             actions.append(q->action("resizeRow"));
             actions.append(popupMenuActions["adjustRow"]);
             actions.append(popupMenuActions["separator5"]);
@@ -1325,14 +1315,15 @@ void CellToolBase::Private::createPopupMenuActions()
     popupMenuActions.insert("listChoose", action);
 }
 
-void CellToolBase::Private::relayoutDocker (bool wide) {
+void CellToolBase::Private::relayoutDocker(bool wide)
+{
     // user input is moved accordingly to the "wide" param, the rest stays in one place
     if (userInput->hasFocus()) return;  // do nothing while the user input has focus
-    widgetLayout->removeWidget (userInput);
+    widgetLayout->removeWidget(userInput);
     if (wide)
-      widgetLayout->addWidget (userInput, 0, 3);
+        widgetLayout->addWidget(userInput, 0, 3);
     else
-      widgetLayout->addWidget (userInput, 1, 0, 1, 5);
+        widgetLayout->addWidget(userInput, 1, 0, 1, 5);
     hasWideLayout = wide;
 }
 
