@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2002-2006 David Faure <faure@kde.org>
- * Copyright (C) 2005-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2005-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Pierre Ducroquet <pinaraf@pinaraf.info>
  * Copyright (C) 2008 Sebastian Sauer <mail@dipe.org>
@@ -23,6 +23,7 @@
 
 // kword includes
 #include "KWDocument.h"
+#include "KWDocument_p.h"
 #include "KWFactory.h"
 #include "KWView.h"
 #include "KWCanvas.h"
@@ -50,8 +51,9 @@
 #include <KoOdfWriteStore.h>
 #include <KoToolManager.h>
 #include <KoShapeRegistry.h>
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 #include <KoStyleManager.h>
+#include <KoResourceManager.h>
 #include <KoInteractionTool.h>
 #include <KoInlineTextObjectManager.h>
 #include <KoDocumentInfo.h>
@@ -59,9 +61,13 @@
 #include <KoParagraphStyle.h>
 #include <KoListStyle.h>
 #include <KoListLevelProperties.h>
-#include <KoDataCenter.h>
 #include <KoTextShapeData.h>
 
+#include <rdf/KoDocumentRdfBase.h>
+#ifdef SHOULD_BUILD_RDF
+#include <rdf/KoDocumentRdf.h>
+#endif
+
 // KDE + Qt includes
 #include <klocale.h>
 #include <kstandardaction.h>
@@ -124,16 +130,15 @@ void MagicCurtain::revealFramesForPage(int pageNumber, qreal moveFrames)
 
 
 // KWDocument
-KWDocument::KWDocument(QWidget *parentWidget, QObject* parent, bool singleViewMode)
+KWDocument::KWDocument(QWidget *parentWidget, QObject *parent, bool singleViewMode)
         : KoDocument(parentWidget, parent, singleViewMode),
         m_frameLayout(&m_pageManager, m_frameSets),
         m_magicCurtain(0),
         m_mainFramesetEverFinished(false)
 {
     m_frameLayout.setDocument(this);
+    resourceManager()->setOdfDocument(this);
 
-    connect(documentInfo(), SIGNAL(infoUpdated(const QString &, const QString &)),
-            inlineTextObjectManager(), SLOT(documentInformationUpdated(const QString &, const QString &)));
     setComponentData(KWFactory::componentData(), false);
     setTemplateType("kword_template");
 
@@ -141,14 +146,24 @@ KWDocument::KWDocument(QWidget *parentWidget, QObject* parent, bool singleViewMo
     connect(&m_frameLayout, SIGNAL(removedFrameSet(KWFrameSet*)), this, SLOT(removeFrameSet(KWFrameSet*)));
 
     // Init shape Factories with our frame based configuration panels.
-    // and ask every shapefactory to populate the dataCenterMap
-    QList<KoShapeConfigFactory *> panels = KWFrameDialog::panels(this);
+    QList<KoShapeConfigFactoryBase *> panels = KWFrameDialog::panels(this);
     foreach (const QString &id, KoShapeRegistry::instance()->keys()) {
-        KoShapeFactory *shapeFactory = KoShapeRegistry::instance()->value(id);
+        KoShapeFactoryBase *shapeFactory = KoShapeRegistry::instance()->value(id);
         shapeFactory->setOptionPanels(panels);
-        shapeFactory->populateDataCenterMap(m_dataCenterMap);
     }
 
+    resourceManager()->setUndoStack(undoStack());
+    if (documentRdfBase()) {
+        documentRdfBase()->linkToResourceManager(resourceManager());
+    }
+
+    QVariant variant;
+    variant.setValue(new KoChangeTracker(resourceManager()));
+    resourceManager()->setResource(KoText::ChangeTracker, variant);
+
+    connect(documentInfo(), SIGNAL(infoUpdated(const QString &, const QString &)),
+            inlineTextObjectManager(), SLOT(documentInformationUpdated(const QString &, const QString &)));
+
     m_config.load(this);
     clear();
 }
@@ -158,7 +173,6 @@ KWDocument::~KWDocument()
     delete m_magicCurtain;
     saveConfig();
     qDeleteAll(m_frameSets);
-    qDeleteAll(m_dataCenterMap);
 }
 
 void KWDocument::addShape(KoShape *shape)
@@ -173,7 +187,7 @@ void KWDocument::addShape(KoShape *shape)
         fs->setName(shape->shapeId());
         frame = new KWFrame(shape, fs);
     }
-    Q_ASSERT(frame->frameSet()); //sebsauer, 2008-08-16, seems this can happen on "Delete page" && 2x undo
+    Q_ASSERT(frame->frameSet());
     addFrameSet(frame->frameSet());
 
     foreach (KoView *view, views()) {
@@ -184,10 +198,6 @@ void KWDocument::addShape(KoShape *shape)
 
 void KWDocument::removeShape(KoShape *shape)
 {
-    foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
-        canvas->shapeManager()->remove(shape);
-    }
     KWFrame *frame = dynamic_cast<KWFrame*>(shape->applicationData());
     if (frame) { // not all shapes have to have a frame. Only top-level ones do.
         KWFrameSet *fs = frame->frameSet();
@@ -196,26 +206,31 @@ void KWDocument::removeShape(KoShape *shape)
             removeFrameSet(fs); // frame and frameset will be deleted when the shape is deleted
         else
             fs->removeFrame(frame);
+    } else { // not a frame, but we still have to remove it from views.
+        foreach (KoView *view, views()) {
+            KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
+            canvas->shapeManager()->remove(shape);
+        }
     }
 }
 
-void KWDocument::paintContent(QPainter&, const QRect& rect)
+void KWDocument::paintContent(QPainter&, const QRect &rect)
 {
     // TODO
 }
 
-KoView* KWDocument::createViewInstance(QWidget* parent)
+KoView *KWDocument::createViewInstance(QWidget *parent)
 {
     KWView *view = new KWView(m_viewMode, this, parent);
     if (m_magicCurtain)
-        view->kwcanvas()->shapeManager()->add(m_magicCurtain, false);
+        view->kwcanvas()->shapeManager()->add(m_magicCurtain, KoShapeManager::AddWithoutRepaint);
 
     bool switchToolCalled = false;
     foreach (KWFrameSet *fs, m_frameSets) {
         if (fs->frameCount() == 0)
             continue;
         foreach (KWFrame *frame, fs->frames())
-            view->kwcanvas()->shapeManager()->add(frame->shape(), false);
+            view->kwcanvas()->shapeManager()->add(frame->shape(), KoShapeManager::AddWithoutRepaint);
         if (switchToolCalled)
             continue;
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
@@ -258,7 +273,7 @@ void KWDocument::removePage(int pageNumber)
 
     KWPage page = m_pageManager.page(pageNumber);
     if (! page.isValid()) {
-        kWarning() << "remove page requested for a non exiting page!\n";
+        kWarning(32001) << "remove page requested for a non exiting page!";
         return;
     }
     addCommand(new KWPageRemoveCommand(this, page));
@@ -266,7 +281,8 @@ void KWDocument::removePage(int pageNumber)
 
 void KWDocument::firePageSetupChanged()
 {
-    inlineTextObjectManager()->setProperty(KoInlineObject::PageCount, pageCount());
+    if (inlineTextObjectManager())
+        inlineTextObjectManager()->setProperty(KoInlineObject::PageCount, pageCount());
     emit pageSetupChanged();
 }
 
@@ -278,7 +294,7 @@ void KWDocument::removeFrameSet(KWFrameSet *fs)
         removeFrame(frame);
     foreach (KoView *view, views()) {
         KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
-        canvas->resourceProvider()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
+        canvas->resourceManager()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
     }
 }
 
@@ -348,7 +364,7 @@ void KWDocument::addFrame(KWFrame *frame)
             canvas->shapeManager()->add(frame->outlineShape()->parent());
         else
             canvas->shapeManager()->add(frame->shape());
-        canvas->resourceProvider()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
+        canvas->resourceManager()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
     }
     if (frame->loadingPageNumber() > 0) {
         if (m_magicCurtain == 0) {
@@ -369,7 +385,8 @@ void KWDocument::removeFrame(KWFrame *frame)
     removeFrameFromViews(frame);
     KWPage page = pageManager()->page(frame->shape());
     if (!page.isValid()) return;
-    if (page != pageManager()->last())
+    if (!page.isAutoGenerated()) return;
+    if (page != pageManager()->last() || page == pageManager()->begin())
         return; // can only delete last page.
     foreach (KWFrameSet *fs, m_frameSets) {
         foreach (KWFrame *f, fs->frames()) {
@@ -388,7 +405,7 @@ void KWDocument::mainTextFrameSetLayoutDone()
     m_mainFramesetEverFinished = true;
 }
 
-KWFrameSet *KWDocument::frameSetByName(const QString & name)
+KWFrameSet *KWDocument::frameSetByName(const QString &name)
 {
     foreach (KWFrameSet *fs, m_frameSets) {
         if (fs->name() == name)
@@ -404,24 +421,25 @@ KWTextFrameSet *KWDocument::mainFrameSet() const
 
 KoInlineTextObjectManager *KWDocument::inlineTextObjectManager() const
 {
-    return dynamic_cast<KoInlineTextObjectManager*>(dataCenterMap()["InlineTextObjectManager"]);
+    QVariant var = resourceManager()->resource(KoText::InlineTextObjectManager);
+    return var.value<KoInlineTextObjectManager*>();
 }
 
-QString KWDocument::uniqueFrameSetName(const QString& suggestion)
+QString KWDocument::uniqueFrameSetName(const QString &suggestion)
 {
     // make up a new name for the frameset, use "[base] [digits]" as template.
     // Fully translatable naturally :)
     return renameFrameSet("", suggestion);
 }
 
-QString KWDocument::suggestFrameSetNameForCopy(const QString& base)
+QString KWDocument::suggestFrameSetNameForCopy(const QString &base)
 {
     // make up a new name for the frameset, use Copy[digits]-[base] as template.
     // Fully translatable naturally :)
     return renameFrameSet(i18n("Copy"), base);
 }
 
-QString KWDocument::renameFrameSet(const QString &prefix, const QString& base)
+QString KWDocument::renameFrameSet(const QString &prefix, const QString &base)
 {
     if (! frameSetByName(base))
         return base;
@@ -459,7 +477,8 @@ void KWDocument::initEmpty()
 
     appendPage("Standard");
 
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(dataCenterMap()["StyleManager"]);
+    Q_ASSERT(resourceManager()->hasResource(KoText::StyleManager));
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
     Q_ASSERT(styleManager);
     KoParagraphStyle *parag = new KoParagraphStyle();
     parag->setName(i18n("Head 1"));
@@ -484,7 +503,7 @@ void KWDocument::initEmpty()
 
     parag = new KoParagraphStyle();
     parag->setName(i18n("Bullet List"));
-    KoListStyle * list = new KoListStyle(parag);
+    KoListStyle *list = new KoListStyle(parag);
     KoListLevelProperties llp = list->levelProperties(0);
     llp.setStyle(KoListStyle::DiscItem);
     list->setLevelProperties(llp);
@@ -516,16 +535,13 @@ void KWDocument::clear()
     padding.right = MM_TO_POINT(3);
     m_pageManager.setPadding(padding);
 
-    inlineTextObjectManager()->setProperty(KoInlineObject::PageCount, pageCount());
+    if (inlineTextObjectManager())
+        inlineTextObjectManager()->setProperty(KoInlineObject::PageCount, pageCount());
 }
 
-bool KWDocument::loadOdf(KoOdfReadStore & odfStore)
+bool KWDocument::loadOdf(KoOdfReadStore &odfStore)
 {
     clear();
-    foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
-        canvas->resourceProvider()->setResource(KoCanvasResource::DocumentIsLoading, true);
-    }
     KWOdfLoader loader(this);
     bool rc = loader.load(odfStore);
     if (rc)
@@ -533,12 +549,8 @@ bool KWDocument::loadOdf(KoOdfReadStore & odfStore)
     return rc;
 }
 
-bool KWDocument::loadXML(const KoXmlDocument & doc, KoStore *store)
+bool KWDocument::loadXML(const KoXmlDocument &doc, KoStore *store)
 {
-    foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
-        canvas->resourceProvider()->setResource(KoCanvasResource::DocumentIsLoading, true);
-    }
     clear();
     KoXmlElement root = doc.documentElement();
     KWDLoader loader(this, store);
@@ -563,7 +575,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
 
     // insert pages
     qreal maxBottom = 0;
-    foreach (KWFrameSet* fs, m_frameSets) {
+    foreach (KWFrameSet *fs, m_frameSets) {
         foreach (KWFrame *frame, fs->frames())
         maxBottom = qMax(maxBottom, frame->shape()->boundingRect().bottom());
     }
@@ -601,7 +613,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
     for (int i = frameSetCount() - 1; i > -1; i--) {
         KWFrameSet *fs = frameSet(i);
         if (!fs) {
-            kWarning() << "frameset " << i << " is NULL!!" << endl;
+            kWarning() << "frameset " << i << " is NULL!!";
             m_lstFrameSet.remove(i);
             continue;
         }
@@ -611,24 +623,24 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
             for (int f = fs->frameCount() - 1; f >= 0; f--) {
                 KWFrame *frame = fs->frame(f);
                 if (frame->left() < 0) {
-                    kWarning() << fs->name() << " frame " << f << " pos.x is < 0, moving frame" << endl;
+                    kWarning() << fs->name() << " frame " << f << " pos.x is < 0, moving frame";
                     frame->moveBy(0 - frame->left(), 0);
                 }
                 if (frame->right() > m_pageLayout.ptWidth) {
                     kWarning() << fs->name() << " frame " << f << " rightborder outside page ("
-                    << frame->right() << ">" << m_pageLayout.ptWidth << "), shrinking" << endl;
+                    << frame->right() << ">" << m_pageLayout.ptWidth << "), shrinking";
                     frame->setRight(m_pageLayout.ptWidth);
                 }
                 if (fs->isProtectSize())
                     continue; // don't make frames bigger of a protected frameset.
                 if (frame->height() < s_minFrameHeight) {
                     kWarning() << fs->name() << " frame " << f << " height is so small no text will fit, adjusting (was: "
-                    << frame->height() << " is: " << s_minFrameHeight << ")" << endl;
+                    << frame->height() << " is: " << s_minFrameHeight << ")";
                     frame->setHeight(s_minFrameHeight);
                 }
                 if (frame->width() < s_minFrameWidth) {
                     kWarning() << fs->name() << " frame " << f << " width is so small no text will fit, adjusting (was: "
-                    << frame->width() << " is: " << s_minFrameWidth  << ")" << endl;
+                    << frame->width() << " is: " << s_minFrameWidth  << ")";
                     frame->setWidth(s_minFrameWidth);
                 }
             }
@@ -641,7 +653,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
                 fs->addFrame(frame);
             }
         } else if (fs->frameCount() == 0) {
-            kWarning() << "frameset " << i << " " << fs->name() << " has no frames" << endl;
+            kWarning() << "frameset " << i << " " << fs->name() << " has no frames";
             removeFrameSet(fs);
             if (fs->type() == FT_PART)
                 delete static_cast<KWPartFrameSet *>(fs)->getChild();
@@ -651,7 +663,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
         if (fs->frameCount() > 0) {
             KWFrame *frame = fs->frame(0);
             if (frame->isCopy()) {
-                kWarning() << "First frame in a frameset[" << fs->name() << "] was set to be a copy; resetting\n";
+                kWarning() << "First frame in a frameset[" << fs->name() << "] was set to be a copy; resetting";
                 frame->setCopy(false);
             }
         }
@@ -698,25 +710,20 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
     setModified(false);
 }
 
-bool KWDocument::completeLoading(KoStore *store)
-{
-    bool ok = true;
-    foreach (KoDataCenter *dataCenter, m_dataCenterMap) {
-        ok = ok && dataCenter->completeLoading(store);
-    }
-    foreach (KoView *view, views()) {
-        KWCanvas *canvas = static_cast<KWView*>(view)->kwcanvas();
-        canvas->resourceProvider()->setResource(KoCanvasResource::DocumentIsLoading, false);
-    }
-    return ok;
-}
-
 bool KWDocument::saveOdf(SavingContext &documentContext)
 {
     KWOdfWriter writer(this);
     return writer.save(documentContext.odfStore, documentContext.embeddedSaver);
 }
 
+QStringList KWDocument::extraNativeMimeTypes(ImportExportType importExportType) const
+{
+    QStringList answer = KoDocument::extraNativeMimeTypes(importExportType);
+    if (importExportType == KoDocument::ForExport)
+        answer.removeAll("application/x-kword"); // we can't save this, only load.
+    return answer;
+}
+
 void KWDocument::requestMoreSpace(KWTextFrameSet *fs)
 {
     // kDebug(32002) << fs;
@@ -724,13 +731,13 @@ void KWDocument::requestMoreSpace(KWTextFrameSet *fs)
     Q_ASSERT(fs->frameCount() > 0);
     Q_ASSERT(QThread::currentThread() == thread());
 
-    KWFrame *lastFrame = fs->frames()[ fs->frameCount()-1 ];
+    KWFrame *lastFrame = fs->frames().last();
 
     QString masterPageName;
     if (fs == mainFrameSet()) {
         KoShape *shape = lastFrame->shape();
         if (shape) {
-            KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(shape->userData());
+            KoTextShapeData *data = qobject_cast<KoTextShapeData*>(shape->userData());
             if (data) {
                 QTextBlock block = fs->document()->findBlock(data->endPosition() + 1);
                 if (block.isValid()) {
@@ -741,15 +748,23 @@ void KWDocument::requestMoreSpace(KWTextFrameSet *fs)
     }
 
     KWPage page = m_pageManager.page(lastFrame->shape());
-    int pageDiff =  m_pageManager.pageCount() - 1 - page.pageNumber();
+    int pageDiff = m_pageManager.pageCount() - page.pageNumber();
     if (page.pageSide() == KWPage::PageSpread)
         pageDiff--;
     if (pageDiff >= (lastFrame->frameOnBothSheets() ? 1 : 2)) {
         // its enough to just create a new frame.
-        m_frameLayout.createNewFrameForPage(fs, page.pageNumber() +
-                                            (lastFrame->frameOnBothSheets() ? 1 : 2));
+        m_frameLayout.createNewFrameForPage(fs, page.pageNumber()
+                + (lastFrame->frameOnBothSheets() ? 1 : 2));
     } else {
-        KWPage newPage = appendPage(masterPageName);
+        int afterPageNum = 0;
+        KWPage last = m_pageManager.last();
+        if (last.isValid())
+            afterPageNum = last.pageNumber();
+        KWPageInsertCommand cmd(this, afterPageNum, masterPageName);
+        cmd.redo();
+        KWPage newPage = cmd.page();
+        Q_ASSERT(newPage.isValid());
+        newPage.setAutoGenerated(true);
         if (m_magicCurtain)
             m_magicCurtain->revealFramesForPage(newPage.pageNumber(), newPage.offsetInDocument());
     }
@@ -770,7 +785,7 @@ void KWDocument::updateHeaderFooter(KWTextFrameSet *tfs)
     }
 }
 
-void KWDocument::showStartUpWidget(KoMainWindow* parent, bool alwaysShow)
+void KWDocument::showStartUpWidget(KoMainWindow *parent, bool alwaysShow)
 {
     // print error if kotext not available
     if (KoShapeRegistry::instance()->value(TextShape_SHAPEID) == 0)
@@ -806,18 +821,18 @@ void KWDocument::printDebug()
     kDebug(32001) << "                 Debug info";
     kDebug(32001) << "Document:" << this;
     /*kDebug(32001) <<"Type of document:" << (m_pageStyle.hasMainTextFrame()?"WP":"DTP"); */
-    kDebug(32001) <<"Units:" << KoUnit::unitName( unit() );
+    kDebug(32001) << "Units:" << KoUnit::unitName(unit());
     kDebug(32001) << "# Framesets:" << frameSetCount();
     int i = 0;
     foreach (KWFrameSet *fs, m_frameSets) {
-        kDebug(32001) << "Frameset" << i++ << ": '" <<
-        fs->name() << "' (" << fs << ")" << endl;
+        kDebug(32001) << "Frameset" << i++ << ":" <<
+        fs->name() << '(' << fs << ')';
         fs->printDebug();
     }
 
     kDebug(32001) << "PageManager holds" << pageCount() << " pages";
     KWPage page = m_pageManager.begin();
-    while(page.isValid()) {
+    while (page.isValid()) {
         int pgnum = page.pageNumber();
         QString side = "[Left] ";
         QString num = QString::number(pgnum);
@@ -829,6 +844,8 @@ void KWDocument::printDebug()
             num += '-' + QString::number(pgnum);
         }
         kDebug(32001) << "Page" << num << side << " width:" << page.width() << " height:" << page.height() << "following" << page.pageStyle().name();
+        if (page.isAutoGenerated())
+            kDebug(32001) << "     Auto-generated page (inserted for text layout)";
         page = page.next();
     }
 
@@ -841,11 +858,13 @@ void KWDocument::printDebug()
             kDebug(32001) << " +-- Columns:" << columns.columns << columns.columnSpacing << "pt spacing";
         KoPageLayout layout = style.pageLayout();
         kDebug(32001) << "     PageSize: " << layout.width << "x" << layout.height;
-        kDebug(32001) << "     Indents: (tlbr, edge, binding)" << layout.top << layout.left
-                << layout.bottom << layout.right << layout.pageEdge << layout.bindingSide;
+        kDebug(32001) << "     Indents: (tlbr, edge, binding)"
+                      << layout.topMargin << layout.leftMargin
+                      << layout.bottomMargin << layout.rightMargin
+                      << layout.pageEdge << layout.bindingSide;
     }
 
-    kDebug(32001) << "  The height of the doc (in pt) is:" << pageManager()->bottomOfPage(pageManager()->pageCount() - 1) << endl;
+    kDebug(32001) << "  The height of the doc (in pt) is:" << pageManager()->bottomOfPage(pageManager()->pageCount() - 1);
 }
 #endif
 
@@ -866,8 +885,8 @@ void KWDocument::saveConfig()
 {
     if (!isReadWrite())
         return;
-//   KConfigGroup group( KoGlobal::kofficeConfig(), "Spelling" );
-//   group.writeEntry( "PersonalDict", m_spellCheckPersonalDict );
+//   KConfigGroup group(KoGlobal::kofficeConfig(), "Spelling");
+//   group.writeEntry("PersonalDict", m_spellCheckPersonalDict);
 
     if (isEmbedded())
         return;
@@ -881,6 +900,7 @@ void KWDocument::saveConfig()
 
 // ************* PageProcessingQueue ************
 PageProcessingQueue::PageProcessingQueue(KWDocument *parent)
+    : QObject(parent)
 {
     m_document = parent;
     m_triggered = false;
@@ -896,12 +916,12 @@ void PageProcessingQueue::addPage(KWPage page)
 
 void PageProcessingQueue::process()
 {
+    m_triggered = false;
     const bool docIsEmpty = m_document->isEmpty();
     const bool docIsModified = m_document->isModified();
     foreach (const KWPage &page, m_pages) {
         if (! page.isValid())
             continue;
-        emit m_document->pageSetupChanged();
         m_document->m_frameLayout.createNewFramesForPage(page.pageNumber());
     }
     if (docIsEmpty)
@@ -910,4 +930,5 @@ void PageProcessingQueue::process()
         m_document->setModified(false);
     m_pages.clear();
     deleteLater();
+    emit m_document->pageSetupChanged();
 }
