@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2006-2008 Jan Hambrecht <jaham@gmx.net>
-   Copyright (C) 2007 Thomas Zander <zander@kde.org>
+   Copyright (C) 2007-2009 Thomas Zander <zander@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,6 +20,7 @@
 */
 
 #include "KoPathShape.h"
+#include "KoPathShape_p.h"
 #include "KoPathPoint.h"
 #include "KoPointGroup.h"
 #include "KoShapeBorderModel.h"
@@ -30,6 +31,7 @@
 #include "KoShapeShadow.h"
 #include "KoShapeBackground.h"
 #include "KoShapeContainer.h"
+#include "KoFilterEffectStack.h"
 
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
@@ -42,33 +44,74 @@
 #include <KDebug>
 #include <QtGui/QPainter>
 
-class KoPathShape::Private
+#include <qnumeric.h> // for qIsNaN
+static bool qIsNaNPoint(const QPointF &p) {
+    return qIsNaN(p.x()) || qIsNaN(p.y());
+}
+
+KoPathShapePrivate::KoPathShapePrivate(KoPathShape *q)
+    : KoShapePrivate(q),
+    fillRule(Qt::OddEvenFill)
 {
-public:
-    Private() : fillRule(Qt::OddEvenFill) {
+}
+
+QRectF KoPathShapePrivate::handleRect(const QPointF &p, qreal radius) const
+{
+    return QRectF(p.x() - radius, p.y() - radius, 2*radius, 2*radius);
+}
+
+void KoPathShapePrivate::applyViewboxTransformation(const KoXmlElement &element)
+{
+    Q_Q(KoPathShape);
+    // apply viewbox transformation
+    QRectF viewBox = q->loadOdfViewbox(element);
+    if (! viewBox.isEmpty()) {
+        // load the desired size
+        QSizeF size;
+        size.setWidth(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "width", QString())));
+        size.setHeight(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "height", QString())));
+
+        // load the desired position
+        QPointF pos;
+        pos.setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x", QString())));
+        pos.setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y", QString())));
+
+        // create matrix to transform original path data into desired size and position
+        QMatrix viewMatrix;
+        viewMatrix.translate(-viewBox.left(), -viewBox.top());
+        viewMatrix.scale(size.width() / viewBox.width(), size.height() / viewBox.height());
+        viewMatrix.translate(pos.x(), pos.y());
+
+        // transform the path data
+        map(viewMatrix);
     }
+}
 
-    Qt::FillRule fillRule;
-};
 
+/////////////////////////
 KoPathShape::KoPathShape()
-        : d(new Private())   // while we don't actually have any private data, just leave it as this.
+    :KoShape(*(new KoPathShapePrivate(this)))
+{
+}
+
+KoPathShape::KoPathShape(KoPathShapePrivate &dd)
+    : KoShape(dd)
 {
 }
 
 KoPathShape::~KoPathShape()
 {
     clear();
-    delete d;
 }
 
 void KoPathShape::saveOdf(KoShapeSavingContext & context) const
 {
+    Q_D(const KoPathShape);
     context.xmlWriter().startElement("draw:path");
     saveOdfAttributes(context, OdfAllAttributes | OdfViewbox);
 
-    context.xmlWriter().addAttribute("svg:d", toString(QMatrix()));
-    context.xmlWriter().addAttribute("koffice:nodeTypes", nodeTypes());
+    context.xmlWriter().addAttribute("svg:d", toString());
+    context.xmlWriter().addAttribute("koffice:nodeTypes", d->nodeTypes());
 
     saveOdfCommonChildElements(context);
     context.xmlWriter().endElement();
@@ -76,6 +119,7 @@ void KoPathShape::saveOdf(KoShapeSavingContext & context) const
 
 bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context)
 {
+    Q_D(KoPathShape);
     loadOdfAttributes(element, context, OdfMandatories | OdfAdditionalAttributes | OdfCommonChildElements);
 
     // first clear the path data from the default path
@@ -113,10 +157,10 @@ bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &c
     } else { // path loading
         KoPathShapeLoader loader(this);
         loader.parseSvg(element.attributeNS(KoXmlNS::svg, "d"), true);
-        loadNodeTypes(element);
+        d->loadNodeTypes(element);
     }
 
-    applyViewboxTransformation(element);
+    d->applyViewboxTransformation(element);
     QPointF pos = normalize();
     setTransformation(QMatrix());
 
@@ -134,6 +178,7 @@ bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &c
 
 QString KoPathShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const
 {
+    Q_D(const KoPathShape);
     style.addProperty("svg:fill-rule", d->fillRule == Qt::OddEvenFill ? "evenodd" : "nonzero");
 
     return KoShape::saveStyle(style, context);
@@ -141,6 +186,7 @@ QString KoPathShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context)
 
 void KoPathShape::loadStyle(const KoXmlElement & element, KoShapeLoadingContext &context)
 {
+    Q_D(KoPathShape);
     KoShape::loadStyle(element, context);
 
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
@@ -178,37 +224,11 @@ QRectF KoPathShape::loadOdfViewbox(const KoXmlElement & element) const
     return viewbox;
 }
 
-void KoPathShape::applyViewboxTransformation(const KoXmlElement & element)
-{
-    // apply viewbox transformation
-    QRectF viewBox = loadOdfViewbox(element);
-    if (! viewBox.isEmpty()) {
-        // load the desired size
-        QSizeF size;
-        size.setWidth(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "width", QString())));
-        size.setHeight(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "height", QString())));
-
-        // load the desired position
-        QPointF pos;
-        pos.setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x", QString())));
-        pos.setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y", QString())));
-
-        // create matrix to transform original path data into desired size and position
-        QMatrix viewMatrix;
-        viewMatrix.translate(-viewBox.left(), -viewBox.top());
-        viewMatrix.scale(size.width() / viewBox.width(), size.height() / viewBox.height());
-        viewMatrix.translate(pos.x(), pos.y());
-
-        // transform the path data
-        map(viewMatrix);
-    }
-}
-
 void KoPathShape::clear()
 {
     foreach(KoSubpath *subpath, m_subpaths) {
         foreach(KoPathPoint *point, *subpath)
-        delete point;
+            delete point;
         delete subpath;
     }
     m_subpaths.clear();
@@ -216,6 +236,7 @@ void KoPathShape::clear()
 
 void KoPathShape::paint(QPainter &painter, const KoViewConverter &converter)
 {
+    Q_D(KoPathShape);
     applyConversion(painter, converter);
     QPainterPath path(outline());
     path.setFillRule(d->fillRule);
@@ -226,15 +247,16 @@ void KoPathShape::paint(QPainter &painter, const KoViewConverter &converter)
 }
 
 #ifndef NDEBUG
-void KoPathShape::paintDebug(QPainter &painter)
+void KoPathShapePrivate::paintDebug(QPainter &painter)
 {
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
+    Q_Q(KoPathShape);
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
     int i = 0;
 
     QPen pen(Qt::black);
     painter.save();
     painter.setPen(pen);
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             ++i;
@@ -263,10 +285,11 @@ void KoPathShape::paintDebug(QPainter &painter)
     kDebug(30006) << "nop =" << i;
 }
 
-void KoPathShape::debugPath()
+void KoPathShapePrivate::debugPath() const
 {
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    Q_Q(const KoPathShape);
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             kDebug(30006) << "p:" << (*pathIt) << "," << *it << "," << (*it)->point() << "," << (*it)->properties() << "," << (*it)->group();
@@ -277,11 +300,12 @@ void KoPathShape::debugPath()
 
 void KoPathShape::paintPoints(QPainter &painter, const KoViewConverter &converter, int handleRadius)
 {
+    Q_D(KoPathShape);
     applyConversion(painter, converter);
 
     KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
 
-    QRectF handle = converter.viewToDocument(handleRect(QPoint(0, 0), handleRadius));
+    QRectF handle = converter.viewToDocument(d->handleRect(QPoint(0, 0), handleRadius));
 
     for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
@@ -290,38 +314,41 @@ void KoPathShape::paintPoints(QPainter &painter, const KoViewConverter &converte
     }
 }
 
-QRectF KoPathShape::handleRect(const QPointF &p, qreal radius) const
-{
-    return QRectF(p.x() - radius, p.y() - radius, 2*radius, 2*radius);
-}
-
-const QPainterPath KoPathShape::outline() const
+QPainterPath KoPathShape::outline() const
 {
     QPainterPath path;
     foreach(KoSubpath * subpath, m_subpaths) {
         KoPathPoint * lastPoint = subpath->first();
         bool activeCP = false;
         foreach(KoPathPoint * currPoint, *subpath) {
-            KoPathPoint::KoPointProperties currProperties = currPoint->properties();
+            KoPathPoint::PointProperties currProperties = currPoint->properties();
             if (currPoint == subpath->first()) {
                 if (currProperties & KoPathPoint::StartSubpath) {
+                    Q_ASSERT(!qIsNaNPoint(currPoint->point()));
                     path.moveTo(currPoint->point());
                 }
             } else if (activeCP && currPoint->activeControlPoint1()) {
+                Q_ASSERT(!qIsNaNPoint(lastPoint->controlPoint2()));
+                Q_ASSERT(!qIsNaNPoint(currPoint->controlPoint1()));
+                Q_ASSERT(!qIsNaNPoint(currPoint->point()));
                 path.cubicTo(
                     lastPoint->controlPoint2(),
                     currPoint->controlPoint1(),
                     currPoint->point());
             } else if( activeCP || currPoint->activeControlPoint1()) {
+                Q_ASSERT(!qIsNaNPoint(lastPoint->controlPoint2()));
+                Q_ASSERT(!qIsNaNPoint(currPoint->controlPoint1()));
                 path.quadTo(
                     activeCP ? lastPoint->controlPoint2() : currPoint->controlPoint1(),
                     currPoint->point());
             } else {
+                Q_ASSERT(!qIsNaNPoint(currPoint->point()));
                 path.lineTo(currPoint->point());
             }
             if (currProperties & KoPathPoint::CloseSubpath && currProperties & KoPathPoint::StopSubpath) {
                 // add curve when there is a curve on the way to the first point
                 KoPathPoint * firstPoint = subpath->first();
+                Q_ASSERT(!qIsNaNPoint(firstPoint->point()));
                 if (currPoint->activeControlPoint2() && firstPoint->activeControlPoint1()) {
                     path.cubicTo(
                         currPoint->controlPoint2(),
@@ -329,9 +356,11 @@ const QPainterPath KoPathShape::outline() const
                         firstPoint->point());
                 }
                 else if(currPoint->activeControlPoint2() || firstPoint->activeControlPoint1()) {
+                    Q_ASSERT(!qIsNaNPoint(currPoint->point()));
+                    Q_ASSERT(!qIsNaNPoint(currPoint->controlPoint1()));
                     path.quadTo(
                         currPoint->activeControlPoint2() ? currPoint->controlPoint2() : firstPoint->controlPoint1(),
-                        firstPoint->point());                
+                        firstPoint->point());
                 }
                 path.closeSubpath();
             }
@@ -349,23 +378,35 @@ const QPainterPath KoPathShape::outline() const
 
 QRectF KoPathShape::boundingRect() const
 {
-    QRectF bb(outline().boundingRect());
+    QMatrix transform = absoluteTransformation(0);
+    // calculate the bounding rect of the transformed outline
+    QRectF bb(transform.map(outline()).boundingRect());
     if (border()) {
         KoInsets inset;
         border()->borderInsets(this, inset);
-        bb.adjust(-inset.left, -inset.top, inset.right, inset.bottom);
+
+        // calculate transformed border insets
+        QPointF center = transform.map(QPointF());
+        QPointF tl = transform.map(QPointF(-inset.left,-inset.top)) - center;
+        QPointF br = transform.map(QPointF(inset.right,inset.bottom)) -center;
+        qreal left = qMin(tl.x(),br.x());
+        qreal right = qMax(tl.x(),br.x());
+        qreal top = qMin(tl.y(),br.y());
+        qreal bottom = qMax(tl.y(),br.y());
+        bb.adjust(left, top, right, bottom);
     }
-    //qDebug() << "KoPathShape::boundingRect = " << bb;
-    bb = absoluteTransformation(0).mapRect(bb);
     if (shadow()) {
         KoInsets insets;
-        shadow()->insets(this, insets);
+        shadow()->insets(insets);
         bb.adjust(-insets.left, -insets.top, insets.right, insets.bottom);
     }
+    if (filterEffectStack()) {
+        QRectF clipRect = filterEffectStack()->clipRectForBoundingRect(QRectF(QPointF(), size()));
+        bb |= transform.mapRect(clipRect);
+    }
     return bb;
 }
 
-
 QSizeF KoPathShape::size() const
 {
     // don't call boundingRect here as it uses absoluteTransformation
@@ -375,10 +416,11 @@ QSizeF KoPathShape::size() const
 
 void KoPathShape::setSize(const QSizeF &newSize)
 {
+    Q_D(KoPathShape);
     QMatrix matrix(resizeMatrix(newSize));
 
     KoShape::setSize(newSize);
-    map(matrix);
+    d->map(matrix);
 }
 
 QMatrix KoPathShape::resizeMatrix( const QSizeF & newSize ) const
@@ -413,23 +455,25 @@ KoPathPoint * KoPathShape::moveTo(const QPointF &p)
 
 KoPathPoint * KoPathShape::lineTo(const QPointF &p)
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     KoPathPoint * lastPoint = m_subpaths.last()->last();
-    updateLast(&lastPoint);
+    d->updateLast(&lastPoint);
     m_subpaths.last()->push_back(point);
     return point;
 }
 
 KoPathPoint * KoPathShape::curveTo(const QPointF &c1, const QPointF &c2, const QPointF &p)
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
     KoPathPoint * lastPoint = m_subpaths.last()->last();
-    updateLast(&lastPoint);
+    d->updateLast(&lastPoint);
     lastPoint->setControlPoint2(c1);
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     point->setControlPoint1(c2);
@@ -439,11 +483,12 @@ KoPathPoint * KoPathShape::curveTo(const QPointF &c1, const QPointF &c2, const Q
 
 KoPathPoint * KoPathShape::curveTo(const QPointF &c, const QPointF &p)
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty())
         moveTo(QPointF(0, 0));
 
     KoPathPoint * lastPoint = m_subpaths.last()->last();
-    updateLast(&lastPoint);
+    d->updateLast(&lastPoint);
     lastPoint->setControlPoint2(c);
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     m_subpaths.last()->push_back(point);
@@ -535,26 +580,29 @@ int KoPathShape::arcToCurve(qreal rx, qreal ry, qreal startAngle, qreal sweepAng
 
 void KoPathShape::close()
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         return;
     }
-    closeSubpath(m_subpaths.last());
+    d->closeSubpath(m_subpaths.last());
 }
 
 void KoPathShape::closeMerge()
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         return;
     }
-    closeMergeSubpath(m_subpaths.last());
+    d->closeMergeSubpath(m_subpaths.last());
 }
 
 QPointF KoPathShape::normalize()
 {
+    Q_D(KoPathShape);
     QPointF tl(outline().boundingRect().topLeft());
     QMatrix matrix;
     matrix.translate(-tl.x(), -tl.y());
-    map(matrix);
+    d->map(matrix);
 
     // keep the top left point of the object
     applyTransformation(matrix.inverted());
@@ -562,10 +610,11 @@ QPointF KoPathShape::normalize()
     return tl;
 }
 
-void KoPathShape::map(const QMatrix &matrix)
+void KoPathShapePrivate::map(const QMatrix &matrix)
 {
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    Q_Q(KoPathShape);
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             (*it)->map(matrix);
@@ -573,13 +622,14 @@ void KoPathShape::map(const QMatrix &matrix)
     }
 }
 
-void KoPathShape::updateLast(KoPathPoint ** lastPoint)
+void KoPathShapePrivate::updateLast(KoPathPoint **lastPoint)
 {
+    Q_Q(KoPathShape);
     // check if we are about to add a new point to a closed subpath
     if ((*lastPoint)->properties() & KoPathPoint::StopSubpath
             && (*lastPoint)->properties() & KoPathPoint::CloseSubpath) {
         // get the first point of the subpath
-        KoPathPoint * subpathStart = m_subpaths.last()->first();
+        KoPathPoint *subpathStart = q->m_subpaths.last()->first();
         // clone the first point of the subpath...
         KoPathPoint * newLastPoint = new KoPathPoint(*subpathStart);
         // ... and make it a normal point
@@ -595,7 +645,7 @@ void KoPathShape::updateLast(KoPathPoint ** lastPoint)
         // now start a new subpath with the cloned start point
         KoSubpath *path = new KoSubpath;
         path->push_back(newLastPoint);
-        m_subpaths.push_back(path);
+        q->m_subpaths.push_back(path);
         *lastPoint = newLastPoint;
     } else {
         // the subpath was not closed so the formerly last point
@@ -630,7 +680,7 @@ QList<KoPathSegment> KoPathShape::segmentsAt(const QRectF &r)
     int subpathCount = m_subpaths.count();
     for (int subpathIndex = 0; subpathIndex < subpathCount; ++subpathIndex) {
         KoSubpath * subpath = m_subpaths[subpathIndex];
-        int pointCount = subpath[subpathIndex].count();
+        int pointCount = subpath->count();
         bool subpathClosed = isClosedSubpath(subpathIndex);
         for (int pointIndex = 0; pointIndex < pointCount; ++pointIndex) {
             if (pointIndex == (pointCount - 1) && ! subpathClosed)
@@ -664,7 +714,8 @@ KoPathPointIndex KoPathShape::pathPointIndex(const KoPathPoint *point) const
 
 KoPathPoint * KoPathShape::pointByIndex(const KoPathPointIndex &pointIndex) const
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(const KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size())
         return 0;
@@ -674,9 +725,10 @@ KoPathPoint * KoPathShape::pointByIndex(const KoPathPointIndex &pointIndex) cons
 
 KoPathSegment KoPathShape::segmentByIndex(const KoPathPointIndex &pointIndex) const
 {
+    Q_D(const KoPathShape);
     KoPathSegment segment(0, 0);
 
-    KoSubpath * subpath = subPath(pointIndex.first);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath != 0 && pointIndex.second >= 0 && pointIndex.second < subpath->size()) {
         KoPathPoint * point = subpath->at(pointIndex.second);
@@ -710,9 +762,10 @@ int KoPathShape::subpathCount() const
     return m_subpaths.count();
 }
 
-int KoPathShape::pointCountSubpath(int subpathIndex) const
+int KoPathShape::subpathPointCount(int subpathIndex) const
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(const KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
         return -1;
@@ -722,22 +775,27 @@ int KoPathShape::pointCountSubpath(int subpathIndex) const
 
 bool KoPathShape::isClosedSubpath(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
         return false;
 
-    return subpath->last()->properties() & KoPathPoint::CloseSubpath;
+    const bool firstClosed = subpath->first()->properties() & KoPathPoint::CloseSubpath;
+    const bool lastClosed = subpath->last()->properties() & KoPathPoint::CloseSubpath;
+
+    return firstClosed && lastClosed;
 }
 
 bool KoPathShape::insertPoint(KoPathPoint* point, const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second > subpath->size())
         return false;
 
-    KoPathPoint::KoPointProperties properties = point->properties();
+    KoPathPoint::PointProperties properties = point->properties();
     properties &= ~KoPathPoint::StartSubpath;
     properties &= ~KoPathPoint::StopSubpath;
     properties &= ~KoPathPoint::CloseSubpath;
@@ -772,15 +830,20 @@ bool KoPathShape::insertPoint(KoPathPoint* point, const KoPathPointIndex &pointI
 
 KoPathPoint * KoPathShape::removePoint(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size())
         return 0;
 
     KoPathPoint * point = subpath->takeAt(pointIndex.second);
 
+    //don't do anything (not even crash), if there was only one point
+    if (pointCount()==0) {
+        return point;
+    }
     // check if we removed the first point
-    if (pointIndex.second == 0) {
+    else if (pointIndex.second == 0) {
         // first point removed, set new StartSubpath
         subpath->first()->setProperty(KoPathPoint::StartSubpath);
         // check if path was closed
@@ -805,10 +868,11 @@ KoPathPoint * KoPathShape::removePoint(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::breakAfter(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
-    if (subpath == 0 || pointIndex.second < 0 || pointIndex.second > subpath->size() - 2
-            || subpath->last()->properties() & KoPathPoint::CloseSubpath)
+    if (!subpath || pointIndex.second < 0 || pointIndex.second > subpath->size() - 2
+        || isClosedSubpath(pointIndex.first))
         return false;
 
     KoSubpath * newSubpath = new KoSubpath;
@@ -830,11 +894,12 @@ bool KoPathShape::breakAfter(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::join(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
-    KoSubpath * nextSubpath = subPath(subpathIndex + 1);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
+    KoSubpath *nextSubpath = d->subPath(subpathIndex + 1);
 
-    if (subpath == 0 || nextSubpath == 0 || subpath->last()->properties() & KoPathPoint::CloseSubpath
-            || nextSubpath->last()->properties() & KoPathPoint::CloseSubpath)
+    if (!subpath || !nextSubpath || isClosedSubpath(subpathIndex)
+            || isClosedSubpath(subpathIndex+1))
         return false;
 
     // the last point of the subpath does not end the subpath anymore
@@ -844,7 +909,7 @@ bool KoPathShape::join(int subpathIndex)
 
     // append the second subpath to the first
     foreach(KoPathPoint * p, *nextSubpath)
-    subpath->append(p);
+        subpath->append(p);
 
     // remove the nextSubpath from path
     m_subpaths.removeAt(subpathIndex + 1);
@@ -857,11 +922,15 @@ bool KoPathShape::join(int subpathIndex)
 
 bool KoPathShape::moveSubpath(int oldSubpathIndex, int newSubpathIndex)
 {
-    KoSubpath * subpath = subPath(oldSubpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(oldSubpathIndex);
 
     if (subpath == 0 || newSubpathIndex >= m_subpaths.size())
         return false;
 
+    if (oldSubpathIndex == newSubpathIndex)
+        return true;
+
     m_subpaths.removeAt(oldSubpathIndex);
     m_subpaths.insert(newSubpathIndex, subpath);
 
@@ -870,10 +939,11 @@ bool KoPathShape::moveSubpath(int oldSubpathIndex, int newSubpathIndex)
 
 KoPathPointIndex KoPathShape::openSubpath(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
-    if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size()
-            || !(subpath->last()->properties() & KoPathPoint::CloseSubpath))
+    if (!subpath || pointIndex.second < 0 || pointIndex.second >= subpath->size()
+            || !isClosedSubpath(pointIndex.first))
         return KoPathPointIndex(-1, -1);
 
     KoPathPoint * oldStartPoint = subpath->first();
@@ -889,17 +959,18 @@ KoPathPointIndex KoPathShape::openSubpath(const KoPathPointIndex &pointIndex)
     // make the first point a start node
     subpath->first()->setProperty(KoPathPoint::StartSubpath);
     // make the last point an end node
-    subpath->last()->unsetProperty(KoPathPoint::StopSubpath);
+    subpath->last()->setProperty(KoPathPoint::StopSubpath);
 
     return pathPointIndex(oldStartPoint);
 }
 
 KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
-    if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size()
-            || subpath->last()->properties() & KoPathPoint::CloseSubpath)
+    if (!subpath || pointIndex.second < 0 || pointIndex.second >= subpath->size()
+        || isClosedSubpath(pointIndex.first))
         return KoPathPointIndex(-1, -1);
 
     KoPathPoint * oldStartPoint = subpath->first();
@@ -915,13 +986,14 @@ KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
     subpath->first()->setProperty(KoPathPoint::StartSubpath);
     subpath->last()->setProperty(KoPathPoint::StopSubpath);
 
-    closeSubpath(subpath);
+    d->closeSubpath(subpath);
     return pathPointIndex(oldStartPoint);
 }
 
 bool KoPathShape::reverseSubpath(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
         return false;
@@ -937,8 +1009,8 @@ bool KoPathShape::reverseSubpath(int subpathIndex)
     KoPathPoint *first = subpath->first();
     KoPathPoint *last = subpath->last();
 
-    KoPathPoint::KoPointProperties firstProps = first->properties();
-    KoPathPoint::KoPointProperties lastProps = last->properties();
+    KoPathPoint::PointProperties firstProps = first->properties();
+    KoPathPoint::PointProperties lastProps = last->properties();
 
     firstProps |= KoPathPoint::StartSubpath;
     firstProps &= ~KoPathPoint::StopSubpath;
@@ -956,7 +1028,8 @@ bool KoPathShape::reverseSubpath(int subpathIndex)
 
 KoSubpath * KoPathShape::removeSubpath(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath != 0)
         m_subpaths.removeAt(subpathIndex);
@@ -1026,7 +1099,7 @@ bool KoPathShape::separate(QList<KoPathShape*> & separatedPaths)
     return true;
 }
 
-void KoPathShape::closeSubpath(KoSubpath *subpath)
+void KoPathShapePrivate::closeSubpath(KoSubpath *subpath)
 {
     if (! subpath)
         return;
@@ -1035,7 +1108,7 @@ void KoPathShape::closeSubpath(KoSubpath *subpath)
     subpath->first()->setProperty(KoPathPoint::CloseSubpath);
 }
 
-void KoPathShape::closeMergeSubpath(KoSubpath *subpath)
+void KoPathShapePrivate::closeMergeSubpath(KoSubpath *subpath)
 {
     if (! subpath || subpath->size() < 2)
         return;
@@ -1062,12 +1135,13 @@ void KoPathShape::closeMergeSubpath(KoSubpath *subpath)
     }
 }
 
-KoSubpath * KoPathShape::subPath(int subpathIndex) const
+KoSubpath *KoPathShapePrivate::subPath(int subpathIndex) const
 {
-    if (subpathIndex < 0 || subpathIndex >= m_subpaths.size())
+    Q_Q(const KoPathShape);
+    if (subpathIndex < 0 || subpathIndex >= q->m_subpaths.size())
         return 0;
 
-    return m_subpaths.at(subpathIndex);
+    return q->m_subpaths.at(subpathIndex);
 }
 
 QString KoPathShape::pathShapeId() const
@@ -1146,11 +1220,12 @@ char nodeType( const KoPathPoint * point )
     }
 }
 
-QString KoPathShape::nodeTypes() const
+QString KoPathShapePrivate::nodeTypes() const
 {
+    Q_Q(const KoPathShape);
     QString types;
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             if (it == (*pathIt)->constBegin()) {
@@ -1180,13 +1255,14 @@ void updateNodeType( KoPathPoint * point, const QChar & nodeType )
     }
 }
 
-void KoPathShape::loadNodeTypes(const KoXmlElement & element)
+void KoPathShapePrivate::loadNodeTypes(const KoXmlElement &element)
 {
+    Q_Q(KoPathShape);
     if (element.hasAttributeNS(KoXmlNS::koffice, "nodeTypes")) {
         QString nodeTypes = element.attributeNS(KoXmlNS::koffice, "nodeTypes");
         QString::const_iterator nIt(nodeTypes.constBegin());
-        KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-        for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+        KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+        for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
             KoSubpath::const_iterator it((*pathIt)->constBegin());
             for (; it != (*pathIt)->constEnd(); ++it, nIt++) {
                 // be sure not to crash if there are not enough nodes in nodeTypes
@@ -1211,15 +1287,17 @@ void KoPathShape::loadNodeTypes(const KoXmlElement & element)
 
 Qt::FillRule KoPathShape::fillRule() const
 {
+    Q_D(const KoPathShape);
     return d->fillRule;
 }
 
 void KoPathShape::setFillRule(Qt::FillRule fillRule)
 {
+    Q_D(KoPathShape);
     d->fillRule = fillRule;
 }
 
-KoPathShape * KoPathShape::fromQPainterPath(const QPainterPath &path)
+KoPathShape * KoPathShape::createShapeFromPainterPath(const QPainterPath &path)
 {
     KoPathShape * shape = new KoPathShape();
 
@@ -1265,7 +1343,7 @@ bool KoPathShape::hitTest(const QPointF &position) const
         if( outlinePath.contains( point ) )
             return true;
     }
-    
+
     // if there is no shadow we can as well just leave
     if (! shadow())
         return false;
