@@ -36,13 +36,11 @@ class KoPathPoint;
 class KoPathShapePrivate;
 
 typedef QPair<int, int> KoPathPointIndex;
-typedef QMap<KoPathShape *, QSet<KoPathPointIndex> > KoPathShapePointIndexMap;
 
 /// a KoSubpath contains a path from a moveTo until a close or a new moveTo
 typedef QList<KoPathPoint *> KoSubpath;
 typedef QList<KoSubpath *> KoSubpathList;
 /// The position of a path point within a path shape
-typedef QPair<KoSubpath*, int> KoPointPosition;
 /**
  * @brief This is the base for all graphical objects.
  *
@@ -109,9 +107,9 @@ public:
     virtual bool hitTest(const QPointF &position) const;
 
     // reimplemented
-    virtual void saveOdf(KoShapeSavingContext & context) const;
+    virtual void saveOdf(KoShapeSavingContext &context) const;
     // reimplemented
-    virtual bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context);
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
 
     /// Removes all subpaths and their points from the path
     void clear();
@@ -122,7 +120,7 @@ public:
      *
      * @return the newly created point
      */
-    KoPathPoint * moveTo(const QPointF &p);
+    KoPathPoint *moveTo(const QPointF &p);
 
     /**
      * @brief Adds a new line segment
@@ -131,7 +129,7 @@ public:
      *
      * @return the newly created point
      */
-    KoPathPoint * lineTo(const QPointF &p);
+    KoPathPoint *lineTo(const QPointF &p);
 
     /**
      * @brief Adds a new cubic Bezier curve segment.
@@ -145,7 +143,7 @@ public:
      *
      * @return The newly created point
      */
-    KoPathPoint * curveTo(const QPointF &c1, const QPointF &c2, const QPointF &p);
+    KoPathPoint *curveTo(const QPointF &c1, const QPointF &c2, const QPointF &p);
 
     /**
      * @brief Adds a new quadratic Bezier curve segment.
@@ -158,7 +156,7 @@ public:
      *
      * @return The newly created point
      */
-    KoPathPoint * curveTo(const QPointF &c, const QPointF &p);
+    KoPathPoint *curveTo(const QPointF &c, const QPointF &p);
 
     /**
      * @brief Add an arc.
@@ -173,7 +171,7 @@ public:
      *
      * @return The newly created point
      */
-    KoPathPoint * arcTo(qreal rx, qreal ry, qreal startAngle, qreal sweepAngle);
+    KoPathPoint *arcTo(qreal rx, qreal ry, qreal startAngle, qreal sweepAngle);
 
 
     /**
@@ -205,17 +203,17 @@ public:
 
     /**
      * @brief Returns the path points within the given rectangle.
-     * @param r the rectangle the requested points are in
+     * @param rect the rectangle the requested points are in
      * @return list of points within the rectangle
      */
-    QList<KoPathPoint*> pointsAt(const QRectF &r);
+    QList<KoPathPoint*> pointsAt(const QRectF &rect);
 
     /**
      * @brief Returns the list of path segments within the given rectangle.
-     * @param r the rectangle the requested segments are in
+     * @param rect the rectangle the requested segments are in
      * @return list of segments within the rectangle
      */
-    QList<KoPathSegment> segmentsAt(const QRectF &r);
+    QList<KoPathSegment> segmentsAt(const QRectF &rect);
 
     /**
      * @brief Returns the path point index of a given path point
@@ -233,7 +231,7 @@ public:
      *
      * @return KoPathPoint on success, 0 otherwise e.g. out of bounds
      */
-    KoPathPoint * pointByIndex(const KoPathPointIndex &pointIndex) const;
+    KoPathPoint *pointByIndex(const KoPathPointIndex &pointIndex) const;
 
     /**
      * @brief Returns the segment specified by a path point index
@@ -268,7 +266,7 @@ public:
      *
      * @return The number of points in the subpath or -1 if subpath out of bounds
      */
-    int pointCountSubpath(int subpathIndex) const;
+    int subpathPointCount(int subpathIndex) const;
 
     /**
      * @brief Checks if a subpath is closed
@@ -293,7 +291,7 @@ public:
      * @return true on success,
      *         false when pointIndex is out of bounds
      */
-    bool insertPoint(KoPathPoint* point, const KoPathPointIndex &pointIndex);
+    bool insertPoint(KoPathPoint *point, const KoPathPointIndex &pointIndex);
 
     /**
      * @brief Removes a point from the path.
@@ -305,7 +303,7 @@ public:
      * @return The removed point on success,
      *         otherwise 0
      */
-    KoPathPoint * removePoint(const KoPathPointIndex &pointIndex);
+    KoPathPoint *removePoint(const KoPathPointIndex &pointIndex);
 
     /**
      * @brief Breaks the path after the point index
@@ -327,7 +325,7 @@ public:
      * @brief Joins the given subpath with the following one
      *
      * Joins the given subpath with the following one by inserting a segment between
-     * the two subpathes.
+     * the two subpaths.
      * This does nothing if the specified subpath is the last subpath
      * or one of both subpaths is closed.
      *
@@ -384,7 +382,7 @@ public:
      * @param subpathIndex the index of the subpath to remove
      * @return the removed subpath on succes, 0 otherwise.
      */
-    KoSubpath * removeSubpath(int subpathIndex);
+    KoSubpath *removeSubpath(int subpathIndex);
 
     /**
      * @brief Adds a subpath at the given index to the path
@@ -392,7 +390,7 @@ public:
      * @param subpathIndex the index at which the new subpath should be inserted
      * @return true on success, false otherwise e.g. subpathIndex out of bounds
      */
-    bool addSubpath(KoSubpath * subpath, int subpathIndex);
+    bool addSubpath(KoSubpath *subpath, int subpathIndex);
 
     /**
      * @brief Combines two path shapes by appending the data of the specified path.
@@ -406,14 +404,7 @@ public:
      * @param separatedPaths the list which contains the separated path shapes
      * @return true if separating the path was successful, false otherwise
      */
-    bool separate(QList<KoPathShape*> & separatedPaths);
-
-#ifndef NDEBUG
-    /**
-     * @brief print debug information about a the points of the path
-     */
-    void debugPath();
-#endif
+    bool separate(QList<KoPathShape*> &separatedPaths);
 
     /**
      * Returns the specific path shape id.
@@ -431,7 +422,7 @@ public:
     virtual QString pathShapeId() const;
 
     /// Returns a odf/svg string represenatation of the path data with the given matrix applied.
-    QString toString(const QMatrix &matrix) const;
+    QString toString(const QMatrix &matrix = QMatrix()) const;
 
     /// Returns the fill rule for the path object
     Qt::FillRule fillRule() const;
@@ -440,39 +431,17 @@ public:
     void setFillRule(Qt::FillRule fillRule);
 
     /// Creates path shape from given QPainterPath
-    static KoPathShape * fromQPainterPath(const QPainterPath &path);
+    static KoPathShape *createShapeFromPainterPath(const QPainterPath &path);
 
 protected:
-    /// constructor
+    /// constructor \internal
     KoPathShape(KoPathShapePrivate &);
 
-private:
-    // TODO move all the private methods to live on the d pointer object
-    void map(const QMatrix &matrix);
-
-    void updateLast(KoPathPoint ** lastPoint);
-
-    /// closes specified subpath
-    void closeSubpath(KoSubpath *subpath);
-    /// close-merges specified subpath
-    void closeMergeSubpath(KoSubpath *subpath);
-
-    /**
-     * @brief Returns subpath at given index
-     * @param subpathIndex the index of the subpath to return
-     * @return subPath on success, or 0 when subpathIndex is out of bounds
-     */
-    KoSubpath * subPath(int subpathIndex) const;
-#ifndef NDEBUG
-    void paintDebug(QPainter &painter);
-#endif
     /// reimplemented
     virtual QString saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const;
     /// reimplemented
-    virtual void loadStyle(const KoXmlElement & element, KoShapeLoadingContext &context);
+    virtual void loadStyle(const KoXmlElement &element, KoShapeLoadingContext &context);
 
-protected:
-    QRectF handleRect(const QPointF &p, qreal radius) const;
     /**
      * @brief Add an arc.
      *
@@ -487,42 +456,10 @@ protected:
      *
      * @return number of points created by the curve
      */
-    int arcToCurve(qreal rx, qreal ry, qreal startAngle, qreal sweepAngle, const QPointF & offset, QPointF * curvePoints) const;
+    int arcToCurve(qreal rx, qreal ry, qreal startAngle, qreal sweepAngle, const QPointF &offset, QPointF *curvePoints) const;
 
     /// Returns the viewbox from the given xml element.
-    QRectF loadOdfViewbox(const KoXmlElement & element) const;
-    /// Applies the viewbox transformation defined in the given element
-    void applyViewboxTransformation(const KoXmlElement & element);
-
-    /**
-     * @brief Saves the node types
-     *
-     * This is inspired by inkscape and uses the same mechanism as they do.
-     * The only difference is that they use sodipodi:nodeTypes as element and
-     * we use koffice:nodeTyes as attribute.
-     * This attribute contains of a string which has the node type of each point
-     * in it. The following node types exist:
-     *
-     * c corner
-     * s smooth
-     * z symetric
-     *
-     * The first point of a path is always of the type c.
-     * If the path is closed the type of the first point is saved in the last elemeent
-     * E.g. you have a closed path with 2 points in it. The first one (start/end of path)
-     * is symetric and the second one is smooth that will result in the nodeType="czs"
-     * So if there is a closed sub path the nodeTypes contain one more entry then there
-     * are points. That is due to the first and the last pojnt of a closed sub path get
-     * merged into one when they are on the same position.
-     *
-     * @return The node types as string
-     */
-    QString nodeTypes() const;
-
-    /**
-     * @brief Loads node types
-     */
-    void loadNodeTypes(const KoXmlElement & element);
+    QRectF loadOdfViewbox(const KoXmlElement &element) const;
 
     /**
      * Get the resize matrix
@@ -530,7 +467,7 @@ protected:
      * This makes sure that also if the newSize isNull that there will be a
      * very small size of 0.000001 pixels
      */
-    QMatrix resizeMatrix( const QSizeF & newSize ) const;
+    QMatrix resizeMatrix( const QSizeF &newSize ) const;
 
     KoSubpathList m_subpaths;
 
