@@ -31,13 +31,14 @@
 #include <QLabel>
 
 #include <KoCanvasController.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 #include <KoColorBackground.h>
 #include <KoFind.h>
 #include <KoTextDocumentLayout.h>
 #include <KoToolManager.h>
 #include <KoToolProxy.h>
 #include <KoZoomHandler.h>
+#include <KoStandardAction.h>
 #include <KoToolBoxFactory.h>
 #include <KoShapeController.h>
 #include <KoShapeManager.h>
@@ -112,7 +113,6 @@ public:
     KAction *deleteSelectionAction;
 
     KToggleAction *actionViewSnapToGrid;
-    KToggleAction *actionViewShowGuides;
     KToggleAction *actionViewShowMasterPages;
 
     KAction *actionInsertPage;
@@ -215,12 +215,12 @@ void KoPAView::initGUI()
     d->verticalRuler->setUnit(d->doc->unit());
     d->verticalRuler->setShowMousePosition(true);
 
-    new KoRulerController(d->horizontalRuler, d->canvas->resourceProvider());
+    new KoRulerController(d->horizontalRuler, d->canvas->resourceManager());
 
-    connect(d->doc, SIGNAL(unitChanged(KoUnit)),
-            d->horizontalRuler, SLOT(setUnit(KoUnit)));
-    connect(d->doc, SIGNAL(unitChanged(KoUnit)),
-            d->verticalRuler, SLOT(setUnit(KoUnit)));
+    connect(d->doc, SIGNAL(unitChanged(const KoUnit&)),
+            d->horizontalRuler, SLOT(setUnit(const KoUnit&)));
+    connect(d->doc, SIGNAL(unitChanged(const KoUnit&)),
+            d->verticalRuler, SLOT(setUnit(const KoUnit&)));
 
     gridLayout->addWidget(d->horizontalRuler, 0, 1);
     gridLayout->addWidget(d->verticalRuler, 1, 0);
@@ -234,23 +234,18 @@ void KoPAView::initGUI()
             this, SLOT(pageOffsetChanged()));
     connect(d->canvasController, SIGNAL(canvasMousePositionChanged(const QPoint&)),
             this, SLOT(updateMousePosition(const QPoint&)));
-    connect(d->verticalRuler, SIGNAL(guideLineCreated(Qt::Orientation, int)),
-            d->canvasController, SLOT(addGuideLine(Qt::Orientation, int)));
-    connect(d->horizontalRuler, SIGNAL(guideLineCreated(Qt::Orientation, int)),
-            d->canvasController, SLOT(addGuideLine(Qt::Orientation, int)));
+    d->verticalRuler->createGuideToolConnection(d->canvas);
+    d->horizontalRuler->createGuideToolConnection(d->canvas);
 
     KoToolBoxFactory toolBoxFactory(d->canvasController, i18n("Tools") );
-    createDockWidget( &toolBoxFactory );
+    if (shell())
+    {
+        shell()->createDockWidget( &toolBoxFactory );
 
-    KoDockerManager *dockerMng = dockerManager();
-    if (!dockerMng) {
-        dockerMng = new KoDockerManager(this);
-        setDockerManager(dockerMng);
+        connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*) ),
+             shell()->dockerManager(), SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*) ) );
     }
 
-    connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*) ),
-             dockerMng, SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*) ) );
-
     connect(shapeManager(), SIGNAL(selectionChanged()), this, SLOT(selectionChanged()));
     connect(d->canvas, SIGNAL(documentSize(const QSize&)), d->canvasController, SLOT(setDocumentSize(const QSize&)));
     connect(d->canvasController, SIGNAL(moveDocumentOffset(const QPoint&)),
@@ -258,7 +253,7 @@ void KoPAView::initGUI()
 
     if (shell()) {
         KoPADocumentStructureDockerFactory structureDockerFactory( KoDocumentSectionView::ThumbnailMode, d->doc->pageType() );
-        d->documentStructureDocker = qobject_cast<KoPADocumentStructureDocker*>( createDockWidget( &structureDockerFactory ) );
+        d->documentStructureDocker = qobject_cast<KoPADocumentStructureDocker*>( shell()->createDockWidget( &structureDockerFactory ) );
         connect( shell()->partManager(), SIGNAL( activePartChanged( KParts::Part * ) ),
                 d->documentStructureDocker, SLOT( setPart( KParts::Part * ) ) );
         connect(d->documentStructureDocker, SIGNAL(pageChanged(KoPAPageBase*)), this, SLOT(updateActivePage(KoPAPageBase*)));
@@ -297,13 +292,10 @@ void KoPAView::initActions()
     actionCollection()->addAction("view_snaptogrid", d->actionViewSnapToGrid);
     connect( d->actionViewSnapToGrid, SIGNAL( triggered( bool ) ), this, SLOT (viewSnapToGrid( bool )));
 
-    d->actionViewShowGuides  = new KToggleAction( KIcon( "guides" ), i18n( "Show Guides" ), this );
-    d->actionViewShowGuides->setChecked( d->doc->guidesData().showGuideLines() );
-    d->actionViewShowGuides->setCheckedState( KGuiItem( i18n( "Hide Guides" ) ) );
-    d->actionViewShowGuides->setToolTip( i18n( "Shows or hides guides" ) );
-    actionCollection()->addAction( "view_show_guides", d->actionViewShowGuides );
-    connect( d->actionViewShowGuides, SIGNAL(triggered(bool)),
-             this,                    SLOT(viewGuides(bool)) );
+    KToggleAction *actionViewShowGuides = KoStandardAction::showGuides(this, SLOT(viewGuides(bool)), this);
+    actionViewShowGuides->setChecked( d->doc->guidesData().showGuideLines() );
+    actionCollection()->addAction(KoStandardAction::name(KoStandardAction::ShowGuides),
+            actionViewShowGuides );
 
     d->actionViewShowMasterPages = new KToggleAction(i18n( "Show Master Pages" ), this );
     actionCollection()->addAction( "view_masterpages", d->actionViewShowMasterPages );
@@ -360,7 +352,7 @@ void KoPAView::initActions()
     actionCollection()->addAction("configure", d->actionConfigure);
     connect(d->actionConfigure, SIGNAL(triggered()), this, SLOT(configure()));
 
-    d->find = new KoFind( this, d->canvas->resourceProvider(), actionCollection() );
+    d->find = new KoFind( this, d->canvas->resourceManager(), actionCollection() );
 
     actionCollection()->action( "object_group" )->setShortcut( QKeySequence( "Ctrl+G" ) );
     actionCollection()->action( "object_ungroup" )->setShortcut( QKeySequence( "Ctrl+Shift+G" ) );
@@ -663,15 +655,15 @@ void KoPAView::doUpdateActivePage( KoPAPageBase * page )
     KoPageLayout &layout = d->activePage->pageLayout();
     d->horizontalRuler->setRulerLength(layout.width);
     d->verticalRuler->setRulerLength(layout.height);
-    d->horizontalRuler->setActiveRange(layout.left, layout.width - layout.right);
-    d->verticalRuler->setActiveRange(layout.top, layout.height - layout.bottom);
+    d->horizontalRuler->setActiveRange(layout.leftMargin, layout.width - layout.rightMargin);
+    d->verticalRuler->setActiveRange(layout.topMargin, layout.height - layout.bottomMargin);
 
     QSizeF pageSize( layout.width, layout.height );
     d->canvas->setDocumentOrigin(QPointF(layout.width, layout.height));
     // the page is in the center of the canvas
     d->zoomController->setDocumentSize(pageSize * 3);
     d->zoomController->setPageSize(pageSize);
-    d->canvas->resourceProvider()->setResource( KoCanvasResource::PageSize, pageSize );
+    d->canvas->resourceManager()->setResource( KoCanvasResource::PageSize, pageSize );
 
     d->canvas->update();
 
@@ -725,7 +717,7 @@ void KoPAView::setActivePage( KoPAPageBase* page )
     }
     
     // Set the current page number in the canvas resource provider
-    d->canvas->resourceProvider()->setResource( KoCanvasResource::CurrentPage, d->doc->pageIndex(d->activePage)+1 );
+    d->canvas->resourceManager()->setResource( KoCanvasResource::CurrentPage, d->doc->pageIndex(d->activePage)+1 );
 }
 
 void KoPAView::navigatePage( KoPageApp::PageNavigation pageNavigation )
@@ -1113,4 +1105,4 @@ bool KoPAView::isMasterUsed( KoPAPageBase * page )
     return used;
 }
 
-#include "KoPAView.moc"
+#include <KoPAView.moc>
