@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
- * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
+*  Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +25,7 @@
 
 #include <QTextBlock>
 #include <QTextCursor>
+#include <QFontMetrics>
 
 #include <KoOdfLoadingContext.h>
 #include <KoOdfStylesReader.h>
@@ -116,9 +118,10 @@ void KoCharacterStyle::Private::ensureMinimalProperties(QTextCursor &cursor, boo
     QTextCharFormat format = cursor.charFormat();
     QList<int> keys = hardCodedDefaultStyle.keys();
     for (int i = 0; i < keys.count(); i++) {
-        QVariant variant = hardCodedDefaultStyle.value(keys.at(i));
-        if (!variant.isNull() && !format.hasProperty(keys.at(i))) {
-            format.setProperty(keys.at(i), variant);
+        const int key = keys.at(i);
+        QVariant variant = hardCodedDefaultStyle.value(key);
+        if (!variant.isNull() && !format.hasProperty(key)) {
+            format.setProperty(key, variant);
         }
     }
     cursor.mergeCharFormat(format);
@@ -164,8 +167,7 @@ QPen KoCharacterStyle::textOutline() const
 {
     QVariant variant = d->stylesPrivate.value(QTextFormat::TextOutline);
     if (variant.isNull()) {
-        QPen pen(Qt::NoPen);
-        return pen;
+        return QPen(Qt::NoPen);
     }
     return qvariant_cast<QPen>(variant);
 }
@@ -175,8 +177,7 @@ QBrush KoCharacterStyle::background() const
     QVariant variant = d->stylesPrivate.value(QTextFormat::BackgroundBrush);
 
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -190,8 +191,7 @@ QBrush KoCharacterStyle::foreground() const
 {
     QVariant variant = d->stylesPrivate.value(QTextFormat::ForegroundBrush);
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -289,8 +289,10 @@ static void importOdfLine(const QString &type, const QString &style, const QStri
         fixedType.clear();
     else if (fixedType.isEmpty() && !fixedStyle.isEmpty())
         fixedType = "single";
-    else if (!fixedType.isEmpty() && fixedStyle.isEmpty())
+    else if (!fixedType.isEmpty() && fixedType != "none" && fixedStyle.isEmpty()) {
+        // don't set a style when the type is none
         fixedStyle = "solid";
+    }
 
     if (fixedType == "single")
         lineType = KoCharacterStyle::SingleLine;
@@ -700,7 +702,7 @@ QFont::Capitalization KoCharacterStyle::fontCapitalization() const
 
 void KoCharacterStyle::setCountry(const QString &country)
 {
-    if (country.isNull())
+    if (country.isEmpty())
         d->stylesPrivate.remove(KoCharacterStyle::Country);
     else
         d->setProperty(KoCharacterStyle::Country, country);
@@ -708,7 +710,7 @@ void KoCharacterStyle::setCountry(const QString &country)
 
 void KoCharacterStyle::setLanguage(const QString &language)
 {
-    if (language.isNull())
+    if (language.isEmpty())
         d->stylesPrivate.remove(KoCharacterStyle::Language);
     else
         d->setProperty(KoCharacterStyle::Language, language);
@@ -729,6 +731,76 @@ bool KoCharacterStyle::hasProperty(int key) const
     return d->stylesPrivate.contains(key);
 }
 
+static KoCharacterStyle::RotationAngle intToRotationAngle(int angle)
+{
+    KoCharacterStyle::RotationAngle rotationAngle = KoCharacterStyle::Zero;
+    if (angle == 90) {
+        rotationAngle = KoCharacterStyle::Ninety;
+    } else if (angle == 270) {
+        rotationAngle = KoCharacterStyle::TwoHundredSeventy;
+    }
+    return rotationAngle;
+}
+
+static int rotationAngleToInt(KoCharacterStyle::RotationAngle rotationAngle)
+{
+    int angle = 0;
+    if (rotationAngle == KoCharacterStyle::Ninety) {
+        angle = 90;
+    } else if (rotationAngle == KoCharacterStyle::TwoHundredSeventy) {
+        angle = 270;
+    }
+    return angle;
+}
+
+static QString rotationScaleToString(KoCharacterStyle::RotationScale rotationScale)
+{
+    QString scale = "line-height";
+    if (rotationScale == KoCharacterStyle::Fixed) {
+        scale = "fixed";
+    }
+    return scale;
+}
+
+static KoCharacterStyle::RotationScale stringToRotationScale(const QString &scale)
+{
+    KoCharacterStyle::RotationScale rotationScale = KoCharacterStyle::LineHeight;
+    if (scale == "fixed") {
+        rotationScale = KoCharacterStyle::Fixed;
+    }
+    return rotationScale;
+}
+
+void KoCharacterStyle::setTextRotationAngle(RotationAngle angle)
+{
+    d->setProperty(TextRotationAngle, rotationAngleToInt(angle));
+}
+
+KoCharacterStyle::RotationAngle KoCharacterStyle::textRotationAngle() const
+{
+    return intToRotationAngle(d->propertyInt(TextRotationAngle));
+}
+
+void KoCharacterStyle::setTextRotationScale(RotationScale scale)
+{
+    d->setProperty(TextRotationScale, rotationScaleToString(scale));
+}
+
+KoCharacterStyle::RotationScale KoCharacterStyle::textRotationScale() const
+{
+    return stringToRotationScale(d->propertyString(TextRotationScale));
+}
+
+void KoCharacterStyle::setTextScale(int scale)
+{
+    d->setProperty(TextScale, scale);
+}
+
+int KoCharacterStyle::textScale() const
+{
+    return d->propertyInt(TextScale);
+}
+
 //in 1.6 this was defined in KoTextFormat::load(KoOasisContext &context)
 void KoCharacterStyle::loadOdf(KoOdfLoadingContext &context)
 {
@@ -744,7 +816,7 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext &context)
         if (fontFace != 0)
             fontName = fontFace->attributeNS(KoXmlNS::svg, "font-family", "");
     }
-    if (! fontName.isNull()) {
+    if (! fontName.isEmpty()) {
     // Hmm, the remove "'" could break it's in the middle of the fontname...
         fontName = fontName.remove('\'');
 
@@ -808,7 +880,7 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
     if (styleStack.hasProperty(KoXmlNS::style, "font-family"))
         fontName = styleStack.property(KoXmlNS::style, "font-family");
 
-    if (! fontName.isNull()) {
+    if (! fontName.isEmpty()) {
         // Hmm, the remove "'" could break it's in the middle of the fontname...
         fontName = fontName.remove('\'');
 
@@ -969,6 +1041,16 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
             setVerticalAlignment(QTextCharFormat::AlignSuperScript);
         else if (textPosition.startsWith("sub"))
             setVerticalAlignment(QTextCharFormat::AlignSubScript);
+        else {
+            QRegExp re("(-?[\\d.]+)%.*");
+            if (re.exactMatch(textPosition)) {
+                float value = re.capturedTexts()[1].toFloat();
+                if (value > 0)
+                    setVerticalAlignment(QTextCharFormat::AlignSuperScript);
+                else if (value < 0)
+                    setVerticalAlignment(QTextCharFormat::AlignSubScript);
+            }
+        }
     }
 
     // The fo:font-variant attribute provides the option to display text as small capitalized letters.
@@ -1035,6 +1117,12 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
         }
     }
 
+    if (styleStack.hasProperty(KoXmlNS::fo, "letter-spacing")) {
+        qreal space = KoUnit::parseValue(styleStack.property(KoXmlNS::fo, "letter-spacing"));
+        QFontMetrics fm(font());
+        setFontLetterSpacing(100+100*space/fm.averageCharWidth());
+    }
+
     if (styleStack.hasProperty(KoXmlNS::style, "text-outline")) {
         if (styleStack.property(KoXmlNS::style, "text-outline") == "true") {
             setTextOutline(QPen((foreground().style() != Qt::NoBrush)?foreground():QBrush(Qt::black) , 0));
@@ -1044,6 +1132,20 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
         }
     }
 
+    if (styleStack.hasProperty(KoXmlNS::style, "text-rotation-angle")) {
+        int angle = styleStack.property(KoXmlNS::style, "text-rotation-angle").toInt();
+        setTextRotationAngle(intToRotationAngle(angle));
+    }
+
+    if (styleStack.hasProperty(KoXmlNS::style, "text-rotation-scale")) {
+        const QString scale = styleStack.property(KoXmlNS::style, "text-rotation-scale");
+        setTextRotationScale(stringToRotationScale(scale));
+    }
+
+    if (styleStack.hasProperty(KoXmlNS::style, "text-scale")) {
+        const int scale = styleStack.property(KoXmlNS::style, "text-scale").toInt();
+        setTextScale(scale);
+    }
 
 //TODO
 #if 0
@@ -1227,11 +1329,22 @@ void KoCharacterStyle::saveOdf(KoGenStyle &style)
             style.addProperty("fo:country", d->stylesPrivate.value(KoCharacterStyle::Country).toString(), KoGenStyle::TextType);
         } else if (key == KoCharacterStyle::Language) {
             style.addProperty("fo:language", d->stylesPrivate.value(KoCharacterStyle::Language).toString(), KoGenStyle::TextType);
+        } else if (key == QTextCharFormat::FontLetterSpacing) {
+            style.addProperty("fo:letter-spacing", (int) fontLetterSpacing(), KoGenStyle::TextType);
         } else if (key == QTextFormat::TextOutline) {
             QPen outline = textOutline();
             style.addProperty("style:text-outline", outline.style() == Qt::NoPen ? "false" : "true", KoGenStyle::TextType);
         } else if (key == KoCharacterStyle::FontCharset) {
             style.addProperty("style:font-charset", d->stylesPrivate.value(KoCharacterStyle::FontCharset).toString(), KoGenStyle::TextType);
+        } else if (key == KoCharacterStyle::TextRotationAngle) {
+            RotationAngle angle = textRotationAngle();
+            style.addProperty("style:text-rotation-angle", rotationAngleToInt(angle), KoGenStyle::TextType);
+        } else if (key == KoCharacterStyle::TextRotationScale) {
+            RotationScale scale = textRotationScale();
+            style.addProperty("style:text-rotation-scale", rotationScaleToString(scale), KoGenStyle::TextType);
+        } else if (key == KoCharacterStyle::TextScale) {
+            int scale = textScale();
+            style.addProperty("style:text-scale", scale, KoGenStyle::TextType);
         }
     }
     //TODO: font name and family
@@ -1242,4 +1355,9 @@ QVariant KoCharacterStyle::value(int key) const
     return d->stylesPrivate.value(key);
 }
 
-#include "KoCharacterStyle.moc"
+void KoCharacterStyle::removeHardCodedDefaults()
+{
+    d->hardCodedDefaultStyle.clearAll();
+}
+
+#include <KoCharacterStyle.moc>
