@@ -1,11 +1,14 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007,2008 Sebastian Sauer <mail@dipe.org>
+ * Copyright (C) 2007,ducroquet Pierre Ducroquet <pinaraf@gmail.com>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; version 2.
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,6 +41,7 @@ class KoGenStyle;
 class KoGenStyles;
 #include "KoXmlReaderForward.h"
 class KoOdfLoadingContext;
+#include "KoBorder.h"
 
 /**
  * A container for all properties for the paragraph wide style.
@@ -68,8 +72,6 @@ public:
         DropCapsDistance,   ///< Distance between drop caps and text
         DropCapsTextStyle,  ///< Text style of dropped chars. Not supported yet.
         FollowDocBaseline,  ///< If true the baselines will be aligned with the doc-wide grid
-        BreakBefore,    ///< If true, insert a frame break before this paragraph
-        BreakAfter,     ///< If true, insert a frame break after this paragraph
 
         // border stuff
         HasLeftBorder,  ///< If true, paint a border on the left
@@ -120,29 +122,13 @@ public:
         TextProgressionDirection,
 
         MasterPageName,         ///< Optional name of the master-page
-        
-        OutlineLevel            ///< Outline level for headings
+
+        OutlineLevel,            ///< Outline level for headings
+        DefaultOutlineLevel
 // do 15.5.24
 // continue at 15.5.28
     };
 
-    /// The type of paragraph border
-    enum BorderStyle {
-        BorderNone,   ///< no border. This value forces the computed value of 'border-width' to be '0'.
-        BorderDotted,   ///< The border is a series of dots.
-        BorderDashed,   ///< The border is a series of short line segments.
-        BorderSolid,    ///< The border is a single line segment.
-        BorderDouble,   ///< The border is two solid lines. The sum of the two lines and the space between them equals the value of 'border-width'.
-        BorderGroove,   ///< The border looks as though it were carved into the canvas.
-        BorderRidge,    ///< The opposite of 'groove': the border looks as though it were coming out of the canvas.
-        BorderInset,    ///< The border makes the entire box look as though it were embedded in the canvas.
-        BorderOutset,   ///< The opposite of 'inset': the border makes the entire box look as though it were coming out of the canvas.
-
-        // kword legacy
-        BorderDashDotPattern,
-        BorderDashDotDotPattern
-    };
-
     /// Constructor
     KoParagraphStyle(QObject *parent = 0);
     /// Creates a KoParagraphStyle with the given block format, the block character format and \a parent
@@ -330,9 +316,9 @@ public:
     qreal leftInnerBorderWidth();
     void setLeftBorderSpacing(qreal width);
     qreal leftBorderSpacing();
-    void setLeftBorderStyle(BorderStyle style);
-    BorderStyle leftBorderStyle();
-    void setLeftBorderColor(const QColor& color);
+    void setLeftBorderStyle(KoBorder::BorderStyle style);
+    KoBorder::BorderStyle leftBorderStyle();
+    void setLeftBorderColor(const QColor &color);
     QColor leftBorderColor();
     void setTopBorderWidth(qreal width);
     qreal topBorderWidth();
@@ -340,9 +326,9 @@ public:
     qreal topInnerBorderWidth();
     void setTopBorderSpacing(qreal width);
     qreal topBorderSpacing();
-    void setTopBorderStyle(BorderStyle style);
-    BorderStyle topBorderStyle();
-    void setTopBorderColor(const QColor& color);
+    void setTopBorderStyle(KoBorder::BorderStyle style);
+    KoBorder::BorderStyle topBorderStyle();
+    void setTopBorderColor(const QColor &color);
     QColor topBorderColor();
     void setRightBorderWidth(qreal width);
     qreal rightBorderWidth();
@@ -350,9 +336,9 @@ public:
     qreal rightInnerBorderWidth();
     void setRightBorderSpacing(qreal width);
     qreal rightBorderSpacing();
-    void setRightBorderStyle(BorderStyle style);
-    BorderStyle rightBorderStyle();
-    void setRightBorderColor(const QColor& color);
+    void setRightBorderStyle(KoBorder::BorderStyle style);
+    KoBorder::BorderStyle rightBorderStyle();
+    void setRightBorderColor(const QColor &color);
     QColor rightBorderColor();
     void setBottomBorderWidth(qreal width);
     qreal bottomBorderWidth();
@@ -360,9 +346,9 @@ public:
     qreal bottomInnerBorderWidth();
     void setBottomBorderSpacing(qreal width);
     qreal bottomBorderSpacing();
-    void setBottomBorderStyle(BorderStyle style);
-    BorderStyle bottomBorderStyle();
-    void setBottomBorderColor(const QColor& color);
+    void setBottomBorderStyle(KoBorder::BorderStyle style);
+    KoBorder::BorderStyle bottomBorderStyle();
+    void setBottomBorderColor(const QColor &color);
     QColor bottomBorderColor();
 
     KoText::Direction textProgressionDirection() const;
@@ -442,7 +428,7 @@ public:
     /// return the optional name of the master-page or a QString() if this paragraph isn't attached to a master-page.
     QString masterPageName() const;
     /// Set the name of the master-page.
-    void setMasterPageName(const QString& name);
+    void setMasterPageName(const QString &name);
 
 
     /// Set the list start value
@@ -480,6 +466,17 @@ public:
      * Change this block outline level
      */
     void setOutlineLevel(int outline);
+    
+    /**
+     * Return the default outline level of this style, or 0 if there is none.
+     */
+    int defaultOutlineLevel() const;
+
+    /**
+     * Change the default outline level for this style.
+     */
+    void setDefaultOutlineLevel(int outline);
+
 
     /// copy all the properties from the other style to this style, effectively duplicating it.
     void copyProperties(const KoParagraphStyle *style);
@@ -538,7 +535,7 @@ public:
      * @param context the odf loading context
      * @param element the element containing the
      */
-    void loadOdf(const KoXmlElement* element, KoOdfLoadingContext & context);
+    void loadOdf(const KoXmlElement *element, KoOdfLoadingContext &context);
 
     void saveOdf(KoGenStyle &style, KoGenStyles &mainStyles);
 
@@ -573,7 +570,7 @@ private:
      * Load the style from the \a KoStyleStack style stack using the
      * OpenDocument format.
      */
-    void loadOdfProperties(KoStyleStack& styleStack);
+    void loadOdfProperties(KoStyleStack &styleStack);
     qreal propertyDouble(int key) const;
     int propertyInt(int key) const;
     bool propertyBoolean(int key) const;
