@@ -24,33 +24,33 @@
 
 #include <QtCore/QDateTime>
 #include <QtGui/QMatrix>
-#include <Qt3Support/Q3PtrList>
+#include <QList>
 
 #include <kparts/part.h>
 #include <kservice.h>
 #include <kcomponentdata.h>
 
-#include "KoUnit.h"
-#include "KoPageLayout.h"
+#include <KoUnit.h>
+#include <KoPageLayout.h>
 #include "komain_export.h"
 #include "KoGridData.h"
 #include "KoGuidesData.h"
 #include <KoXmlReader.h>
+#include <KoOdfDocument.h>
 
-class QXmlSimpleReader;
 class QUndoCommand;
 
 class KoStore;
 class KoOdfReadStore;
 class KoOdfWriteStore;
 class KoMainWindow;
-class KoChild;
-class KoDocumentChild;
 class KoView;
 class KoDocumentInfo;
+class KoDocumentRdf;
+class KoDocumentRdfBase;
 class KoOpenPane;
-class KoEmbeddedDocumentSaver;
-
+class KUndoStack;
+class KoTextEditor;
 
 class KoVersionInfo
 {
@@ -70,26 +70,17 @@ public:
  *
  *  @short The %KOffice document class
  */
-class KOMAIN_EXPORT KoDocument : public KParts::ReadWritePart
+class KOMAIN_EXPORT KoDocument : public KParts::ReadWritePart, public KoOdfDocument
 {
     Q_OBJECT
 //     Q_PROPERTY( QByteArray dcopObjectId READ dcopObjectId)
     Q_PROPERTY(bool backupFile READ backupFile WRITE setBackupFile)
 
 public:
-    // context passed on saving to saveOdf
-    struct SavingContext {
-        SavingContext(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & embeddedSaver)
-                : odfStore(odfStore)
-                , embeddedSaver(embeddedSaver) {}
-
-        KoOdfWriteStore & odfStore;
-        KoEmbeddedDocumentSaver & embeddedSaver;
-    };
 
     /**
      *  Constructor.
-     * The first 4 arguments are the same as the ones passed to KParts::Factory::createPart.
+     * The first 3 arguments are the same as the ones passed to KParts::Factory::createPart.
      *
      * @param parentWidget the parent widget, in case we create a wrapper widget
      *        (in single view mode).
@@ -120,7 +111,7 @@ public:
     bool isSingleViewMode() const;
 
     /**
-     * Is the document embedded?
+     * @return true if the document is embedded in another odf document
      */
     bool isEmbedded() const;
 
@@ -233,16 +224,24 @@ public:
 
     /**
      * Returns the OASIS OpenDocument mimetype of the document, if supported
-     * This comes from the X-KDE-NativeOasisMimeType key in the .desktop file
+     * This comes from the X-KDE-NativeOasisMimeType key in the
+     * desktop file
+     *
+     * @return the oasis mimetype or, if it hasn't one, the nativeformatmimetype.
      */
     QByteArray nativeOasisMimeType() const;
 
+    enum ImportExportType {
+        ForExport,
+        ForImport
+    };
+
     /// Checks whether a given mimetype can be handled natively.
-    bool isNativeFormat(const QByteArray& mimetype) const;
+    bool isNativeFormat(const QByteArray& mimetype, ImportExportType importExportType) const;
 
     /// Returns a list of the mimetypes considered "native", i.e. which can
     /// be saved by KoDocument without a filter, in *addition* to the main one
-    virtual QStringList extraNativeMimeTypes() const;
+    virtual QStringList extraNativeMimeTypes(ImportExportType importExportType) const;
 
     /// Enum values used by specialOutputFlag - note that it's a bitfield for supportedSpecialFormats
     enum { /*SaveAsKOffice1dot1 = 1,*/ // old and removed
@@ -364,25 +363,6 @@ public:
      */
     virtual KParts::Part *hitTest(QWidget *widget, const QPoint &globalPos);
 
-    /**
-     *  Find the most nested child document which contains the
-     *  requested point. The point is in the coordinate system
-     *  of this part. If no child document contains this point, then
-     *  a pointer to this document is returned.
-     *
-     *  This method can to be reimplemented if the document features child documents.
-     *
-     *  @param pos is in (unzoomed) document coordinates
-     *  @param view the view in which we are testing for a hit. This is used to implement
-     *         logic like "only hit an embedded object if it's selected in the current view"
-     *  @param matrix transforms points from the documents coordinate system
-     *         to the coordinate system of the requested point. This is used by
-     *         transformed child documents, see KoDocumentChild/KoChild.
-     *
-     *  @return Pointer to the document under the mouse at that position
-     */
-    virtual KoDocument *hitTest(const QPoint &pos, KoView* view, const QMatrix& matrix = QMatrix());
-
     /**
      *  Paints the whole document into the given painter object.
      *
@@ -398,28 +378,6 @@ public:
      */
     virtual QPixmap generatePreview(const QSize& size);
 
-    /**
-     *  Paints all of the documents children into the given painter object.
-     *
-     *  @param painter     The painter object onto which will be drawn.
-     *  @param rect        The rect that should be used in the painter object.
-     *  @param view        The KoView is needed to fiddle about with the active widget.
-     *
-     *  @see #paintChild #paintEverything #paintContent
-     */
-    virtual void paintChildren(QPainter &painter, const QRect &rect, KoView *view);
-
-    /**
-     *  Paint a given child. Normally called by paintChildren().
-     *
-     *  @param child       The child to be painted.
-     *  @param painter     The painter object onto which will be drawn.
-     *  @param view        The KoView is needed to fiddle about with the active widget.
-     *
-     *  @see #paintEverything #paintChildren #paintContent
-     */
-    virtual void paintChild(KoDocumentChild *child, QPainter &painter, KoView *view);
-
     /**
      *  Paints the data itself. Normally called by paintEverything(). It does not
      *  paint the children.
@@ -454,9 +412,7 @@ public:
     /**
      *  @return true if the document is empty.
      */
-    virtual bool isEmpty() const {
-        return m_bEmpty;
-    }
+    virtual bool isEmpty() const;
 
     /**
      *  @brief Sets the document to empty.
@@ -466,9 +422,7 @@ public:
      *
      *  @see isEmpty()
      */
-    virtual void setEmpty() {
-        m_bEmpty = true;
-    }
+    virtual void setEmpty();
 
     /**
      *  @brief Loads a document from a store.
@@ -499,19 +453,6 @@ public:
      */
     virtual bool loadXML(const KoXmlDocument & doc, KoStore * store) = 0;
 
-    /**
-     *  Reimplement this method to load the contents of your %KOffice document,
-     *  from the XML document ("content.xml"). The styles have been parsed already,
-     *  you can find them in the odfStore.styles(). The store can be used
-     *  to load images and embedded documents.
-     */
-    virtual bool loadOdf(KoOdfReadStore & odfStore) = 0;
-
-    /**
-     *  Reimplement this method to save the contents of your %KOffice document,
-     *  using the ODF format.
-     */
-    virtual bool saveOdf(SavingContext & documentContext) = 0;
 
     /**
      *  Reimplement this to save the contents of the %KOffice document into
@@ -524,9 +465,8 @@ public:
      *  including processing instruction, complete DOCTYPE tag (with systemId and publicId), and root element.
      *  @param tagName the name of the tag for the root element
      *  @param version the DTD version (usually the application's version).
-     *  @deprecated use KoOdfWriteStore::createOasisXmlWriter instead
      */
-    KDE_DEPRECATED QDomDocument createDomDocument(const QString& tagName, const QString& version) const;
+    QDomDocument createDomDocument(const QString& tagName, const QString& version) const;
 
     /**
      *  Return a correctly created QDomDocument for an old (1.3-style) %KOffice document,
@@ -568,6 +508,8 @@ public:
      */
     virtual bool saveNativeFormat(const QString & file);
 
+public:
+
     /**
      * Activate/deactivate/configure the autosave feature.
      * @param delay in seconds, 0 to disable
@@ -602,24 +544,7 @@ public:
      * Retrieve the default value for autosave in seconds.
      * Called by the applications to use the correct default in their config
      */
-    static int defaultAutoSave() {
-        return s_defaultAutoSave;
-    }
-
-    /**
-     * @return the list of all children. Do not modify the
-     *         returned list.
-     */
-    const Q3PtrList<KoDocumentChild>& children() const;
-
-    /**
-     * @return the KoDocumentChild associated with the given Document, but only if
-     *         @p doc is a direct child of this document.
-     *
-     * This is a convenience function. You could get the same result
-     * by traversing the list returned by children().
-     */
-    KoDocumentChild *child(KoDocument *doc);
+    static int defaultAutoSave();
 
     /**
      * @return the information concerning this document.
@@ -627,8 +552,18 @@ public:
      */
     KoDocumentInfo *documentInfo() const;
 
-    void setViewBuildDocument(KoView *view, const QDomDocument &doc);
-    QDomDocument viewBuildDocument(KoView *view);
+    /**
+     * @return the Rdf metadata for this document.
+     * This method should only be used by code that links to
+     * the RDF system and needs full access to the KoDocumentRdf object.
+     * @see KoDocumentRdf
+     */
+    KoDocumentRdf *documentRdf() const;
+    /**
+     * @return the Rdf metadata for this document.
+     * @see KoDocumentRdf
+     */
+    KoDocumentRdfBase *documentRdfBase() const;
 
     /**
      * Appends the shell to the list of shells which show this
@@ -648,7 +583,7 @@ public:
     /**
      * @return the list of shells for the main window
      */
-    const Q3PtrList<KoMainWindow>& shells() const;
+    const QList<KoMainWindow*>& shells() const;
 
     /**
      * @return the number of shells for the main window
@@ -658,23 +593,10 @@ public:
     /**
      * @return the list of all the currently opened documents
      */
-    static Q3PtrList<KoDocument> *documentList() {
+    static QList<KoDocument*> *documentList() {
         return s_documentList;
     }
 
-    /**
-     * @brief Return a DCOP interface for this document
-     *
-     * %KOffice parts are strongly recommended to reimplement this method,
-     * so that their DCOP interface provides more functionality than the basic KoDocumentIface
-     */
-//     virtual DCOPObject * dcopObject();
-
-    /**
-     * @return the ID of the DCOP interface for this document.
-     **/
-//     QByteArray dcopObjectId() const;
-
     /**
      * Signal the progress of operations such as loading or saving.
      */
@@ -682,10 +604,6 @@ public:
         emit sigProgress(value);
     }
 
-    bool isInOperation() const;
-    virtual void emitBeginOperation();
-    virtual void emitEndOperation();
-
     /**
      * Return true if url() is a real filename, false if url() is
      * an internal url in the store, like "tar:/..."
@@ -699,6 +617,7 @@ public:
      * @see KoPageLayout
      */
     virtual KoPageLayout pageLayout(int pageNumber = 0) const;
+    virtual void setPageLayout(const KoPageLayout &pageLayout);
 
     /**
      * Performs a cleanup of unneeded backup files
@@ -719,23 +638,8 @@ public:
      */
     bool isLoading() const;
 
-    int queryCloseExternalChildren();
     int queryCloseDia();
 
-    /**
-     * @brief Set when we do not want to save external children when saving our 'main' doc.
-     *
-     * This makes it possible to save 'main' doc + all its internal children first, then
-     * go on to save external children. Typically used by query close.
-     * Use:
-     * @code
-     *      doc->setDoNotSaveExtDoc();
-     *      doc->save();    // saves doc and its internal children,
-     *                            //also calls saveExternalChildren() which sets setDoNotSaveExtDoc(false)
-     *      doc->saveExternalChildren();
-     * @endcode
-     */
-    void setDoNotSaveExtDoc(bool on = true);
 
     /**
      * Sets the backup path of the document
@@ -762,6 +666,16 @@ public:
 
     void setTitleModified(const QString &caption, bool mod);
 
+    /**
+     * @return caption of the document
+     *
+     * Caption is of the form "[title] - [url]",
+     * built out of the document info (title) and pretty-printed
+     * document URL.
+     * If the title is not present, only the URL it returned.
+     */
+    QString caption() const;
+
     /**
      * Sets the document URL to empty URL
      * KParts doesn't allow this, but %KOffice apps have e.g. templates
@@ -804,7 +718,14 @@ public:
     /**
      * Sets the unit used to display all measures/distances.
      */
-    void setUnit(KoUnit u);
+    void setUnit(const KoUnit &unit);
+
+    /**
+     * Save the unit to the settings writer
+     *
+     * @param settingsWriter
+     */
+    void saveUnitOdf(KoXmlWriter* settingsWriter) const;
 
     /**
      * Returns the name of the unit used to display all measures/distances.
@@ -860,6 +781,10 @@ public slots:
      */
     virtual void initEmpty();
 
+    /**
+     * Returns the global undo stack
+     */
+    KUndoStack *undoStack();
     /**
      * Adds a command to the undo stack and executes it by calling the redo() function.
      * @param command command to add to the undo stack
@@ -896,17 +821,7 @@ signals:
      * It is common to connect views to it, in order to change the displayed units
      * (e.g. in the rulers)
      */
-    void unitChanged(KoUnit);
-
-    /**
-     * This signal is emitted when a direct or indirect child document changes
-     * and needs to be updated in all views.
-     *
-     * If one of your child documents emits the childChanged signal, then you may
-     * usually just want to redraw this child. In this case you can ignore the parameter
-     * passed by the signal.
-     */
-    void childChanged(KoDocumentChild *child);
+    void unitChanged(const KoUnit &unit);
 
     /**
      * Progress info while loading or saving. The value is in percents (i.e. a number between 0 and 100)
@@ -927,9 +842,6 @@ signals:
      */
     void clearStatusBarMessage();
 
-    void beginOperation();
-    void endOperation();
-
     /**
     * Emitted when the document is modified
     */
@@ -977,6 +889,8 @@ protected:
      *  Applies a filter if necessary, and calls loadNativeFormat in any case
      *  You should not have to reimplement, except for very special cases.
      *
+     * NOTE: this method also creates a new KoView instance!
+     *
      * This method is called from the KReadOnlyPart::openUrl method.
      */
     virtual bool openFile();
@@ -993,7 +907,7 @@ protected:
      * Override this method in your derived class to show a widget in the startup 'dialog'.
      * This widget should allow the user to set settings for a custom document (i.e. one
      * not based on a template).
-     * The returned widget should provide its own button (preferrably 'Create') and
+     * The returned widget should provide its own button (preferably 'Create') and
      * implement the logic to implement the document instance correctly.
      * After initializing the widget should emit a signal called 'documentSelected()' which
      * will remove the startupWidget and show the document.
@@ -1002,43 +916,6 @@ protected:
      */
     virtual QList<CustomDocumentWidgetItem> createCustomDocumentWidgets(QWidget *parent);
 
-    /**
-     *  OLD XML method. For OASIS just call KoDocumentChild::loadOasisDocument
-     *  after KoDocumentChild::loadOasis.
-     *
-     *  You need to overload this function if your document may contain
-     *  embedded documents. This function is called to load embedded documents.
-     *
-     *  An example implementation may look like this:
-     *  @code
-     *  QPtrListIterator<KoDocumentChild> it( children() );
-     *  for( ; it.current(); ++it )
-     *  {
-     *    if ( !it.current()->loadDocument( _store ) )
-     *    {
-     *      return false;
-     *    }
-     *  }
-     *  return true;
-     *  @endcode
-     */
-    virtual bool loadChildren(KoStore*);
-
-    /**
-     *  Saves all internal children (only!).
-     *  @see saveExternalChildren if you have external children.
-     *  Returns true on success.
-     */
-    virtual bool saveChildren(KoStore* store);
-
-    /**
-     *  Saves all internal children (only!), to the store, using the OASIS format.
-     *  This is called automatically during saveNativeFormat.
-     *  @see saveExternalChildren if you have external children.
-     *  Returns true on success.
-     */
-    virtual bool saveChildrenOdf(SavingContext & documentContext);
-
     /**
      *  Overload this function if you have to load additional files
      *  from a store. This function is called after loadXML()
@@ -1058,33 +935,12 @@ protected:
      */
     virtual bool completeSaving(KoStore* store);
 
-    /**
-     * Inserts the new child in the list of children and emits the
-     * childChanged() signal.
-     *
-     * At the same time this method marks this document as modified.
-     *
-     * To remove a child, just delete it. KoDocument will detect this
-     * and remove the child from its lists.
-     *
-     * @see #isModified
-     */
-    virtual void insertChild(KoDocumentChild *child);
 
     /** @internal */
     virtual void setModified() {
         KParts::ReadWritePart::setModified();
     }
 
-    KoPageLayout m_pageLayout;
-
-    /**
-     *  Saves all externally stored children.
-     *  Returns true on success.
-     * @see #saveChildren for internal children
-     */
-    virtual bool saveExternalChildren();
-
     /**
      *  Returns whether or not the current openUrl() or openFile() call is
      *  actually an import operation (like File --> Import).
@@ -1110,21 +966,35 @@ protected:
     KoOpenPane* createOpenPane(QWidget* parent, const KComponentData &instance,
                                const QString& templateType = QString());
 
+
+
+    /// to satisfy KoOdfDocument where it overlaps with kparts
+    virtual KUrl getOdfUrl() const {
+        return url();
+    }
+
+    virtual void setOdfUrl( const KUrl& url ) {
+        setUrl( url );
+    }
+
 private slots:
-    void slotChildChanged(KoChild *c);
-    void slotChildDestroyed();
+
     void slotAutoSave();
     void slotStarted(KIO::Job*);
     void startCustomDocument();
+
     /**
-     * Removes the open widget showed at application start up.
+     * Removes the open widget shown at application start up.
      */
     void deleteOpenPane();
 
 private:
-    virtual void insertChild(QObject *) {
-        Q_ASSERT(0);
-    } // avoid compiler warning, but don't call deprecated insertChild() in QObject
+
+    bool saveNativeFormatODF(KoStore* store, const QByteArray &mimeType);
+    bool saveNativeFormatKOffice(KoStore* store);
+
+    /// @return the current KoMainWindow shell
+    KoMainWindow* currentShell();
 
     KService::Ptr nativeService();
     bool oldLoadAndParse(KoStore* store, const QString& filename, KoXmlDocument& doc);
@@ -1133,12 +1003,14 @@ private:
 
     bool savePreview(KoStore* store);
     bool saveOasisPreview(KoStore* store, KoXmlWriter* manifestWriter);
+
+    QString prettyPathOrUrl() const;
+
     class Private;
     Private * const d;
-    KService::Ptr m_nativeService;
-    bool m_bEmpty;
-    static Q3PtrList<KoDocument> *s_documentList;
-    static const int s_defaultAutoSave;
+
+
+    static QList<KoDocument*> *s_documentList;
 };
 
 #endif
