@@ -22,15 +22,15 @@
 #ifndef _KOFFICE_VERSION_H_
 #define _KOFFICE_VERSION_H_
 
-#define KOFFICE_VERSION_STRING "2.0 beta-7"
-#define KOFFICE_VERSION_MAJOR 1
-#define KOFFICE_VERSION_MINOR 9
-#define KOFFICE_VERSION_RELEASE 98
+#define KOFFICE_VERSION_STRING "2.2.1"
+#define KOFFICE_VERSION_MAJOR 2
+#define KOFFICE_VERSION_MINOR 2
+#define KOFFICE_VERSION_RELEASE 1
 #define KOFFICE_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))
 
-//#define KOFFICE_ALPHA 10
-#define KOFFICE_BETA 7
-//#define KOFFICE_RC
+// #define KOFFICE_ALPHA 1
+// #define KOFFICE_BETA 2
+// #define KOFFICE_RC 1
 
 
 #define KOFFICE_VERSION \
