@@ -1,8 +1,8 @@
 /* This file is part of the KDE project
 
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
-   Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
-   Copyright (C) 2008 Jan Hambrecht <jaham@gmx.net>
+   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+   Copyright (C) 2008-2009 Jan Hambrecht <jaham@gmx.net>
    Copyright (C) 2008 Casper Boemann <cbr@boemann.dk>
 
    This library is free software; you can redistribute it and/or
@@ -24,12 +24,13 @@
 #include "DefaultTool.h"
 #include "DefaultToolWidget.h"
 #include "DefaultToolArrangeWidget.h"
-#include "SnapGuideConfigWidget.h"
 #include "SelectionDecorator.h"
 #include "ShapeMoveStrategy.h"
 #include "ShapeRotateStrategy.h"
 #include "ShapeShearStrategy.h"
 #include "ShapeResizeStrategy.h"
+#include "guidestool/GuidesTool.h"
+#include "guidestool/GuidesToolFactory.h" // for the ID
 
 #include <KoPointerEvent.h>
 #include <KoToolSelection.h>
@@ -44,7 +45,7 @@
 #include <KoDrag.h>
 #include <KoDocument.h>
 #include <KoCanvasBase.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 #include <KoShapeRubberSelectStrategy.h>
 #include <commands/KoShapeMoveCommand.h>
 #include <commands/KoShapeDeleteCommand.h>
@@ -52,7 +53,6 @@
 #include <commands/KoShapeGroupCommand.h>
 #include <commands/KoShapeUngroupCommand.h>
 #include <KoSnapGuide.h>
-#include <tools/KoGuidesTool.h>
 
 #include <KAction>
 #include <QKeyEvent>
@@ -181,70 +181,70 @@ DefaultTool::~DefaultTool()
     delete m_guideLine;
 }
 
-bool DefaultTool::wantsAutoScroll()
+bool DefaultTool::wantsAutoScroll() const
 {
     return true;
 }
 
 void DefaultTool::setupActions()
 {
-    KAction* actionBringToFront = new KAction(KIcon("object-order-front"),
+    KAction* actionBringToFront = new KAction(KIcon("object-order-front-koffice"),
                                                i18n("Bring to &Front"), this);
     addAction("object_order_front", actionBringToFront);
     actionBringToFront->setShortcut(QKeySequence("Ctrl+Shift+]"));
     connect(actionBringToFront, SIGNAL(triggered()), this, SLOT(selectionBringToFront()));
 
-    KAction* actionRaise = new KAction(KIcon("object-order-raise"), i18n("&Raise"), this);
+    KAction* actionRaise = new KAction(KIcon("object-order-raise-koffice"), i18n("&Raise"), this);
     addAction("object_order_raise", actionRaise);
     actionRaise->setShortcut(QKeySequence("Ctrl+]"));
     connect(actionRaise, SIGNAL(triggered()), this, SLOT(selectionMoveUp()));
 
-    KAction* actionLower = new KAction(KIcon("object-order-lower"), i18n("&Lower"), this);
+    KAction* actionLower = new KAction(KIcon("object-order-lower-koffice"), i18n("&Lower"), this);
     addAction("object_order_lower", actionLower);
     actionLower->setShortcut(QKeySequence("Ctrl+["));
     connect(actionLower, SIGNAL(triggered()), this, SLOT(selectionMoveDown()));
 
-    KAction* actionSendToBack = new KAction(KIcon("object-order-back"),
+    KAction* actionSendToBack = new KAction(KIcon("object-order-back-koffice"),
                                              i18n("Send to &Back"), this);
     addAction("object_order_back", actionSendToBack);
     actionSendToBack->setShortcut(QKeySequence("Ctrl+Shift+["));
     connect(actionSendToBack, SIGNAL(triggered()), this, SLOT(selectionSendToBack()));
 
-    KAction* actionAlignLeft = new KAction(KIcon("object-align-horizontal-left"),
+    KAction* actionAlignLeft = new KAction(KIcon("object-align-horizontal-left-koffice"),
                                             i18n("Align Left"), this);
     addAction("object_align_horizontal_left", actionAlignLeft);
     connect(actionAlignLeft, SIGNAL(triggered()), this, SLOT(selectionAlignHorizontalLeft()));
 
-    KAction* actionAlignCenter = new KAction(KIcon("object-align-horizontal-center"),
+    KAction* actionAlignCenter = new KAction(KIcon("object-align-horizontal-center-koffice"),
                                               i18n("Horizontally Center"), this);
     addAction("object_align_horizontal_center", actionAlignCenter);
     connect(actionAlignCenter, SIGNAL(triggered()), this, SLOT(selectionAlignHorizontalCenter()));
 
-    KAction* actionAlignRight = new KAction(KIcon("object-align-horizontal-right"),
+    KAction* actionAlignRight = new KAction(KIcon("object-align-horizontal-right-koffice"),
                                              i18n("Align Right"), this);
     addAction("object_align_horizontal_right", actionAlignRight);
     connect(actionAlignRight, SIGNAL(triggered()), this, SLOT(selectionAlignHorizontalRight()));
 
-    KAction* actionAlignTop = new KAction(KIcon("object-align-vertical-top"), i18n("Align Top"), this);
+    KAction* actionAlignTop = new KAction(KIcon("object-align-vertical-top-koffice"), i18n("Align Top"), this);
     addAction("object_align_vertical_top", actionAlignTop);
     connect(actionAlignTop, SIGNAL(triggered()), this, SLOT(selectionAlignVerticalTop()));
 
-    KAction* actionAlignMiddle = new KAction(KIcon("object-align-vertical-center"),
+    KAction* actionAlignMiddle = new KAction(KIcon("object-align-vertical-center-koffice"),
                                               i18n("Vertically Center"), this);
     addAction("object_align_vertical_center", actionAlignMiddle);
     connect(actionAlignMiddle, SIGNAL(triggered()), this, SLOT(selectionAlignVerticalCenter()));
 
-    KAction* actionAlignBottom = new KAction(KIcon("object-align-vertical-bottom"),
+    KAction* actionAlignBottom = new KAction(KIcon("object-align-vertical-bottom-koffice"),
                                               i18n("Align Bottom"), this);
     addAction("object_align_vertical_bottom", actionAlignBottom);
     connect(actionAlignBottom, SIGNAL(triggered()), this, SLOT(selectionAlignVerticalBottom()));
 
-    KAction* actionGroupBottom = new KAction(KIcon("object-group"),
+    KAction* actionGroupBottom = new KAction(KIcon("object-group-koffice"),
                                               i18n("Group"), this);
     addAction("object_group", actionGroupBottom);
     connect(actionGroupBottom, SIGNAL(triggered()), this, SLOT(selectionGroup()));
 
-    KAction* actionUngroupBottom = new KAction(KIcon("object-ungroup"),
+    KAction* actionUngroupBottom = new KAction(KIcon("object-ungroup-koffice"),
                                                 i18n("Ungroup"), this);
     addAction("object_ungroup", actionUngroupBottom);
     connect(actionUngroupBottom, SIGNAL(triggered()), this, SLOT(selectionUngroup()));
@@ -369,14 +369,7 @@ void DefaultTool::updateCursor()
     QString statusText;
 
     if (koSelection()->count() > 0) { // has a selection
-        bool editable=false;
-        // check if we have at least one shape that is edtiable
-        foreach(KoShape *shape, koSelection()->selectedShapes(KoFlake::StrippedSelection)) {
-            if (shape->isEditable()) {
-                editable = true;
-                break;
-            }
-        }
+        bool editable=editableShapesCount(koSelection()->selectedShapes(KoFlake::StrippedSelection));
 
         if (!m_mouseWasInsideHandles) {
             m_angle = rotationOfHandle(m_lastHandle, true);
@@ -483,24 +476,24 @@ void DefaultTool::updateCursor()
         }
     }
     useCursor(cursor);
-    if (! m_currentStrategy)
+    if (currentStrategy() == 0)
         emit statusTextChanged(statusText);
 }
 
 void DefaultTool::paint(QPainter &painter, const KoViewConverter &converter)
 {
     KoInteractionTool::paint(painter, converter);
-    if (m_currentStrategy  == 0 && koSelection()->count() > 0) {
+    if (currentStrategy() == 0 && koSelection()->count() > 0) {
         SelectionDecorator decorator(m_mouseWasInsideHandles ? m_lastHandle : KoFlake::NoHandle,
                  true, true);
         decorator.setSelection(koSelection());
-        decorator.setHandleRadius(m_canvas->resourceProvider()->handleRadius());
+        decorator.setHandleRadius(canvas()->resourceManager()->handleRadius());
         decorator.setHotPosition(m_hotPosition);
         decorator.paint(painter, converter);
     }
     painter.save();
     KoShape::applyConversion(painter, converter);
-    m_canvas->snapGuide()->paint(painter, converter);
+    canvas()->snapGuide()->paint(painter, converter);
     painter.restore();
 }
 
@@ -513,7 +506,7 @@ void DefaultTool::mousePressEvent(KoPointerEvent *event)
 void DefaultTool::mouseMoveEvent(KoPointerEvent *event)
 {
     KoInteractionTool::mouseMoveEvent(event);
-    if (m_currentStrategy == 0 && koSelection()->count() > 0) {
+    if (currentStrategy() == 0 && koSelection()->count() > 0) {
         QRectF bound = handlesSize();
         if (bound.contains(event->point)) {
             bool inside;
@@ -530,7 +523,7 @@ void DefaultTool::mouseMoveEvent(KoPointerEvent *event)
             m_mouseWasInsideHandles = false;
 
             if (m_guideLine->isSelected()) {
-                KoGuidesTool * guidesTool = KoToolManager::instance()->guidesTool(m_canvas);
+                GuidesTool *guidesTool = dynamic_cast<GuidesTool*>(KoToolManager::instance()->toolById(canvas(), GuidesToolId));
                 if (guidesTool) {
                     guidesTool->moveGuideLine(m_guideLine->orientation(), m_guideLine->index());
                     activateTemporary(guidesTool->toolId());
@@ -541,7 +534,7 @@ void DefaultTool::mouseMoveEvent(KoPointerEvent *event)
         }
     } else {
         if (m_guideLine->isSelected()) {
-            KoGuidesTool * guidesTool = KoToolManager::instance()->guidesTool(m_canvas);
+            GuidesTool *guidesTool = dynamic_cast<GuidesTool*>(KoToolManager::instance()->toolById(canvas(), GuidesToolId));
             if (guidesTool) {
                 guidesTool->moveGuideLine(m_guideLine->orientation(), m_guideLine->index());
                 activateTemporary(guidesTool->toolId());
@@ -560,10 +553,10 @@ void DefaultTool::selectGuideAtPosition(const QPointF &position)
     Qt::Orientation orientation = Qt::Horizontal;
 
     // check if we are on a guide line
-    KoGuidesData * guidesData = m_canvas->guidesData();
+    KoGuidesData * guidesData = canvas()->guidesData();
     if (guidesData && guidesData->showGuideLines()) {
-        qreal handleRadius = m_canvas->resourceProvider()->handleRadius();
-        qreal minDistance = m_canvas->viewConverter()->viewToDocumentX(handleRadius);
+        qreal grabSensitivity = canvas()->resourceManager()->grabSensitivity();
+        qreal minDistance = canvas()->viewConverter()->viewToDocumentX(grabSensitivity);
         uint i = 0;
         foreach (qreal guidePos, guidesData->horizontalGuideLines()) {
             qreal distance = qAbs(guidePos - position.y());
@@ -598,9 +591,9 @@ QRectF DefaultTool::handlesSize()
 {
     QRectF bound = koSelection()->boundingRect();
     // expansion Border
-    if (!m_canvas || !m_canvas->viewConverter()) return bound;
+    if (!canvas() || !canvas()->viewConverter()) return bound;
 
-    QPointF border = m_canvas->viewConverter()->viewToDocument(QPointF(HANDLE_DISTANCE, HANDLE_DISTANCE));
+    QPointF border = canvas()->viewConverter()->viewToDocument(QPointF(HANDLE_DISTANCE, HANDLE_DISTANCE));
     bound.adjust(-border.x(), -border.y(), border.x(), border.y());
     return bound;
 }
@@ -620,11 +613,11 @@ void DefaultTool::mouseDoubleClickEvent(KoPointerEvent *event)
             shapes.append(shape);
     }
     if (shapes.count() == 0) { // nothing in the selection was clicked on.
-        KoShape *shape = m_canvas->shapeManager()->shapeAt (event->point, KoFlake::ShapeOnTop);
+        KoShape *shape = canvas()->shapeManager()->shapeAt (event->point, KoFlake::ShapeOnTop);
         if (shape) {
             shapes.append(shape);
         } else if (m_guideLine->isSelected()) {
-            KoGuidesTool * guidesTool = KoToolManager::instance()->guidesTool(m_canvas);
+            GuidesTool *guidesTool = dynamic_cast<GuidesTool*>(KoToolManager::instance()->toolById(canvas(), GuidesToolId));
             if (guidesTool) {
                 guidesTool->editGuideLine(m_guideLine->orientation(), m_guideLine->index());
                 activateTool(guidesTool->toolId());
@@ -641,25 +634,28 @@ bool DefaultTool::moveSelection(int direction, Qt::KeyboardModifiers modifiers)
 {
     qreal x=0.0, y=0.0;
     if (direction == Qt::Key_Left)
-        x=-0.5;
+        x = -5;
     else if (direction == Qt::Key_Right)
-        x=0.5;
+        x = 5;
     else if (direction == Qt::Key_Up)
-        y=-0.5;
+        y = -5;
     else if (direction == Qt::Key_Down)
-        y=0.5;
+        y = 5;
 
     if (x != 0.0 || y != 0.0) { // actually move
-        if ((modifiers & Qt::ShiftModifier) == 0) { // no shift used
+        if ((modifiers & Qt::ShiftModifier) != 0) {
             x *= 10;
             y *= 10;
+        } else if ((modifiers & Qt::AltModifier) != 0) { // more precise
+            x /= 5;
+            y /= 5;
         }
 
         QList<QPointF> prevPos;
         QList<QPointF> newPos;
         QList<KoShape*> shapes;
-        foreach(KoShape* shape, koSelection()->selectedShapes(KoFlake::StrippedSelection)) {
-            if (shape->isLocked())
+        foreach(KoShape* shape, koSelection()->selectedShapes(KoFlake::TopLevelSelection)) {
+            if (shape->isGeometryProtected())
                 continue;
             shapes.append(shape);
             QPointF p = shape->position();
@@ -677,7 +673,7 @@ bool DefaultTool::moveSelection(int direction, Qt::KeyboardModifiers modifiers)
                 m_moveCommand->redo();
             } else {
                 m_moveCommand = new KoShapeMoveCommand(shapes, prevPos, newPos);
-                m_canvas->addCommand(m_moveCommand);
+                canvas()->addCommand(m_moveCommand);
             }
             m_lastUsedMoveCommand = QTime::currentTime();
             return true;
@@ -689,7 +685,7 @@ bool DefaultTool::moveSelection(int direction, Qt::KeyboardModifiers modifiers)
 void DefaultTool::keyPressEvent(QKeyEvent *event)
 {
     KoInteractionTool::keyPressEvent(event);
-    if (m_currentStrategy == 0) {
+    if (currentStrategy() == 0) {
         switch (event->key()) {
         case Qt::Key_Left:
         case Qt::Key_Right:
@@ -703,7 +699,7 @@ void DefaultTool::keyPressEvent(QKeyEvent *event)
         case Qt::Key_3:
         case Qt::Key_4:
         case Qt::Key_5:
-            m_canvas->resourceProvider()->setResource(HotPosition, event->key()-Qt::Key_1);
+            canvas()->resourceManager()->setResource(HotPosition, event->key()-Qt::Key_1);
             event->accept();
             break;
         default:
@@ -729,7 +725,7 @@ void DefaultTool::customMoveEvent(KoPointerEvent * event)
             m_customEventStrategy->finishInteraction(event->modifiers());
             QUndoCommand *command = m_customEventStrategy->createCommand();
             if (command)
-                m_canvas->addCommand(command);
+                canvas()->addCommand(command);
             delete m_customEventStrategy;
             m_customEventStrategy = 0;
             repaintDecorations();
@@ -742,16 +738,16 @@ void DefaultTool::customMoveEvent(KoPointerEvent * event)
     if (zoom > move && zoom > rotate) {
         // zoom
         if (! m_customEventStrategy)
-            m_customEventStrategy = new ShapeResizeStrategy(this, m_canvas, event->point, KoFlake::TopLeftHandle);
+            m_customEventStrategy = new ShapeResizeStrategy(this, event->point, KoFlake::TopLeftHandle);
     } else if (move > zoom && move > rotate) { // check if x-/y-movement is dominant
         // move
         if (! m_customEventStrategy)
-            m_customEventStrategy = new ShapeMoveStrategy(this, m_canvas, event->point);
+            m_customEventStrategy = new ShapeMoveStrategy(this, event->point);
     } else if (rotate > zoom && rotate > move) // rotation is dominant
     {
         // rotate
         if (! m_customEventStrategy)
-            m_customEventStrategy = new ShapeRotateStrategy(this, m_canvas, event->point, event->buttons());
+            m_customEventStrategy = new ShapeRotateStrategy(this, event->point, event->buttons());
     }
 
     if (m_customEventStrategy)
@@ -764,12 +760,12 @@ void DefaultTool::repaintDecorations()
 {
     Q_ASSERT(koSelection());
     if (koSelection()->count() > 0)
-        m_canvas->updateCanvas(handlesSize());
+        canvas()->updateCanvas(handlesSize());
 }
 
 void DefaultTool::copy() const
 {
-    QList<KoShape *> shapes = m_canvas->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection);
+    QList<KoShape *> shapes = canvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection);
     if (!shapes.empty()) {
         KoShapeOdfSaveHelper saveHelper(shapes);
         KoDrag drag;
@@ -781,13 +777,13 @@ void DefaultTool::copy() const
 void DefaultTool::deleteSelection()
 {
     QList<KoShape *> shapes;
-    foreach (KoShape *s, m_canvas->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
-        if (s->isLocked())
+    foreach (KoShape *s, canvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
+        if (s->isGeometryProtected())
             continue;
         shapes << s;
     }
     if (!shapes.empty()) {
-        m_canvas->addCommand(m_canvas->shapeController()->removeShapes(shapes));
+        canvas()->addCommand(canvas()->shapeController()->removeShapes(shapes));
     }
 }
 
@@ -797,15 +793,15 @@ bool DefaultTool::paste()
 
     bool success = false;
     if (data->hasFormat(KoOdf::mimeType(KoOdf::Text))) {
-        KoShapeManager * shapeManager = m_canvas->shapeManager();
-        int zIndex = 0;
-        foreach (KoShape *shape, shapeManager->shapes())
-        {
-            zIndex = qMax(zIndex, shape->zIndex());
-        }
-
-        KoShapePaste paste(m_canvas, zIndex + 1, shapeManager->selection()->activeLayer());
+        KoShapeManager * shapeManager = canvas()->shapeManager();
+        KoShapePaste paste(canvas(), shapeManager->selection()->activeLayer());
         success = paste.paste(KoOdf::Text, data);
+        if (success) {
+            shapeManager->selection()->deselectAll();
+            foreach(KoShape *shape, paste.pastedShapes()) {
+                shapeManager->selection()->select(shape);
+            }
+        }
     }
     return success;
 }
@@ -819,9 +815,9 @@ QStringList DefaultTool::supportedPasteMimeTypes() const
 
 KoSelection *DefaultTool::koSelection()
 {
-    Q_ASSERT(m_canvas);
-    Q_ASSERT(m_canvas->shapeManager());
-    return m_canvas->shapeManager()->selection();
+    Q_ASSERT(canvas());
+    Q_ASSERT(canvas()->shapeManager());
+    return canvas()->shapeManager()->selection();
 }
 
 KoFlake::SelectionHandle DefaultTool::handleAt(const QPointF &point, bool *innerHandleMeaning)
@@ -843,14 +839,14 @@ KoFlake::SelectionHandle DefaultTool::handleAt(const QPointF &point, bool *inner
         return KoFlake::NoHandle;
 
     recalcSelectionBox();
-    const KoViewConverter *converter = m_canvas->viewConverter();
+    const KoViewConverter *converter = canvas()->viewConverter();
     if (!converter) return KoFlake::NoHandle;
 
     if (innerHandleMeaning != 0)
     {
         QPainterPath path;
         path.addPolygon(m_selectionOutline);
-        *innerHandleMeaning =  path.contains(point);
+        *innerHandleMeaning = path.contains(point) || path.intersects(handlePaintRect(point));
     }
     for (int i = 0; i < KoFlake::NoHandle; ++i) {
         KoFlake::SelectionHandle handle = handleOrder[i];
@@ -911,12 +907,11 @@ void DefaultTool::recalcSelectionBox()
     }
 }
 
-void DefaultTool::activate(bool temporary)
+void DefaultTool::activate(ToolActivation, const QSet<KoShape*> &)
 {
-    Q_UNUSED(temporary);
     m_mouseWasInsideHandles = false;
     m_lastHandle = KoFlake::NoHandle;
-    useCursor(Qt::ArrowCursor, true);
+    useCursor(Qt::ArrowCursor);
     repaintDecorations();
     delete m_guideLine;
     m_guideLine = new GuideLine();
@@ -925,38 +920,31 @@ void DefaultTool::activate(bool temporary)
 
 void DefaultTool::selectionAlignHorizontalLeft()
 {
-
     selectionAlign(KoShapeAlignCommand::HorizontalLeftAlignment);
 }
 
 void DefaultTool::selectionAlignHorizontalCenter()
 {
-
     selectionAlign(KoShapeAlignCommand::HorizontalCenterAlignment);
 }
 
-
 void DefaultTool::selectionAlignHorizontalRight()
 {
-
     selectionAlign(KoShapeAlignCommand::HorizontalRightAlignment);
 }
 
 void DefaultTool::selectionAlignVerticalTop()
 {
-
     selectionAlign(KoShapeAlignCommand::VerticalTopAlignment);
 }
 
 void DefaultTool::selectionAlignVerticalCenter()
 {
-
     selectionAlign(KoShapeAlignCommand::VerticalCenterAlignment);
 }
 
 void DefaultTool::selectionAlignVerticalBottom()
 {
-
     selectionAlign(KoShapeAlignCommand::VerticalBottomAlignment);
 }
 
@@ -971,25 +959,21 @@ void DefaultTool::selectionGroup()
 
     // only group shapes with an unselected parent
     foreach (KoShape* shape, selectedShapes) {
-        if (! selectedShapes.contains(shape->parent())) {
+        if (! selectedShapes.contains(shape->parent()) && shape->isEditable()) {
             groupedShapes << shape;
         }
     }
     KoShapeGroup *group = new KoShapeGroup();
-    // is this needed?
-    if (selection->activeLayer()) {
-        selection->activeLayer()->addChild(group);
-    }
     // TODO what if only one shape is left?
     QUndoCommand *cmd = new QUndoCommand(i18n("Group shapes"));
-    m_canvas->shapeController()->addShapeDirect(group, cmd);
-    new KoShapeGroupCommand(group, groupedShapes, cmd);
-    m_canvas->addCommand(cmd);
+    canvas()->shapeController()->addShapeDirect(group, cmd);
+    KoShapeGroupCommand::createCommand(group, groupedShapes, cmd);
+    canvas()->addCommand(cmd);
 }
 
 void DefaultTool::selectionUngroup()
 {
-    KoSelection* selection = m_canvas->shapeManager()->selection();
+    KoSelection* selection = canvas()->shapeManager()->selection();
     if (! selection)
         return;
 
@@ -998,7 +982,7 @@ void DefaultTool::selectionUngroup()
 
     // only ungroup shape groups with an unselected parent
     foreach (KoShape* shape, selectedShapes) {
-        if (!selectedShapes.contains(shape->parent())) {
+        if (!selectedShapes.contains(shape->parent()) && shape->isEditable()) {
             containerSet << shape;
         }
     }
@@ -1010,18 +994,20 @@ void DefaultTool::selectionUngroup()
         KoShapeGroup *group = dynamic_cast<KoShapeGroup*>(shape);
         if (group) {
             cmd = cmd ? cmd : new QUndoCommand(i18n("Ungroup shapes"));
-            new KoShapeUngroupCommand(group, group->iterator(), cmd);
-            m_canvas->shapeController()->removeShape(group, cmd);
+            new KoShapeUngroupCommand(group, group->childShapes(),
+                                      group->parent()? QList<KoShape*>(): canvas()->shapeManager()->topLevelShapes(),
+                                      cmd);
+            canvas()->shapeController()->removeShape(group, cmd);
         }
     }
     if (cmd) {
-        m_canvas->addCommand(cmd);
+        canvas()->addCommand(cmd);
     }
 }
 
 void DefaultTool::selectionAlign(KoShapeAlignCommand::Align align)
 {
-    KoSelection* selection = m_canvas->shapeManager()->selection();
+    KoSelection* selection = canvas()->shapeManager()->selection();
     if (! selection)
         return;
 
@@ -1029,20 +1015,27 @@ void DefaultTool::selectionAlign(KoShapeAlignCommand::Align align)
     if (selectedShapes.count() < 1)
         return;
 
+    QList<KoShape*> editableShapes = filterEditableShapes(selectedShapes);
+
     // TODO add an option to the widget so that one can align to the page
     // with multiple selected shapes too
 
     QRectF bb;
 
     // single selected shape is automatically aligned to document rect
-    if (selectedShapes.count() == 1 && m_canvas->resourceProvider()->hasResource(KoCanvasResource::PageSize))
-        bb = QRectF(QPointF(0,0), m_canvas->resourceProvider()->sizeResource(KoCanvasResource::PageSize));
-    else
-        bb = selection->boundingRect();
+    if (editableShapes.count() == 1 ) {
+        if (!canvas()->resourceManager()->hasResource(KoCanvasResource::PageSize))
+            return;
+        bb = QRectF(QPointF(0,0), canvas()->resourceManager()->sizeResource(KoCanvasResource::PageSize));
+    } else {
+        foreach( KoShape * shape, editableShapes ) {
+            bb |= shape->boundingRect();
+        }
+    }
 
-    KoShapeAlignCommand *cmd = new KoShapeAlignCommand(selectedShapes, align, bb);
+    KoShapeAlignCommand *cmd = new KoShapeAlignCommand(editableShapes, align, bb);
 
-    m_canvas->addCommand(cmd);
+    canvas()->addCommand(cmd);
     selection->updateSizeAndPosition();
 }
 
@@ -1068,7 +1061,7 @@ void DefaultTool::selectionSendToBack()
 
 void DefaultTool::selectionReorder(KoShapeReorderCommand::MoveShapeType order)
 {
-    KoSelection* selection = m_canvas->shapeManager()->selection();
+    KoSelection* selection = canvas()->shapeManager()->selection();
     if (! selection)
         return;
 
@@ -1076,8 +1069,14 @@ void DefaultTool::selectionReorder(KoShapeReorderCommand::MoveShapeType order)
     if (selectedShapes.count() < 1)
         return;
 
-    QUndoCommand * cmd = KoShapeReorderCommand::createCommand(selectedShapes, m_canvas->shapeManager(), order);
-    m_canvas->addCommand(cmd);
+    QList<KoShape*> editableShapes = filterEditableShapes(selectedShapes);
+    if (editableShapes.count() < 1)
+        return;
+
+    QUndoCommand * cmd = KoShapeReorderCommand::createCommand(editableShapes, canvas()->shapeManager(), order);
+    if (cmd) {
+        canvas()->addCommand(cmd);
+    }
 }
 
 QMap<QString, QWidget *> DefaultTool::createOptionWidgets()
@@ -1085,7 +1084,7 @@ QMap<QString, QWidget *> DefaultTool::createOptionWidgets()
     QMap<QString, QWidget *> widgets;
     widgets.insert(i18n("Arrange"), new DefaultToolArrangeWidget(this));
     widgets.insert(i18n("Geometry"), new DefaultToolWidget(this));
-    widgets.insert(i18n("Snapping"), new SnapGuideConfigWidget(m_canvas->snapGuide()));
+    widgets.insert(i18n("Snapping"), canvas()->createSnapGuideConfigWidget());
     return widgets;
 }
 
@@ -1099,21 +1098,16 @@ void DefaultTool::resourceChanged(int key, const QVariant & res)
 
 KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
 {
-    // reset the move by keys when a new strategy is created otherwise we might change the 
+    // reset the move by keys when a new strategy is created otherwise we might change the
     // command after a new command was added. This happend when you where faster than the timer.
     m_moveCommand = 0;
 
-    KoShapeManager *shapeManager = m_canvas->shapeManager();
+    KoShapeManager *shapeManager = canvas()->shapeManager();
     KoSelection *select = shapeManager->selection();
-    bool insideSelection, editableShape=false;
+    bool insideSelection;
     KoFlake::SelectionHandle handle = handleAt(event->point, &insideSelection);
 
-    foreach (KoShape* shape, select->selectedShapes()) {
-        if (shape->isEditable()) {
-            editableShape = true;
-            break;
-        }
-    }
+    bool editableShape = editableShapesCount(select->selectedShapes());
 
     if (event->buttons() & Qt::MidButton) {
         // change the hot selection position when middle clicking on a handle
@@ -1134,7 +1128,7 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
         default:
         {
             // check if we had hit the center point
-            const KoViewConverter * converter = m_canvas->viewConverter();
+            const KoViewConverter * converter = canvas()->viewConverter();
             QPointF pt = converter->documentToView(event->point-select->absolutePosition());
             if (qAbs(pt.x()) < HANDLE_DISTANCE && qAbs(pt.y()) < HANDLE_DISTANCE)
                 newHotPosition = KoFlake::CenteredPosition;
@@ -1142,7 +1136,7 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
         }
         }
         if (m_hotPosition != newHotPosition)
-            m_canvas->resourceProvider()->setResource(HotPosition, newHotPosition);
+            canvas()->resourceManager()->setResource(HotPosition, newHotPosition);
         return 0;
     }
 
@@ -1155,29 +1149,29 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
             if (event->buttons() == Qt::LeftButton) {
                 // resizing or shearing only with left mouse button
                 if (insideSelection)
-                    return new ShapeResizeStrategy(this, m_canvas, event->point, handle);
+                    return new ShapeResizeStrategy(this, event->point, handle);
                 if (handle == KoFlake::TopMiddleHandle || handle == KoFlake::RightMiddleHandle ||
                         handle == KoFlake::BottomMiddleHandle || handle == KoFlake::LeftMiddleHandle)
-                    return new ShapeShearStrategy(this, m_canvas, event->point, handle);
+                    return new ShapeShearStrategy(this, event->point, handle);
             }
             // rotating is allowed for rigth mouse button too
             if (handle == KoFlake::TopLeftHandle || handle == KoFlake::TopRightHandle ||
                 handle == KoFlake::BottomLeftHandle || handle == KoFlake::BottomRightHandle)
-                return new ShapeRotateStrategy(this, m_canvas, event->point, event->buttons());
+                return new ShapeRotateStrategy(this, event->point, event->buttons());
         }
         if (! (selectMultiple || selectNextInStack) && event->buttons() == Qt::LeftButton) {
             const QPainterPath outlinePath = select->transformation().map(select->outline());
-            if (outlinePath.contains(event->point))
-                return new ShapeMoveStrategy(this, m_canvas, event->point);
+            if (outlinePath.contains(event->point) || outlinePath.intersects(handlePaintRect(event->point)))
+                return new ShapeMoveStrategy(this, event->point);
         }
     }
 
     if ((event->buttons() & Qt::LeftButton) == 0)
         return 0;  // Nothing to do for middle/right mouse button
 
-    KoShape * object(shapeManager->shapeAt(event->point, selectNextInStack ? KoFlake::NextUnselected : KoFlake::ShapeOnTop));
+    KoShape *shape(shapeManager->shapeAt(event->point, selectNextInStack ? KoFlake::NextUnselected : KoFlake::ShapeOnTop));
 
-    if (!object && handle == KoFlake::NoHandle) {
+    if (!shape && handle == KoFlake::NoHandle) {
         // check if we have hit a guide
         if (m_guideLine->isValid()) {
             m_guideLine->select();
@@ -1187,22 +1181,22 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
             repaintDecorations();
             select->deselectAll();
         }
-        return new KoShapeRubberSelectStrategy(this, m_canvas, event->point);
+        return new KoShapeRubberSelectStrategy(this, event->point);
     }
 
-    if (select->isSelected(object)) {
+    if (select->isSelected(shape)) {
         if (selectMultiple) {
             repaintDecorations();
-            select->deselect(object);
+            select->deselect(shape);
         }
     }
-    else if (handle == KoFlake::NoHandle) { // clicked on object which is not selected
+    else if (handle == KoFlake::NoHandle) { // clicked on shape which is not selected
         repaintDecorations();
         if (! selectMultiple)
             shapeManager->selection()->deselectAll();
-        select->select(object, selectNextInStack ? false : true);
+        select->select(shape, selectNextInStack ? false : true);
         repaintDecorations();
-        return new ShapeMoveStrategy(this, m_canvas, event->point);
+        return new ShapeMoveStrategy(this, event->point);
     }
     return 0;
 }
@@ -1226,12 +1220,13 @@ void DefaultTool::updateActions()
         return;
     }
 
-    bool enable = selection->count () > 0;
+    QList<KoShape*> editableShapes = filterEditableShapes(selection->selectedShapes(KoFlake::TopLevelSelection));
+    bool enable = editableShapes.count () > 0;
     action("object_order_front")->setEnabled(enable);
     action("object_order_raise")->setEnabled(enable);
     action("object_order_lower")->setEnabled(enable);
     action("object_order_back")->setEnabled(enable);
-    enable = selection->count() > 1 || (enable && m_canvas->resourceProvider()->hasResource(KoCanvasResource::PageSize));
+    enable = (editableShapes.count () > 1) || (enable && canvas()->resourceManager()->hasResource(KoCanvasResource::PageSize));
     action("object_align_horizontal_left")->setEnabled(enable);
     action("object_align_horizontal_center")->setEnabled(enable);
     action("object_align_horizontal_right")->setEnabled(enable);
@@ -1239,10 +1234,9 @@ void DefaultTool::updateActions()
     action("object_align_vertical_center")->setEnabled(enable);
     action("object_align_vertical_bottom")->setEnabled(enable);
 
-    QList<KoShape *> shapes = selection->selectedShapes(KoFlake::TopLevelSelection);
-    action("object_group")->setEnabled(shapes.count() > 1);
+    action("object_group")->setEnabled(editableShapes.count() > 1);
     bool groupShape = false;
-    foreach (KoShape * shape, shapes) {
+    foreach (KoShape * shape, editableShapes) {
         if (dynamic_cast<KoShapeGroup *>(shape)) {
             groupShape = true;
             break;
@@ -1258,4 +1252,26 @@ KoToolSelection* DefaultTool::selection()
     return m_selectionHandler;
 }
 
-#include "DefaultTool.moc"
+QList<KoShape*> DefaultTool::filterEditableShapes( const QList<KoShape*> &shapes )
+{
+    QList<KoShape*> editableShapes;
+    foreach( KoShape * shape, shapes ) {
+        if (shape->isEditable())
+            editableShapes.append(shape);
+    }
+
+    return editableShapes;
+}
+
+uint DefaultTool::editableShapesCount( const QList<KoShape*> &shapes )
+{
+    uint count = 0;
+    foreach( KoShape * shape, shapes ) {
+        if (shape->isEditable())
+            count++;
+    }
+
+    return count;
+}
+
+#include <DefaultTool.moc>
