@@ -1,9 +1,10 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  * Copyright (C) 2008 Roopesh Chander <roop@forwardbias.in>
  * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
+ * Copyright (C) 2009-2010 Casper Boemann <cbo@boemann.dk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -45,13 +46,16 @@
 #include <KoChangeTracker.h>
 #include <KoChangeTrackerElement.h>
 #include <KoGenChange.h>
+#include <KoTextBlockPaintStrategyBase.h>
+#include <KoImageData.h>
+#include <KoImageCollection.h>
 
 #include <KDebug>
 #include <QTextList>
 #include <QStyle>
 #include <QFontMetrics>
 #include <QTextTableCell>
-
+#include <QTextFragment>
 
 // ---------------- layout helper ----------------
 Layout::Layout(KoTextDocumentLayout *parent)
@@ -63,6 +67,7 @@ Layout::Layout(KoTextDocumentLayout *parent)
         m_isRtl(false),
         m_inTable(false),
         m_parent(parent),
+        m_textShape(0),
         m_demoText(false),
         m_endOfDemoText(false),
         m_defaultTabSizing(MM_TO_POINT(15)),
@@ -99,7 +104,7 @@ void Layout::reset()
     m_reset = true;
 }
 
-bool Layout::interrupted()
+bool Layout::isInterrupted() const
 {
     return m_reset;
 }
@@ -178,6 +183,10 @@ bool Layout::addLine(QTextLine &line)
     }
 
     qreal height = m_format.doubleProperty(KoParagraphStyle::FixedLineHeight);
+    if (line.textLength() == 1 && m_block.text().at(line.textStart()) == QChar::ObjectReplacementCharacter && line.descent() == 0.0) {
+        // thats an (non inline) anchor then, anchors takes no height.
+        height = 0.1;
+    }
     qreal objectHeight = 0.0;
     bool useFixedLineHeight = height != 0.0;
     if (m_dropCapsNChars > 0)
@@ -210,11 +219,11 @@ bool Layout::addLine(QTextLine &line)
                              m_block.position() + line.textStart() + line.textLength() - 1))) {
                     m_fragmentIterator++;
                     if (!m_changeTracker
-                        || !m_changeTracker->isEnabled()
+                        || !m_changeTracker->displayChanges()
                         || !m_changeTracker->containsInlineChanges(m_fragmentIterator.fragment().charFormat())
                         || !m_changeTracker->elementById(m_fragmentIterator.fragment().charFormat().property(KoCharacterStyle::ChangeTrackerId).toInt())->isEnabled()
                         || (m_changeTracker->elementById(m_fragmentIterator.fragment().charFormat().property(KoCharacterStyle::ChangeTrackerId).toInt())->getChangeType() != KoGenChange::deleteChange)
-                        || m_changeTracker->displayDeleted()) {
+                        || m_changeTracker->displayChanges()) {
                         height = qMax(height, m_fragmentIterator.fragment().charFormat().fontPointSize());
                         objectHeight = qMax(objectHeight, inlineCharHeight(m_fragmentIterator.fragment()));
                     }
@@ -255,7 +264,8 @@ bool Layout::addLine(QTextLine &line)
         if (m_data->endPosition() == -1) // no text at all fit in the shape!
             m_data->setEndPosition(m_data->position());
         m_data->wipe();
-        m_textShape->markLayoutDone();
+        if (m_textShape) // kword uses a dummy shape which is not a text shape
+            m_textShape->markLayoutDone();
         nextShape();
         if (m_data)
             m_data->setPosition(m_block.position() + (ignoreParagraph ? 0 : line.textStart()));
@@ -268,7 +278,7 @@ bool Layout::addLine(QTextLine &line)
 
     // add linespacing
     if (! useFixedLineHeight) {
-        qreal linespacing = m_format.doubleProperty(KoParagraphStyle::LineSpacing);;
+        qreal linespacing = m_format.doubleProperty(KoParagraphStyle::LineSpacing);
         if (linespacing == 0.0) { // unset
             int percent = m_format.intProperty(KoParagraphStyle::PercentLineHeight);
             if (percent != 0)
@@ -307,6 +317,29 @@ bool Layout::addLine(QTextLine &line)
         }
     }
 
+    // call update to repaint the new line
+    QRectF repaintRect = line.rect();
+    if (layout->lineCount() > 1) { // cover big linespacing.
+        QTextLine prevLine = layout->lineAt(layout->lineCount()-2);
+        const qreal bottom = prevLine.y() + prevLine.height();
+        if (repaintRect.top() > bottom) {
+            repaintRect.setTop(bottom); //expand to cover space between lines.
+        }
+    } else { // cover space between paragraphs
+        QTextBlock prev = m_block.previous();
+        if (prev.isValid() && prev.layout()->lineCount() > 0) {
+            QTextLine prevLine = prev.layout()->lineAt(layout->lineCount()-1);
+            const qreal bottom = prevLine.y() + prevLine.height();
+            if (repaintRect.top() > bottom) {
+                repaintRect.setTop(bottom); //expand to cover space between paragraphs.
+            }
+        }
+    }
+    repaintRect.moveTop(repaintRect.y() - docOffsetInShape());
+    repaintRect.setX(0.0); // just take full width since we can't force a repaint of
+    repaintRect.setWidth(shape->size().width()); // where lines were before layout.
+    shape->update(repaintRect);
+
     return false;
 }
 
@@ -326,8 +359,7 @@ bool Layout::nextParag()
         }
         qreal borderBottom = m_y;
         if (m_block.isValid() && !m_newShape) { // only add bottom of prev parag if we did not go to a new shape for this parag.
-            if (m_format.pageBreakPolicy() == QTextFormat::PageBreak_AlwaysAfter ||
-                    m_format.boolProperty(KoParagraphStyle::BreakAfter)) {
+            if (m_format.pageBreakPolicy() & QTextFormat::PageBreak_AlwaysAfter) {
                 m_data->setEndPosition(m_block.position() - 1);
                 m_data->wipe();
                 nextShape();
@@ -359,7 +391,9 @@ bool Layout::nextParag()
         shape->update(QRectF(0.0, offsetInShape, shape->size().width(), shape->size().height() - offsetInShape));
         // cleanup and repaint rest of shapes.
         m_data->wipe();
-        m_textShape->markLayoutDone();
+        if (m_textShape) { // may be null in relation to setFollowupShape
+            m_textShape->markLayoutDone();
+        }
         cleanupShapes();
         return false;
     }
@@ -407,8 +441,7 @@ bool Layout::nextParag()
         m_blockData->setCounterWidth(0.0);
     }
 
-    bool pagebreak = m_format.pageBreakPolicy() == QTextFormat::PageBreak_AlwaysBefore ||
-                      m_format.boolProperty(KoParagraphStyle::BreakBefore);
+    bool pagebreak = m_format.pageBreakPolicy() & QTextFormat::PageBreak_AlwaysBefore;
 
     const QVariant masterPageName = m_format.property(KoParagraphStyle::MasterPageName);
     if (! masterPageName.isNull() && m_currentMasterPage != masterPageName.toString()) {
@@ -443,10 +476,13 @@ bool Layout::nextParag()
     QList<QTextOption::Tab> tabs;
     QVariant variant = m_format.property(KoParagraphStyle::TabPositions);
     if (!variant.isNull()) {
+        const qreal tabOffset = x();
         foreach(const QVariant &tv, qvariant_cast<QList<QVariant> >(variant)) {
             KoText::Tab koTab = tv.value<KoText::Tab>();
             QTextOption::Tab tab;
-            tab.position = koTab.position * qt_defaultDpiY() / 72.; // convertion here is required because Qt thinks in device units and we don't
+
+            // convertion here is required because Qt thinks in device units and we don't
+            tab.position = (koTab.position - tabOffset) * qt_defaultDpiY() / 72.;
             tab.type = koTab.type;
             tab.delimiter = koTab.delimiter;
             tabs.append(tab);
@@ -736,6 +772,7 @@ void Layout::nextShape()
 
     shape = 0;
     m_data = 0;
+    m_textShape = 0;
 
     QList<KoShape *> shapes = m_parent->shapes();
     for (shapeNumber++; shapeNumber < shapes.count(); shapeNumber++) {
@@ -758,6 +795,7 @@ void Layout::nextShape()
         QTextCursor cursor(m_textShape->footnoteDocument());
         cursor.select(QTextCursor::Document);
         cursor.removeSelectedText();
+        Q_ASSERT(!m_textShape->hasFootnoteDocument());
     }
     m_shapeBorder = shape->borderInsets();
     m_y += m_shapeBorder.top;
@@ -811,6 +849,7 @@ void Layout::resetPrivate()
     shape = 0;
     layout = 0;
     m_newShape = true;
+    m_textShape = 0;
     m_blockData = 0;
     m_newParag = true;
     m_block = m_parent->document()->begin();
@@ -843,6 +882,7 @@ void Layout::resetPrivate()
                 // block has been layouted. So use its offset.
                 m_y = m_block.layout()->lineAt(0).position().y();
                 if (m_y < data->documentOffset() - 0.126) { // 0.126 to account of rounding in Qt-scribe
+                    // the last layed-out parag
                     Q_ASSERT(shapeNumber > 0);
                     // since we only recalc whole parags; we need to go back a little.
                     shapeNumber--;
@@ -877,16 +917,15 @@ void Layout::resetPrivate()
         return;
     Q_ASSERT(shapeNumber < shapes.count());
     shape = shapes[shapeNumber];
-
+    m_data = qobject_cast<KoTextShapeData*>(shape->userData());
     m_textShape = dynamic_cast<TextShape*>(shape);
     Q_ASSERT(m_textShape);
-    if (m_textShape->hasFootnoteDocument()) {
+    if (m_textShape->hasFootnoteDocument() && m_y == m_data->documentOffset()) {
         QTextCursor cursor(m_textShape->footnoteDocument());
         cursor.select(QTextCursor::Document);
         cursor.removeSelectedText();
     }
     m_demoText = m_textShape->demoText();
-    m_data = qobject_cast<KoTextShapeData*>(shape->userData());
     m_shapeBorder = shape->borderInsets();
     if (m_y == 0)
         m_y = m_shapeBorder.top;
@@ -899,7 +938,7 @@ void Layout::updateBorders()
 {
     Q_ASSERT(m_data);
     m_borderInsets = m_data->shapeMargins();
-    KoTextBlockBorderData border(QRectF(this->x() - listIndent(), m_y + m_borderInsets.top, width(), 1.));
+    KoTextBlockBorderData border(QRectF(this->x() - resolveTextIndent() - listIndent(), m_y + m_borderInsets.top, width() + resolveTextIndent(), 1));
     border.setEdge(border.Left, m_format, KoParagraphStyle::LeftBorderStyle,
                    KoParagraphStyle::LeftBorderWidth, KoParagraphStyle::LeftBorderColor,
                    KoParagraphStyle::LeftBorderSpacing, KoParagraphStyle::LeftInnerBorderWidth);
@@ -957,7 +996,7 @@ qreal Layout::topMargin()
     bool allowMargin = true; // wheather to allow margins at top of shape
     if (m_newShape) {
         allowMargin = false; // false by default, but check 2 exceptions.
-        if (m_format.boolProperty(KoParagraphStyle::BreakBefore))
+        if (m_format.pageBreakPolicy () & QTextFormat::PageBreak_AlwaysBefore)
             allowMargin = true;
         else if (m_styleManager && m_format.topMargin() > 0) {
             // also allow it when the paragraph has the margin, but the style has a different one.
@@ -996,12 +1035,18 @@ void Layout::drawFrame(QTextFrame *frame, QPainter *painter, const KoTextDocumen
     for (it = frame->begin(); !(it.atEnd()); ++it) {
         QTextBlock block = it.currentBlock();
         QTextTable *table = qobject_cast<QTextTable*>(it.currentFrame());
+        QTextFrame *subFrame = it.currentFrame();
 
         if (table) {
             m_tableLayout.setTable(table);
             m_tableLayout.drawBackground(painter);
             drawFrame(table, painter, context, inTable+1); // this actually only draws the text inside
-            m_tableLayout.drawBorders(painter);
+            QPainterPath accuBlankBorders;
+            m_tableLayout.drawBorders(painter, &accuBlankBorders);
+            painter->strokePath(accuBlankBorders, QPen(QColor(0,0,0,96)));
+            continue;
+        } else if (subFrame) {
+            drawFrame(subFrame, painter, context, inTable);
             continue;
         } else {
             if (!block.isValid())
@@ -1012,11 +1057,26 @@ void Layout::drawFrame(QTextFrame *frame, QPainter *painter, const KoTextDocumen
 
         if (!painter->hasClipping() || ! clipRegion.intersect(QRegion(layout->boundingRect().toRect())).isEmpty()) {
             started = true;
-            QBrush bg = block.blockFormat().background();
+
+            KoTextBlockData *blockData = dynamic_cast<KoTextBlockData*>(block.userData());
+            KoTextBlockBorderData *border = 0;
+            KoTextBlockPaintStrategyBase *paintStrategy = 0;
+            if (blockData) {
+                border = blockData->border();
+                paintStrategy = blockData->paintStrategy();
+            }
+            KoTextBlockPaintStrategyBase dummyPaintStrategy;
+            if (paintStrategy == 0)
+                paintStrategy = &dummyPaintStrategy;
+            if (!paintStrategy->isVisible())
+                continue; // this paragraph shouldn't be shown so just skip it
+
+            QBrush bg = paintStrategy->background(block.blockFormat().background());
             if (bg != Qt::NoBrush)
                 painter->fillRect(layout->boundingRect(), bg);
+            paintStrategy->applyStrategy(painter);
             painter->save();
-            drawListItem(painter, block);
+            drawListItem(painter, block, context.imageCollection);
             painter->restore();
 
             QVector<QTextLayout::FormatRange> selections;
@@ -1030,7 +1090,7 @@ void Layout::drawFrame(QTextFrame *frame, QPainter *painter, const KoTextDocumen
                 if (end < block.position() || begin > block.position() + block.length())
                     continue; // selection does not intersect this block.
                 if (!m_changeTracker
-                    || m_changeTracker->displayDeleted()
+                    || m_changeTracker->displayChanges()
                     || !m_changeTracker->containsInlineChanges(selection.format)
                     || !m_changeTracker->elementById(selection.format.property(KoCharacterStyle::ChangeTrackerId).toInt())->isEnabled()
                     || (m_changeTracker->elementById(selection.format.property(KoCharacterStyle::ChangeTrackerId).toInt())->getChangeType() != KoGenChange::deleteChange)) {
@@ -1048,10 +1108,6 @@ void Layout::drawFrame(QTextFrame *frame, QPainter *painter, const KoTextDocumen
             layout->draw(painter, QPointF(0, 0), selections);
             decorateParagraph(painter, block, selectionStart - block.position(), selectionEnd - block.position(), context.viewConverter);
 
-            KoTextBlockBorderData *border = 0;
-            KoTextBlockData *blockData = dynamic_cast<KoTextBlockData*>(block.userData());
-            if (blockData)
-                border = dynamic_cast<KoTextBlockBorderData*>(blockData->border());
             if (lastBorder && lastBorder != border) {
                 painter->save();
                 lastBorder->paint(*painter);
@@ -1155,12 +1211,12 @@ static void drawDecorationText(QPainter *painter, const QTextLine &line, const Q
     painter->setPen(oldPen);
 }
 
-static void drawDecorationWords(QPainter *painter, const QTextLine &line, const QString &text, const QColor &color, KoCharacterStyle::LineType type, KoCharacterStyle::LineStyle style, const QString& decorText, qreal width, const qreal y)
+static void drawDecorationWords(QPainter *painter, const QTextLine &line, const QString &text, const QColor &color, KoCharacterStyle::LineType type, KoCharacterStyle::LineStyle style, const QString& decorText, qreal width, const qreal y, const int fragmentToLineOffset, const int startOfFragmentInBlock)
 {
     qreal wordBeginX = -1;
-    int j = line.textStart();
-    while (j < line.textLength() + line.textStart()) {
-        if (text[j].isSpace()) {
+    int j = line.textStart()+fragmentToLineOffset;
+    while (j < line.textLength() + line.textStart() && j-startOfFragmentInBlock<text.size()) {
+        if (text[j-startOfFragmentInBlock].isSpace()) {
             if (wordBeginX != -1) {
                 if (decorText.isEmpty())
                     drawDecorationLine(painter, color, type, style, width, wordBeginX, line.cursorToX(j), y);
@@ -1307,7 +1363,7 @@ void Layout::drawTrackedChangeItem(QPainter *painter, QTextBlock &block, int sel
                 startOfBlock = currentFragment.position(); // start of this block w.r.t. the document
             if (m_changeTracker->containsInlineChanges(fmt)) {
                 KoChangeTrackerElement *changeElement = m_changeTracker->elementById(fmt.property(KoCharacterStyle::ChangeTrackerId).toInt());
-                if (m_changeTracker->isEnabled() && changeElement->isEnabled()) {
+                if (m_changeTracker->displayChanges() && changeElement->isEnabled()) {
                     int firstLine = layout->lineForTextPosition(currentFragment.position() - startOfBlock).lineNumber();
                     int lastLine = layout->lineForTextPosition(currentFragment.position() + currentFragment.length() - startOfBlock).lineNumber();
                     for (int i = firstLine ; i <= lastLine ; i++) {
@@ -1318,14 +1374,16 @@ void Layout::drawTrackedChangeItem(QPainter *painter, QTextBlock &block, int sel
 
                             switch(changeElement->getChangeType()) {
                                 case (KoGenChange::insertChange):
-                                    painter->fillRect(x1, line.y(), x2-x1, line.height(), QColor(0,255,0,255));
+                                    if (m_changeTracker->displayChanges())
+                                        painter->fillRect(x1, line.y(), x2-x1, line.height(), m_changeTracker->getInsertionBgColor());
                                     break;
                                 case (KoGenChange::formatChange):
-                                    painter->fillRect(x1, line.y(), x2-x1, line.height(), QColor(0,0,255,255));
+                                    if (m_changeTracker->displayChanges())
+                                        painter->fillRect(x1, line.y(), x2-x1, line.height(), m_changeTracker->getFormatChangeBgColor());
                                     break;
                                 case (KoGenChange::deleteChange):
-                                    if (m_changeTracker->displayDeleted())
-                                        painter->fillRect(x1, line.y(), x2-x1, line.height(), QColor(255,0,0,255));
+                                    if (m_changeTracker->displayChanges())
+                                        painter->fillRect(x1, line.y(), x2-x1, line.height(), m_changeTracker->getDeletionBgColor());
                                     break;
                             }
                         }
@@ -1342,7 +1400,7 @@ void Layout::drawTrackedChangeItem(QPainter *painter, QTextBlock &block, int sel
                     QTextCharFormat rFormat = r.format;
                     if ((rEnd >= fragmentBegin && rEnd <= fragmentEnd) || (fragmentEnd >= rStart && fragmentEnd <= rEnd)) { //intersect
                         ranges.erase(iter);
-                        
+
                         break;
                     }
                     ++iter;
@@ -1383,7 +1441,7 @@ void Layout::drawTrackedChangeItem(QPainter *painter, QTextBlock &block, int sel
 //    layout->setAdditionalFormats(ranges);
 }
 
-static void drawStrikeOuts(QPainter *painter, const QTextFragment& currentFragment, const QTextLine& line, qreal x1, qreal x2)
+void Layout::drawStrikeOuts(QPainter *painter, const QTextFragment &currentFragment, const QTextLine &line, qreal x1, qreal x2, const int startOfFragmentInBlock, const int fragmentToLineOffset) const
 {
     QTextCharFormat fmt = currentFragment.charFormat();
     KoCharacterStyle::LineStyle strikeOutStyle = (KoCharacterStyle::LineStyle)
@@ -1392,7 +1450,22 @@ static void drawStrikeOuts(QPainter *painter, const QTextFragment& currentFragme
             fmt.intProperty(KoCharacterStyle::StrikeOutType);
     if ((strikeOutStyle != KoCharacterStyle::NoLineStyle) &&
             (strikeOutType != KoCharacterStyle::NoLineType)) {
-        qreal y = line.position().y() + line.height() / 2;
+        QTextCharFormat::VerticalAlignment valign = fmt.verticalAlignment();
+
+        QFont font(fmt.font());
+        if (valign == QTextCharFormat::AlignSubScript
+                || valign == QTextCharFormat::AlignSuperScript)
+            font.setPointSize(qRound(font.pointSize() * 2 / 3.));
+        QFontMetricsF metrics(font, m_parent->paintDevice());
+
+        qreal y = line.position().y();
+        if (valign == QTextCharFormat::AlignSubScript)
+            y += line.height() - metrics.descent() - metrics.strikeOutPos();
+        else if (valign == QTextCharFormat::AlignSuperScript)
+            y += metrics.ascent() - metrics.strikeOutPos();
+        else
+            y += line.ascent() - metrics.strikeOutPos();
+
         QColor color = fmt.colorProperty(KoCharacterStyle::StrikeOutColor);
         if (!color.isValid())
             color = fmt.foreground().color();
@@ -1400,16 +1473,21 @@ static void drawStrikeOuts(QPainter *painter, const QTextFragment& currentFragme
             (KoCharacterStyle::LineMode) fmt.intProperty(KoCharacterStyle::StrikeOutMode);
 
         QString strikeOutText = fmt.stringProperty(KoCharacterStyle::StrikeOutText);
-        qreal width = 0;
+        qreal width = 0; // line thickness
         if (strikeOutText.isEmpty()) {
             width = computeWidth(
                         (KoCharacterStyle::LineWeight) fmt.intProperty(KoCharacterStyle::StrikeOutWeight),
                         fmt.doubleProperty(KoCharacterStyle::StrikeOutWidth),
-                        painter->font());
+                        font);
         }
+        if (valign == QTextCharFormat::AlignSubScript
+                || valign == QTextCharFormat::AlignSuperScript) // adjust size.
+            width = width * 2 / 3;
 
         if (strikeOutMode == KoCharacterStyle::SkipWhiteSpaceLineMode) {
-            drawDecorationWords(painter, line, currentFragment.text(), color, strikeOutType, strikeOutStyle, strikeOutText, width, y);
+            drawDecorationWords(painter, line, currentFragment.text(), color, strikeOutType,
+                    strikeOutStyle, strikeOutText, width, y, fragmentToLineOffset,
+                    startOfFragmentInBlock);
         } else {
             if (strikeOutText.isEmpty())
                 drawDecorationLine(painter, color, strikeOutType, strikeOutStyle, width, x1, x2, y);
@@ -1419,27 +1497,45 @@ static void drawStrikeOuts(QPainter *painter, const QTextFragment& currentFragme
     }
 }
 
-static void drawUnderlines(QPainter *painter, const QTextFragment& currentFragment, const QTextLine& line, qreal x1, qreal x2)
+void Layout::drawUnderlines(QPainter *painter, const QTextFragment &currentFragment, const QTextLine &line, qreal x1, qreal x2, const int startOfFragmentInBlock, const int fragmentToLineOffset) const
 {
     QTextCharFormat fmt = currentFragment.charFormat();
     KoCharacterStyle::LineStyle fontUnderLineStyle = (KoCharacterStyle::LineStyle) fmt.intProperty(KoCharacterStyle::UnderlineStyle);
     KoCharacterStyle::LineType fontUnderLineType = (KoCharacterStyle::LineType) fmt.intProperty(KoCharacterStyle::UnderlineType);
     if ((fontUnderLineStyle != KoCharacterStyle::NoLineStyle) &&
             (fontUnderLineType != KoCharacterStyle::NoLineType)) {
-        QFontMetrics metrics(fmt.font());
-        qreal y = line.position().y() + line.ascent() + metrics.underlinePos();
+        QTextCharFormat::VerticalAlignment valign = fmt.verticalAlignment();
+
+        QFont font(fmt.font());
+        if (valign == QTextCharFormat::AlignSubScript
+                || valign == QTextCharFormat::AlignSuperScript)
+            font.setPointSize(font.pointSize() * 2 / 3);
+        QFontMetricsF metrics(font, m_parent->paintDevice());
+
+        qreal y = line.position().y();
+        if (valign == QTextCharFormat::AlignSubScript)
+            y += line.height() - metrics.descent() + metrics.underlinePos();
+        else if (valign == QTextCharFormat::AlignSuperScript)
+            y += metrics.ascent() + metrics.underlinePos();
+        else
+            y += line.ascent() + metrics.underlinePos();
+
         QColor color = fmt.underlineColor();
         if (!color.isValid())
             color = fmt.foreground().color();
         KoCharacterStyle::LineMode underlineMode =
             (KoCharacterStyle::LineMode) fmt.intProperty(KoCharacterStyle::UnderlineMode);
-        qreal width = computeWidth(
+        qreal width = computeWidth( // line thickness
                           (KoCharacterStyle::LineWeight) fmt.intProperty(KoCharacterStyle::UnderlineWeight),
                           fmt.doubleProperty(KoCharacterStyle::UnderlineWidth),
-                          fmt.font());
+                          font);
+        if (valign == QTextCharFormat::AlignSubScript
+                || valign == QTextCharFormat::AlignSuperScript) // adjust size.
+            width = width * 2 / 3;
+
         if (underlineMode == KoCharacterStyle::SkipWhiteSpaceLineMode) {
             drawDecorationWords(painter, line, currentFragment.text(), color, fontUnderLineType,
-                    fontUnderLineStyle, QString(), width, y);
+                    fontUnderLineStyle, QString(), width, y, fragmentToLineOffset, startOfFragmentInBlock);
         } else {
             drawDecorationLine(painter, color, fontUnderLineType, fontUnderLineStyle, width, x1, x2, y);
         }
@@ -1472,6 +1568,7 @@ void Layout::decorateParagraph(QPainter *painter, const QTextBlock &block, int s
             int firstLine = layout->lineForTextPosition(currentFragment.position() - startOfBlock).lineNumber();
             int lastLine = layout->lineForTextPosition(currentFragment.position() + currentFragment.length()
                     - startOfBlock).lineNumber();
+            int startOfFragmentInBlock = currentFragment.position() - startOfBlock;
             for (int i = firstLine ; i <= lastLine ; i++) {
                 QTextLine line = layout->lineAt(i);
                 if (layout->isValidCursorPosition(currentFragment.position() - startOfBlock)) {
@@ -1479,8 +1576,11 @@ void Layout::decorateParagraph(QPainter *painter, const QTextBlock &block, int s
                     qreal x2 = line.cursorToX(currentFragment.position() + currentFragment.length() - startOfBlock);
                     x2 = qMin(line.naturalTextWidth() + line.cursorToX(line.textStart()), x2);
 
-                    drawStrikeOuts(painter, currentFragment, line, x1, x2);
-                    drawUnderlines(painter, currentFragment, line, x1, x2);
+                    // sometimes a fragment starts in the middle of a line, so calc offset
+                    int fragmentToLineOffset = qMax(currentFragment.position() - startOfBlock - line.textStart(),0);
+
+                    drawStrikeOuts(painter, currentFragment, line, x1, x2, startOfFragmentInBlock, fragmentToLineOffset);
+                    drawUnderlines(painter, currentFragment, line, x1, x2, startOfFragmentInBlock, fragmentToLineOffset);
                     decorateTabs(painter, tabList, line, currentFragment, startOfBlock);
 
                     bool misspelled = fmt.boolProperty(KoCharacterStyle::Spelling);
@@ -1496,7 +1596,7 @@ void Layout::decorateParagraph(QPainter *painter, const QTextBlock &block, int s
     painter->setFont(oldFont);
 }
 
-void Layout::drawListItem(QPainter *painter, const QTextBlock &block)
+void Layout::drawListItem(QPainter *painter, const QTextBlock &block, KoImageCollection *imageCollection)
 {
     KoTextBlockData *data = dynamic_cast<KoTextBlockData*>(block.userData());
     if (data == 0)
@@ -1510,6 +1610,12 @@ void Layout::drawListItem(QPainter *painter, const QTextBlock &block)
         if (m_styleManager) {
             const int id = listFormat.intProperty(KoListStyle::CharacterStyleId);
             KoCharacterStyle *cs = m_styleManager->characterStyle(id);
+            if (!cs) {
+                KoParagraphStyle *ps = m_styleManager->paragraphStyle(
+                                       block.blockFormat().intProperty(KoParagraphStyle::StyleId));
+                if (ps)
+                    cs = ps->characterStyle();
+            }
             if (cs) {
                 cs->applyStyle(cf);
                 filled = true;
@@ -1546,6 +1652,7 @@ void Layout::drawListItem(QPainter *painter, const QTextBlock &block)
             layout.setTextOption(option);
             layout.beginLayout();
             QTextLine line = layout.createLine();
+            line.setLineWidth(data->counterWidth());
             layout.endLayout();
             QPointF counterPosition = data->counterPosition();
             if (block.layout()->lineCount() > 0) {
@@ -1639,6 +1746,16 @@ void Layout::drawListItem(QPainter *painter, const QTextBlock &block)
                 break;
             default:; // others we ignore.
             }
+        } else if (listStyle == KoListStyle::ImageItem && imageCollection) {
+            QFontMetricsF fm(cf.font(), m_parent->paintDevice());
+            qreal x = qMax(qreal(1), data->counterPosition().x());
+            qreal width = qMax(listFormat.doubleProperty(KoListStyle::Width), (qreal)1.0);
+            qreal height = qMax(listFormat.doubleProperty(KoListStyle::Height), (qreal)1.0);
+            qreal y = data->counterPosition().y() + fm.ascent() - fm.xHeight()/2 - height/2; // centered
+            qint64 key = listFormat.property(KoListStyle::BulletImageKey).value<qint64>();
+            KoImageData idata;
+            imageCollection->fillFromKey(idata, key);
+            painter->drawPixmap(x, y, width, height, idata.pixmap());
         }
     }
 }
@@ -1656,6 +1773,7 @@ bool Layout::setFollowupShape(KoShape *followupShape)
 
     m_newShape = false;
     shape = followupShape;
+    m_textShape = 0;
     m_data->setDocumentOffset(m_y);
     m_shapeBorder = shape->borderInsets();
     return true;
@@ -1679,7 +1797,7 @@ int Layout::cursorPosition() const
     int answer = m_block.position();
     if (!m_newParag && layout && layout->lineCount()) {
         QTextLine tl = layout->lineAt(layout->lineCount() - 1);
-        answer += tl.textStart() + tl.textLength() - 1;
+        answer += tl.textStart() + tl.textLength();
     }
     return answer;
 }
@@ -1709,6 +1827,7 @@ bool Layout::previousParag()
         QList<KoShape *> shapes = m_parent->shapes();
         for (--shapeNumber; shapeNumber >= 0; shapeNumber--) {
             shape = shapes[shapeNumber];
+            m_textShape = dynamic_cast<TextShape*>(shape);
             m_data = qobject_cast<KoTextShapeData*>(shape->userData());
             if (m_data != 0)
                 break;
@@ -1739,7 +1858,7 @@ qreal Layout::inlineCharHeight(const QTextFragment &fragment)
 
 qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
 {
-    if (m_parent->inlineTextObjectManager() == 0)
+    if (m_parent->inlineTextObjectManager() == 0 || m_textShape == 0)
         return 0;
     Q_ASSERT(oldLength);
     QString text = m_block.text();
@@ -1748,17 +1867,34 @@ qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
     while (pos >= 0 && pos <= line.textStart() + line.textLength()) {
         QTextCursor c1(m_block);
         c1.setPosition(m_block.position() + pos);
+        pos = text.indexOf(QChar(0xFFFC), pos + 1);
         c1.setPosition(c1.position() + 1, QTextCursor::KeepAnchor);
         KoInlineNote *note = dynamic_cast<KoInlineNote*>(m_parent->inlineTextObjectManager()->inlineTextObject(c1));
         if (note && note->type() == KoInlineNote::Footnote) {
+            QTextBlock footnoteBlock = m_textShape->footnoteDocument()->begin();
+            bool alreadyPresent = false;
+            while (footnoteBlock.isValid()) {
+                if (footnoteBlock.blockFormat().property(13471293).value<void*>() == note) {
+                    alreadyPresent = true;
+                    break;
+                }
+                footnoteBlock = footnoteBlock.next();
+            }
+            if (alreadyPresent)
+                continue;
             QTextCursor cursor(m_textShape->footnoteDocument());
-            cursor.setPosition(m_textShape->footnoteDocument()->characterCount()-1);
+            cursor.movePosition(QTextCursor::End);
             if (firstFootnote) {
                 (*oldLength) = cursor.position();
                 firstFootnote = false;
             }
             if (cursor.position() > 1)
-                cursor.insertText("\n");
+                cursor.insertBlock();
+            QTextBlockFormat bf;
+            QVariant variant;
+            variant.setValue<void*>(note);
+            bf.setProperty(13471293, variant);
+            cursor.mergeBlockFormat(bf);
 
             QTextCharFormat cf;
             cf.setVerticalAlignment(QTextCharFormat::AlignSuperScript);
@@ -1766,9 +1902,8 @@ qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
             cursor.insertText(note->label() + ' ');
             cf.setVerticalAlignment(QTextCharFormat::AlignNormal);
             cursor.mergeCharFormat(cf);
-            cursor.insertText(note->text());
+            cursor.insertFragment(note->text());
         }
-        pos = text.indexOf(QChar(0xFFFC), pos + 1);
     }
     if (m_textShape->hasFootnoteDocument())
         return m_textShape->footnoteDocument()->size().height();
