@@ -62,6 +62,7 @@ private slots:
     void testNumberedList();
     void testInterruptedLists(); // consecutiveNumbering
     void testNestedLists();
+    void testNestedPrefixedLists();
     void testAutoRestartList();
     void testListParagraphIndent();
     void testRomanNumbering();
@@ -70,6 +71,8 @@ private slots:
     void testRightToLeftList();
     void testLetterSynchronization();
     void testInvalidateLists();
+    void testCenteredItems();
+    void testMultiLevel();
 
 // relativeBulletSize
 
@@ -83,12 +86,12 @@ private:
     void initForNewTest(const QString &initText = QString());
 
 private:
-    MockTextShape *shape1;
-    QTextDocument *doc;
-    KoTextDocumentLayout *layout;
-    QTextLayout *blockLayout;
-    QString loremIpsum;
-    KoStyleManager *styleManager;
+    MockTextShape *m_shape1;
+    QTextDocument *m_doc;
+    KoTextDocumentLayout *m_layout;
+    QTextLayout *m_blockLayout;
+    QString m_loremIpsum;
+    KoStyleManager *m_styleManager;
     Layout *m_textLayout;
 };
 
@@ -98,7 +101,7 @@ public:
     MockTextShape()
         : TextShape(0)
     {
-        layout = dynamic_cast<KoTextDocumentLayout*>(textShapeData()->document()->documentLayout());
+        layout = qobject_cast<KoTextDocumentLayout*>(textShapeData()->document()->documentLayout());
     }
     void paint(QPainter &painter, const KoViewConverter &converter) {
         Q_UNUSED(painter);
